/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//{{AFX_INCLUDES()
#include "mschart.h"
//}}AFX_INCLUDES
#if !defined(AFX_CHARTDLG_H__F233388B_678E_11D4_948A_00C04F89E392__INCLUDED_)
#define AFX_CHARTDLG_H__F233388B_678E_11D4_948A_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// ChartDlg.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CChartDlg dialog
#include "ChartCtrl.h"
#include "MSChartDefs.h"
#define MAX_DATA_COLORS 10

class CChartDlg : public CDialog
{
    // Construction
public:
    CChartDlg(CWnd* pParent = NULL);   // standard constructor
    virtual ~CChartDlg();
    // Dialog Data
    //{{AFX_DATA(CChartDlg)
    enum { IDD = IDD_CHART_DIALOG };
    CChartCtrl  m_ctlChart;
    //}}AFX_DATA

    void SetTitleText(const CString& strTitle);
    void SetTitleBkColor(COLORREF colTitleBk);
    void SetTitleTextColor(COLORREF colTitleText);
    void SetTitleFont(const CString& strFont);
    void SetPlotBkColor(COLORREF colPlotBk);
    void SetChartData(const VARIANT& varData);
    void SetSeriesStyle(int nSeries,VtChSeriesType nStyle);
    void SetAxisLabel(VtChAxisId nAxis,const CString& strText);
    void SetAxisMinimum(VtChAxisId nAxis,int nMinimum);
    void SetAxisMaximum(VtChAxisId nAxis,int nMaximum);
    void SetAxisLabelFont(const CString&);
    void SetGridColor(COLORREF);
    void SetAxisLabelColor(COLORREF);
    void SetAxisTextColor(COLORREF);
    void ShowLegend(BOOL bShow);
    void SetLegendText(int nSeries,const CString& strText);
    void SetDataPoint(int nIndex,int nSeries,double dValue);
    void SetDataColors(const VARIANT& varColors);
    void SetSeriesOnSecondaryAxis(int nSeries,BOOL bSet);
    void SetZoomRange(int nStartPoint,int nEndPoint);
    void SetSeriesAnnotation(int nSeries,CString lpszNewValue);
    void RemoveSeries(CString strSeries);

    void GetLegendText(int nSeries,CString& strText);
    void GetTitleText(CString& strTitle);
    COLORREF GetTitleBkColor();
    COLORREF GetPlotBkColor();
    VtChSeriesType GetSeriesStyle(int nSeries);
    void GetAxisLabel(VtChAxisId nAxis,CString& strText);
    int GetAxisMinimum(VtChAxisId nAxis);
    int GetAxisMaximum(VtChAxisId nAxis);
    COLORREF GetTitleTextColor();
    void GetTitleFont(CString& strFont);
    void GetAxisLabelFont(CString&);
    COLORREF GetGridColor();
    COLORREF GetAxisLabelColor();
    COLORREF GetAxisTextColor();
    double GetDataPoint(int nIndex,int nSeries);
    void GetDataColors(VARIANT& varColors);
    BOOL GetSeriesOnSecondaryAxis(int nSeries);
    void GetSeriesAnnotation(int nSeries,CString& lpszNewValue);
    void GetChartData(VARIANT& vaData);

    void AddDataPoint(int nSeries,double dValue);
    void CopyToClipboard();

    void EnableZoom(BOOL bEnable)
    {
        m_bZoomEnabled = bEnable;
    }
    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CChartDlg)
protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    //}}AFX_VIRTUAL

    // Implementation
protected:

    // Generated message map functions
    //{{AFX_MSG(CChartDlg)
    virtual BOOL OnInitDialog();
    afx_msg BOOL OnEraseBkgnd(CDC *pDC);
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg void OnMouseMove(UINT nFlags, CPoint point);
    afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
    DECLARE_EVENTSINK_MAP()
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()

    const COleVariant covOptional;
    int m_nCurrentGroup;
    COLORREF m_colBackground;
    COLORREF m_colData[MAX_DATA_COLORS];

    void LayoutChart();

    int m_nStep;
    CPoint m_ptZoomSelStart;

    typedef struct
    {
        double dValue;
        bool bNull;
    }
    DataPoint;

    typedef CList <DataPoint,DataPoint&> DataList;

    CArray <DataList*,DataList*> m_arGroupData;

    CStringArray m_arAnnotation;
    CStringArray m_arSeriesAnnotation;

    void UpdateChartData();

    int m_nZoomStart,m_nZoomEnd;

    int m_nDragStart;
    int m_nPrevPos;
    int GetDataPointFromPoint(int nXpos);

    BOOL m_bZoomEnabled;

    CBitmap m_bmpScreen;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CHARTDLG_H__F233388B_678E_11D4_948A_00C04F89E392__INCLUDED_)
