/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#if !defined(AFX_INJECTORMONDLG_H__0C210C5E_0BC9_11D4_BD8C_00C04F89E392__INCLUDED_)
#define AFX_INJECTORMONDLG_H__0C210C5E_0BC9_11D4_BD8C_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// InjectorMonDlg.h : header file
//
#include "CORBA/CORBAClient.h"
#include "TestManager.h"
#include "Repository/CyrStRepository.h"
#include "CYRWrapLib\CECTestOutput.h"

#define WMU_DYN_GRAPH                       (WM_USER+9602)
#define WMU_UPDATE_TITLE                    (WM_USER+9603)

class CInjectorData;
/////////////////////////////////////////////////////////////////////////////
// CInjectorMonDlg dialog

class CInjectorMonDlg : public CDialog
{
    typedef enum {enInjectorBmp, enAboveThresholdBmp,enBelowThresholdBmp} InjectorListImages;

    typedef enum { enNameCol,enIPCol,enCPUCol,enSwapCol,enNetCol}InjectorListColumns;
    //      enTestCol,enStartCol,enPIDCol,enVUsCol,enTotalVUsCol}

public:
    typedef enum {enCPUType,enNetType,enSwapType,enNumDefQueryTypes} DefQueryTypes;

    // Construction
public:
    CInjectorMonDlg(CWnd* pParent = NULL);   // standard constructor
    virtual ~CInjectorMonDlg();

    void StartMonitor(CString strRegName);
    void SetRefreshDelay(int nDelay);
    // Dialog Data
    //{{AFX_DATA(CInjectorMonDlg)
    enum { IDD = IDD_INJECTOR_MON_DLG };
    CListCtrl   m_ctlInjectorList;
    //}}AFX_DATA

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CInjectorMonDlg)
protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    //}}AFX_VIRTUAL

    // Implementation
protected:

    // Generated message map functions
    //{{AFX_MSG(CInjectorMonDlg)
    virtual BOOL OnInitDialog();
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg void OnInjectorsSelectColumns();
    afx_msg void OnDestroy();
    afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
    afx_msg void OnPopupShowgraphs();
    afx_msg void OnUpdatePopupShowgraphs(CCmdUI* pCmdUI);
    afx_msg BOOL OnHelpInfo(HELPINFO* pHelpInfo);
    afx_msg void OnFullHelp();
    //}}AFX_MSG
    afx_msg LRESULT OnDataReady(WPARAM,LPARAM);
    DECLARE_MESSAGE_MAP()

private:
    CString m_szHtmlHelpFile;

    CORBAClient* m_pCorbaClt;
    CyrStTestRun_var m_TestRun;
    int m_nOTADataIndex;
    int m_nOTAHeaderIndex;
    CImageList m_InjectorImages;
    CString m_strManagerName;

    volatile BOOL m_bQuitting;
    volatile int m_nRefreshDelay;

    CMapStringToPtr m_mapIPtoInjector;
    CMapStringToPtr m_mapNdxToInjector;
    CMapStringToString m_mapIPToName;

    void InitOTADataIndexes();
    BOOL IsOTADescriptionLine(CString strOTALine);
    void AddOTAQueryType(const CString& strInjectorIP,const CString& strQueryType,int nDataNdx);
    CString GetComputerNameFromIP(const CString& strIP);
    void UpdateInjectorList();
    void FilterColumns(CListCtrl *pListCtrl);
    void LoadThresholds();
    BOOL IsShuttingDown();

    void SaveSettings();
    void RestoreSettings();

    static UINT DataRefreshThread(LPVOID lpParam);
    CWinThread *m_pRefreshThread;

    void GetNodeList(TestLevel_ptr pTestLevel);
};

class CInjectorData
{
public:
    typedef struct
    {
        int nNdx;
        double dValue;
        double dThreshold;
    }
    OtaData;

public:
    CInjectorData()
    {
        for(int nNdx=0;nNdx<CInjectorMonDlg::enNumDefQueryTypes;nNdx++)
        {
            m_arOTAData[nNdx].nNdx=-1;
            m_arOTAData[nNdx].dValue=-1;
            m_arOTAData[nNdx].dThreshold = -1;
        }
        m_nListItem=-1;
    }

    CString strName;
    CString strIPAddress;
    int m_nListItem;
    OtaData m_arOTAData[CInjectorMonDlg::enNumDefQueryTypes];
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_INJECTORMONDLG_H__0C210C5E_0BC9_11D4_BD8C_00C04F89E392__INCLUDED_)
