/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MultipleNTQueryDlg.cpp : implementation file
//

#include "stdafx.h"
#include "xotaeditor.h"
#include "MultipleNTQueryDlg.h"
#include "GenericPage.h"
#include "..\..\HTMLHelp\XOTAEditor\Multiple_NT.inl"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMultipleNTQueryDlg dialog


CMultipleNTQueryDlg::CMultipleNTQueryDlg(CGenericPage* pParent,const CString& strQuery)
        : CDialog(CMultipleNTQueryDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CMultipleNTQueryDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_strQuery = strQuery;
    m_pParent = pParent;
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}


void CMultipleNTQueryDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CMultipleNTQueryDlg)
    DDX_Control(pDX, IDC_EDIT_QUERY, m_ctlEditQuery);
    DDX_Control(pDX, IDC_DELETE_QUERY, m_ctlDeleteQuery);
    DDX_Control(pDX, IDC_QUERY_LIST, m_ctlQueryList);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CMultipleNTQueryDlg, CDialog)
//{{AFX_MSG_MAP(CMultipleNTQueryDlg)
ON_BN_CLICKED(IDC_DELETE_QUERY, OnDeleteQuery)
ON_BN_CLICKED(IDC_ADD_QUERY, OnAddQuery)
ON_BN_CLICKED(IDC_EDIT_QUERY, OnEditQuery)
ON_LBN_SELCHANGE(IDC_QUERY_LIST, OnSelchangeQueryList)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMultipleNTQueryDlg message handlers

void CMultipleNTQueryDlg::OnDeleteQuery()
{
    const int nSelCount = m_ctlQueryList.GetSelCount();
    int *pSelection = new int[nSelCount];

    m_ctlQueryList.GetSelItems(nSelCount,pSelection);

    int nCount=0;
    for(int nNdx=0;nNdx<nSelCount;nNdx++)
    {
        m_ctlQueryList.DeleteString(pSelection[nNdx]-nCount++);
    }

    delete pSelection;
    UpdateControls();
}

void CMultipleNTQueryDlg::OnAddQuery()
{
    CString strQuery;
    if((m_pParent!=NULL) && m_pParent->EditQuery(CGenericPage::enNTPerformanceQuery,strQuery))
    {
        if(m_ctlQueryList.FindStringExact(-1,strQuery)==LB_ERR)
        {
            m_ctlQueryList.AddString(strQuery);
        }
    }
}

BOOL CMultipleNTQueryDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    CString strTemp(m_strQuery);

    char *pszToken = strtok(strTemp.GetBuffer(0),"\n");
    while(pszToken!=NULL)
    {
        m_ctlQueryList.AddString(pszToken);
        pszToken = strtok(NULL,"\n");
    }

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CMultipleNTQueryDlg::OnEditQuery()
{
    int nCurSel;
    if(m_ctlQueryList.GetSelItems(1,&nCurSel)!=LB_ERR)
    {
        CString strQuery;
        m_ctlQueryList.GetText(nCurSel,strQuery);
        if((m_pParent!=NULL) && m_pParent->EditQuery(CGenericPage::enNTPerformanceQuery,strQuery))
        {
            m_ctlQueryList.DeleteString(nCurSel);
            if(m_ctlQueryList.FindStringExact(-1,strQuery)==LB_ERR)
            {
                m_ctlQueryList.InsertString(nCurSel,strQuery);
            }
        }
    }
}

void CMultipleNTQueryDlg::OnSelchangeQueryList()
{
    UpdateControls();
}

void CMultipleNTQueryDlg::OnOK()
{
    m_strQuery.Empty();

    CString strQuery;
    int nQueryCount = m_ctlQueryList.GetCount();
    for(int nQuery=0;nQuery<nQueryCount;nQuery++)
    {
        m_ctlQueryList.GetText(nQuery,strQuery);
        m_strQuery += (strQuery + "\n");
    }

    CDialog::OnOK();
}

void CMultipleNTQueryDlg::UpdateControls()
{
    int nSelCount = m_ctlQueryList.GetSelCount();

    //Enable the edit button if only one query is selected
    m_ctlEditQuery.EnableWindow(nSelCount == 1);

    //Enable the delete button if one or more items are selected
    m_ctlDeleteQuery.EnableWindow(nSelCount >= 1);
}

BOOL CMultipleNTQueryDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
