/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XOTAEditorCtl.cpp : Implementation of the CXOTAEditorCtrl ActiveX Control class.

#include "stdafx.h"
#include "XOTAEditor.h"
#include "XOTAEditorCtl.h"
#include "XOTAEditorPpg.h"
#include "TScript.h"
#include "mdbexcept.h"
#include <direct.h>
#include "..\..\studio\interfaces\cyrplugins.h"
#include "mdb_ctime.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CXOTAEditorCtrl, COleControl)


/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CXOTAEditorCtrl, COleControl)
//{{AFX_MSG_MAP(CXOTAEditorCtrl)
ON_WM_CREATE()
ON_WM_SIZE()
ON_WM_ERASEBKGND()
//}}AFX_MSG_MAP
ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CXOTAEditorCtrl, COleControl)
//{{AFX_DISPATCH_MAP(CXOTAEditorCtrl)
DISP_FUNCTION(CXOTAEditorCtrl, "LoadTOF", LoadTOF, VT_BOOL, VTS_VARIANT)
DISP_FUNCTION(CXOTAEditorCtrl, "SaveTOF", SaveTOF, VT_BOOL, VTS_VARIANT)
//}}AFX_DISPATCH_MAP
DISP_FUNCTION_ID(CXOTAEditorCtrl, "AboutBox", DISPID_ABOUTBOX, AboutBox, VT_EMPTY, VTS_NONE)
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CXOTAEditorCtrl, COleControl)
//{{AFX_EVENT_MAP(CXOTAEditorCtrl)
// NOTE - ClassWizard will add and remove event map entries
//    DO NOT EDIT what you see in these blocks of generated code !
//}}AFX_EVENT_MAP
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CXOTAEditorCtrl, 1)
PROPPAGEID(CXOTAEditorPropPage::guid)
END_PROPPAGEIDS(CXOTAEditorCtrl)


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CXOTAEditorCtrl, "XOTAEDITOR.XOTAEditorCtrl.1",
                       0x1d6cac76, 0x9cd6, 0x11d3, 0xbd, 0x5b, 0, 0xc0, 0x4f, 0x89, 0xe3, 0x92)


/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CXOTAEditorCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DXOTAEditor =
    {
        0x1d6cac74, 0x9cd6, 0x11d3, { 0xbd, 0x5b, 0, 0xc0, 0x4f, 0x89, 0xe3, 0x92 }
    };
const IID BASED_CODE IID_DXOTAEditorEvents =
    {
        0x1d6cac75, 0x9cd6, 0x11d3, { 0xbd, 0x5b, 0, 0xc0, 0x4f, 0x89, 0xe3, 0x92 }
    };


/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwXOTAEditorOleMisc =
    OLEMISC_ACTIVATEWHENVISIBLE |
    OLEMISC_SETCLIENTSITEFIRST |
    OLEMISC_INSIDEOUT |
    OLEMISC_CANTLINKINSIDE |
    OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CXOTAEditorCtrl, IDS_XOTAEDITOR, _dwXOTAEditorOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CXOTAEditorCtrl::CXOTAEditorCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CXOTAEditorCtrl

BOOL CXOTAEditorCtrl::CXOTAEditorCtrlFactory::UpdateRegistry(BOOL bRegister)
{
    // TODO: Verify that your control follows apartment-model threading rules.
    // Refer to MFC TechNote 64 for more information.
    // If your control does not conform to the apartment-model rules, then
    // you must modify the code below, changing the 6th parameter from
    // afxRegApartmentThreading to 0.

    if (bRegister)
        return AfxOleRegisterControlClass(
                   AfxGetInstanceHandle(),
                   m_clsid,
                   m_lpszProgID,
                   IDS_XOTAEDITOR,
                   IDB_XOTAEDITOR,
                   afxRegApartmentThreading,
                   _dwXOTAEditorOleMisc,
                   _tlid,
                   _wVerMajor,
                   _wVerMinor);
    else
        return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CXOTAEditorCtrl::CXOTAEditorCtrl - Constructor

CXOTAEditorCtrl::CXOTAEditorCtrl()
{
    m_pRules = NULL;
    m_pHeaderRecord = NULL;

    InitializeIIDs(&IID_DXOTAEditor, &IID_DXOTAEditorEvents);
}


/////////////////////////////////////////////////////////////////////////////
// CXOTAEditorCtrl::~CXOTAEditorCtrl - Destructor

CXOTAEditorCtrl::~CXOTAEditorCtrl()
{
    // TODO: Cleanup your control's instance data here.
    delete m_pRules;
    m_pRules = NULL;
    delete m_pHeaderRecord;
    m_pHeaderRecord=NULL;
}


/////////////////////////////////////////////////////////////////////////////
// CXOTAEditorCtrl::OnDraw - Drawing function

void CXOTAEditorCtrl::OnDraw(
    CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
    if(!m_EditorSheet.GetSafeHwnd())
    {
        pdc->FillRect(rcBounds, CBrush::FromHandle((HBRUSH)GetStockObject(GRAY_BRUSH)));
        pdc->TextOut(0,0,"XOTAEditor OCX");
    }
}


/////////////////////////////////////////////////////////////////////////////
// CXOTAEditorCtrl::DoPropExchange - Persistence support

void CXOTAEditorCtrl::DoPropExchange(CPropExchange* pPX)
{
    ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
    COleControl::DoPropExchange(pPX);

    // TODO: Call PX_ functions for each persistent custom property.

}


/////////////////////////////////////////////////////////////////////////////
// CXOTAEditorCtrl::OnResetState - Reset control to default state


void CXOTAEditorCtrl::OnResetState()
{
    COleControl::OnResetState();  // Resets defaults found in DoPropExchange

    // TODO: Reset any other control state here.
}


/////////////////////////////////////////////////////////////////////////////
// CXOTAEditorCtrl::AboutBox - Display an "About" box to the user


void CXOTAEditorCtrl::AboutBox()
{
    CDialog dlgAbout(IDD_ABOUTBOX_XOTAEDITOR);
    dlgAbout.DoModal();
}


/////////////////////////////////////////////////////////////////////////////
// CXOTAEditorCtrl message handlers

int CXOTAEditorCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (COleControl::OnCreate(lpCreateStruct) == -1)
        return -1;
    //Look for odl at CYRANO_HOME\\Engines\\Mon\\ota.odl
    CString strHomeDir = getenv("CYRANO_HOME");
    string szOdlFile = "ota.odl";
    if(!strHomeDir.IsEmpty())
    {
        int nLength = strHomeDir.GetLength();
        if(strHomeDir[nLength-1]!='\\')
        {
            strHomeDir += "\\";
        }

        szOdlFile = strHomeDir + "Engines\\Mon\\ota.odl";
    }

    try
    {
        m_pRules = new TScriptRule(szOdlFile);
    }
    catch(mdbIOException& e)
    {
        string strWhat = e.what();

        CString strMessage("An error occurred opening the file ""ota.odl""\n\nError message: ");
        strMessage += strWhat.c_str();

        MessageBox(strMessage, "OpenSTA",MB_ICONSTOP);
        delete m_pRules;
        m_pRules = NULL;
        return -1;
    }

    ModifyStyleEx (0, WS_EX_CONTROLPARENT);

    m_EditorSheet.Construct("OTA Editor",this);
    m_EditorSheet.Create(this,WS_CHILD|WS_VISIBLE);

    //ensure all the pages have been created before trying to fill the grids
    m_EditorSheet.SetRedraw(FALSE);
    int nPageCount = m_EditorSheet.GetPageCount();
    for(int nPage=nPageCount;nPage>=0;nPage--)
    {
        m_EditorSheet.SetActivePage(nPage);
    }
    m_EditorSheet.SetRedraw(TRUE);


    return 0;
}


void CXOTAEditorCtrl::OnSize(UINT nType, int cx, int cy)
{
    COleControl::OnSize(nType, cx, cy);

    //Size the editor dialog to fit the control
    if(m_EditorSheet.GetSafeHwnd())
    {
        m_EditorSheet.MoveWindow(0,0,cx,cy,TRUE);
    }
}

BOOL CXOTAEditorCtrl::PreTranslateMessage(MSG* pMsg)
{
    //Give editor dialog first go at messages
    if(m_EditorSheet.PreTranslateMessage(pMsg))
        return true;

    return COleControl::PreTranslateMessage(pMsg);
}

//
//  ReadTOF
//  =======
//
//  Load the data from a TOF file into the grids
//
BOOL CXOTAEditorCtrl::ReadTOF(const CString & strTOF)
{
    bool bResult = false;
    TScript *pScript=NULL;
    try
    {
        AtvfInput atvfInput(strTOF);
        pScript = new TScript(m_pRules,atvfInput);

        //Make a copy of the header record for use if
        //we save this file
        TRecord *pHeader = pScript->header();
        m_pHeaderRecord = pHeader->duplicate();

        //Loop through loading all records into editor
        UINT nNumRecords = pScript->length();
        for(int nRecord=0;nRecord<nNumRecords;nRecord++)
        {
            TRecord *pRecord = (*pScript)[nRecord];

            CString strRecordName = pRecord->token().c_str();

            CStringArray arstrParams;
            UINT uiNumParams = pRecord->numParams();
            for(int nParam=0;nParam<uiNumParams;nParam++)
            {
                const TParam *pParam = pRecord->param(nParam);

                arstrParams.Add(pParam->token().c_str());
                arstrParams.Add(pParam->strVal(0).c_str());
            }

            m_EditorSheet.AddDataEntry(strRecordName,arstrParams);
        }
        bResult=true;
    }
    catch(mdbParseException& e)
    {
        string strWhat = e.what();
        CString strMessage;
        strMessage.Format("An error occurred opening the file \n\n\"%s\"\n\nError message: %s",
                          strTOF,
                          strWhat.c_str());

        MessageBox(strMessage, "OpenSTA",MB_ICONSTOP);
        bResult=false;
    }
    catch(mdbIOException& e)
    {
        string strWhat = e.what();
        CString strMessage;
        strMessage.Format("An error occurred opening the file \n\n\"%s\"\n\nError message: %s",
                          strTOF,
                          strWhat.c_str());

        MessageBox(strMessage, "OpenSTA",MB_ICONSTOP);
        bResult=false;
    }

    delete pScript;

    return bResult;
}

//
//  WriteTOF
//  ========
//
// Write the data from the grids into a TOF file
//
BOOL CXOTAEditorCtrl::WriteTOF(const CString& strTOF)
{
    TScript *pScript = new TScript(m_pRules);

    TRecord *pHeader = pScript->header();

    //get the current date and time
    string strDate = mdb_ctime();

    if(m_pHeaderRecord == NULL)
    {
        //There is no existing header to copy so create a new one
        pHeader->setPrmVal(strDate,"date");
        pHeader->setPrmVal("","name");
        pHeader->setPrmVal("","description");
    }
    else
    {
        //Copy the existing header
        UINT uiNumParams = m_pHeaderRecord->numParams();
        for(int nParam=0;nParam<uiNumParams;nParam++)
        {
            const TParam *pParam = m_pHeaderRecord->param(nParam);

            pHeader->setPrmVal(pParam->strVal(0).c_str(),pParam->token().c_str());
        }
    }

    //Set last modified date
    pHeader->setPrmVal(strDate, string("lastedit"));

    //
    // Loop through the editors pages, retrieving all the data as we go...
    //
    int nNumPages = m_EditorSheet.GetPageCount();
    for(int nPage=0;nPage<nNumPages;nPage++)
    {
        CGenericPage *pPage = dynamic_cast<CGenericPage *>(m_EditorSheet.GetPage(nPage));

        if(pPage == NULL)
        {
            ASSERT(FALSE);
            continue;
        }

        //Get the token name for the records on this page
        string strToken = pPage->GetTokenName();

        //
        //Look for the key which corresponds to this token
        //(we need the key in order to create new records)
        //
        keyvect_t *pKeys = m_pRules->getRecordsKeys();
        for(keyvect_t::iterator atvfkey = pKeys->begin();atvfkey != pKeys->end(); atvfkey++)
        {
            //Create a temp working record
            TRecord *pTempRecord = m_pRules->makeNewRecord(*atvfkey);

            //If the tokens match we've found the key
            if(strToken == pTempRecord->token())
            {
                delete pTempRecord;
                break;
            }
            delete pTempRecord;
        }

        //
        //  loop through all the data entries on this page
        //
        CStringArray arstrParams;
        int nNumDataEntries = pPage->GetNumDataEntries();
        for(int nEntry = 0;nEntry<nNumDataEntries;nEntry++)
        {
            //Create a new record
            TRecord *pRecord = m_pRules->makeNewRecord(*atvfkey);

            //
            // Get some data from the page
            //(The data array "arstrParams is organised as {Key1,value1,key2,value2...}
            //
            arstrParams.RemoveAll();
            if(!pPage->GetDataEntry(nEntry,arstrParams))
            {
                //something went wrong on this item so move onto the next
                continue;
            }

            //
            //loop through the entries in this data entry
            //
            int nNumKeys = arstrParams.GetSize();
            for(int nKey=0;nKey<nNumKeys;nKey+=2)
            {
                string strKey = arstrParams[nKey];
                string strValue = arstrParams[nKey+1];

                //Add a parameter to the record
                pRecord->setPrmVal(strValue,strKey);
            }

            //Add this record to the script
            pScript->add(pRecord);
        }
    }

    CString strTofFilename = strTOF;
    ofstream tofFile(strTofFilename.GetBuffer(0));
    pScript->save(tofFile);

    delete pScript;

    return true;
}

BOOL CXOTAEditorCtrl::LoadTOF(const VARIANT FAR& varPath)
{
    //Check the odl was read properly
    if(m_pRules == NULL)
        return false;

    //Remove the stored header record
    if(m_pHeaderRecord!=NULL)
    {
        delete m_pHeaderRecord;
        m_pHeaderRecord = NULL;
    }

    //Remove all the old data from the editor dialog
    //  ASSERT(FALSE);
    //  m_EditorDlg.ResetData();

    //If an empty file name is passed in just clear the data
    CString strTOF(varPath.bstrVal);
    if(strTOF.IsEmpty())
    {
        return true;
    }

    // KH   - Temp take out read coz it GPFs'
    return ReadTOF(strTOF);
    //  return false;
}


BOOL CXOTAEditorCtrl::SaveTOF(const VARIANT FAR& varPath)
{
    //Check the odl was read properly
    if(m_pRules == NULL)
        return false;

    return WriteTOF(varPath.bstrVal);
}


BOOL CXOTAEditorCtrl::OnEraseBkgnd(CDC* pDC)
{
    return true;
}
