/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// @doc     Document
// @module  CXProjectViewerCtrl | ActiveX Control Main class, which handles
//          method calls and fires events.
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_XPROJECTVIEWERCTL_H__8E709F74_F8A3_11D2_BDC5_00608CB45114__INCLUDED_)
#define AFX_XPROJECTVIEWERCTL_H__8E709F74_F8A3_11D2_BDC5_00608CB45114__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// XProjectViewerCtl.h : Declaration of the CXProjectViewerCtrl ActiveX Control class.
#include "CECProjectSelector.h"
#include "CECXFloorDlg.h"

class CYRServer;
/////////////////////////////////////////////////////////////////////////////
// CXProjectViewerCtrl : See XProjectViewerCtl.cpp for implementation.

// @class Project Viewer Control class
// @base public | COleControl
class CXProjectViewerCtrl : public COleControl
{
    DECLARE_DYNCREATE(CXProjectViewerCtrl)

    // @access Public Members
public:
    // @cmember Constructor
    CXProjectViewerCtrl();

    // @cmember Project Selection Dialog
    CECProjectSelector m_Selector;

    // @cmember XFloorWnd dialog
    CECXFloorDlg m_XFloorDlg;

    // @cmember Connects to the repository server if the connection is not already made
    BOOL IsServerRunning();

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CXProjectViewerCtrl)
public:
    virtual void OnDraw(CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid);
    virtual void DoPropExchange(CPropExchange* pPX);
    virtual void OnResetState();
    //}}AFX_VIRTUAL

    // @access Protected Members
protected:
    // @cmember Desstructor
    ~CXProjectViewerCtrl();

    DECLARE_OLECREATE_EX(CXProjectViewerCtrl)    // Class factory and guid
    DECLARE_OLETYPELIB(CXProjectViewerCtrl)      // GetTypeInfo
    DECLARE_PROPPAGEIDS(CXProjectViewerCtrl)     // Property page IDs
    DECLARE_OLECTLTYPE(CXProjectViewerCtrl)     // Type name and misc status

    // Message maps
    //{{AFX_MSG(CXProjectViewerCtrl)
    afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg BOOL OnEraseBkgnd(CDC* pDC);
    afx_msg void OnPaint();
    //}}AFX_MSG
    afx_msg LRESULT OnProjectChanged(WPARAM,LPARAM);
    afx_msg LRESULT OnExecuteTest(WPARAM,LPARAM);
    afx_msg LRESULT OnOpenTestRun(WPARAM,LPARAM);
    afx_msg LRESULT OnEditFile(WPARAM,LPARAM);
    afx_msg LRESULT OnViewResults(WPARAM,LPARAM);
    afx_msg LRESULT OnNewTest(WPARAM,LPARAM);
    afx_msg LRESULT OnNewScript(WPARAM,LPARAM);
    afx_msg LRESULT OnExportFile(WPARAM,LPARAM);
    afx_msg LRESULT OnOpenFile(WPARAM,LPARAM);
    afx_msg LRESULT OnViewHttpStats(WPARAM,LPARAM);
    DECLARE_MESSAGE_MAP()

    // Dispatch maps
    //{{AFX_DISPATCH(CXProjectViewerCtrl)
    afx_msg long GetServer();
    afx_msg void SetServer(long nNewValue);
    afx_msg void SetActiveProject(LPCTSTR ProjectName);
    afx_msg void XInitialize(LPCTSTR strProjType, LPCTSTR strConfig);
    afx_msg void XRefresh();
    afx_msg void XEnableMenus(LPCTSTR strPage, BOOL bEnabled);
    //}}AFX_DISPATCH
    DECLARE_DISPATCH_MAP()

    afx_msg void AboutBox();

    // Event maps
    //{{AFX_EVENT(CXProjectViewerCtrl)
    void FireXChangeActiveProject(LPCTSTR ProjectName)
    {
        FireEvent(eventidXChangeActiveProject,EVENT_PARAM(VTS_BSTR), ProjectName);
    }
    void FireXExecuteTest(LPCTSTR TestName)
    {
        FireEvent(eventidXExecuteTest,EVENT_PARAM(VTS_BSTR), TestName);
    }
    void FireXOpenTestRun(LPCTSTR TestName, LPCTSTR TestRunName)
    {
        FireEvent(eventidXOpenTestRun,EVENT_PARAM(VTS_BSTR  VTS_BSTR), TestName, TestRunName);
    }
    void FireXEditFile(LPCTSTR ScriptName)
    {
        FireEvent(eventidXEditFile,EVENT_PARAM(VTS_BSTR), ScriptName);
    }
    void FireXViewResults(LPCTSTR strTestName, LPCTSTR strTestRunName, short nType)
    {
        FireEvent(eventidXViewResults,EVENT_PARAM(VTS_BSTR  VTS_BSTR  VTS_I2), strTestName, strTestRunName, nType);
    }
    void FireXRequestNewTest()
    {
        FireEvent(eventidXRequestNewTest,EVENT_PARAM(VTS_NONE));
    }
    void FireXRequestNewScript()
    {
        FireEvent(eventidXRequestNewScript,EVENT_PARAM(VTS_NONE));
    }
    void FireXExportFile(LPCTSTR strFileName)
    {
        FireEvent(eventidXExportFile,EVENT_PARAM(VTS_BSTR), strFileName);
    }
    void FireXOpenFile(LPCTSTR strFileName)
    {
        FireEvent(eventidXOpenFile,EVENT_PARAM(VTS_BSTR), strFileName);
    }
    void FireXViewHttpStats(LPCTSTR TestName, LPCTSTR TestRunName)
    {
        FireEvent(eventidXViewHttpStats,EVENT_PARAM(VTS_BSTR  VTS_BSTR), TestName, TestRunName);
    }
    //}}AFX_EVENT
    DECLARE_EVENT_MAP()

    // Dispatch and event IDs
    // @access Public Members
public:
    enum {
        //{{AFX_DISP_ID(CXProjectViewerCtrl)
        dispidServer = 1L,
        dispidSetActiveProject = 2L,
        dispidXInitialize = 3L,
        dispidXRefresh = 4L,
        dispidXEnableMenus = 5L,
        eventidXChangeActiveProject = 1L,
        eventidXExecuteTest = 2L,
        eventidXOpenTestRun = 3L,
        eventidXEditFile = 4L,
        eventidXViewResults = 5L,
        eventidXRequestNewTest = 6L,
        eventidXRequestNewScript = 7L,
        eventidXExportFile = 8L,
        eventidXOpenFile = 9L,
        eventidXViewHttpStats = 10L,
        //}}AFX_DISP_ID
    };

    // @access Private Members
private:
    // @cmember Reference to the repository server
    CYRServer*  m_pServer;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_XPROJECTVIEWERCTL_H__8E709F74_F8A3_11D2_BDC5_00608CB45114__INCLUDED)
