/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CXSourceTree.cpp : implementation file
//

#include "stdafx.h"
#include "XReposTree.h"
#include "CXSourceTree.h"
//#include "PropertiesDlg.h"
#include "..\..\Studio\Common\CECRegistryTools.h"
#include "..\..\Studio\Common\CECXServerError.h"
#include "..\..\Studio\Common\CECTargetEnvironment.h"

#include "StringEx.h"
#include "..\..\include\Solutions.h"
#include "server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define FOLDER_ICON                 0
#define SCL_UNLOCKED_ICON           1
#define SCL_LOCKED_ICON             2

/////////////////////////////////////////////////////////////////////////////
// CXSourceTree dialog


CXSourceTree::CXSourceTree(BOOL bUseSCC, CWnd* pParent /*=NULL*/)
        : CDialog(CXSourceTree::IDD, pParent)
{
    //{{AFX_DATA_INIT(CXSourceTree)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_bUseSCC = bUseSCC;
    m_pServer = NULL;
}


void CXSourceTree::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CXSourceTree)
    DDX_Control(pDX, IDC_XTREE_SOURCE, m_XSourceTree);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CXSourceTree, CDialog)
//{{AFX_MSG_MAP(CXSourceTree)
ON_WM_SIZE()
//}}AFX_MSG_MAP
ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXSourceTree message handlers

BOOL CXSourceTree::OnInitDialog()
{
    CDialog::OnInitDialog();
    AfxEnableControlContainer(NULL);

    //
    // Define node types and context menu entries for each type
    //
    if (m_bUseSCC)
    {
        m_XSourceTree.DefineNodeType(SOURCE_FOLDER, "&New Script,1", FALSE, FALSE);
        m_XSourceTree.DefineNodeType(SCRIPT_FILE,   "E&dit,1|Check &In,0|Check &Out,1|E&xport,1|&Delete,1", FALSE, FALSE);
    }
    else
    {
        m_XSourceTree.DefineNodeType(SOURCE_FOLDER, "&New Script,1", FALSE, FALSE);
        m_XSourceTree.DefineNodeType(SCRIPT_FILE,   "E&dit,1|E&xport,1|&Delete,1", FALSE, FALSE);
    }
    //
    // Add bitmaps to the image list
    //

    // Add folder icon to image list index 0 (#defined to FOLDER_ICON)
    CBitmap* bmp = new CBitmap;
    bmp->LoadBitmap(IDB_FOLDER);
    m_XSourceTree.AddImageToList((long)bmp->m_hObject);
    delete bmp;

    // Add script file icon (unlocked) to image list index 1 (#defined to SCL_UNLOCKED_ICON)
    bmp = new CBitmap;
    bmp->LoadBitmap(IDB_SCL_FILE);
    m_XSourceTree.AddImageToList((long)bmp->m_hObject);
    delete bmp;

    // Add script file icon (locked) to image list index 4 (#defined to SCL_LOCKED_ICON)
    bmp = new CBitmap;
    bmp->LoadBitmap(IDB_SCL_FILE_LOCKED);
    m_XSourceTree.AddImageToList((long)bmp->m_hObject);
    delete bmp;

    //
    // If a project is already selected add a Sourecs folder item.
    //
    if (m_strProject != "<No Project Selected>" && m_strProject != "")
    {
        m_XSourceTree.AddItem(-1, SOURCE_FOLDER ,"Scripts", FOLDER_ICON, TRUE);
    }

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CXSourceTree::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    if( m_XSourceTree.GetSafeHwnd() != NULL )
    {
        m_XSourceTree.SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
}

//
// @mfunc   Initialize the tree control
//
//
void CXSourceTree::Activate()
{
    m_XSourceTree.DeleteAllItems();
    if (m_strProject != "<No Project Selected>" && m_strProject != "")
    {
        m_XSourceTree.AddItem(-1, SOURCE_FOLDER ,"Scripts", 0, TRUE);
    }
}

//
// @mfunc   Add tree items for each source file under the given parent item.
//
//
void CXSourceTree::AddSources(HTREEITEM parent)
{
    //
    // Make sure the parent allows children
    //
    m_XSourceTree.SetChildren((long)parent, TRUE);

    int scriptsAdded = 0;

    //
    //    ...Recover a list of the source in this project...
    //
    CStringArray arstrFiles;
    m_pServer->GetFileList(CYRServer::enSourceFolder,"",arstrFiles);

    //
    //    ...and add them to the tree.
    for(int nNdx=0;nNdx<arstrFiles.GetSize();nNdx++)
    {
        CString sourceName = arstrFiles[nNdx];

        CString strExt = sourceName.Right(3);
        int nLen = sourceName.GetLength();
        CString name = sourceName.Left(nLen-4);

        if (strExt.CompareNoCase("scl") == 0 && name.Right(3) != "-TH")
        {
            if (m_bUseSCC && IsFileLocked(SCRIPT_FILE, sourceName))
            {
                m_XSourceTree.AddItem((long)parent, SCRIPT_FILE, name, SCL_LOCKED_ICON,FALSE);
            }
            else
            {
                m_XSourceTree.AddItem((long)parent, SCRIPT_FILE, name, SCL_UNLOCKED_ICON,FALSE);
            }
            scriptsAdded++;
        }
    }
    //
    // No scripts were added so set the source folder to have no children
    //
    if (scriptsAdded == 0)
    {
        m_XSourceTree.SetChildren((long)parent, FALSE);
    }

}

//
// @mfunc   Initialize a pointer to a data server
//
//
void CXSourceTree::SetServer(CYRServer* pServer)
{
    ASSERT(pServer != NULL);
    m_pServer =pServer;
}


BEGIN_EVENTSINK_MAP(CXSourceTree, CDialog)
//{{AFX_EVENTSINK_MAP(CXSourceTree)
ON_EVENT(CXSourceTree, IDC_XTREE_SOURCE, 2 /* XTreeExpanding */, OnXTreeExpanding, VTS_I4 VTS_I2 VTS_BSTR)
ON_EVENT(CXSourceTree, IDC_XTREE_SOURCE, 1 /* LBDblClick */, OnLBDblClick, VTS_I4 VTS_I2 VTS_BSTR)
ON_EVENT(CXSourceTree, IDC_XTREE_SOURCE, 3 /* XTreeContextClick */, OnXTreeContextClick, VTS_BSTR VTS_I2 VTS_BSTR)
ON_EVENT(CXSourceTree, IDC_XTREE_SOURCE, 4 /* XTreeItemProperties */, OnXTreeItemProperties, VTS_BSTR VTS_I2)
ON_EVENT(CXSourceTree, IDC_XTREE_SOURCE, 5 /* XShowContext */, OnXShowContext, VTS_I2 VTS_BSTR)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()


void CXSourceTree::OnXTreeExpanding(long hTreeItem, short nNodeType, LPCTSTR szPath)
{
    BeginWaitCursor();

    switch (nNodeType)
    {
    case SOURCE_FOLDER:
        {
            AddSources((HTREEITEM)hTreeItem);
            break;
        }
    }

    EndWaitCursor();
}



void CXSourceTree::SetActiveProject(CString strProject)
{
    m_strProject=strProject;
}

void CXSourceTree::OnLBDblClick(long hTreeItem, short nNodeType, LPCTSTR szPath)
{
    //
    // Get the final part of the given path
    //
    CString tempString = szPath;
    int nLen = tempString.GetLength();
    int nPos = tempString.ReverseFind('|');

    //
    // Determine which type of script file was clicked on and send a message containing
    // the script name to open the script for editing.
    //
    if (nNodeType == SCRIPT_FILE)
    {
        tempString = tempString.Right(nLen-nPos-1)+".scl";
        // And invoke the modeller on it
        CECTargetEnvironment te;
        te.InvokeModeller (TARGET_UNKNOWN ,tempString);
    }
}

void CXSourceTree::OnXTreeContextClick(LPCTSTR strPath, short nType, LPCTSTR strEntry)
{
    CString Entry = strEntry;
    CString tempString = strPath;
    int nLen = tempString.GetLength();
    int nPos = tempString.ReverseFind('|');

    if (Entry == "&New Script")
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_ADD_SCRIPT,0,0);
        return;
    }
    //
    // If Edit has been requested, add the file extension and send a message to the
    // control class to fire the event back to host.
    //

    if (nType != SCRIPT_FILE)
        return;

    LPCTSTR script;
    tempString = tempString.Right(nLen-nPos-1)+".scl";
    script = tempString;

    if (Entry == "E&dit")
    {
        // And invoke the modeller on it
        CECTargetEnvironment te;
        te.InvokeModeller (TARGET_UNKNOWN ,script);
    }
    else if (Entry == "Check &In")
    {
        BOOL bRet = ::SendNotifyMessage(GetParent()->m_hWnd,WMU_CHECK_IN,(UINT)(LPCTSTR)m_pServer->GetFolderName(CYRServer::enSourceFolder),(long)script);
        if (bRet)
        {
            m_XSourceTree.EnableMenuItem(SCRIPT_FILE,"Check &In", FALSE);
            m_XSourceTree.EnableMenuItem(SCRIPT_FILE,"Check &Out", TRUE);
            RefreshTree();
        }
    }
    else if (Entry == "Check &Out")
    {
        BOOL bRet = ::SendNotifyMessage(GetParent()->m_hWnd,WMU_CHECK_OUT,(UINT)(LPCTSTR)m_pServer->GetFolderName(CYRServer::enSourceFolder),(long)script);
        if (bRet)
        {
            m_XSourceTree.EnableMenuItem(SCRIPT_FILE,"Check &In", TRUE);
            m_XSourceTree.EnableMenuItem(SCRIPT_FILE,"Check &Out", FALSE);
            RefreshTree();
        }

    }
    else if (Entry == "&Delete")
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DELETE_FILE,(UINT)(LPCTSTR)m_pServer->GetFolderName(CYRServer::enSourceFolder),(long)script);
        RefreshTree();
    }
    else if(Entry == "E&xport")
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_EXPORT_FILE,0,(long)script);
    }
}

void CXSourceTree::OnXTreeItemProperties(LPCTSTR strPath, short nType)
{
    /*  CPropertiesDlg  dlg;
        dlg.SetServerInfo(m_pServer, m_strProject);
        dlg.SetItemInfo(strPath, nType);
        dlg.DoModal();
    */
}

void CXSourceTree::RefreshTree()
{
    m_XSourceTree.RefreshTree();
}

void CXSourceTree::OnXShowContext(short nNodeType, LPCTSTR strPath)
{
    if (nNodeType != SCRIPT_FILE)
        return;

    CStringEx path = strPath;
    int nCount = path.GetFieldCount('|');
    CString item = path.GetField('|',nCount-1);
    item = item + ".scl";

    if (IsFileLocked(nNodeType, item))
    {
        m_XSourceTree.EnableMenuItem(nNodeType, "Check &In", FALSE);
        m_XSourceTree.EnableMenuItem(nNodeType, "Check &Out", TRUE);
    }
    else
    {
        m_XSourceTree.EnableMenuItem(nNodeType, "Check &In", TRUE);
        m_XSourceTree.EnableMenuItem(nNodeType, "Check &Out", FALSE);
    }

}

BOOL CXSourceTree::IsFileLocked(short nNodeType, CString fileName)
{
    return false;
    /*  CString KeyName = SOLUTIONS_SERVER_KEY;
        CString ValueName = "ProjectDir";
     
        CECRegistryTools reg;
        CString workDir = reg.GetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName);
     
        CString localFile = workDir+m_strProject+"\\"+CECComSourceFolder::FOLDER_NAME+"\\"+fileName;
        
        try
        {
            CFile f(localFile, CFile::modeRead);
            CFileStatus status;
            f.GetStatus(status);
            f.Close();
     
            if (status.m_attribute & 0x01)
            {
                return TRUE;
            }
            else
            {
                return FALSE;
            }
        }
        catch (...)
        {
            return TRUE;
        }   */
}

void CXSourceTree::EnableMenus(BOOL bEnabaled)
{
    m_XSourceTree.EnableMenus(bEnabaled);
}

BOOL CXSourceTree::OnEraseBkgnd(CDC *pDC)
{
    return true;
}
