/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XTestTree.cpp : implementation file
//

#include "stdafx.h"
#include "xrepostree.h"
#include "XTestTree.h"
#include "..\..\Studio\Interfaces\CyrPlugins.h"
#include "..\..\Studio\common\CyrTestProperties.h"
#include "Solutions.h"
#include "..\..\Studio\common\CECSourceFileUtils.h"
#include "..\..\Studio\common\CECRegistryTools.h"
#include "..\..\Studio\common\CECTargetEnvironment.h"
#include "..\..\Studio\common\CYRExternalFile.h"
#include "Server.h"
#include "..\..\Studio\common\TestWizardDefs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define FOLDER_ICON             0
#define TEST_ICON               1
#define SCRIPT_ICON             2
#define SCRIPT_MISSING_ICON     3
#define INCL_ICON               4
#define INCL_MISSING_ICON       5
#define OTA_ICON                6
#define TEST_MISSING_ICON       7
/////////////////////////////////////////////////////////////////////////////
// CXTestTree dialog


CXTestTree::CXTestTree(BOOL bUseSCC, CWnd* pParent /*=NULL*/)
        : CDialog(CXTestTree::IDD, pParent)
{
    //{{AFX_DATA_INIT(CXTestTree)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_bUseSCC = bUseSCC;
    m_strProject="";
    m_pServer = NULL;
    // Get the TOG mode

    m_hSimpleNode = NULL;
    m_hGroupNode  = NULL;
}


void CXTestTree::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CXTestTree)
    DDX_Control(pDX, IDC_XTREE_TEST, m_XTreeTest);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CXTestTree, CDialog)
//{{AFX_MSG_MAP(CXTestTree)
ON_WM_SIZE()
ON_WM_PAINT()
//}}AFX_MSG_MAP
ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXTestTree message handlers
void CXTestTree::SetActiveProject(CString strProject)
{
    m_strProject=strProject;
    if (m_strProject == "<No Project Selected>")
    {
        m_XTreeTest.DeleteAllItems();
    }
}

void CXTestTree::SetServer(CYRServer* pServer)
{
    ASSERT(pServer != NULL);

    m_pServer = pServer;
}

void CXTestTree::Activate()
{}


BOOL CXTestTree::OnInitDialog()
{
    AfxEnableControlContainer(NULL);
    CDialog::OnInitDialog();

    //
    // Define node types and context menu entries for each type
    //
    m_XTreeTest.DefineNodeType(SIMPLE_TEST, "&New Test,1", FALSE, FALSE);
    m_XTreeTest.DefineNodeType(GROUP_TEST,  "&New Group Test,1", FALSE, FALSE);
    m_XTreeTest.DefineNodeType(TEST,    "E&xecute,1|&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeTest.DefineNodeType(SCRIPT_FILE, "&Edit,1|&Delete,1|E&xport,0", FALSE ,FALSE);
    m_XTreeTest.DefineNodeType(OTA_TOF, "", FALSE ,FALSE);
    //
    // Add bitmaps to the image list
    //

    // Add folder icon to image list index 0 (#defined to FOLDER_ICON)
    CBitmap bmp;
    bmp.LoadBitmap(IDB_FOLDER);
    m_XTreeTest.AddImageToList((long)bmp.m_hObject);
    bmp.DeleteObject();

    bmp.LoadBitmap(IDB_TEST);
    m_XTreeTest.AddImageToList((long)bmp.m_hObject);
    bmp.DeleteObject();

    bmp.LoadBitmap(IDB_SCL_UNLOCKED);
    m_XTreeTest.AddImageToList((long)bmp.m_hObject);
    bmp.DeleteObject();

    bmp.LoadBitmap(IDB_SCL_MISSING);
    m_XTreeTest.AddImageToList((long)bmp.m_hObject);
    bmp.DeleteObject();


    bmp.LoadBitmap(IDB_INCL_UNLOCKED);
    m_XTreeTest.AddImageToList((long)bmp.m_hObject);
    bmp.DeleteObject();

    bmp.LoadBitmap(IDB_INCL_MISSING);
    m_XTreeTest.AddImageToList((long)bmp.m_hObject);
    bmp.DeleteObject();

    bmp.LoadBitmap(IDB_OTA_UNLOCKED);
    m_XTreeTest.AddImageToList((long)bmp.m_hObject);
    bmp.DeleteObject();

    bmp.LoadBitmap(IDB_TEST_MISSING);
    m_XTreeTest.AddImageToList((long)bmp.m_hObject);
    bmp.DeleteObject();

    if (!m_strProject .IsEmpty() && m_strProject != "<No Project Selected>")
    {
        m_hSimpleNode = (HTREEITEM) m_XTreeTest.AddItem(-1, SIMPLE_TEST ,"Tests", FOLDER_ICON, FALSE);
        m_hGroupNode = (HTREEITEM) m_XTreeTest.AddItem(-1, GROUP_TEST ,"Group Tests", FOLDER_ICON, FALSE);
    }

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CXTestTree::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    if( m_XTreeTest.GetSafeHwnd() != NULL )
    {
        m_XTreeTest.SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );

    }

}

BEGIN_EVENTSINK_MAP(CXTestTree, CDialog)
//{{AFX_EVENTSINK_MAP(CXTestTree)
ON_EVENT(CXTestTree, IDC_XTREE_TEST, 2 /* XTreeExpanding */, OnXTreeExpanding, VTS_I4 VTS_I2 VTS_BSTR)
ON_EVENT(CXTestTree, IDC_XTREE_TEST, 1 /* LBDblClick */, OnLBDblClick, VTS_I4 VTS_I2 VTS_BSTR)
ON_EVENT(CXTestTree, IDC_XTREE_TEST, 3 /* XTreeContextClick */, OnXTreeContextClick, VTS_BSTR VTS_I2 VTS_BSTR)
ON_EVENT(CXTestTree, IDC_XTREE_TEST, 6 /* XTreeItemDelete */, OnXTreeItemDelete, VTS_BSTR VTS_I2)
ON_EVENT(CXTestTree, IDC_XTREE_TEST, 5 /* XShowContext */, OnXShowContext, VTS_I2 VTS_BSTR)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CXTestTree::OnXTreeExpanding(long hTreeItem, short nNodeType, LPCTSTR szPath)
{
    BeginWaitCursor();

    switch (nNodeType)
    {
    case SIMPLE_TEST:
        {
            AddTestTypes((HTREEITEM)hTreeItem, SIMPLE_TEST);
            break;
        }
    case GROUP_TEST:
        {
            AddTestTypes((HTREEITEM)hTreeItem, GROUP_TEST);
            break;
        }
    case TEST:
        {
            AddTestDependencies((HTREEITEM)hTreeItem);
            break;
        }
    case SCRIPT_FILE:
        {
            AddScriptDependencies((HTREEITEM)hTreeItem);
            break;
        }
    }

    EndWaitCursor();
}


void CXTestTree::AddTestTypes(HTREEITEM parent, int testType)
{
    //
    // Make sure the parent allows children
    //
    m_XTreeTest.SetChildren((long)parent, TRUE);

    CYRTests tests(m_pServer);
    CStringArray testNames;

    if(testType == SIMPLE_TEST)
    {
        tests.GetSimpleTests(testNames);
    }
    else if (testType == GROUP_TEST)
    {
        tests.GetGroupTests(testNames);
    }

    for (int i = 0; i<testNames.GetSize(); i++)
    {
        m_XTreeTest.AddItem((long)parent, TEST, testNames[i], TEST_ICON ,true);

    }

    if (testNames.GetSize() == 0)
    {
        m_XTreeTest.SetChildren((long)parent, FALSE);
    }

}

//
// FindChildren
// ============
//
// See if the tree item has any tests defined
//
void CXTestTree::FindChildren(HTREEITEM parent)
{
    // Get access to the tests
    CYRTests Tests(m_pServer);

    BOOL bChildren = FALSE;

    if (parent==m_hGroupNode)
    {
        // And see if any group tests exist
        CStringArray arstrGroupTests;
        Tests.GetGroupTests(arstrGroupTests);


        bChildren = (arstrGroupTests.GetSize() > 0) ? TRUE : FALSE;


    }
    else if (parent==m_hSimpleNode)
    {
        // And see if any simple tests exist
        CStringArray arstrSimpleTests;
        Tests.GetSimpleTests(arstrSimpleTests);

        bChildren = (arstrSimpleTests.GetSize() > 0) ? TRUE : FALSE;
    }

    // And set up the parent node accordingly
    m_XTreeTest.SetChildren((long) parent, bChildren);
}

void CXTestTree::AddTestDependencies(HTREEITEM parent)
{
    static initSimple=0;
    static initGroup=0;
    //
    // Make sure the parent allows children
    //
    m_XTreeTest.SetChildren((long)parent, TRUE);

    //
    // Get the path of the parent item. If the top level item is Group Tests
    // add the child tests but don't allow expanding
    BOOL bAllowChildren = TRUE;

    CString strPath = m_XTreeTest.GetItemPath((long)parent);
    int nPos = strPath.Find("|" ,0);
    if (strPath.Left(nPos) == "Group Tests")
    {
        bAllowChildren = FALSE;
    }


    CYRTests Tests(m_pServer);

    //
    // Recover the test name from the tree item
    //
    CString path = m_XTreeTest.GetItemPath((long)parent);
    int nFindPos = path.ReverseFind('|');
    int nLen = path.GetLength();
    CString strTest = path.Right(nLen-nFindPos-1);

    //
    // Construct the file path of the wizard file for this test
    //
    CString strTestFile(strTest + ".WIZ");
    CString strTestPath(m_pServer->GetFolderPath(CYRServer::enTestsFolder) + strTest + "\\" + strTestFile);

    //
    // Read the number of dependencies
    //
    CString strNumber;
    DWORD dwMaxBufferSize=200;
    GetPrivateProfileString(WIZARD_DEPENDENCIES,WIZARD_DEPENDENCIES_NUMBER,"0",strNumber.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strTestPath);
    strNumber.ReleaseBuffer();
    int nNumDependencies = atoi(strNumber);

    if ((parent==m_hGroupNode) && (initGroup==0))
    {
        initGroup++;
        if (nNumDependencies == 0)
        {
            m_XTreeTest.SetChildren((long)parent, FALSE);
        }

        return;
    }

    if ((parent==m_hSimpleNode) && (initSimple==0))
    {
        initSimple++;
        if (nNumDependencies == 0)
        {
            m_XTreeTest.SetChildren((long)parent, FALSE);
        }
        return;
    }

    for (int i = 1; i<=nNumDependencies; i++)
    {
        //
        // Read the next dependency...
        //
        CString strIndex;
        strIndex.Format(WIZARD_DEPENDENCIES_FORMAT,i);

        CString strDepFile;
        DWORD dwMaxBufferSize=200;
        GetPrivateProfileString(WIZARD_DEPENDENCIES,strIndex,"0",strDepFile.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strTestPath);
        strDepFile.ReleaseBuffer();

        //
        // ... then add a tree item for the dependency type.
        //
        int len = strDepFile.GetLength();
        CString strExt = strDepFile.Right(3);
        CString strDep = strDepFile.Left(len-4);

        if (strExt.CompareNoCase("tss") == 0)
        {
            if(Tests.DoesTestExist(strDep))
            {
                m_XTreeTest.AddItem((long)parent, TEST, strDep, TEST_ICON ,bAllowChildren);
            }
            else
            {
                m_XTreeTest.AddItem((long)parent, TEST, strDep, TEST_MISSING_ICON ,bAllowChildren);
            }
        }
        if (strExt.CompareNoCase("scl") == 0)
        {
            //
            // If the script file does not exist then use the missing icon
            //
            CString strSourcePath(m_pServer->GetFolderPath(CYRServer::enSourceFolder)+strDepFile);
            CFileStatus fs;
            if (CFile::GetStatus(strSourcePath,fs))
            {
                // And see if this script has any dependencies - if so add the plus icon
                HTREEITEM hScript = (HTREEITEM) m_XTreeTest.AddItem((long)parent, SCRIPT_FILE, strDep, SCRIPT_ICON ,true);
                AddScriptDependencies(hScript);
            }
            else
            {
                // And see if this script has any dependencies - if so add the plus icon
                HTREEITEM hScript = (HTREEITEM) m_XTreeTest.AddItem((long)parent, SCRIPT_FILE, strDep, SCRIPT_MISSING_ICON ,true);
                AddScriptDependencies(hScript);

            }
        }
        if (strExt.CompareNoCase("tof") == 0)
        {
            m_XTreeTest.AddItem((long)parent, OTA_TOF, strDep, OTA_ICON ,false);
        }
    }

    if (nNumDependencies == 0)
    {
        m_XTreeTest.SetChildren((long)parent, FALSE);
    }

}

void CXTestTree::AddScriptDependencies(HTREEITEM parent)
{
    //
    // Make sure the parent allows children
    //
    m_XTreeTest.SetChildren((long)parent, TRUE);

    CString strPath = m_XTreeTest.GetItemPath((long)parent);
    int nPos = strPath.ReverseFind('|');
    int nLen = strPath.GetLength();
    CString strScript = strPath.Right(nLen-nPos-1);

    CString sourcePath = m_pServer->GetFolderPath(CYRServer::enSourceFolder) + strScript+".scl";


    CECSourceFileUtils depInfo(m_pServer,sourcePath);
    int nCount = depInfo.ReadDependencies();

    if (nCount == 0)
    {
        m_XTreeTest.SetChildren((long)parent, FALSE);
    }
    else
    {
        int nAdded = 0;

        for (int i = 0; i<nCount; i++)
        {
            DEPENDENCY d = depInfo.GetDependency(i);
            int nLen = d.file.GetLength();
            int nPos = d.file.ReverseFind('\\');
            CString dependFile = d.file.Right(nLen-nPos-1);

            // For now - ignore include files
            /*
                        if (d.type == CECSourceFileUtils::INCL)
                        {
                            m_XTreeTest.AddItem((long)parent, INCL_FILE, dependFile, INCL_ICON,FALSE);
                            nAdded++;
                        }
            */
        }

        if(nAdded == 0)
        {
            m_XTreeTest.SetChildren((long)parent, FALSE);
        }
    }
}

void CXTestTree::OnLBDblClick(long hTreeItem, short nNodeType, LPCTSTR szPath)
{
    CYRTests Tests(m_pServer);

    switch (nNodeType)
    {
    case TEST:
        {
            // Invoke wizard to edit the test
            CString strPath(szPath);
            int nPos = strPath.ReverseFind('|');
            int nLen = strPath.GetLength();
            CString strTest = strPath.Right(nLen-nPos-1);
            if (Tests.IsGroupTest(strTest))
            {
                InvokeGroupTestWizard(strTest);
            }
            if (Tests.IsSimpleTest(strTest))
            {
                InvokeSimpleTestWizard(strTest);
            }

            break;
        }
    case SCRIPT_FILE:
        {
            // Get the Path
            CString strPath(szPath);
            int nPos = strPath.ReverseFind('|');
            int nLen = strPath.GetLength();

            // Strip out the file name
            CString szFile = strPath.Right(nLen-nPos-1)+ ".scl";

            // And invoke the modeller on it
            CECTargetEnvironment te;
            te.InvokeModeller (TARGET_UNKNOWN ,szFile);

            break;
        }
    case OTA_TOF:
        {
            CString strPath(szPath);
            int nPos = strPath.ReverseFind('|');
            int nLen = strPath.GetLength();
            CString strOTA = strPath.Right(nLen-nPos-1)+".tof";

            CWnd* pMainWnd = AfxGetMainWnd();
            CString strParams;
            strParams.Format("%s|%s" ,SOLUTIONS_GENERIC_DLL,strOTA);
            pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,(WPARAM)WMU_EDIT_OTA_TOF,(LPARAM)&strParams);
            return;

            break;
        }
    }
}

void CXTestTree::OnXTreeContextClick(LPCTSTR strPath, short nType, LPCTSTR strEntry)
{
    CYRTests Tests(m_pServer);

    CString entry(strEntry);
    CString tempString = strPath;
    int nLen = tempString.GetLength();
    int nPos = tempString.ReverseFind('|');
    tempString = tempString.Right(nLen-nPos-1);
    LPCTSTR itemName = tempString;

    switch (nType)
    {
    case SIMPLE_TEST:
        {
            if (entry == "&New Test")
            {
                InvokeSimpleTestWizard("");
            }
            break;
        }
    case GROUP_TEST:
        {
            if (entry == "&New Group Test")
            {
                InvokeGroupTestWizard("");
            }
            break;
        }
    case TEST:
        {

            if (entry == "&Edit")
            {
                // Invoke wizard to edit the test
                CString szPath(strPath);
                int nPos = szPath.ReverseFind('|');
                int nLen = szPath.GetLength();
                CString strTest = szPath.Right(nLen-nPos-1);
                if (Tests.IsGroupTest(strTest))
                {
                    InvokeGroupTestWizard(strTest);
                }
                if (Tests.IsSimpleTest(strTest))
                {
                    InvokeSimpleTestWizard(strTest);
                }
            }
            if (entry == "E&xecute")
            {
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_INIT_TEST,0,(long)itemName);
                return;
            }
            if (entry == "&Delete")
            {
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DELETE_TEST,(UINT)(LPCTSTR)m_pServer->GetFolderName(CYRServer::enTestsFolder),(long)itemName);
                RefreshTree();
                return;
            }
            break;
        }
    case SCRIPT_FILE:
        {
            if (entry == "&Edit")
            {
                // Add the scl suffix onto the item in question
                CString szScriptName(itemName);
                szScriptName += ".scl";

                // And invoke the modeller on it
                CECTargetEnvironment te;
                te.InvokeModeller (TARGET_UNKNOWN ,szScriptName);

            }
            if (entry == "&Delete")
            {
                CString sourcePath = m_pServer->GetFolderPath(CYRServer::enSourceFolder);
                sourcePath +=tempString+".scl";
                CFileStatus fs;
                if (CFile::GetStatus(sourcePath,fs))
                {
                    CString szFile = tempString+ ".scl";
                    LPCTSTR scriptName = szFile;
                    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DELETE_FILE,(UINT)(LPCTSTR)m_pServer->GetFolderName(CYRServer::enSourceFolder),(long)scriptName);
                    RefreshTree();
                }
                else
                {
                    MessageBox("File Does not exist", "OpenSTA");
                }
            }
            if (entry == "E&xport")
            {
                // Get access to the tests
                CString sourcePath = m_pServer->GetFolderPath(CYRServer::enSourceFolder);
                sourcePath +=tempString+".SCL";
                CFileStatus fs;
                if (CFile::GetStatus(sourcePath,fs))
                {
                    CYRExternalFile exporter;
                    exporter.Export(tempString+".SCL");
                }
                else
                {
                    MessageBox("File Does not exist", "OpenSTA");
                }
            }
            break;
        }
    }
}

void CXTestTree::InvokeGroupTestWizard(CString strTest)
{
    CWnd* pMainWnd = AfxGetMainWnd();

    CString wizInfo = "Group Test|"+strTest;
    pMainWnd->SendMessage (WMU_WIZARD_START ,(LPARAM)"Generic0", (WPARAM)wizInfo.GetBuffer(0));
    RefreshTree();
}

void CXTestTree::InvokeSimpleTestWizard(CString strTest)
{
    CWnd* pMainWnd = AfxGetMainWnd();
    CString wizInfo = "Test|"+strTest;
    pMainWnd->SendMessage (WMU_WIZARD_START ,(LPARAM)"OpenSTA", (WPARAM)wizInfo.GetBuffer(0));
    RefreshTree();
}

void CXTestTree::RefreshTree()
{
    // First see if our Root nodes have any children ...
    FindChildren(m_hSimpleNode);
    FindChildren(m_hGroupNode);

    // and refresh the Test Tree. Typically called after adding/removing items from
    // the tree or just as a general validation mechanism.
    m_XTreeTest.RefreshTree();
}

void CXTestTree::EnableMenus(BOOL bEnabaled)
{
    m_XTreeTest.EnableMenus(bEnabaled);
}

void CXTestTree::OnXTreeItemDelete(LPCTSTR strPath, short nType)
{
    CYRTests Tests(m_pServer);

    CString szPath(strPath);
    int nPos = szPath.ReverseFind('|');
    int nLen = szPath.GetLength();
    CString item = szPath.Right(nLen-nPos-1);

    switch (nType)
    {
    case TEST:
        {
            LPCTSTR testName = item;
            ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DELETE_TEST,(UINT)(LPCTSTR)m_pServer->GetFolderName(CYRServer::enTestsFolder),(long)testName);
            RefreshTree();

            break;
        }
    case SCRIPT_FILE:
        {
            CString sourcePath = m_pServer->GetFolderPath(CYRServer::enSourceFolder);
            sourcePath +=item+".scl";
            CFileStatus fs;
            if (CFile::GetStatus(sourcePath,fs))
            {
                CString szFile = item+ ".scl";
                LPCTSTR scriptName = szFile;
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DELETE_FILE,(UINT)(LPCTSTR)m_pServer->GetFolderName(CYRServer::enSourceFolder),(long)scriptName);
                RefreshTree();
            }
            else
            {
                MessageBox("File Does not exist", "OpenSTA");
            }

            break;
        }
    }
}

BOOL CXTestTree::OnEraseBkgnd(CDC *pDC)
{
    return true;
}

void CXTestTree::OnPaint()
{
    CPaintDC dc(this); // device context for painting

    // TODO: Add your message handler code here

    // Do not call CDialog::OnPaint() for painting messages
}


void CXTestTree::OnXShowContext(short nNodeType, LPCTSTR strPath)
{
    if (nNodeType == SCRIPT_FILE)
    {
        CString szPath(strPath);
        int nPos = szPath.ReverseFind('|');
        int nLen = szPath.GetLength();
        CString strScript = szPath.Right(nLen-nPos-1);

        // Get access to the tests
        CString sourcePath = m_pServer->GetFolderPath(CYRServer::enSourceFolder);
        sourcePath += strScript + ".SCL";

        CFileStatus fs;
        if (CFile::GetStatus(sourcePath,fs))
        {
            m_XTreeTest.EnableMenuItem(nNodeType, "E&xport", TRUE);
        }
        else
        {
            m_XTreeTest.EnableMenuItem(nNodeType, "E&xport", FALSE);
        }
    }
}
