/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XResultsCtl.cpp : Implementation of the CXResultsCtrl ActiveX Control class.

#include "stdafx.h"
#include "XResults.h"
#include "XResultsCtl.h"
#include "XResultsPpg.h"
#include "server.h"
#include "..\..\studio\interfaces\cyrplugins.h"

#include "..\..\Studio\Common\CECXServerError.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CXResultsCtrl, COleControl)


/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CXResultsCtrl, COleControl)
//{{AFX_MSG_MAP(CXResultsCtrl)
ON_WM_SIZE()
ON_WM_CREATE()
//}}AFX_MSG_MAP
ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
ON_MESSAGE (WMU_UPDATE_TITLE, OnUpdateTitle)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CXResultsCtrl, COleControl)
//{{AFX_DISPATCH_MAP(CXResultsCtrl)
DISP_FUNCTION(CXResultsCtrl, "XSetServerInfo", XSetServerInfo, VT_EMPTY, VTS_BSTR VTS_BSTR)
DISP_FUNCTION(CXResultsCtrl, "XInitialize", XInitialize, VT_EMPTY, VTS_BSTR VTS_BSTR VTS_BSTR VTS_I2)
//}}AFX_DISPATCH_MAP
DISP_FUNCTION_ID(CXResultsCtrl, "AboutBox", DISPID_ABOUTBOX, AboutBox, VT_EMPTY, VTS_NONE)
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CXResultsCtrl, COleControl)
//{{AFX_EVENT_MAP(CXResultsCtrl)
EVENT_CUSTOM("XUpdateTitle", FireXUpdateTitle, VTS_BSTR  VTS_BSTR  VTS_I2)
//}}AFX_EVENT_MAP
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CXResultsCtrl, 1)
PROPPAGEID(CXResultsPropPage::guid)
END_PROPPAGEIDS(CXResultsCtrl)


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CXResultsCtrl, "XRESULTS.XResultsCtrl.1",
                       0x502a94c6, 0xcd97, 0x11d3, 0x9a, 0x95, 0, 0x50, 0x4, 0x47, 0xfc, 0x27)


/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CXResultsCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DXResults =
    {
        0x502a94c4, 0xcd97, 0x11d3, { 0x9a, 0x95, 0, 0x50, 0x4, 0x47, 0xfc, 0x27 }
    };
const IID BASED_CODE IID_DXResultsEvents =
    {
        0x502a94c5, 0xcd97, 0x11d3, { 0x9a, 0x95, 0, 0x50, 0x4, 0x47, 0xfc, 0x27 }
    };


/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwXResultsOleMisc =
    OLEMISC_ACTIVATEWHENVISIBLE |
    OLEMISC_SETCLIENTSITEFIRST |
    OLEMISC_INSIDEOUT |
    OLEMISC_CANTLINKINSIDE |
    OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CXResultsCtrl, IDS_XRESULTS, _dwXResultsOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CXResultsCtrl::CXResultsCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CXResultsCtrl

BOOL CXResultsCtrl::CXResultsCtrlFactory::UpdateRegistry(BOOL bRegister)
{
    // TODO: Verify that your control follows apartment-model threading rules.
    // Refer to MFC TechNote 64 for more information.
    // If your control does not conform to the apartment-model rules, then
    // you must modify the code below, changing the 6th parameter from
    // afxRegApartmentThreading to 0.

    if (bRegister)
        return AfxOleRegisterControlClass(
                   AfxGetInstanceHandle(),
                   m_clsid,
                   m_lpszProgID,
                   IDS_XRESULTS,
                   IDB_XRESULTS,
                   afxRegApartmentThreading,
                   _dwXResultsOleMisc,
                   _tlid,
                   _wVerMajor,
                   _wVerMinor);
    else
        return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CXResultsCtrl::CXResultsCtrl - Constructor

CXResultsCtrl::CXResultsCtrl()
{
    InitializeIIDs(&IID_DXResults, &IID_DXResultsEvents);
    m_pServer=NULL;
    // TODO: Initialize your control's instance data here.
}


/////////////////////////////////////////////////////////////////////////////
// CXResultsCtrl::~CXResultsCtrl - Destructor


CXResultsCtrl::~CXResultsCtrl()
{
    // TODO: Cleanup your control's instance data here.
}


/////////////////////////////////////////////////////////////////////////////
// CXResultsCtrl::OnDraw - Drawing function


void CXResultsCtrl::OnDraw(
    CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
    // TODO: Replace the following code with your own drawing code.
    if(!m_Results.GetSafeHwnd())
    {
        pdc->FillRect(rcBounds, CBrush::FromHandle((HBRUSH)GetStockObject(WHITE_BRUSH)));
        pdc->Ellipse(rcBounds);
    }
}


/////////////////////////////////////////////////////////////////////////////
// CXResultsCtrl::DoPropExchange - Persistence support

void CXResultsCtrl::DoPropExchange(CPropExchange* pPX)
{
    ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
    COleControl::DoPropExchange(pPX);

    // TODO: Call PX_ functions for each persistent custom property.

}


/////////////////////////////////////////////////////////////////////////////
// CXResultsCtrl::OnResetState - Reset control to default state


void CXResultsCtrl::OnResetState()
{
    COleControl::OnResetState();  // Resets defaults found in DoPropExchange

    // TODO: Reset any other control state here.
}


/////////////////////////////////////////////////////////////////////////////
// CXResultsCtrl::AboutBox - Display an "About" box to the user


void CXResultsCtrl::AboutBox()
{
    CDialog dlgAbout(IDD_ABOUTBOX_XRESULTS);
    dlgAbout.DoModal();
}


/////////////////////////////////////////////////////////////////////////////
// CXResultsCtrl message handlers

void CXResultsCtrl::XSetServerInfo(LPCTSTR Username, LPCTSTR Password)
{
    m_Password = Password;
    m_Username = Username;
    IsServerRunning();

}

void CXResultsCtrl::XInitialize(LPCTSTR ProjectName, LPCTSTR TestName, LPCTSTR RunName, short nType)
{
    BeginWaitCursor();

    if (m_Results.m_hWnd == NULL)
    {
        m_Results.Create (IDD_RESULTS ,this);
        m_Results.ShowWindow(SW_SHOW);
        m_Results.SetServer(m_pServer);
    }
    m_Results.Init(ProjectName, TestName, RunName, nType);


    CRect r;
    GetClientRect(r);

    m_Results.SetWindowPos(&wndTop, 0,0, r.Width(), r.Height(),SWP_NOZORDER);
    m_Results.Invalidate();

    EndWaitCursor();
}

//
// @mfunc   IsServerRunning
//
//    Determine if the server is running and connects to it.
//
BOOL CXResultsCtrl::IsServerRunning()
{
    if(m_pServer)
        return true;
    // Attempt to connect to server...
    m_pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);

    if(!m_pServer)
    {
        m_pServer = new CYRServer;
    }
    return true;
}


void CXResultsCtrl::OnSize(UINT nType, int cx, int cy)
{
    COleControl::OnSize(nType, cx, cy);

    if (m_Results.m_hWnd != NULL)
    {
        m_Results.SetWindowPos (&wndTop ,0 ,0 ,cx ,cy ,SWP_NOZORDER);
    }
}

int CXResultsCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (COleControl::OnCreate(lpCreateStruct) == -1)
        return -1;

    AfxEnableControlContainer();

    return 0;
}

LRESULT CXResultsCtrl::OnUpdateTitle(WPARAM wParam,LPARAM lParam)
{
    FireXUpdateTitle(m_Results.m_szTest, m_Results.m_szRun, m_Results.m_nType);
    return 0;
}

BOOL CXResultsCtrl::PreTranslateMessage(MSG* pMsg)
{
    //Give dialog first crack at handling key presses
    if(pMsg->message >= WM_KEYFIRST && pMsg->message <=WM_KEYLAST)
    {
        if(m_Results.PreTranslateMessage(pMsg))
            return true;
    }

    return COleControl::PreTranslateMessage(pMsg);
}
