/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECTestGraphDlg.cpp : implementation file
//

#include "stdafx.h"
#include "XTestDynamicGraphs.h"
#include "CECTestGraphDlg.h"
#include "CECGraphSelect.h"
#include "CECRefreshDlg.h"

#include "CyrWrapLib\CECOrbTestRun.h"
#include "resource.h"
#include "XChartCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static UINT uiCORBARefCount=0;
static int DEFAULT_REFRESH_RATE = 500;
/////////////////////////////////////////////////////////////////////////////
// CECTestGraphDlg dialog
const int VtChSeriesType2dLine = 6;
#define IDT_POLL_DATA 1

CECTestGraphDlg::CECTestGraphDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECTestGraphDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECTestGraphDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pCorbaClt=new CORBAServer ( 0, NULL );
    m_pCorbaClt->Ready(true);

    uiCORBARefCount++;

    m_nNextGroup = 0;
    m_OTACount = -1;

    m_nOTADataIndex = 0;
    m_nOTAHeaderIndex = 0;

    m_TM = NULL;
    m_TestRun = NULL;
    m_nRefreshRate = DEFAULT_REFRESH_RATE;
}

CECTestGraphDlg::~CECTestGraphDlg()
{
    if((--uiCORBARefCount) == 0)
    {
        m_pCorbaClt->GetBOA()->destroy();
        m_pCorbaClt->GetORB()->NP_destroy();
    }
    delete m_pCorbaClt;

    for(int n=0;n<m_selectedTypes.GetSize();n++)
        delete m_selectedTypes[n].pChart;
    m_selectedTypes.RemoveAll();
}

void CECTestGraphDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECTestGraphDlg)
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECTestGraphDlg, CDialog)
ON_WM_CONTEXTMENU()
//{{AFX_MSG_MAP(CECTestGraphDlg)
ON_WM_SIZE()
ON_WM_TIMER()
ON_BN_CLICKED(IDC_CONFIG, OnConfig)
ON_WM_DESTROY()
ON_COMMAND(ID_POPUP_REFRESH, OnPopupRefresh)
ON_COMMAND(ID_POPUP_CONFIG, OnPopupConfig)
//}}AFX_MSG_MAP
ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECTestGraphDlg message handlers

BOOL CECTestGraphDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    //  FindAvailableGraphTypes();

    //  OnConfig();

    //  SetTimer(IDT_POLL_DATA,50,NULL);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECTestGraphDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    int nNumGraphs = m_selectedTypes.GetSize();
    double dHeight =0;
    if(nNumGraphs)
        dHeight = cy / nNumGraphs;

    CRect rcGraph(0,0,cx,dHeight);

    for(int nGraph=0;nGraph<nNumGraphs;nGraph++)
    {
        CXChartCtrl *pChart = m_selectedTypes[nGraph].pChart;
        pChart->MoveWindow(rcGraph);

        rcGraph.OffsetRect(0,dHeight);
    }
}

void CECTestGraphDlg::OnTimer(UINT nIDEvent)
{

    switch(nIDEvent)
    {
    case IDT_POLL_DATA:
        {
            int nGraphs = m_selectedTypes.GetSize();

            for (int i = 0; i<nGraphs; i++)
            {
                GRAPH_TYPE type = m_selectedTypes[i];
                type.graphIndex = i;
                if (type.nType == TESTINFO_TYPE && type.szName == "Active Users")
                {
                    RollActiveThreads(type);
                }
            }

            if (m_nOTAHeaderIndex > 0)
            {
                RollOTAGraphs();
            }
        }
        break;
    }

    CDialog::OnTimer(nIDEvent);
}

void CECTestGraphDlg::StartMonitor(CString strRegName,CString strInjector)
{
    m_strInjector = strInjector;
    m_strManager = strRegName;
    try
    {
        //Get the Test manager status object
        CORBA::Object_var obj;
        obj=m_pCorbaClt->GetObjectFromClass(TestManager::CLASS_NAME, strRegName.GetBuffer(0));
        m_TM = narrow_object((TestManager_ptr)NULL,obj);
        m_ActiveLevel = m_TM->GetTestLevel();
    }
    catch(...)
    {
        MessageBox("An error occured starting the monitor.", "OpenSTA");
        GetParent()->PostMessage(WM_CLOSE);
        return;
    }

    OnConfig();

    SetTimer(IDT_POLL_DATA,m_nRefreshRate,NULL);
}



void CECTestGraphDlg::FindAvailableGraphTypes()
{
    m_availableTypes.RemoveAll();

    // Setup standard graph types if no injector is given
    if(m_strInjector.IsEmpty())
    {
        GRAPH_TYPE typ;
        typ.nType = TESTINFO_TYPE;
        typ.szName = "Active Users";
        typ.graphStyle = HISTORY;
        typ.pChart = NULL;
        m_availableTypes.Add(typ);
        return;
    }


    // Find out if OTA is running
    if (m_TM == NULL)
    {
        return;
    }

    CORBA::Object_var obj;
    try
    {
        obj=m_TM->GetTestRunAsObject();
        m_TestRun=narrow_object((CyrStTestRun_ptr )NULL,obj);

        long nEntries = m_TestRun->GetNoOutputEntries(CECOtaHeader::OUTPUT_TYPE);

        for (m_nOTAHeaderIndex; m_nOTAHeaderIndex<nEntries; m_nOTAHeaderIndex++)
        {
            CString headerInfo = m_TestRun->GetOutput(CECOtaHeader::OUTPUT_TYPE, m_nOTAHeaderIndex);

            CString node = strtok(headerInfo.GetBuffer(0),",");
            CString strIndex = strtok(NULL,",");
            int nIndex = atoi(strIndex);
            if(m_strInjector == node)
            {
                GRAPH_TYPE typ;
                typ.nType = OTA_TYPE;
                typ.graphStyle = HISTORY;
                typ.OTAIndex = nIndex;
                typ.OTANode = node;
                CString name = strtok(NULL,",");
                name.TrimRight(",");
                typ.szName = name;
                typ.pChart = NULL;
                m_availableTypes.Add(typ);
            }
        }

        // Get the start offset for the OTA Data
        m_nOTADataIndex = m_TestRun->GetOutputOffset(CECOtaSimpleQuery::OUTPUT_TYPE);

    }
    catch(...)
    {}

}

void CECTestGraphDlg::SetupGraphs()
{
    m_nNextGroup = 0;

    CRect rcDisplay;
    GetClientRect(&rcDisplay);

    int nNumGraphs = m_selectedTypes.GetSize();
    if(!nNumGraphs)
        return;

    double dHeight = rcDisplay.Height()/nNumGraphs;

    CRect rcGraph;
    for (int i = 0; i<nNumGraphs; i++)
    {
        if (i != (nNumGraphs-1))
        {
            rcGraph.SetRect(0,i*dHeight,rcDisplay.Width(),(i+1)*dHeight-1);
        }
        // Last one so make sure it goes to bottom of display.
        else
            rcGraph.SetRect(0,i*dHeight,rcDisplay.Width(),rcDisplay.Height());

        GRAPH_TYPE &type = m_selectedTypes[i];
        if(type.pChart != NULL)
            delete type.pChart;

        type.pChart = CreateDisplay(rcGraph);

        type.graphIndex = i;
        AddGraphDisplay(type);
    }
}


void CECTestGraphDlg::AddGraphDisplay(GRAPH_TYPE &type)
{
    //
    // Add a default display component setting graph/axis styles and colours.
    //
    type.pChart->AddDataPoint(1,0.0);

    //setup initial data
    if (type.graphStyle == PERFORMANCE)
    {
        for(int i=1;i<=50;i++)
        {
            type.pChart->SetDataPoint(i,1,0);
        }
    }

    type.pChart->SetSeriesStyle(1,VtChSeriesType2dLine);
    type.pChart->EnableZoom(FALSE);

    const int nMaxGraphs = 7;
    const COLORREF colLines[nMaxGraphs]=
        {
            RGB(255,0,0),   RGB(0,255,0),
            RGB(0,0,255),   RGB(255,255,0),
            RGB(255,0,255), RGB(0,255,255),
            RGB(255,255,255)
        };


    // Only put titles in for OTA Graphs as the Test Status graph title
    // will appear in the frame title
    if (type.nType == OTA_TYPE)
    {
        type.pChart->SetYAxisLabel(type.szName);
    }
    type.dataGroup = m_nNextGroup;
    m_nNextGroup++;
}

void CECTestGraphDlg::OnDestroy()
{
    CDialog::OnDestroy();

    KillTimer(IDT_POLL_DATA);
}

void CECTestGraphDlg::RollActiveThreads(const GRAPH_TYPE& type)
{
    try
    {
        long nTotalThreads = m_ActiveLevel->NoChildThreads(true);
        if (type.graphStyle == PERFORMANCE)
        {
            //          m_ctlGraph.GetGraph()->RollGroup(type.dataGroup, nTotalThreads);
        }

        else
        {
            type.pChart->AddDataPoint(1,nTotalThreads);
        }
    }
    catch(...)
    {
        GetParent()->PostMessage(WM_CLOSE);
    }
}

void CECTestGraphDlg::RollOTAGraphs()
{

    try
    {
        CORBA::Object_var obj;
        obj=m_pCorbaClt->GetObjectFromClass(TestManager::CLASS_NAME, m_strManager.GetBuffer(0));

        CString strData;
        int nNumEntries = m_TestRun->GetNoOutputEntries(CECOtaSimpleQuery::OUTPUT_TYPE);
        for(;m_nOTADataIndex < nNumEntries;m_nOTADataIndex++)
        {
            strData = m_TestRun->GetOutput(CECOtaSimpleQuery::OUTPUT_TYPE, m_nOTADataIndex);

            CString node = strtok(strData.GetBuffer(0),",");
            ;
            CString index = strtok(NULL,",");
            CString start = strtok(NULL,",");
            CString stop = strtok(NULL,",");
            CString value = strtok(NULL,",");

            double dVal = atof(value);

            for (int i = 0; i<m_selectedTypes.GetSize(); i++)
            {
                if (m_selectedTypes[i].OTANode == node &&
                        m_selectedTypes[i].OTAIndex == atoi(index))
                {
                    if (m_selectedTypes[i].graphStyle == PERFORMANCE)
                    {
                        m_selectedTypes[i].pChart->AddDataPoint(1,dVal);
                        break;
                    }
                    else
                    {
                        m_selectedTypes[i].pChart->AddDataPoint(1,dVal);
                    }
                }
            }
        }
    }
    catch(...)
    {
        GetParent()->PostMessage(WM_CLOSE);
    }
}

void CECTestGraphDlg::OnContextMenu(CWnd*, CPoint point)
{

    // CG: This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;
    VERIFY(menu.LoadMenu(CG_IDR_POPUP_ECTEST_GRAPH_DLG));

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,
                           this);

}

#if __CLOSE_ON_EXCEPTIONS__
void CECTestGraphDlg::RemoveSelectedType(const GRAPH_TYPE &type)
{
    m_selectedTypes[type.graphIndex].nNumExceptions++;
    delete m_selectedTypes[type.graphIndex].pChart;
    m_selectedTypes.RemoveAt(type.graphIndex);

    for(int nNdx=0;nNdx<m_selectedTypes.GetSize();nNdx++)
    {
        m_selectedTypes[nNdx].graphIndex = nNdx;
    }

    TRACE("Remove type\n");
    //  if(m_selectedTypes.GetSize()==0)
    //CloseWindow();
}

#endif

CXChartCtrl* CECTestGraphDlg::CreateDisplay(const CRect& rcGraph)
{
    CXChartCtrl *pChart = new CXChartCtrl;

    if(!pChart)
        return NULL;

    pChart->Create(NULL,WS_CHILD|WS_VISIBLE,rcGraph,this,0);

    pChart->SetPlotBkColor(RGB(155,155,155));

    pChart->SetLabelColor(RGB(255,255,255));

    pChart->SetAxisTextColor(RGB(90,90,90));

    pChart->SetGridColor(RGB(90,90,90));

    return pChart;
}

void CECTestGraphDlg::OnConfig()
{
    for(int n=0;n<m_selectedTypes.GetSize();n++)
        delete m_selectedTypes[n].pChart;
    m_selectedTypes.RemoveAll();

    //m_strInjector
    KillTimer(IDT_POLL_DATA);
    FindAvailableGraphTypes();

    for(n=0;n<m_availableTypes.GetSize();n++)
        m_selectedTypes.Add(m_availableTypes[n]);

    SetupGraphs();

    SetTimer(IDT_POLL_DATA,m_nRefreshRate,NULL);

}

void CECTestGraphDlg::OnPopupRefresh()
{
    // Allow the selection of the refresh rate
    CECRefreshDlg dlg(m_nRefreshRate);
    int nRet = dlg.DoModal();

    if (nRet != IDOK)
    {
        return;
    }

    // Get the new Refresh Rate and restart the timer
    m_nRefreshRate = dlg.GetRefreshRate();

    KillTimer(IDT_POLL_DATA);

    SetTimer(IDT_POLL_DATA,m_nRefreshRate,NULL);
}

BOOL CECTestGraphDlg::OnEraseBkgnd(CDC *pDC)
{
    return true;
}

void CECTestGraphDlg::OnPopupConfig()
{
    CECGraphSelect selector;
    selector.SetAccess(this);
    int nRet = selector.DoModal();

    if (nRet == IDCANCEL)
        return;

    SetupGraphs();
    Invalidate();
}
