/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECTestStatus.cpp : implementation file
//

#include "stdafx.h"
#include "XTestMon.h"
#include "CECTestStatus.h"
#include "resource.h"
#include "SelectColumnDlg.h"
#include "winsock2.h"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "..\..\HTMLHelp\XTestMon\Test_Monitor.inl"

/////////////////////////////////////////////////////////////////////////////
// CECTestStatus dialog

//keep a count of the number of times corba has been initialised
//across all instances of this ocx
static UINT uiCORBARefCount=0;

CECTestStatus::CECTestStatus(CWnd* pParent /*=NULL*/)
        : CDialog(CECTestStatus::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECTestStatus)
    m_bShowSubTests = FALSE;
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_pCorbaClt=new CORBAClient ( 0, NULL );
    uiCORBARefCount++;

    m_TM = NULL;
    m_ActiveLevel = NULL;

    m_nExpandLevel = 0;

    m_nSelTest = -1;
    m_bAllThreads = FALSE;
    m_bUpdateNow = false;
    m_bQuit = false;
    m_bAborted = false;
}

CECTestStatus::~CECTestStatus()
{
    //Only do the destroys if this is the last occurrence of this ocx
    if((--uiCORBARefCount) == 0)
    {
        m_pCorbaClt->GetBOA()->destroy();
        m_pCorbaClt->GetORB()->NP_destroy();
    }
    delete m_pCorbaClt;
    m_mapThreadIterations.RemoveAll();
}

void CECTestStatus::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECTestStatus)
    DDX_Control(pDX, IDC_SHOW_GRAPH, m_ctlShowGraphs);
    DDX_Control(pDX, IDC_SHOW_SUBTESTS, m_ctlSubTests);
    DDX_Control(pDX, IDC_BACK, m_BackButton);
    DDX_Control(pDX, IDC_THREAD_LABEL, m_ThreadLabel);
    DDX_Control(pDX, IDC_TOTAL_THREADS, m_TotalThreads);
    DDX_Control(pDX, IDC_THREAD_LIST, m_ThreadList);
    DDX_Control(pDX, IDC_LEVEL_LIST, m_LevelList);
    DDX_Control(pDX, IDC_EXPAND, m_ExpandButton);
    DDX_Control(pDX, IDC_ALL_THREADS, m_AllThreads);
    DDX_Check(pDX, IDC_SHOW_SUBTESTS, m_bShowSubTests);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECTestStatus, CDialog)
ON_WM_CONTEXTMENU()
//{{AFX_MSG_MAP(CECTestStatus)
ON_WM_SIZE()
ON_NOTIFY(NM_CLICK, IDC_LEVEL_LIST, OnClickLevelList)
ON_BN_CLICKED(IDC_EXPAND, OnExpand)
ON_BN_CLICKED(IDC_BACK, OnBack)
ON_BN_CLICKED(IDC_ALL_THREADS, OnAllThreads)
ON_WM_CLOSE()
ON_NOTIFY(NM_DBLCLK, IDC_LEVEL_LIST, OnDblclkLevelList)
ON_BN_CLICKED(IDC_SHOW_SUBTESTS, OnShowSubtests)
ON_WM_DESTROY()
ON_WM_PAINT()
ON_COMMAND(ID_LEVEL_SELECT_COLUMNS, OnLevelSelectColumns)
ON_COMMAND(ID_THREAD_SELECT_COLUMNS, OnThreadSelectColumns)
ON_COMMAND(ID_THREAD_VU_DETAILS, OnThreadVuDetails)
ON_NOTIFY(NM_DBLCLK, IDC_THREAD_LIST, OnDblclkThreadList)
ON_BN_CLICKED(IDC_SHOW_GRAPH, OnShowGraph)
ON_WM_HELPINFO()
ON_COMMAND(ID_FULL_HELP, OnFullHelp)
//}}AFX_MSG_MAP

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECTestStatus message handlers

BOOL CECTestStatus::OnInitDialog()
{
    CDialog::OnInitDialog();

    m_ExpandButton.EnableWindow(FALSE);
    m_BackButton.EnableWindow(FALSE);

    m_LevelList.SetExtendedStyle(LVS_EX_FULLROWSELECT);
    m_ImageList.Create(IDB_THREADLIST_BMP,16,0,RGB(0,255,0));
    m_ThreadList.SetImageList(&m_ImageList,LVSIL_SMALL);
    m_ThreadList.SetExtendedStyle(LVS_EX_FULLROWSELECT);

    SetupColumns();
    RestoreSettings();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECTestStatus::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    if(m_ThreadLabel.GetSafeHwnd()== NULL)
        return;

    UpdateData(TRUE);

    const int nSeparator = 10;

    //Get sizes of all unsizable controls
    CRect rcTotalThreads,rcThreadLabel,rcAllThreads,rcExpand,rcBack,rcSubTest,rcShowGraphs;
    m_ThreadLabel.GetWindowRect(&rcThreadLabel);
    m_TotalThreads.GetWindowRect(&rcTotalThreads);
    m_AllThreads.GetWindowRect(&rcAllThreads);
    m_ExpandButton.GetWindowRect(&rcExpand);
    m_BackButton.GetWindowRect(&rcBack);
    m_ctlSubTests.GetWindowRect(&rcSubTest);
    m_ctlShowGraphs.GetWindowRect(&rcShowGraphs);

    int nTopOfLists = 2*nSeparator + rcTotalThreads.Height();
    int nTopOfBottomControls =cy-rcExpand.Height()-nSeparator ;
    if(nTopOfBottomControls< nTopOfLists)
    {
        nTopOfBottomControls = nTopOfLists;
    }

    //Position controls in top right
    int nXOffset = cx - rcShowGraphs.Width() - nSeparator;

    m_ctlShowGraphs.SetWindowPos(NULL,nXOffset,nSeparator/2,0,0,SWP_NOSIZE|SWP_NOZORDER);
    nXOffset -= rcTotalThreads.Width() + nSeparator;

    m_TotalThreads.SetWindowPos(NULL,nXOffset,nSeparator/2, 0,0, SWP_NOSIZE|SWP_NOZORDER);
    nXOffset -= rcThreadLabel.Width()+nSeparator;

    m_ThreadLabel.SetWindowPos(NULL,nXOffset,nSeparator,0,0,SWP_NOSIZE|SWP_NOZORDER);

    //Position controls in bottom left
    nXOffset = nSeparator;

    m_AllThreads.SetWindowPos(NULL,nXOffset,nTopOfBottomControls,0,0,SWP_NOSIZE|SWP_NOZORDER);
    nXOffset+=rcAllThreads.Width()+nSeparator;

    m_ctlSubTests.SetWindowPos(NULL,nXOffset,nTopOfBottomControls,0,0,SWP_NOZORDER|SWP_NOSIZE);
    nXOffset+=rcSubTest.Width()+nSeparator;

    //Position controls in bottom right
    int nLeftMostPos = cx - rcExpand.Width() - 2*nSeparator - rcBack.Width();
    if(nLeftMostPos>nXOffset)
    {
        nXOffset = nLeftMostPos;
    }

    m_BackButton.SetWindowPos(NULL,nXOffset,nTopOfBottomControls,0,0,SWP_NOSIZE|SWP_NOZORDER);
    nXOffset+=rcBack.Width()+nSeparator;

    m_ExpandButton.SetWindowPos(NULL,nXOffset,nTopOfBottomControls, 0, 0, SWP_NOSIZE|SWP_NOZORDER);


    //fit list boxes into the remaining space
    int nTotalListSpace = nTopOfBottomControls-nTopOfLists-nSeparator;
    if(m_bShowSubTests)
    {
        int nListHeight = (nTotalListSpace-nSeparator)/2;
        m_LevelList.SetWindowPos(NULL,nSeparator, nTopOfLists, cx-2*nSeparator, nListHeight, SWP_NOZORDER);
        m_ThreadList.SetWindowPos(NULL,nSeparator, nTopOfLists+nListHeight+nSeparator, cx-2*nSeparator, nListHeight, SWP_NOZORDER);
    }
    else
    {
        m_ThreadList.SetWindowPos(NULL,nSeparator,nTopOfLists, cx-2*nSeparator, nTotalListSpace, SWP_NOZORDER);
    }

    RedrawWindow();
}

void CECTestStatus::SetupColumns()
{
    CRect r;
    m_LevelList.GetClientRect(r);

    m_LevelList.InsertColumn(0, "Script Name",  LVCFMT_CENTER, r.Width()/4 , 0);
    m_LevelList.InsertColumn(1, "Start Time",   LVCFMT_CENTER, r.Width()/4 , 1);
    m_LevelList.InsertColumn(2, "Injector",         LVCFMT_CENTER, r.Width()/4 , 2);
    m_LevelList.InsertColumn(3, "Note Text",    LVCFMT_CENTER, r.Width()/4 , 3);


    m_ThreadList.GetClientRect(r);

    int nNumColumns=9;
    int nWidth = (r.Width()-30) /(nNumColumns-1);

    int nCol=0;
    m_ThreadList.InsertColumn(nCol, "VU ID",        LVCFMT_CENTER, 30 ,     nCol++);
    m_ThreadList.InsertColumn(nCol, "Test Name",    LVCFMT_CENTER, nWidth , nCol++);
    m_ThreadList.InsertColumn(nCol, "Script",       LVCFMT_CENTER, nWidth , nCol++);
    m_ThreadList.InsertColumn(nCol, "Iteration",    LVCFMT_CENTER, nWidth , nCol++);
    m_ThreadList.InsertColumn(nCol, "Injector",     LVCFMT_CENTER, nWidth , nCol++);
    m_ThreadList.InsertColumn(nCol, "Status",       LVCFMT_CENTER, nWidth , nCol++);
    m_ThreadList.InsertColumn(nCol, "Start",        LVCFMT_CENTER, nWidth , nCol++);
    m_ThreadList.InsertColumn(nCol, "Duration",     LVCFMT_CENTER, nWidth , nCol++);
    m_ThreadList.InsertColumn(nCol, "Information",  LVCFMT_CENTER, nWidth , nCol++);
}

void CECTestStatus::ShowTestStatus(CString strManagerName)
{
    m_strRegManager = strManagerName;

    // Initialize the Manager status object
    BOOL bInit = InitManagerStatus();

    m_bQuit = false;
    m_pUpdateThread = AfxBeginThread(&UpdateThread,this);
}

UINT CECTestStatus::UpdateThread(LPVOID pParam)
{
    CECTestStatus *pParent = (CECTestStatus*)pParam;

    DWORD dwStartTime = GetTickCount();

    while(!pParent->m_bQuit)
    {
        try
        {
            CORBA::Object_var varTestManager=pParent->m_pCorbaClt->GetObjectFromClass(TestManager::CLASS_NAME, pParent->m_strRegManager.GetBuffer(0));
        }
        catch(...)
        {
            pParent->m_bAborted = true;
            CWnd* pWnd = pParent->GetParent();
            if (pWnd != NULL)
                pWnd->PostMessage(WM_CLOSE);

            CoUninitialize();
            return 0;
        }

        pParent->UpdateSubTestList();
        pParent->UpdateThreadList();

        while(1)
        {
            if(pParent->m_bUpdateNow ||
                    (GetTickCount()-dwStartTime >= 5000))
            {
                dwStartTime = GetTickCount();
                pParent->m_bUpdateNow=false;
                break;
            }

            Sleep(1000);
        }
    }

    pParent->m_bQuit = false;

    return 0;
}

BOOL CECTestStatus::InitManagerStatus()
{
    BOOL bStatus=TRUE;
    try
    {
        //Get the Test manager status object
        CORBA::Object_var obj;
        obj=m_pCorbaClt->GetObjectFromClass(TestManager::CLASS_NAME, m_strRegManager.GetBuffer(0));
        m_TM = narrow_object((TestManager_ptr)NULL,obj);
    }
    catch(...)
    {
        bStatus = FALSE;
    }

    return bStatus;
}

void CECTestStatus::UpdateSubTestList()
{
    if(!m_bShowSubTests)
        return;

    m_LevelList.SetRedraw(FALSE);
    m_LevelList.DeleteAllItems();
    m_TestLevelList.clear();

    try
    {
        if (m_ActiveLevel==NULL)
            m_ActiveLevel = m_TM->GetTestLevel();
        //
        //  Get the Test name of the active Level and fire an event to the host to inform
        //  of the curent view of the monitor.
        //
        CString strScript = m_ActiveLevel->ScriptName();
        // Strip off script path.
        int nLastSlashPos = strScript.ReverseFind('\\');
        strScript.Delete(0,nLastSlashPos+1);

        // If there is an extension remove it for display
        strScript = strScript.SpanExcluding(".");

        LPCTSTR testName = strScript;
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_CURRENT_LEVEL,0,(long)testName);

        //
        // If any child levels exist read the status information into the List View
        //
        TestLevelList TLList;
        long nChildTests = m_ActiveLevel->GetChildTestLevelList(TLList);
        if (nChildTests > 0)
        {
            for(int i=0;i<TLList.length();i++)
            {
                TestLevel_var childLevel=TLList[i];

                //
                // Read the status information from the child level
                //
                strScript = childLevel->ScriptName();

                // Strip off script path.
                int nLastSlashPos = strScript.ReverseFind('\\');
                strScript.Delete(0,nLastSlashPos+1);

                // If there is an extension remove it for display
                strScript = strScript.SpanExcluding(".");

                CString strNode = GetComputerNameFromIP(childLevel->GetNode());
                CString strNoteText = childLevel->GetNoteText();
                CTTime StartTime(childLevel->StartTime());
                string TimeString = StartTime.Format("%H:%M:%S %d/%m/%Y" );
                CString strTime(TimeString.c_str());

                //
                // Insert the information into a new row in the list view.
                //
                m_LevelList.InsertItem(i,strScript, 0);
                m_LevelList.SetItem(i, 1, LVIF_TEXT , strTime, 0, 0, LVIF_STATE, NULL);
                m_LevelList.SetItem(i, 2, LVIF_TEXT , strNode, 0, 0, LVIF_STATE, NULL);
                m_LevelList.SetItem(i, 3, LVIF_TEXT , strNoteText, 0, 0, LVIF_STATE, NULL);
            }
        }
    }
    catch(...)
    {}

    m_LevelList.SetRedraw();
}

void CECTestStatus::UpdateThreadList()
{
    try
    {
        if (m_ActiveLevel==NULL)
            m_ActiveLevel = m_TM->GetTestLevel();

        long nChildThreads = m_ActiveLevel->NoChildThreads(false);
        long nTotalThreads = m_ActiveLevel->NoChildThreads(true);

        CString threadCount;
        threadCount.Format("%d", nTotalThreads);
        m_TotalThreads.SetWindowText(threadCount);

        CThreadInfo threadInfo;
        CMap <UINT,UINT&,CThreadInfo,CThreadInfo&> mapNewThreads;
        // Add the child threads for this level
        if (nChildThreads > 0)
        {
            mapNewThreads.InitHashTable(nChildThreads);
            TestExecuterList TEList;
            unsigned long no_executers = m_ActiveLevel->GetTestExecuterList(TEList);
            for(long TEiter=0;TEiter<TEList.length();TEiter++)
            {
                threadInfo.strInjector=(*TEList[TEiter]).GetNodeName();
                threadInfo.strTestName=(*TEList[TEiter]).TestName();
                ThreadInfoList TIList;
                CORBA::ULong no_threads=TEList[TEiter]->GetThreadInformation(TIList);
                for(long TIiter=0;TIiter<TIList.length();TIiter++)
                {
                    LoadThreadInfo(TIList[TIiter],threadInfo);
                    mapNewThreads.SetAt(threadInfo.uiIdent,threadInfo);
                }
            }
        }
        // If "Show All Threads" is enabled search through the child levels
        // to find any more running threads
        if (m_bAllThreads)
        {
            AddChildThreads(m_ActiveLevel,mapNewThreads);
        }

        UpdateThreadListView(mapNewThreads);
        mapNewThreads.RemoveAll();
    }
    catch(...)
    {
        TRACE("Caught exception\n");
    }
}

void CECTestStatus::UpdateThreadListView(CMap<UINT,UINT&,CThreadInfo,CThreadInfo&> &newThreads)
{
    m_ThreadList.SetRedraw(FALSE);

    UINT uiIdent;
    CThreadInfo threadInfoNew,threadInfoOld;

    bool bFoundVUDlgThread=false;
    bool bModified = false;
    CString strIteration,strStatus;
    //Check that every item in the list exists in newThread
    int nNumListItems = m_ThreadList.GetItemCount();

    int nNumInView = m_ThreadList.GetCountPerPage()+1;
    int nRefreshed = 0;
    int nFirstIndex = GetFirstInView(&m_ThreadList);

    for(int nItem=nFirstIndex;nItem<nNumListItems;nItem++)
    {
        uiIdent = m_ThreadList.GetItemData(nItem);

        if (nRefreshed < nNumInView)
        {
            bModified = false;
            if(newThreads.Lookup(uiIdent,threadInfoNew))
            {
                //The thread is still running so check it is uptodate
                m_mapThreads.Lookup(uiIdent,threadInfoOld);

                if(threadInfoOld != threadInfoNew)
                {
                    bModified = true;

                    if(threadInfoNew.nIteration == -1)
                        strIteration = "0";
                    else
                        strIteration.Format("%d",threadInfoNew.nIteration);
                    switch(threadInfoNew.nStatus)
                    {
                    case ACTIVE:
                        strStatus = "Active";
                        break;
                    case SUSPENDED:
                        strStatus = "Suspended";
                        break;
                    }

                    string strTemp = threadInfoNew.rwStartTime.Format("%H:%M:%S %d/%m/%Y" );
                    CString strStartTime(strTemp.c_str());

                    strTemp = threadInfoNew.rwDuration.Format("%H:%M:%S");
                    CString strDuration(strTemp.c_str());

                    m_ThreadList.SetItemText(nItem,0,threadInfoNew.strId);
                    m_ThreadList.SetItemText(nItem,1,threadInfoNew.strTestName);
                    m_ThreadList.SetItemText(nItem,2,threadInfoNew.strScript);
                    m_ThreadList.SetItemText(nItem,3,strIteration);
                    m_ThreadList.SetItemText(nItem,4,threadInfoNew.strInjector);
                    m_ThreadList.SetItemText(nItem,5,strStatus);
                    m_ThreadList.SetItemText(nItem,6,strStartTime);
                    m_ThreadList.SetItemText(nItem,7,strDuration);

                    //Hold on to the information field if there is no new info
                    if(!threadInfoOld.strInformation.IsEmpty())
                    {
                        if(threadInfoNew.strInformation.IsEmpty() ||
                                threadInfoNew.strInformation == threadInfoOld.strInformation.SpanExcluding("\r"))
                        {
                            //If the new line is empty or is the same as
                            //the previous line, ignore it
                            threadInfoNew.strInformation = threadInfoOld.strInformation;
                        }
                        else
                            threadInfoNew.strInformation += "\r\n" + threadInfoOld.strInformation;


                        //Limit info field to nMaxInfo characters
                        const int nMaxInfo = 300;
                        if(threadInfoNew.strInformation.GetLength() > nMaxInfo)
                        {
                            threadInfoNew.strInformation = threadInfoNew.strInformation.Left(nMaxInfo);
                            //Trim back to last newline
                            int nNewLinePos = threadInfoNew.strInformation.ReverseFind('\r');
                            threadInfoNew.strInformation = threadInfoNew.strInformation.Left(nNewLinePos);
                        }
                    }

                    m_ThreadList.SetItemText(nItem,8,threadInfoNew.strInformation.SpanExcluding("\r"));

                    m_mapThreads.SetAt(uiIdent,threadInfoNew);
                }

                //Update VU details dlg
                if(m_dlgVUDetails.GetCurrentThread()==uiIdent)
                {
                    bFoundVUDlgThread = true;
                    if(bModified)
                        m_dlgVUDetails.SetThreadInfo(threadInfoNew);
                }
            }

            nRefreshed++;
        }


        if(!newThreads.Lookup(uiIdent,threadInfoNew))
        {
            //The thread no longer exists so remove it from the view and map
            m_mapThreads.RemoveKey(uiIdent);
            m_ThreadList.DeleteItem(nItem--);
            nNumListItems--;
        }

    }

    if(!bFoundVUDlgThread)
        m_dlgVUDetails.Clear();

    int nCurrentNdx = nNumListItems;
    POSITION pos = newThreads.GetStartPosition();
    while(pos != NULL)
    {
        newThreads.GetNextAssoc(pos,uiIdent,threadInfoNew);

        if(!m_mapThreads.Lookup(uiIdent,threadInfoOld))
        {
            m_mapThreads.SetAt(uiIdent,threadInfoNew);

            if(threadInfoNew.nIteration == -1)
                strIteration = "-";
            else
                strIteration.Format("%d",threadInfoNew.nIteration);
            switch(threadInfoNew.nStatus)
            {
            case ACTIVE:
                strStatus = "Active";
                break;
            case SUSPENDED:
                strStatus = "Suspended";
                break;
            }

            string strTemp = threadInfoNew.rwStartTime.Format("%H:%M:%S %d/%m/%Y" );
            CString strStartTime(strTemp.c_str());

            strTemp = threadInfoNew.rwDuration.Format("%H:%M:%S");
            CString strDuration(strTemp.c_str());

            nCurrentNdx = m_ThreadList.InsertItem(nCurrentNdx,threadInfoNew.strId, enThread);
            m_ThreadList.SetItemText(nCurrentNdx,1,threadInfoNew.strTestName);
            m_ThreadList.SetItemText(nCurrentNdx,2,threadInfoNew.strScript);
            m_ThreadList.SetItemText(nCurrentNdx,3,strIteration);
            m_ThreadList.SetItemText(nCurrentNdx,4,threadInfoNew.strInjector);
            m_ThreadList.SetItemText(nCurrentNdx,5,strStatus);
            m_ThreadList.SetItemText(nCurrentNdx,6,strStartTime);
            m_ThreadList.SetItemText(nCurrentNdx,7,strDuration);
            m_ThreadList.SetItemText(nCurrentNdx,8,threadInfoNew.strInformation.SpanExcluding("\r"));
            m_ThreadList.SetItemData(nCurrentNdx,uiIdent);
            nCurrentNdx++;
        }
    }
    m_ThreadList.SetRedraw(TRUE);
}

char* CECTestStatus::GetNextField(char sep, char*& szIn)
{
    if(szIn==NULL)
        return NULL;

    char szSearch[2] = {sep,'\0'};
    char *szSepPos = strstr(szIn,szSearch);
    szSepPos[0] = '\0';

    char *pszResult = szIn;
    szIn = ++szSepPos;
    return pszResult;
}

void CECTestStatus::OnContextMenu(CWnd* pWnd, CPoint point)
{
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;

    switch(pWnd->GetDlgCtrlID())
    {
    case IDC_THREAD_LIST:
        VERIFY(menu.LoadMenu(IDR_THREADLIST_POPUP));
        break;
    case IDC_LEVEL_LIST:
        VERIFY(menu.LoadMenu(IDR_LEVELLIST_POPUP));
        break;
    default:
        VERIFY(menu.LoadMenu(IDR_FULL_HELP_POPUP));
    }

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    // Show the popup menu
    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);
}

void CECTestStatus::OnClickLevelList(NMHDR* pNMHDR, LRESULT* pResult)
{
    m_ExpandButton.EnableWindow(m_LevelList.GetFirstSelectedItemPosition() != NULL);

    *pResult = 0;
}


void CECTestStatus::OnExpand()
{
    try
    {
        m_ActiveLevel = m_TestLevelList[m_nSelTest];
        m_nExpandLevel++;
        m_BackButton.EnableWindow(TRUE);
        m_ExpandButton.EnableWindow(FALSE);

        m_bUpdateNow = true;
    }
    catch(...)
    {}


}

void CECTestStatus::OnBack()
{
    try
    {
        if (m_nExpandLevel>0)
        {
            m_ActiveLevel = m_ActiveLevel->GetParentTestLevel();
            m_nExpandLevel--;
            m_bUpdateNow=true;

            if (m_nExpandLevel == 0)
            {
                m_BackButton.EnableWindow(FALSE);
            }
        }
    }
    catch(...)
    {}


}

void CECTestStatus::OnAllThreads()
{
    m_bAllThreads = (BOOL)m_AllThreads.GetCheck();
    m_bUpdateNow=true;
}

void CECTestStatus::AddChildThreads(TestLevel_var level,CMap<UINT,UINT&,CThreadInfo,CThreadInfo&> &mapNewThreads)
{
    long nChildTests = level->NoChildTestLevels(false);
    if (nChildTests > 0)
    {
        TestLevelList TLList;
        CORBA::ULong no_testlevels=level->GetChildTestLevelList(TLList);
        CThreadInfo threadInfo;
        for(int nTest=0;nTest<TLList.length();nTest++)
        {
            TestLevel_var childLevel = TLList[nTest];
            m_TestLevelList.push_back(TestLevel::_duplicate(childLevel));
            // Add thread information for the child level
            TestExecuterList TEList;
            unsigned long no_executers = childLevel->GetTestExecuterList(TEList);
            for(long TEiter=0;TEiter<TEList.length();TEiter++)
            {
                threadInfo.strInjector=(*TEList[TEiter]).GetNodeName();
                threadInfo.strTestName=(*TEList[TEiter]).TestName();
                ThreadInfoList TIList;
                CORBA::ULong no_threads=TEList[TEiter]->GetThreadInformation(TIList);
                for(long TIiter=0;TIiter<TIList.length();TIiter++)
                {
                    LoadThreadInfo(TIList[TIiter],threadInfo);
                    mapNewThreads.SetAt(threadInfo.uiIdent,threadInfo);
                }
            }
            //          if (nChildThreads > 0)
            //          {
            //              char* ThreadIdents = childLevel->GetChildTestThreadIdents();
            //              char *pszThreadIdent = strtok(ThreadIdents,"|");
            //              while(pszThreadIdent != NULL)
            //              {
            //                  LoadThreadInfo(childLevel,pszThreadIdent,threadInfo);
            //
            //                  mapNewThreads.SetAt(threadInfo.uiIdent,threadInfo);
            //
            //                  pszThreadIdent = strtok(NULL,"|");
            //              }
            //          }

            // Then add thread info from levels below this child
            AddChildThreads(childLevel,mapNewThreads);
        }
    }
}

void CECTestStatus::LoadThreadInfo(const ThreadInfo& info,CThreadInfo& threadInfo)
{
    //UNIQUE ID
    threadInfo.uiIdent = HashKey((const char*)info.UIdent);

    //Get the VU ID
    char *pszID = strrchr((const char*)info.UIdent,'-');
    if(pszID == NULL)
        threadInfo.strId.Empty();
    else
        threadInfo.strId = pszID + 1;

    //TEST NAME is the same for all Threads so do not change

    //SCRIPT NAME
    threadInfo.strScript = info.ScriptName;

    // Strip off script path.
    int nLastSlashPos = threadInfo.strScript.ReverseFind('\\');
    if (nLastSlashPos != -1)
        threadInfo.strScript.Delete(0,nLastSlashPos);

    // If there is an extension remove it for display
    threadInfo.strScript = threadInfo.strScript.SpanExcluding(".");

    //ITERATION
    CString strNote = info.NoteText;
    if(strNote.Left(15).CompareNoCase("LOG: ITERATION ") == 0)
    {
        strNote.Delete(0,15);
        threadInfo.nIteration = atoi(strNote);
        m_mapThreadIterations.SetAt(threadInfo.uiIdent,threadInfo.nIteration);
        strNote.Empty();
    }
    else
    {
        if(!m_mapThreadIterations.Lookup(threadInfo.uiIdent,threadInfo.nIteration))
            threadInfo.nIteration = -1;
    }

    //INJECTOR  is the same for all Threads so do not change

    //STATUS
    try
    {
        threadInfo.nStatus = info.Status;
    }
    catch(...)
    {}
    //START TIME

    CTTime StartTime(info.StartTime);
    threadInfo.rwStartTime = StartTime;

    //DURATION
    threadInfo.rwDuration = CTTime();
    if(threadInfo.rwDuration <= threadInfo.rwStartTime)
        //If current time is before start time! Set duration to one second as
        //zero seconds doesn't format correctly
        threadInfo.rwDuration -= (threadInfo.rwDuration.getSeconds()-1);
    else
        threadInfo.rwDuration -= threadInfo.rwStartTime.getSeconds();

    //INFORMATION
    threadInfo.strInformation = strNote;
}

void CECTestStatus::OnClose()
{

    CDialog::OnClose();
}

void CECTestStatus::OnDblclkLevelList(NMHDR* pNMHDR, LRESULT* pResult)
{
    POSITION pos = m_LevelList.GetFirstSelectedItemPosition();
    if(pos == NULL)
        return;

    m_nSelTest = m_LevelList.GetNextSelectedItem(pos);
    OnExpand();

    *pResult = 0;
}

void CECTestStatus::OnShowSubtests()
{
    UpdateData(TRUE);

    m_LevelList.ShowWindow(m_bShowSubTests ? SW_SHOW :SW_HIDE);

    CRect rcClient;
    GetClientRect(&rcClient);
    SendMessage(WM_SIZE,(WPARAM)SIZE_RESTORED,MAKELPARAM(rcClient.Width(),rcClient.Height()));
    m_bUpdateNow=true;
}

void CECTestStatus::SaveSettings()
{
    UpdateData(TRUE);

    AfxGetApp()->WriteProfileInt("Options","Show All Threads",m_bAllThreads);
    AfxGetApp()->WriteProfileInt("Options","Show Sub Tests",m_bShowSubTests);

    //Store settings for thread list
    CString strEntry;
    int nNumColumns = m_ThreadList.GetHeaderCtrl()->GetItemCount();
    AfxGetApp()->WriteProfileInt("Thread List Columns","Num Columns",nNumColumns);

    int nWidth = 0;
    for(int nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        nWidth = m_ThreadList.GetColumnWidth(nColumn);
        strEntry.Format("%d",nColumn);
        AfxGetApp()->WriteProfileInt("Thread List Columns",strEntry,nWidth);
    }

    //Store settings for sub test list
    nNumColumns = m_LevelList.GetHeaderCtrl()->GetItemCount();
    AfxGetApp()->WriteProfileInt("SubTest List Columns","Num Columns",nNumColumns);

    for(nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        nWidth = m_LevelList.GetColumnWidth(nColumn);

        strEntry.Format("%d",nColumn);
        AfxGetApp()->WriteProfileInt("SubTest List Columns",strEntry,nWidth);
    }
}

void CECTestStatus::RestoreSettings()
{
    m_bAllThreads = AfxGetApp()->GetProfileInt("Options","Show All Threads",false);
    m_bShowSubTests = AfxGetApp()->GetProfileInt("Options","Show Sub Tests",true);

    //Restore settings for thread list
    CHeaderCtrl* pheader = m_ThreadList.GetHeaderCtrl();

    int nWidth;
    CString strEntry;
    int nNumColumns = AfxGetApp()->GetProfileInt("Thread List Columns","Num Columns",0);
    for(int nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        strEntry.Format("%d",nColumn);
        nWidth = AfxGetApp()->GetProfileInt("Thread List Columns",strEntry,0);
        m_ThreadList.SetColumnWidth(nColumn,nWidth);
    }

    //Restore settings for sub test list
    pheader = m_LevelList.GetHeaderCtrl();

    nNumColumns = AfxGetApp()->GetProfileInt("SubTest List Columns","Num Columns",0);
    for(nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        strEntry.Format("%d",nColumn);
        nWidth = AfxGetApp()->GetProfileInt("SubTest List Columns",strEntry,0);

        m_LevelList.SetColumnWidth(nColumn,nWidth);
    }
    UpdateData(FALSE);
    OnShowSubtests();
    m_AllThreads.SetCheck(m_bAllThreads);
}

void CECTestStatus::OnDestroy()
{
    //End thread
    if(!m_bAborted)
    {
        int nCount =100;
        m_bQuit = true;
        while(m_bQuit && nCount--)
            Sleep(10);
        if(!nCount)
        {
            DWORD dwExit =0;
            TerminateThread(m_pUpdateThread->m_hThread,dwExit);
        }
    }

    CDialog::OnDestroy();

    SaveSettings();
}

void CECTestStatus::OnPaint()
{
    CPaintDC dc(this); // device context for painting

    CRect rc;
    m_ThreadList.GetWindowRect(&rc);
    ScreenToClient(&rc);
    if(rc.IntersectRect(rc,&dc.m_ps.rcPaint))
    {
        m_ThreadList.Invalidate();
    }

    m_LevelList.GetWindowRect(&rc);
    ScreenToClient(&rc);
    if(rc.IntersectRect(rc,&dc.m_ps.rcPaint))
    {
        m_LevelList.Invalidate();
    }

    // Do not call CDialog::OnPaint() for painting messages
}


void CECTestStatus::OnLevelSelectColumns()
{
    FilterColumns(&m_LevelList);
}

void CECTestStatus::OnThreadSelectColumns()
{
    FilterColumns(&m_ThreadList);
}

void CECTestStatus::FilterColumns(CListCtrl *pListCtrl)
{
    CSelectColumnDlg dlg;

    CHeaderCtrl *pHeader = pListCtrl->GetHeaderCtrl();
    int nNumCols = pHeader->GetItemCount();

    const int nBufferSize=50;
    char szBuffer[nBufferSize];
    LVCOLUMN lvColumn;
    lvColumn.mask = LVCF_TEXT|LVCF_WIDTH;
    lvColumn.pszText = szBuffer;
    lvColumn.cchTextMax = nBufferSize;

    for(int nCol=0;nCol<nNumCols;nCol++)
    {
        pListCtrl->GetColumn(nCol,&lvColumn);
        dlg.AddOption(lvColumn.pszText,lvColumn.cx);
    }

    if(dlg.DoModal()==IDOK)
    {
        CRect rcList;
        pListCtrl->GetClientRect(rcList);

        int nNumColumns = dlg.GetNumOptions();
        int nNumSelColumns = 0;
        for(nCol=0;nCol<nNumColumns;nCol++)
        {
            if(dlg.IsSelected(nCol))
                nNumSelColumns++;
        }

        int nColWidth = rcList.Width()/nNumSelColumns;
        for(nCol=0;nCol<nNumColumns;nCol++)
        {
            if(dlg.IsSelected(nCol))
                pListCtrl->SetColumnWidth(nCol,nColWidth);
            else
                pListCtrl->SetColumnWidth(nCol,0);
        }
    }
}

CString CECTestStatus::GetComputerNameFromIP(const CString& strIP)
{
    CString strName;
    if(m_mapIPToName.Lookup(strIP,strName))
        return strName;

    CString strIPCopy(strIP);
    strIPCopy.Replace('_','.');
    DWORD dwAddr = inet_addr(strIPCopy);

    HOSTENT *pHost = gethostbyaddr((char*)&dwAddr,4,AF_INET);
    if(pHost==NULL)
    {
        TRACE("Net error %d\n",WSAGetLastError());
        strName = strIP;
    }
    else
    {
        strName = pHost->h_name;
        strName = strName.SpanExcluding(".");
    }
    m_mapIPToName.SetAt(strIP,strName);
    return strName;
}

void CECTestStatus::OnThreadVuDetails()
{
    POSITION pos = m_ThreadList.GetFirstSelectedItemPosition();
    if(pos != NULL)
    {
        int nNdx = m_ThreadList.GetNextSelectedItem(pos);
        UINT uiIdent = m_ThreadList.GetItemData(nNdx);

        CThreadInfo threadInfo;
        if(m_mapThreads.Lookup(uiIdent,threadInfo))
        {
            m_dlgVUDetails.SetThreadInfo(threadInfo);
            if(m_dlgVUDetails.GetSafeHwnd()==NULL)
            {
                m_dlgVUDetails.Create(IDD_VUDETAILS_DLG,this);
            }
        }
    }
}



void CECTestStatus::OnDblclkThreadList(NMHDR* pNMHDR, LRESULT* pResult)
{
    OnThreadVuDetails();

    *pResult = 0;
}

void CECTestStatus::OnShowGraph()
{
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DYN_GRAPH,(WPARAM)(LPCTSTR)"", (LPARAM)(LPCTSTR)m_strRegManager);
}

BOOL CECTestStatus::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

int CECTestStatus::GetFirstInView(CListCtrl* pList)
{
    CPoint pt;
    pt.x = 5;
    pt.y = 5;

    int nHit = pList->HitTest(pt);

    if (nHit == -1)
        nHit = 0;

    return nHit;
}

void CECTestStatus::OnFullHelp()
{
    HtmlHelp(m_hWnd, m_szHtmlHelpFile, HH_DISPLAY_TOPIC, (DWORD)"xtestmon.htm");
}
