/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XTestMonCtl.cpp : Implementation of the CXTestMonCtrl ActiveX Control class.

#include "stdafx.h"
#include "XTestMon.h"
#include "XTestMonCtl.h"
#include "XTestMonPpg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CXTestMonCtrl, COleControl)


/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CXTestMonCtrl, COleControl)
//{{AFX_MSG_MAP(CXTestMonCtrl)
ON_WM_CREATE()
ON_WM_SIZE()
ON_WM_ERASEBKGND()
ON_WM_DESTROY()
//}}AFX_MSG_MAP
ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
ON_MESSAGE (WMU_CURRENT_LEVEL, OnUpdateActiveTest)
ON_MESSAGE(WMU_DYN_GRAPH, OnShowDynGraph)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CXTestMonCtrl, COleControl)
//{{AFX_DISPATCH_MAP(CXTestMonCtrl)
DISP_FUNCTION(CXTestMonCtrl, "XShowTestStatus", XShowTestStatus, VT_EMPTY, VTS_BSTR)
//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CXTestMonCtrl, COleControl)
//{{AFX_EVENT_MAP(CXTestMonCtrl)
EVENT_CUSTOM("XUpdateActiveTestName", FireXUpdateActiveTestName, VTS_BSTR)
EVENT_CUSTOM("ShowDynGraph", FireShowDynGraph, VTS_BSTR)
EVENT_CUSTOM("ClosingControl", FireClosingControl, VTS_NONE)
//}}AFX_EVENT_MAP
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CXTestMonCtrl, 1)
PROPPAGEID(CXTestMonPropPage::guid)
END_PROPPAGEIDS(CXTestMonCtrl)


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CXTestMonCtrl, "XTESTMON.XTestMonCtrl.1",
                       0x6cf93b26, 0x9051, 0x11d3, 0x9a, 0x95, 0, 0x50, 0x4, 0x47, 0xfc, 0x27)


/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CXTestMonCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DXTestMon =
    {
        0x6cf93b24, 0x9051, 0x11d3, { 0x9a, 0x95, 0, 0x50, 0x4, 0x47, 0xfc, 0x27 }
    };
const IID BASED_CODE IID_DXTestMonEvents =
    {
        0x6cf93b25, 0x9051, 0x11d3, { 0x9a, 0x95, 0, 0x50, 0x4, 0x47, 0xfc, 0x27 }
    };


/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwXTestMonOleMisc =
    OLEMISC_ACTIVATEWHENVISIBLE |
    OLEMISC_SETCLIENTSITEFIRST |
    OLEMISC_INSIDEOUT |
    OLEMISC_CANTLINKINSIDE |
    OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CXTestMonCtrl, IDS_XTESTMON, _dwXTestMonOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CXTestMonCtrl::CXTestMonCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CXTestMonCtrl

BOOL CXTestMonCtrl::CXTestMonCtrlFactory::UpdateRegistry(BOOL bRegister)
{
    // TODO: Verify that your control follows apartment-model threading rules.
    // Refer to MFC TechNote 64 for more information.
    // If your control does not conform to the apartment-model rules, then
    // you must modify the code below, changing the 6th parameter from
    // afxRegApartmentThreading to 0.

    if (bRegister)
        return AfxOleRegisterControlClass(
                   AfxGetInstanceHandle(),
                   m_clsid,
                   m_lpszProgID,
                   IDS_XTESTMON,
                   IDB_XTESTMON,
                   afxRegApartmentThreading,
                   _dwXTestMonOleMisc,
                   _tlid,
                   _wVerMajor,
                   _wVerMinor);
    else
        return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CXTestMonCtrl::CXTestMonCtrl - Constructor

CXTestMonCtrl::CXTestMonCtrl()
{
    InitializeIIDs(&IID_DXTestMon, &IID_DXTestMonEvents);

    // TODO: Initialize your control's instance data here.
}


/////////////////////////////////////////////////////////////////////////////
// CXTestMonCtrl::~CXTestMonCtrl - Destructor


CXTestMonCtrl::~CXTestMonCtrl()
{
    // TODO: Cleanup your control's instance data here.
}


/////////////////////////////////////////////////////////////////////////////
// CXTestMonCtrl::OnDraw - Drawing function


void CXTestMonCtrl::OnDraw(
    CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
    // TODO: Replace the following code with your own drawing code.
    if(m_TestStatus.GetSafeHwnd()==NULL)
    {
        pdc->FillRect(rcBounds, CBrush::FromHandle((HBRUSH)GetStockObject(WHITE_BRUSH)));
        pdc->TextOut(0,0,"XTestMon");
    }
}


/////////////////////////////////////////////////////////////////////////////
// CXTestMonCtrl::DoPropExchange - Persistence support

void CXTestMonCtrl::DoPropExchange(CPropExchange* pPX)
{
    ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
    COleControl::DoPropExchange(pPX);

    // TODO: Call PX_ functions for each persistent custom property.

}


/////////////////////////////////////////////////////////////////////////////
// CXTestMonCtrl::OnResetState - Reset control to default state


void CXTestMonCtrl::OnResetState()
{
    COleControl::OnResetState();  // Resets defaults found in DoPropExchange

    // TODO: Reset any other control state here.
}


/////////////////////////////////////////////////////////////////////////////
// CXTestMonCtrl message handlers


int CXTestMonCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (COleControl::OnCreate(lpCreateStruct) == -1)
        return -1;

    m_TestStatus.Create(IDD_TESTMON, this);
    m_TestStatus.ShowWindow(SW_SHOW);
    m_TestStatus.Invalidate();
    Invalidate();

    return 0;
}

void CXTestMonCtrl::OnSize(UINT nType, int cx, int cy)
{
    COleControl::OnSize(nType, cx, cy);

    if(m_TestStatus.m_hWnd != NULL)
    {
        m_TestStatus.SetWindowPos(this, 0,0,cx,cy,SWP_NOZORDER);
    }
}

void CXTestMonCtrl::XShowTestStatus(LPCTSTR strManagerName)
{
    m_TestStatus.ShowTestStatus(strManagerName);
}

BOOL CXTestMonCtrl::PreTranslateMessage(MSG* pMsg)
{
    if(m_TestStatus.PreTranslateMessage(pMsg))
        return true;

    return COleControl::PreTranslateMessage(pMsg);
}

LRESULT CXTestMonCtrl::OnUpdateActiveTest(WPARAM id,LPARAM ptrTestName)
{
    LPCSTR testString = (LPCSTR) ptrTestName;
    CString szActiveTest = testString;

    FireXUpdateActiveTestName(szActiveTest);
    return 0;
}


BOOL CXTestMonCtrl::OnEraseBkgnd(CDC* pDC)
{
    return true;
}

LRESULT CXTestMonCtrl::OnShowDynGraph(WPARAM,LPARAM lParam)
{
    LPCTSTR szRegManager = (LPCTSTR)lParam;
    FireShowDynGraph(szRegManager);
    return 0;
}

void CXTestMonCtrl::OnDestroy()
{
    COleControl::OnDestroy();

    FireClosingControl();
}
