/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CORBAClient
///////////////////////////////////////////////////////////////////////////////

#ifndef _CORBAClient_h_
#define _CORBAClient_h_

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CORBA/CORBAGeneral.h>
#include <CORBA/CORBAHelper.inl>

///////////////////////////////////////////////////////////////////////////////
// @class   This class allows easier client access to CORBA objects.  Any
//          object references returned by this class will still have to be
//          narrowed to the appropriate type.
// @base    public | CORBAGeneral
class CYR_EXPORT CORBAClient : public CORBAGeneral
{
    // @access Public Members
public:
    CORBAClient( int argc, char** argv );       // @cmember Constructor.
    CORBAClient( CORBA::ORB_ptr orb, CORBA::BOA_ptr boa );    // @cmember Constructor.

    ///////////////////////////////////////////////////////////////////////////
    // @cmember T* | Narrow | ( T** cls, CORBA::Object_ptr obj ) | Narrowing
    //          method for CORBA objects.
    // @mfunc   T* | CORBAClient | Narrow | This templated method allows CORBA
    //          'Object_ptr's to be narrowed to an appropriate type.  If the
    //          narrowing fails then an exception is raised.
    // @tfarg   class | T | This is the type of the class the method is
    //          narrowing the object pointer to.
    // @rdesc   A pointer to the class we wish to narrow to.
    // @parm    T** | cls | A pointer to the class we wish to narrow to.
    // @parm    Object_ptr | obj | The object pointer to be narrowed.
    //
    // @ex      An example of narrowing a CORBA object: |
    //          ...
    //          CORBA::Object_ptr obj = pClient->GetObjectFromClass( Factory::CLASS_NAME, "Object" );
    //          Factory_ptr factory;
    //          pClient->Narrow( factory, obj );
    //          ...
    template<class T>
    T* Narrow( T** cls, CORBA::Object_ptr obj )
    {
        *cls = narrow_object((T*)NULL, obj );
        return *cls;
    }
};

#endif
