/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CORBAGeneral
///////////////////////////////////////////////////////////////////////////////

#ifndef _CORBAGeneral_h
#define _CORBAGeneral_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes

#include <CyrException.h>
#include <CORBA/CORBAHelper.inl>
///////////////////////////////////////////////////////////////////////////////
// CORBA includes
#include <omniORB2/CORBA.h>
#include <omniORB2/Naming.hh>

///////////////////////////////////////////////////////////////////////////////
// @class   This class is the base class for the CORBA wrappers.  It contains
//          any generic functionality used by CORBA.
class CYR_EXPORT CORBAGeneral
{
    // @access Public Members
public:
    CORBAGeneral( int argc, char** argv );      // @cmember Constructor.
    CORBAGeneral( CORBA::ORB_ptr orb, CORBA::BOA_ptr boa );   // @cmember Constructor.
    virtual ~CORBAGeneral();                    // @cmember Destructor.

    // @cmember Informs application that CORBA is ready.
    void Ready( bool block );

    // @cmember Lists all objects of a particular type.
    string GetRegisteredList( const char* type = NULL );

    // Accessor methods
    // @cmember Returns the ORB pointer.
    CORBA::ORB_ptr GetORB();
    // @cmember Returns the BOA pointer.
    CORBA::BOA_ptr GetBOA();

    // @cmember Retrieve a reference to a CORBA object.
    CORBA::Object_ptr GetObjectFromClass( const char* cls, const char* name );
    // @cmember Retrieve a reference to a CORBA object.
    CORBA::Object_ptr GetObjectFromIOR( const char* ior );

    // @access Protected Members
protected:
    CORBA::ORB_ptr m_orb;          // @cmember ORB reference.
    CORBA::BOA_ptr m_boa;          // @cmember BOA reference.

    // @cmember Get reference to NamingService
    CosNaming::NamingContext_ptr GetNamingService();

    // @access Private Members
private:
    bool m_bInit;           // @cmember Did this class initialise ORB?
}
;

#endif

