/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrWrapLib
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECOrbFolder_h
#define _CECOrbFolder_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrWrapLib/CECOrbProject.h>
#include <CyrWrapLib/CECOrbLink.h>

///////////////////////////////////////////////////////////////////////////////
// @class   CECOrbFolder | This class is a wrapper around the CORBA interface
//          to OpenSTARep's Folder objects.  It provides a basic set of methods
//          for creation and retrieval of Folders.  Specific Folder
//          instances, such as a Project Settings Folder, are derived from
//          this class and are expected to initialize themselves correctly.
// @base    public | CECOrbLink
class CECOrbFolder : public CECOrbLink
{
    // @access Public Members
public:
    // @cmember Constructor for retrieving root Folders
    CECOrbFolder( CECOrbProject* project );
    // @cmember Constructor for retrieving child Folders
    CECOrbFolder( CECOrbFolder* parent, const char* name );
    // @cmember Constructor for creating new child Folders
    CECOrbFolder( CECOrbFolder* parent, const char* name, const char* desc );

    string GetName() ;         // @cmember Returns name of Folder
    string GetFolders( long& count ) const;  // @cmember Returns list of child Folders
    string GetFiles( long& count ) const;    // @cmember Returns list of files in Folder

    void Refresh();

    CyrStFolder_ptr GetServant();  // @cmember Returns underlying Servant object
    CECOrbServer*  GetParentServer();
    CECOrbProject* GetParentProject();
    CECOrbFolder*  GetParentFolder();

    // @access Protected Members
protected:
    CECOrbFolder( CECOrbFolder* parent );
    CyrStFolder_var m_servant;  // @cmember Reference to underlying CORBA servant

    // @cmember Returns underlying servant reference as an item for linking.
    CyrStLink_ptr GetLinkServant()
    {
        return GetServant();
    }

    CECOrbProject* m_project;
    CECOrbFolder*  m_folder;
private:
    string     m_cacheName;      // @cmember cached FolderName to save on CORBA calls
}
;

#endif

