/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrWrapLib
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECOrbTestRun_h
#define _CECOrbTestRun_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrWrapLib/CECOrbLink.h>
#include <CyrWrapLib/CECOrbFolder.h>
#include <CyrWrapLib/CECTestOutput.h>

///////////////////////////////////////////////////////////////////////////////
// @class   CECOrbTestRun | This class is a wrapper around the CORBA interface
//          to a Test Run component stored in OpenSTARep.  It provides a basic
//          set of methods for retrieving general information about a specific
//          Test Run.  It also acts as a factory for the creation and retrieval
//          of Test Output.  Test Runs can only be created as children of Test
//          Stores.
// @base    public | CECOrbFolder
class CECOrbTestRun : public CECOrbFolder
{
    // @access Public Members
public:
    // @cmember Constructor for retrieval of Test Run
    CECOrbTestRun( CECOrbFolder* pParent, const char* name );
    // @cmember Constructor for creation of Test Run
    CECOrbTestRun( CECOrbFolder* pParent, const char* desc, const char* node, const char* executor );
    // @cmember Constructor for connection to active Test Run
    CECOrbTestRun( CECOrbFolder* pParent, CyrStTestRun_ptr pTestRun );
    virtual ~CECOrbTestRun();  // @cmember Destructor

    string GetDescription();     // @cmember Returns Test Run description
    string GetTestName();        // @cmember Returns name of Test being run
    CTTime    GetTimeStarted();     // @cmember Returns Test Run start time
    CTTime    GetTimeStopped();     // @cmember Returns Test Run stop time
    string GetNodeName();        // @cmember Returns Test Run node name
    string GetExecutorName();    // @cmember Returns Test Run executor name
    long      GetTestVersion();     // @cmember Returns version of Test being run

    // @cmember Adds a single item of test output to the database
    void AddTestOutput( CECTestOutput* output );
    string GetFirstOutputString( const char* outType );
    string GetNextOutputString( const char* outType );

    // @cmember Start iterating through timer values
    CECTimerOutput* GetFirstTimer();
    // @cmember Get next timer in iteration
    CECTimerOutput* GetNextTimer();
    // @cmember Start iterating through test log values
    CECTestLog* GetFirstTestLog();
    // @cmember Get next test log in iteration
    CECTestLog* GetNextTestLog();

    CyrStTestRun_ptr GetServant();  // @cmember Returns underlying servant object

    void StopTestRun();     // @cmember Stops test run.

    // @access Protected Members
protected:
    bool m_readonly;            // @cmember Determines if Test Run is new or not
}
;

#endif
