/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//    DiagnosticsImpl.h                                                 //
//    =============                                                         //
//                                                                          //
//    This is the header file for the DiagnosticsImpl Class.                //
//                                                                          //
//    This class is derived to provide diagnostic facilities                //
//                                                                          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
//
//    Only load if not already loaded
//
#ifndef DiagnosticsImpl_Loaded
#define DiagnosticsImpl_Loaded

//
//    Incvlude the CORBA header file
//
#include "Diagnostics.h"

//
//    The stub for the diagnostics registry key
//
#define DIAG_REGKEY "SOFTWARE\\CYRANO\\DIAGNOSTICS"
#define DIAG_CLASSKEY   "Classes"
#define DIAG_LEVELKEY   "Level"

//
class DiagnosticsImpl : public _sk_Diagnostics
{

    //
    //    Public Member Functions
    //
public:

    DiagnosticsImpl ();

    virtual ~DiagnosticsImpl();

    long    AddClass        (const char* Executable ,const char* User ,const char* Node ,const char* Class ,long Level);
    // @cmember Add a diagnostic class for an executable

    long    DeleteClass     (const char* Executable ,const char* User ,const char* Node ,const char* Class);
    // @cmember Delete a class from an executable

    long    DeleteAllClasses    (const char* Executable ,const char* User ,const char* Node);
    // @cmember Delete all diagnostic classes for an executable

    long    IsTraced        (const char* Executable ,const char* User ,const char* Node);
    // @cmember Determine if tracing is active for executable

    long    GetClasses      (const char* Executable ,const char* User ,const char* Node
                             ,char* &ClassBuff ,unsigned long RetBufLen
                             ,char*& LevelBuf  ,unsigned long LevelBufLen);
    // @cmember Return the defined diagnostic classes for an executable

protected:
    HKEY    OpenExecutableKey   (const char* Executable ,const char* User ,const char* Node ,char* BaseKeyOpened);
    // @cmember Open the key for the CLASSES of the executable

}
;
#endif
