/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc Factory
///////////////////////////////////////////////////////////////////////////////

#ifndef _ParamList_i_h
#define _ParamList_i_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Factory/Provider.h>

///////////////////////////////////////////////////////////////////////////////
#include <string>
#include <map>
using std::string;
using std::map;
///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <functional>

typedef map<string,string,std::less<string> > ParamValues;

///////////////////////////////////////////////////////////////////////////////
// @class   This class represents the implementation of the ParamList CORBA
//          class.  The ParamList class contains a list of named parameters
//          that can be used by the Factory to construct a Product.
//
//          There is no requirement to register this object with the
//          NameService.  It will be created usually by the client object
//          prior to the Factory::New method being called.
// @base    public | _sk_ParamList
class ParamListImpl : public virtual POA_ParamList,public PortableServer::RefCountServantBase
{
    // @access Public Members
public:
    ParamListImpl()
    {}

    ;        // @cmember Constructor.
    virtual ~ParamListImpl();  // @cmember Destructor.

    char* GetValue( const char* name ); // @cmember Retrieve parameter.
    void AddValue( const char* name, const char* value ); // @cmember Add new parameter.
    // @cmember,mfunc Clears the list of parameters.
    void Clear()
    {
        m_array.clear();
    }

    // @access Protected Members
protected:
    ParamValues  m_array;  // @cmember The list of parameters
}
;

///////////////////////////////////////////////////////////////////////////////
// @class   This class is derived from the base parameter list class.  It
//          simply provides a constructor for initialising the paremeter list
//          with a default set of parameters.
// @base    public | ParamListImpl
class FactoryParamList : public ParamListImpl
{
    // @access Public Members
public:
    // @cmember Constructor.
    FactoryParamList( const char* cls, const char* name, long period );
};

#endif










