/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc Logging
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrLogFile_h
#define _CyrLogFile_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Logging/CyrLog.h>
#include <OSTAORB/OSTAORB.h>
#include <CECInetInfo.h>
#include <ctime.hpp>

///////////////////////////////////////////////////////////////////////////////
#include <map>
#include <list>
#include <fstream>
using std::map;
using std::list;
using std::fstream;
///////////////////////////////////////////////////////////////////////////////
// Typedefs
typedef map<string,long,std::less<string> > TraceLevelMap;
typedef list<string> TraceList;

#define CYR_DAEMON  "CyranoDaemon"
#define CYR_VDK002  "CyranoVDK002"
#define CYR_FACTORY "CyranoFactory"
#define CYR_LOG     "CyranoLog"
#define CYR_EVENTS  "CyranoEvents"
#define CYR_OTA     "CyranoOTA"

///////////////////////////////////////////////////////////////////////////////
// Constants
// @const int | MIN_FLUSH | The minimum number of entries before flushing begins.
// @const int | NUM_FLUSH | The maximum number of entries to flush each time.
const int MIN_FLUSH = 60;
const int NUM_FLUSH = 10;

///////////////////////////////////////////////////////////////////////////////
// @class   CyrLogFile | This class is derived from the basic Cyrano logging
//          class.  Its main task is to save logging information to a file in
//          the same directory as the currently running application.
// @base    public | CyrLogImpl
class CyrLogFile : public virtual POA_CyrLog,public PortableServer::RefCountServantBase
{
    // @access Public Members
public:
    CyrLogFile( CORBAServer* pServer, long level );  // @cmember Constructor
    virtual ~CyrLogFile();          // @cmember Destructor

    string GetFullPath();        // @cmember Gets full path for log object

    // @access IDL generated methods

    // @cmember Adds a single trace message to the list
    void AddTrace( const char* category, long level, const char* msg );
    void SetLevel( const char* category, long level );  // @cmember Sets current trace level
    long GetLevel( const char* category );              // @cmember Gets current trace level
    char* GetFirstTrace();           // @cmember Gets first trace entry
    char* GetNextTrace();            // @cmember Gets next trace entry

    TraceCat* GetTraceCategories();  // @cmember Gets list of trace levels

    // @access Protected Members
protected:
    string m_node;  // @cmember The current node name
    string m_user;  // @cmember The current user name
    string m_exec;  // @cmember The current executor name
    string m_filename;

    long m_defaultLevel;        // @cmember The default level for new categories
    TraceLevelMap m_catLevels;  // @cmember List of all current trace levels

    // @cmember Flushes entries from list.
    void FlushEntries();
    std::fstream* m_file;             // @cmember File to flush entries to

    TraceList::iterator m_iterator;          // @cmember Iteration marker
    TraceList m_traceList;      // @cmember Trace list

    CORBAServer* m_pServer;     // @cmember Pointer to CORBA helper functionality
}
;

#endif
