/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrStRepository
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrStTestRun_i_h
#define _CyrStTestRun_i_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Repository/CyrStFolder_i.h>
#include <CyrffApi/CECFFTestRun.h>

///////////////////////////////////////////////////////////////////////////////
// @class   CyrStTestRunImpl | This class represents the implementation of the
//          CORBA interface CyrStTestRun.  This interface alows creation and
//          storage of a Test Run's properties.
// @devnote Currently this is very simply implemented as a collection of
//          Folders.  If the OpenSTARep component is added then it should be
//          changed to use that interface.
// @devnote There are several parameters that are stringified instances of
//          other variable types, e.g. version should be a long.  However,
//          these parameters arrive as strings and are stored as strings so
//          it would be inefficient to alter them into their correct forms.
// @base    public | POA_CyrStTestRun
class CyrStTestRunImpl : public virtual POA_CyrStTestRun, public CyrStFolderImpl
{
    // @access Public Members
public:
    // @cmember Constructor
    CyrStTestRunImpl( CyrStProvider* parent, ParamList_ptr params );
    virtual ~CyrStTestRunImpl();

    char* GetDescription();    // @cmember Returns the description of this Test run
    char* GetTestName();       // @cmember Returns the name of the Test being executed
    long  GetTimeStarted();    // @cmember Returns the time the Test run was started
    long  GetTimeStopped();    // @cmember Returns the time the test run was stopped
    char* GetNodeName();       // @cmember Returns the name of the node that the test was executed on
    char* GetExecutorName();   // @cmember Returns the name of the executor that is executing the test
    long  GetTestVersion();    // @cmember Returns the verson of the Test being run

    // @cmember Sets the stop time for the Test Run
    void StopTestRun();

    // @cmember Adds an output string of the specified type
    void  AddTestOutput( const char* outType, const char* entry );
    // @cmember Retrieves complete list of output for specified type
    char* GetFirstOutput( const char* outType );
    char* GetNextOutput( const char* outType );
    virtual char*  GetOutput(const char*  outType, CORBA::Long  Iter);
    virtual CORBA::Long  GetNoOutputEntries(const char*  outType);

    // @cmember Retrieves number of purged values for specified type
    virtual CORBA::Long  GetOutputOffset(const char*  outType);
};

#endif



