/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrRandom
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrRandom_h
#define _CyrRandom_h

///////////////////////////////////////////////////////////////////////////////
// @class   This class implements a pseudo-random number generator.  The one
//          used is the one suggested by Wichmann, B. A. & Hill, I. D. (1982)
//          Algorithm AS 183: An efficient and portable pseudo-random number
//          generator Applied Statistics 31 (1982) 188-190
class CyrRandom
{
    // @access Public Members
public:
    CyrRandom();                        // @cmember Constructor.
    CyrRandom( short seed );            // @cmember Constructor.
    CyrRandom( int x, int y, int z );   // @cmember Constructor.

    void Seed( short seed );            // @cmember Set random number seed.
    void Seed( int x, int y, int z );   // @cmember Set random number seed.
    double Random();                    // @cmember Generate number.

    long    RandInt(long min, long max);
    short   RandInt(short min, short max);
    int     RandInt(int min, int max);

    double  Uniform(double min, double max);
    float   Uniform(float min, float max);

    /*  dg 29apr99 replaced Templates functions RandInt and Uniform with full
        implementation versions contained in the cpp file. To overcome problems
        encountered with templates funcs in VC++6
        ///////////////////////////////////////////////////////////////////////////
        // 
        template<class T>
        T RandInt( T min, T max )
        {
            double result = Random() * (max + 1 - min);
            return min + (T)result;
        }
     
     
        ///////////////////////////////////////////////////////////////////////////
        //
        template<class T>
        T Uniform( T min, T max )
        {
            T result = min + ( max - min ) * (T)Random();
            return result;
        }
     
    */
    // @access Private Members
private:
    int _x, _y, _z;  // @cmember Seed values
}
;

#endif

