/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// FontComboEx.cpp : implementation file
//

#include "stdafx.h"
#include "concertlib.h"
#include "FontComboEx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFontComboEx

///////////////////////////////////////////////////////////////////////////
//
// CFontComboEx
//Initial values of the text and highlight stuff
//---------------------------------------------------------------------------

CFontComboEx::CFontComboEx()
{
    m_enStyle = FONTS;
    m_clrHilight = GetSysColor (COLOR_HIGHLIGHT);
    m_clrNormalText = GetSysColor (COLOR_WINDOWTEXT);
    m_clrHilightText = GetSysColor (COLOR_HIGHLIGHTTEXT);
    m_clrBkgnd = GetSysColor (COLOR_WINDOW);
    m_bInitOver = FALSE;
}

//---------------------------------------------------------------------------

CFontComboEx::CFontComboEx (STYLE enStyle)
{
    m_enStyle = enStyle;
    m_clrHilight = GetSysColor (COLOR_HIGHLIGHT);
    m_clrNormalText = GetSysColor (COLOR_WINDOWTEXT);
    m_clrHilightText = GetSysColor (COLOR_HIGHLIGHTTEXT);
    m_clrBkgnd = GetSysColor (COLOR_WINDOW);
    m_bInitOver =FALSE;
}

//---------------------------------------------------------------------------

CFontComboEx::~CFontComboEx()
{}

//---------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CFontComboEx, CComboBox)
//{{AFX_MSG_MAP(CFontComboEx)
ON_WM_CREATE()
ON_WM_DESTROY()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////
//
// CFontComboEx message handlers
//---------------------------------------------------------------------------

void CFontComboEx::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
    //I might want to add something else someday
    switch (m_enStyle)
    {
    case FONTS:
        DrawFont(lpDrawItemStruct);
        break;
    }
}


//---------------------------------------------------------------------------

//I dont need the MeasureItem to do anything. Whatever the system says, it stays
void CFontComboEx::MeasureItem(LPMEASUREITEMSTRUCT lpMeasureItemStruct)
{}

//---------------------------------------------------------------------------


void CFontComboEx::DrawFont(LPDRAWITEMSTRUCT lpDIS)
{
    CDC* pDC = CDC::FromHandle(lpDIS->hDC);
    CRect rect;

    TRACE0 ("In Draw Font\n");

    // draw the colored rectangle portion
    rect.CopyRect(&lpDIS->rcItem);

    pDC->SetBkMode( TRANSPARENT );

    if (lpDIS->itemState & ODS_SELECTED)
    {
        pDC->FillSolidRect (rect,m_clrHilight);
        pDC->SetTextColor (m_clrHilightText);
    }
    else
    {
        pDC->FillSolidRect (rect,m_clrBkgnd);
        pDC->SetTextColor (m_clrNormalText);
    }

    if ((int)(lpDIS->itemID) < 0) // Well its negetive so no need to draw text
    {}

    else
    {
        CString strText;
        GetLBText (lpDIS->itemID,strText);
        CFont newFont;
        CFont *pOldFont;

        ((LOGFONT*)lpDIS->itemData)->lfHeight = 90; //9 point size
        ((LOGFONT*)lpDIS->itemData)->lfWidth = 0;
        newFont.CreatePointFontIndirect ((LOGFONT*)lpDIS->itemData);
        pOldFont = pDC->SelectObject (&newFont);
        pDC->DrawText(strText, rect, DT_LEFT | DT_VCENTER | DT_SINGLELINE);
        pDC->SelectObject (pOldFont);
        newFont.DeleteObject ();
    }
}

//---------------------------------------------------------------------------

void CFontComboEx::InitFonts ()
{
    CDC *pDC = GetDC ();
    ResetContent (); //Delete whatever is there
    EnumFonts (pDC->GetSafeHdc(),NULL,(FONTENUMPROC) EnumFontProc,(LPARAM)this);//Enumerate
    m_bInitOver = TRUE;
}

//---------------------------------------------------------------------------

BOOL CALLBACK CFontComboEx::EnumFontProc (LPLOGFONT lplf, LPTEXTMETRIC lptm, DWORD dwType, LPARAM lpData)
{
    if ( ( dwType |= RASTER_FONTTYPE|TRUETYPE_FONTTYPE ) && (lplf->lfPitchAndFamily & FIXED_PITCH ) ) //Add only TTF fellows, If you want you can change it to check for others
    {
        int index = ((CFontComboEx *) lpData)->AddString(lplf->lfFaceName);
        LPLOGFONT lpLF;
        lpLF = new LOGFONT;
        CopyMemory ((PVOID) lpLF,(CONST VOID *) lplf,sizeof (LOGFONT));
        ((CFontComboEx *) lpData)->SetItemData (index,(DWORD) lpLF);
    }
    return TRUE;
}

//---------------------------------------------------------------------------

int CFontComboEx::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CComboBox::OnCreate(lpCreateStruct) == -1)
        return -1;

    // TODO: Add your specialized creation code here
    if (m_enStyle == FONTS)
    {
        InitFonts ();
    }
    return 0;
}

//---------------------------------------------------------------------------

void CFontComboEx::OnDestroy()
{

    if (m_enStyle == FONTS)
    {
        int nCount;
        nCount = GetCount ();
        for (int i = 0; i <  nCount; i++)
        {
            delete ((LOGFONT*)GetItemData (i)); //delete the LOGFONTS actually created..
        }

    }
    // TODO: Add your message handler code here
    CComboBox::OnDestroy();
}

//---------------------------------------------------------------------------

void CFontComboEx::FillFonts ()
{
    m_enStyle = FONTS;
    InitFonts ();
}

//---------------------------------------------------------------------------

int  CFontComboEx::GetSelFont (LOGFONT& lf)
{
    int index = GetCurSel ();
    if (index == LB_ERR)
        return LB_ERR;
    LPLOGFONT lpLF = (LPLOGFONT) GetItemData (index);
    CopyMemory ((PVOID)&lf, (CONST VOID *) lpLF, sizeof (LOGFONT));
    return index; //return the index here.. Maybe the user needs it:-)
}

//---------------------------------------------------------------------------


void CFontComboEx::SetSelFont (LOGFONT *lf)
{
    int nCount = GetCount();
    for( int nIndex = 0 ; nIndex < nCount ; nIndex++ )
    {
        CString strFont ;
        GetLBText( nIndex , strFont ) ;
        if( stricmp( lf->lfFaceName , strFont ) == 0 )
        {
            SetCurSel( nIndex ) ;
            break ;
        }
    }
}

//---------------------------------------------------------------------------

void CFontComboEx::PreSubclassWindow()
{
    // TODO: Add your specialized code here and/or call the base class
    //Tried to do what Roger Onslow did for the button.. Did not work..?? Any R&D guys around :-)
}

