/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// AbstractTree.cpp: implementation of the CAbstractTree class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AbstractTree.h"

#include "CyrRes.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------

CAbstractTree::CAbstractTree(BOOL bDestroyInfoForDisplay /*= TRUE*/ ):
        m_pInsertInfo(NULL) ,
        m_ContextMenu( IDM_FULL_CHOICE, 0 ),
        m_wStatus(0) ,
        m_wRecurCount( 0 ),
        m_bDestroyInfoForDisplay(bDestroyInfoForDisplay)
{
}

//--------------------------------------------------------------------


CAbstractTree::~CAbstractTree()
{
    if( m_pInsertInfo )
        delete m_pInsertInfo ;
}


///////////////////////////////////////////////////////////////////////
/////           INSERT ITEMS            ///////////////////////////////
///////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------
//create new item
BOOL CAbstractTree::NewInsertInfo()
{
    if ( m_pInsertInfo != NULL )
    {
        delete m_pInsertInfo ;
        m_pInsertInfo = NULL ;
    }

    try
    {
        m_pInsertInfo = new TInsertInfo() ;
    }
    catch (...)
    {
        m_pInsertInfo = NULL ;
    }

    return ( m_pInsertInfo != NULL ) ;
}

//--------------------------------------------------------------------
//store items which already have their parents
BOOL CAbstractTree::StoreItem( DWORD dwID , HTREEITEM hItem )
{
    ASSERT( m_pInsertInfo != NULL ) ;

    try
    {
        m_pInsertInfo->m_MapIDToItem.SetAt( dwID , (LPVOID)hItem ) ;
    }
    catch (...)
    {
        return FALSE ;
    }

    return TRUE ;
}

//--------------------------------------------------------------------

//store items which don't have already their parents
BOOL CAbstractTree::StoreOrphan( TItemInfo & info )
{
    ASSERT( m_pInsertInfo != NULL ) ;

    try
    {
        m_pInsertInfo->m_Orphans.Add( info ) ;
    }
    catch (...)
    {
        return FALSE ;
    }

    return TRUE ;
}

//--------------------------------------------------------------------

//Insert Item before hBeforeItem in the tree

BOOL CAbstractTree::InsertItemBefore( HTREEITEM & hItem ,
                                      HTREEITEM   hBeforeItem ,
                                      LPCTSTR     lpszItem ,
                                      int         nImage ,
                                      int         nSelectedImage ,
                                      DWORD       dwID ,
                                      DWORD       dwParentID /*=0L*/ )
{
    HTREEITEM hAfter = NULL ;
    CTreeCtrl & tree = GetTreeCtrl() ;
    //----

    if( hBeforeItem )
        hAfter = tree.GetNextItem( hBeforeItem , TVGN_PREVIOUS ) ;

    if( hAfter == NULL )
        hAfter = (HTREEITEM) TVI_FIRST ;

    return InsertItem( hItem ,
                       lpszItem ,
                       nImage ,
                       nSelectedImage ,
                       dwID ,
                       dwParentID ,
                       hAfter ) ;
}

//--------------------------------------------------------------------

//Insert Item before hAfter in the tree

BOOL CAbstractTree::InsertItemAfter( HTREEITEM & hItem ,
                                     HTREEITEM   hAfter ,
                                     LPCTSTR     lpszItem ,
                                     int         nImage ,
                                     int         nSelectedImage ,
                                     DWORD       dwID ,
                                     DWORD       dwParentID /*=0L*/ )
{
    return InsertItem( hItem ,
                       lpszItem ,
                       nImage ,
                       nSelectedImage ,
                       dwID ,
                       dwParentID ,
                       hAfter ) ;
}

//--------------------------------------------------------------------

//Insert Item in the tree

BOOL CAbstractTree::InsertItem( HTREEITEM & hItem ,
                                LPCTSTR     lpszItem ,
                                int         nImage ,
                                int         nSelectedImage ,
                                DWORD       dwID ,
                                DWORD       dwParentID /*=0L*/ ,
                                HTREEITEM   hAfter /*= TVI_LAST*/ )
{
    BOOL bRet = TRUE ;
    // ----
    hItem = (HTREEITEM)NULL ;

    if ( m_pInsertInfo != NULL || NewInsertInfo() != FALSE )
    {
        CTreeCtrl & tree = GetTreeCtrl() ;
        // ----
        if ( dwParentID != 0 ) // Parent is specified
        {
            LPVOID hParentItem = NULL ;
            // ----
            //look for the parent. If the parent exists, insert item in the tree
            //and insert item in m_pInsertInfo->m_MapIDToItem
            if ( m_pInsertInfo->m_MapIDToItem.Lookup( dwParentID, hParentItem ) != FALSE )
            {
                if ( hParentItem != NULL )
                {
                    hItem = tree.InsertItem( lpszItem, nImage, nSelectedImage, (HTREEITEM)hParentItem , hAfter ) ;
                    tree.SetItemData(hItem, dwID);
                    bRet  = StoreItem( dwID , hItem ) ;
                    // Look for inserting item from m_pInsertInfo->m_Orphans in the tree
                    LayOut( FALSE ) ;
                }
                else
                {
                    ASSERT(FALSE) ;
                }
            }
            //If the parent does not exist,
            //insert item in m_pInsertInfo->m_Orphans
            else
            {
                TItemInfo itemInfo ;
                // ----
                itemInfo.m_dwID           = dwID ;
                itemInfo.m_dwParentID     = dwParentID ;
                itemInfo.m_strText        = lpszItem ;
                itemInfo.m_nImage         = nImage ;
                itemInfo.m_nSelectedImage = nSelectedImage ;

                bRet = StoreOrphan( itemInfo ) ;
            }
        }
        else
        {
            hItem = tree.InsertItem( lpszItem, nImage, nSelectedImage, TVI_ROOT , hAfter ) ;
            tree.SetItemData( hItem, dwID );

            bRet  = StoreItem( dwID , hItem ) ;
            LayOut( FALSE ) ;
        }
    }

    return bRet ;
}

//--------------------------------------------------------------------

// Insert orphan items whose parents are in 'm_pInsertInfo->m_MapIDToItem'
// and delete those items from 'm_pInsertInfo->m_Orphans'
// if bFinalize = TRUE it means every items have been inserted
// if not ASSERT appear.

BOOL CAbstractTree::LayOut( BOOL bFinalize /*=TRUE*/ )
{
    if ( m_pInsertInfo != NULL )
    {
        BOOL bLoop = TRUE ;
        // ----
        CTreeCtrl &tree = GetTreeCtrl() ;
        while ( bLoop )
        {
            //look for items not inserted
            int nCount = m_pInsertInfo->m_Orphans.GetSize() ;
            // ----
            if ( nCount == 0 )
            {
                break ; // Nothing to insert
            }

            //look if parent has been inserted and insert its children

            for ( int i = 0 ; i < nCount ; /**/ )
            {
                TItemInfo & itemInfo    = m_pInsertInfo->m_Orphans.ElementAt(i) ;
                LPVOID      hParentItem = NULL ;
                // ----
                bLoop = FALSE ;

                if ( m_pInsertInfo->m_MapIDToItem.Lookup( itemInfo.m_dwParentID, hParentItem ) != FALSE )
                {
                    HTREEITEM hItem = tree.InsertItem( itemInfo.m_strText ,
                                                       itemInfo.m_nImage ,
                                                       itemInfo.m_nSelectedImage ,
                                                       (HTREEITEM)hParentItem ,
                                                       TVI_LAST ) ;
                    tree.SetItemData( hItem, itemInfo.m_dwID );
                    // ----
                    if ( StoreItem( itemInfo.m_dwID , hItem ) == FALSE )
                    {
                        return FALSE ; // Too bad !
                    }
                    // remove the children inserted

                    m_pInsertInfo->m_Orphans.RemoveAt(i) ;
                    nCount-- ;
                    bLoop = ( nCount > 0 ) ;
                }
                else
                {
                    i++ ;
                }
            }
        }

        if ( bFinalize != FALSE )
        {
            ASSERT(m_pInsertInfo->m_Orphans.GetSize()==0) ;
            if( m_bDestroyInfoForDisplay )
            {
                delete m_pInsertInfo ;
                m_pInsertInfo = NULL ;
            }
        }
    }
    return TRUE ;
}

///////////////////////////////////////////////////////////////////////
/////           COLLAPSE EXPAND         ///////////////////////////////
///////////////////////////////////////////////////////////////////////
// --------------------------------------------------------------------------

//return TRUE if can expand, else FALSE
BOOL CAbstractTree::CanExpandAll() const
{
    // 'GetRootItem()' is const, so 'const_cast' is OK.
    CTreeCtrl &TreeCtrl = const_cast<CAbstractTree *>(this)->GetTreeCtrl() ;
    HTREEITEM hRootItem = TreeCtrl.GetRootItem() ;
    BOOL bRet = FALSE ;
    // ----
    if ( hRootItem != (HTREEITEM)NULL )
    {
        bRet = bRet || CanExpandBranch( hRootItem ) ;
        while( hRootItem != NULL && bRet == FALSE )
        {
            hRootItem = TreeCtrl.GetNextItem( hRootItem , TVGN_NEXT ) ;
            bRet = bRet || CanExpandBranch( hRootItem ) ;
        }
    }

    return bRet ;
}

// --------------------------------------------------------------------------

//return TRUE if can collapse, else FALSE
BOOL CAbstractTree::CanCollapseAll() const
{
    const CTreeCtrl & tree      = const_cast<CAbstractTree *>(this)->GetTreeCtrl() ;
    HTREEITEM         hRootItem = tree.GetRootItem() ;
    BOOL bRet = FALSE ;
    UINT nItemState = 0 ;
    // ----
    while ( hRootItem != (HTREEITEM)NULL && !bRet )
    {
        nItemState = tree.GetItemState( hRootItem  , TVIS_EXPANDED ) ;
        bRet = bRet || ( ( nItemState & TVIS_EXPANDED ) == TVIS_EXPANDED ) ;
        hRootItem = tree.GetNextItem( hRootItem , TVGN_NEXT ) ;
    }

    return bRet ;
}

// --------------------------------------------------------------------------

//return TRUE if can expand all items , else FALSE
BOOL CAbstractTree::ExpandAll()
{
    BOOL     bRet;
    CTreeCtrl & tree      = GetTreeCtrl() ;
    HTREEITEM   hRootItem = tree.GetRootItem() ;
    HTREEITEM   hCurrItem = hRootItem ;
    // ----
    if ( hRootItem != (HTREEITEM)NULL )
    {
        tree.SetRedraw( FALSE ) ;
        while ( hCurrItem != NULL )
        {
            bRet = ExpandBranch( hCurrItem ) ;
            hCurrItem = tree.GetNextSiblingItem( hCurrItem );
        }
        tree.SetRedraw( TRUE ) ;
        tree.Invalidate() ;

        return TRUE ;
    }

    return FALSE ;
}

// --------------------------------------------------------------------------

//return TRUE if can collapse all , else FALSE
BOOL CAbstractTree::CollapseAll()
{
    BOOL     bRet;
    CTreeCtrl & tree      = GetTreeCtrl() ;
    HTREEITEM   hRootItem = tree.GetRootItem() ;
    HTREEITEM   hCurrItem = hRootItem ;
    // ----
    if ( hRootItem != (HTREEITEM)NULL )
    {
        tree.SetRedraw( FALSE ) ;
        while ( hCurrItem != NULL )
        {
            bRet = CollapseBranch( hCurrItem ) ;
            hCurrItem = tree.GetNextSiblingItem( hCurrItem );
        }
        tree.SetRedraw( TRUE ) ;
        tree.Invalidate() ;

        return TRUE ;
    }

    return FALSE ;
}

// --------------------------------------------------------------------------

//return TRUE if can expand one branch, else FALSE
BOOL CAbstractTree::CanExpandBranch( HTREEITEM _hItem /*=(HTREEITEM)NULL*/ ) const
{
    // 'GetRootItem()' is const, so 'const_cast' is OK.
    const CTreeCtrl & tree     = const_cast<CAbstractTree *>(this)->GetTreeCtrl() ;
    HTREEITEM         hSelItem = ( _hItem == (HTREEITEM)NULL ? tree.GetSelectedItem() : _hItem ) ;
    // ----
    if ( hSelItem != (HTREEITEM)NULL )
    {
        if ( tree.ItemHasChildren( hSelItem ) )
        {
            UINT nItemState = tree.GetItemState( hSelItem  , TVIS_EXPANDED ) ;
            // ----
            return ( ( nItemState & TVIS_EXPANDED ) != TVIS_EXPANDED ) ;
        }
    }
    return FALSE ;
}

// --------------------------------------------------------------------------

//return TRUE if can collapse one branch, else FALSE
BOOL CAbstractTree::CanCollapseBranch( HTREEITEM _hItem /*=(HTREEITEM)NULL*/ ) const
{
    const CTreeCtrl & tree     = const_cast<CAbstractTree *>(this)->GetTreeCtrl() ;
    HTREEITEM         hSelItem = ( _hItem == (HTREEITEM)NULL ? tree.GetSelectedItem() : _hItem ) ;
    // ----
    if ( hSelItem != (HTREEITEM)NULL )
    {
        UINT nItemState = tree.GetItemState( hSelItem  , TVIS_EXPANDED ) ;
        // ----
        return ( ( nItemState & TVIS_EXPANDED ) == TVIS_EXPANDED ) ;
    }
    return FALSE ;
}

// --------------------------------------------------------------------------

//return TRUE if can expand one branch, else FALSE
BOOL CAbstractTree::ExpandBranch( HTREEITEM _hItem /*=(HTREEITEM)NULL*/ )
{
    BOOL bRet = FALSE ;
    // ----
    m_wRecurCount++ ;

    if ( m_wRecurCount <= 10 )
    {
        CTreeCtrl & tree  = GetTreeCtrl() ;
        HTREEITEM   hItem = ( _hItem == (HTREEITEM)NULL ? tree.GetSelectedItem() : _hItem ) ;
        // ----
        if ( hItem != (HTREEITEM)NULL )
        {
            if ( tree.ItemHasChildren( hItem ) )
            {
                tree.Expand( hItem , TVE_EXPAND ) ;

                hItem = tree.GetChildItem( hItem );

                if ( m_wRecurCount == 0 )
                {
                    tree.SetRedraw(FALSE) ;

                    do
                    {
                        ExpandBranch( hItem ) ;
                    }
                    while( ( hItem = tree.GetNextSiblingItem( hItem )) != NULL ) ;

                    tree.SetRedraw(TRUE) ;
                }
                else
                {
                    do
                    {
                        ExpandBranch( hItem ) ;
                    }
                    while( ( hItem = tree.GetNextSiblingItem( hItem )) != NULL ) ;
                }
            }

            tree.EnsureVisible( tree.GetSelectedItem() );
            bRet = TRUE ;
        }
    }

    m_wRecurCount-- ;
    return bRet ;
}

// --------------------------------------------------------------------------

//return TRUE if can collapse one branch, else FALSE
BOOL CAbstractTree::CollapseBranch( HTREEITEM _hItem /*=(HTREEITEM)NULL*/ )
{
    CTreeCtrl & tree     = GetTreeCtrl() ;
    HTREEITEM   hSelItem = ( _hItem == (HTREEITEM)NULL ? tree.GetSelectedItem() : _hItem ) ;
    // ----
    if ( hSelItem != (HTREEITEM)NULL )
    {
        return tree.Expand( hSelItem , TVE_COLLAPSE );
    }

    return FALSE ;
}


///////////////////////////////////////////////////////////////////////
/////           MENUS                   ///////////////////////////////
///////////////////////////////////////////////////////////////////////
// --------------------------------------------------------------------------

BOOL CAbstractTree::GetStatus( Status status ) const
{
    return ( ( m_wStatus & (WORD)status ) != 0 ) ;
}

// --------------------------------------------------------------------------

void CAbstractTree::SetStatus( Status status , BOOL bSet )
{
    if ( ! bSet )
        m_wStatus &= ~(WORD)status ;
    else
        m_wStatus |= (WORD)status ;
}

// --------------------------------------------------------------------------

//status of the popup menu
BOOL CAbstractTree::GetPostPopupMenu() const
{
    return GetStatus( status_post_popup_menu ) ;
}

// --------------------------------------------------------------------------

void CAbstractTree::SetPostPopupMenu( BOOL bActivatePostMode )
{
    SetStatus( status_post_popup_menu , bActivatePostMode ) ;
}

// --------------------------------------------------------------------------

//implement default popupMenu
BOOL CAbstractTree::DoPopupMenu( CPoint clientPoint )
{
    CWnd *pWnd ;
    //--
    //Get the Wnd associated with the menu
    GetClientWindow( &pWnd ) ;
    return SUCCEEDED( CyrLibDoDefaultPopupMenu( this ,
                      &clientPoint ,
                      CDynMenu32DefaultImpl::mi_expand_group ) ) ;
}

// --------------------------------------------------------------------------

//enable popup menu
BOOL CAbstractTree::CanDoPopupMenu() const
{
    return TRUE ;
}

// --------------------------------------------------------------------------

//define our own menu
HRESULT CAbstractTree::OnPreMenuPopup( /*[in]*/ CMenu * pMenu )
{
    if ( pMenu != NULL )
    {
        ASSERT( ::IsMenu( pMenu->m_hMenu ) ) ;
        DoOwnerContextMenu( pMenu ) ;
        return S_OK ;
    }

    return E_INVALIDARG ;
}

