/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"
#include "BmpUtil.h"

//--------------------------------------------------------------------

size_t  HexToBinLength( LPCTSTR pczHex ) ;
HRESULT HexToBin( LPCTSTR pczHex , LPVOID pBin , size_t nMaxBinLength ) ;
HRESULT BinToHex( LPVOID pBin , LPTSTR pczHex , size_t nBinLength ) ;

//--------------------------------------------------------------------

static inline BYTE _ShortToHexChar( short n )
{
    BYTE byHex ;
    // ----
    switch ( n )
    {
    case 0 :
        byHex = (BYTE)'0' ;
        break ;
    case 1 :
        byHex = (BYTE)'1' ;
        break ;
    case 2 :
        byHex = (BYTE)'2' ;
        break ;
    case 3 :
        byHex = (BYTE)'3' ;
        break ;
    case 4 :
        byHex = (BYTE)'4' ;
        break ;
    case 5 :
        byHex = (BYTE)'5' ;
        break ;
    case 6 :
        byHex = (BYTE)'6' ;
        break ;
    case 7 :
        byHex = (BYTE)'7' ;
        break ;
    case 8 :
        byHex = (BYTE)'8' ;
        break ;
    case 9 :
        byHex = (BYTE)'9' ;
        break ;
    case 10 :
        byHex = (BYTE)'A' ;
        break ;
    case 11 :
        byHex = (BYTE)'B' ;
        break ;
    case 12 :
        byHex = (BYTE)'C' ;
        break ;
    case 13 :
        byHex = (BYTE)'D' ;
        break ;
    case 14 :
        byHex = (BYTE)'E' ;
        break ;
    case 15 :
        byHex = (BYTE)'F' ;
        break ;
    default:
        byHex = (BYTE)'Z' ;
        break ;
    }

    return byHex ;
}

//--------------------------------------------------------------------

static inline short _HexCharToShort( BYTE c )
{
    int n ;
    // ----
    switch ( c )
    {
    case (BYTE)'0' :
                    n = 0  ;
        break ;
    case (BYTE)'1' :
                    n = 1  ;
        break ;
    case (BYTE)'2' :
                    n = 2  ;
        break ;
    case (BYTE)'3' :
                    n = 3  ;
        break ;
    case (BYTE)'4' :
                    n = 4  ;
        break ;
    case (BYTE)'5' :
                    n = 5  ;
        break ;
    case (BYTE)'6' :
                    n = 6  ;
        break ;
    case (BYTE)'7' :
                    n = 7  ;
        break ;
    case (BYTE)'8' :
                    n = 8  ;
        break ;
    case (BYTE)'9' :
                    n = 9  ;
        break ;
    case (BYTE)'A' :
                case (BYTE)'a' :
                        n = 10 ;
        break ;
    case (BYTE)'B' :
                case (BYTE)'b' :
                        n = 11 ;
        break ;
    case (BYTE)'C' :
                case (BYTE)'c' :
                        n = 12 ;
        break ;
    case (BYTE)'D' :
                case (BYTE)'d' :
                        n = 13 ;
        break ;
    case (BYTE)'E' :
                case (BYTE)'e' :
                        n = 14 ;
        break ;
    case (BYTE)'F' :
                case (BYTE)'f' :
                        n = 15 ;
        break ;
    default:
        n = -1 ;
        break ;
    }

    return n ;
}

//--------------------------------------------------------------------

static inline BYTE _HexCharToEscape( BYTE c , BYTE bLowcase = 0 )
{
    BYTE e ;
    // ----
    switch ( c )
    {
    case (BYTE)'0' :
                    e = ( bLowcase ? (BYTE)'g' : (BYTE)'G' ) ;
        break ;
    case (BYTE)'1' :
            e = ( bLowcase ? (BYTE)'h' : (BYTE)'H' ) ;
        break ;
    case (BYTE)'2' :
                    e = ( bLowcase ? (BYTE)'i' : (BYTE)'I' ) ;
        break ;
    case (BYTE)'3' :
            e = ( bLowcase ? (BYTE)'j' : (BYTE)'J' ) ;
        break ;
    case (BYTE)'4' :
                    e = ( bLowcase ? (BYTE)'k' : (BYTE)'K' ) ;
        break ;
    case (BYTE)'5' :
            e = ( bLowcase ? (BYTE)'l' : (BYTE)'L' ) ;
        break ;
    case (BYTE)'6' :
                    e = ( bLowcase ? (BYTE)'m' : (BYTE)'M' ) ;
        break ;
    case (BYTE)'7' :
            e = ( bLowcase ? (BYTE)'n' : (BYTE)'N' ) ;
        break ;
    case (BYTE)'8' :
                    e = ( bLowcase ? (BYTE)'o' : (BYTE)'O' ) ;
        break ;
    case (BYTE)'9' :
            e = ( bLowcase ? (BYTE)'p' : (BYTE)'P' ) ;
        break ;
    case (BYTE)'A' :
                case (BYTE)'a' :
                        e = ( bLowcase ? (BYTE)'q' : (BYTE)'Q' ) ;
        break ;
    case (BYTE)'B' :
        case (BYTE)'b' :
                        e = ( bLowcase ? (BYTE)'r' : (BYTE)'R' ) ;
        break ;
    case (BYTE)'C' :
        case (BYTE)'c' :
                        e = ( bLowcase ? (BYTE)'s' : (BYTE)'S' ) ;
        break ;
    case (BYTE)'D' :
        case (BYTE)'d' :
                        e = ( bLowcase ? (BYTE)'t' : (BYTE)'T' ) ;
        break ;
    case (BYTE)'E' :
        case (BYTE)'e' :
                        e = ( bLowcase ? (BYTE)'u' : (BYTE)'U' ) ;
        break ;
    case (BYTE)'F' :
        case (BYTE)'f' :
                        e = ( bLowcase ? (BYTE)'v' : (BYTE)'V' ) ;
        break ;
    default:
            e = (BYTE)'Z' ;
        break ;
    }
    return e ;
}

//--------------------------------------------------------------------

static inline BYTE _EscapeToHexChar( BYTE c )
{
    BYTE e ;
    // ----
    switch ( c )
    {
    case (BYTE)'G' :
                case (BYTE)'g' :
                        e = (BYTE)'0' ;
        break ;
    case (BYTE)'H' :
                case (BYTE)'h' :
                        e = (BYTE)'1' ;
        break ;
    case (BYTE)'I' :
                case (BYTE)'i' :
                        e = (BYTE)'2' ;
        break ;
    case (BYTE)'J' :
                case (BYTE)'j' :
                        e = (BYTE)'3' ;
        break ;
    case (BYTE)'K' :
                case (BYTE)'k' :
                        e = (BYTE)'4' ;
        break ;
    case (BYTE)'L' :
                case (BYTE)'l' :
                        e = (BYTE)'5' ;
        break ;
    case (BYTE)'M' :
                case (BYTE)'m' :
                        e = (BYTE)'6' ;
        break ;
    case (BYTE)'N' :
                case (BYTE)'n' :
                        e = (BYTE)'7' ;
        break ;
    case (BYTE)'O' :
                case (BYTE)'o' :
                        e = (BYTE)'8' ;
        break ;
    case (BYTE)'P' :
                case (BYTE)'p' :
                        e = (BYTE)'9' ;
        break ;
    case (BYTE)'Q' :
                case (BYTE)'q' :
                        e = (BYTE)'A' ;
        break ;
    case (BYTE)'R' :
                case (BYTE)'r' :
                        e = (BYTE)'B' ;
        break ;
    case (BYTE)'S' :
                case (BYTE)'s' :
                        e = (BYTE)'C' ;
        break ;
    case (BYTE)'T' :
                case (BYTE)'t' :
                        e = (BYTE)'D' ;
        break ;
    case (BYTE)'U' :
                case (BYTE)'u' :
                        e = (BYTE)'E' ;
        break ;
    case (BYTE)'V' :
                case (BYTE)'v' :
                        e = (BYTE)'F' ;
        break ;
    default:
        e = (BYTE)'Z' ;
        break ;
    }
    return e ;
}

//--------------------------------------------------------------------

static inline DWORD _EncodeRep( BYTE c , DWORD dwRepCount , LPSTR pzDest )
{
    ASSERT( _EscapeToHexChar( c ) != (BYTE)'Z' ) ;
    ASSERT( dwRepCount > 1 ) ;
    ASSERT( pzDest != NULL ) ;

    BYTE  cc ;
    BYTE  bLeftDigit = 0 ; // Optim
    DWORD nLen = 1 ;
    // ----
    *pzDest = c ;

    if ( ( cc = HIBYTE( HIWORD( dwRepCount ) ) ) != 0 )
    {
        if ( ( cc & 0xF0 ) != 0 )
        {
            *( pzDest + nLen++ ) = _HexCharToEscape( _ShortToHexChar( ( cc & 0xF0 ) >> 4 ) , 1 ) ;
        }
        *( pzDest + nLen++ ) = _HexCharToEscape( _ShortToHexChar( cc & 0x0F ) , 1 ) ;
        bLeftDigit = 1 ;
    }

    if ( ( cc = LOBYTE( HIWORD( dwRepCount ) ) ) != 0 )
    {
        if ( bLeftDigit || ( cc & 0xF0 ) != 0 )
        {
            *( pzDest + nLen++ ) = _HexCharToEscape( _ShortToHexChar( ( cc & 0xF0 ) >> 4 ) , 1 ) ;
        }
        *( pzDest + nLen++ ) = _HexCharToEscape( _ShortToHexChar( cc & 0x0F ) , 1 ) ;
        bLeftDigit = 1 ;
    }

    if ( ( cc = HIBYTE( LOWORD( dwRepCount ) ) ) != 0 )
    {
        if ( bLeftDigit || ( cc & 0xF0 ) != 0 )
        {
            *( pzDest + nLen++ ) = _HexCharToEscape( _ShortToHexChar( ( cc & 0xF0 ) >> 4 ) , 1 ) ;
        }
        *( pzDest + nLen++ ) = _HexCharToEscape( _ShortToHexChar( cc & 0x0F ) , 1 ) ;
        bLeftDigit = 1 ;
    }

    if ( ( cc = LOBYTE( LOWORD( dwRepCount ) ) ) != 0 )
    {
        if ( bLeftDigit || ( cc & 0xF0 ) != 0 )
        {
            *( pzDest + nLen++ ) = _HexCharToEscape( _ShortToHexChar( ( cc & 0xF0 ) >> 4 ) , 1 ) ;
        }
        *( pzDest + nLen++ ) = _HexCharToEscape( _ShortToHexChar( cc & 0x0F ) , 1 ) ;
    }

    return nLen ;
}

//--------------------------------------------------------------------

static inline DWORD _DecodeRepLen( LPCSTR pzRepCountEscape , DWORD * pdwEscapeLenIncrement )
{
    DWORD dwCount = 0 ;
    // ----
    if ( pzRepCountEscape != NULL && *pzRepCountEscape != 0 )
    {
        const BYTE  cMax = sizeof( "FF00FF00" ) - 1 ;
        BYTE        cc , i ;
        // ----
        ASSERT( _EscapeToHexChar( *pzRepCountEscape ) != (BYTE)'Z' ) ;

        // Seek end of escape hex number :
        // Nb Rep counts are coded in LOWERCASE :
        for ( i = 0 ; i < cMax && _EscapeToHexChar( ( cc = *( pzRepCountEscape + i ) ) ) != (BYTE)'Z' && islower( static_cast<int>(cc) ) ; i++ )
        {
            ;
        }

        if ( pdwEscapeLenIncrement != NULL )
        {
            *pdwEscapeLenIncrement += i ;
        }

        ASSERT( static_cast<signed char>(i) > 0 ) ;
        i -= ( static_cast<signed char>(i) > 0 ? 1 : 0 ) ; // Safe !

        // Reverse-read it :
        for ( BYTE j = 0 ; static_cast<signed char>(i) >= 0 ; i-- , j++ )
        {
            ASSERT( j <= cMax ) ;

            cc = _EscapeToHexChar( *( pzRepCountEscape + i ) ) ;

#ifdef _DEBUG
            short _nNibble = _HexCharToShort( cc ) ;
#endif

            dwCount += ( (DWORD)_HexCharToShort( cc ) << ( 4 * j ) ) ;
        }
    }
    else
    {
        ASSERT(FALSE) ;
    }

    return dwCount ;
}

//--------------------------------------------------------------------

static HRESULT Pack( LPCSTR pczHex , LPSTR pz_P_Hex )
{
    HRESULT hRes = S_OK ;
    // ----
    if ( pczHex != NULL )
    {
        LPCSTR pczSource = pczHex ;
        LPSTR  pzDest    = pz_P_Hex ;

#ifdef _DEBUG
        if ( pczHex == pz_P_Hex )
        {
            int nBreakPointHere = 0 ;
        } // One might be interested in knowing
#endif

        DWORD dwRepCount = 0 ; // N.B : "repeat" means SECOND or more occurence of the same.
        BYTE  cLastChar  = 0 ;
        BYTE  cc ;
        // ----
        for ( ; /* ( cc = *pczSource ) != 0 && */ _HexCharToShort( cc = *pczSource  ) >= 0 || ( cc == 0 ) ; pczSource++ )
        {
            if ( cLastChar != cc ) // Break
            {
                if ( dwRepCount > 1 ) // Save at least one position
                {
                    DWORD dwEncodeLen = _EncodeRep( _HexCharToEscape( cLastChar ) , dwRepCount + 1 , pzDest - ( dwRepCount + 1 ) ) ;
                    DWORD dwOffset    = dwEncodeLen - ( dwRepCount + 1 ) ; // (see "repeat" comment)
                    // ----
                    pzDest += dwOffset ;
                }
                else
                {
                    ASSERT(TRUE) ;
                } // Might appreciate a breakpoint.

                dwRepCount = 0 ;
            }
            else
            {
                dwRepCount++ ;
            }

            *pzDest++ = cLastChar = cc ;

            if ( cc == 0 )
            {
                break ;
            } // That'it it folks !
        }


        *pzDest++ = 0 ;
        *pzDest++ = 0 ;
    }
    else
    {
        hRes = E_INVALIDARG ;
    }

    return hRes ;
}

//--------------------------------------------------------------------
// Length of un-packed hex string.
// Does NOT include terminating "00".

static DWORD UnPackLen( LPCSTR pcz_P_Hex )
{
    DWORD dwCount = 0 ;
    // ----
    if ( pcz_P_Hex != NULL )
    {
        BYTE cc ;
        // ----
        for ( DWORD dw = 0 ; ( cc = *( pcz_P_Hex + dw ) ) != 0 ; dw++ )
        {
            if ( _HexCharToShort( cc ) < 0 )
            {
                // Nb Rep counts are coded in LOWERCASE :
                if ( _EscapeToHexChar( cc ) != (BYTE)'Z' )
                {
                    DWORD dwEscapeLen = 0 ; // Length of escape characters sequence ("ghij...")
                    // ----
                    ASSERT( _EscapeToHexChar( *( pcz_P_Hex + dw + 1 ) ) != (BYTE)'Z' ) ;
                    dwCount += _DecodeRepLen( pcz_P_Hex + dw + 1 , &dwEscapeLen ) ;
                    ASSERT( dwEscapeLen > 0 ) ;
                    dw += dwEscapeLen ;
                }
                else
                {
                    break ;
                }
            }
            else
            {
                dwCount++ ;
            }
        }
    }

    ASSERT( dwCount % 2 == 0 ) ;

    return dwCount ;
}

//--------------------------------------------------------------------
// N.B : '*ppzHex' is re-allocated to fit the un-packed hex string
// (make sure it was previously allocated via 'malloc' or is NULL).

static HRESULT UnPack( LPCSTR pcz_P_Hex , LPSTR * _ppzHex )
{
    HRESULT hRes = S_OK ;
    // ----
    if ( pcz_P_Hex != NULL && _ppzHex != NULL )
    {
        DWORD dwUnPack = UnPackLen( pcz_P_Hex ) ;
#ifdef _DEBUG
        DWORD dw_check_bounds = 0 ;
#endif
        // ----
        if ( dwUnPack > 0 )
        {
            LPSTR pzHex = (LPSTR)malloc( dwUnPack + sizeof( "00" ) ) ;
            // ----
            if ( pzHex != NULL )
            {
                LPSTR pzDest = pzHex ;
                BYTE  cc ;
                // ----
                for ( DWORD dw = 0 ; ( cc = *( pcz_P_Hex + dw ) ) != 0 ; dw++ )
                {
                    if ( _HexCharToShort( cc ) < 0 )
                    {
                        if ( _EscapeToHexChar( cc ) != (BYTE)'Z' )
                        {
                            DWORD dwEscapeLen = 0 ; // Length of escape characters sequence ("ghij...")
                            DWORD dwCount     = _DecodeRepLen( pcz_P_Hex + dw + 1 , &dwEscapeLen ) ;
                            BYTE  cHexDigit   =  _EscapeToHexChar( cc ) ;
                            //----
                            ASSERT( dwEscapeLen > 0 ) ;

                            for ( DWORD x = 0 ; x < dwCount ; x++ )
                            {
                                *pzDest++ = cHexDigit ;
                                ASSERT( dw_check_bounds++ < dwUnPack ) ;
                            }

                            dw += dwEscapeLen ;
                        }
                        else
                        {
                            break ;
                        }
                    }
                    else
                    {
                        *pzDest++ = cc ;
                        ASSERT( dw_check_bounds++ < dwUnPack ) ;
                    }
                }

                *pzDest++ = 0 ;
                *pzDest++ = 0 ;

                free( *_ppzHex ) ;
                *_ppzHex = pzHex ;
            }
            else
            {
                hRes = E_OUTOFMEMORY ;
            }
        }
        else
        {
            ASSERT(FALSE) ;
            hRes = E_FAIL ;
        }
    }
    else
    {
        hRes = E_INVALIDARG ;
    }

    return hRes ;
}

//--------------------------------------------------------------------

HRESULT HexToBin( LPCSTR pczHex , LPVOID pBin , size_t nMaxBinLength )
{
    HRESULT hRes   = S_OK ;
    size_t  nIndex = 0 ;
    // ----
    if ( pczHex != NULL && pBin != NULL )
    {
        LPCTSTR pczCursor  = pczHex ;
        short   nHi        = _HexCharToShort( *( pczCursor ) ) ;
        short   nLo        = _HexCharToShort( *( pczCursor + 1 ) ) ;
        BYTE    cHi , cLo ;
        // ----
        while ( nHi >= 0 && nLo >= 0 && nIndex < nMaxBinLength )
        {
            reinterpret_cast<BYTE *>(pBin)[nIndex] = static_cast<BYTE>( ( nHi << 4 ) + nLo ) ;

            pczCursor += 2 ;
            nIndex++ ;

            cHi = *( pczCursor ) ;
            cLo = *( pczCursor + 1 ) ;

            if ( cHi && cLo )
            {
                nHi = _HexCharToShort( cHi ) ;
                nLo = _HexCharToShort( cLo ) ;
            }
            else
            {
                break ;
            }
        }
    }
    else
    {
        hRes = E_INVALIDARG ;
    }

    return hRes ;
}

//--------------------------------------------------------------------

HRESULT BinToHex( LPVOID pBin , LPTSTR pczHex , size_t nBinLength )
{
    HRESULT hRes = S_OK ;
    // ----
    if ( pczHex != NULL && pBin != NULL )
    {
        BYTE cc ;
        // ----
        for ( size_t index = 0 ; index < nBinLength ; index++ )
        {
            const BYTE cByte = reinterpret_cast<BYTE *>(pBin)[index] ;
            // ----
            if ( ( cc = _ShortToHexChar( ( cByte & 0xF0 ) >> 4 ) ) == (BYTE)'Z' )
            {
                hRes = E_INVALIDARG ;
                break ;
            }
            else
            {
                *( pczHex + ( 2 * index ) )     = cc ;
            }

            if ( ( cc = _ShortToHexChar( cByte & 0x0F ) ) == (BYTE)'Z' )
            {
                hRes = E_INVALIDARG ;
                break ;
            }
            else
            {
                *( pczHex + ( 2 * index ) + 1 ) = cc ;
            }
        }

        *( pczHex + ( 2 * index ) )     = (BYTE)0 ;
        *( pczHex + ( 2 * index ) + 1 ) = (BYTE)0 ;
    }
    else
    {
        hRes = E_INVALIDARG ;
    }

    return hRes ;
}

//--------------------------------------------------------------------

static inline _IsHexChar( BYTE c )
{
    return _HexCharToShort( c ) >= 0 ;
}

//--------------------------------------------------------------------

size_t HexToBinLength( LPCTSTR pczHex )
{
    size_t  nBinLength = 0 ;

    if ( pczHex != NULL )
    {
        LPCTSTR pczCursor  = pczHex ;
        BYTE    cHi        = *( pczCursor ) ;
        BYTE    cLo        = *( pczCursor + 1 ) ;
        // ----
        while ( cHi != 0 && cLo != 0 )
        {
            if ( _IsHexChar( cHi ) && _IsHexChar( cLo ) )
            {
                nBinLength++ ;
                pczCursor += 2 ;

                cHi = *( pczCursor ) ;
                cLo = *( pczCursor + 1 ) ;
            }
            else
            {
                break ;
            }
        }
    }

    return nBinLength ;
}

//--------------------------------------------------------------------

static inline size_t _BinLenToHexLen( size_t nBinLen )
{
    return ( 2 * nBinLen ) + 2 ; // One BYTE translates into 2 chars and the string is punctuated by "00".
}

//--------------------------------------------------------------------


HRESULT BMPFileToHex( LPCTSTR pczBMPFile , LPTSTR * ppzMallocHexBuffer )
{
    HRESULT hRes = S_OK ;
    CFile   file ;
    // ----
    if ( pczBMPFile == NULL || ppzMallocHexBuffer == NULL )
    {
        hRes = E_INVALIDARG ;
    }
    else
    {
        if ( !file.Open( pczBMPFile, CFile::modeRead) )
        {
            hRes = E_FAIL ;
        }
        else
        {
            BITMAPFILEHEADER bmfHeader;
            // ----
            // Read file header
            if ( file.Read((LPSTR)&bmfHeader, sizeof(bmfHeader)) != sizeof(bmfHeader) )
            {
                hRes = E_FAIL ;
            }
            else
            {
                // File type should be 'BM'
                if ( bmfHeader.bfType != ((WORD) ('M' << 8) | 'B') )
                {
                    hRes = E_FAIL ;
                }
                else
                {
                    // Get length of the remainder of the file and allocate memory
                    DWORD  nPackedDIBLen = file.GetLength() - sizeof(BITMAPFILEHEADER) ;
                    LPVOID pBin          = malloc( nPackedDIBLen ) ;
                    // ----
                    if ( pBin != NULL )
                    {
                        if ( file.ReadHuge((LPSTR)pBin, nPackedDIBLen) != nPackedDIBLen )
                        {
                            hRes = E_FAIL ;
                        }
                        else
                        {
                            size_t nHexLen = _BinLenToHexLen( nPackedDIBLen ) ;
                            // ----
                            *ppzMallocHexBuffer = (LPTSTR)malloc( nHexLen ) ;

                            if ( *ppzMallocHexBuffer != NULL )
                            {
                                hRes = BinToHex( pBin , *ppzMallocHexBuffer , nPackedDIBLen ) ;
                            }
                            else
                            {
                                hRes = E_OUTOFMEMORY ;
                            }
                        }

                        free( pBin ) ;
                    }
                    else
                    {
                        hRes = E_OUTOFMEMORY ;
                    }
                }
            }
        }
    }

    return hRes ;
}

//--------------------------------------------------------------------
// LoadBMPImage - Loads a BMP file and creates a bitmap GDI object
//  also creates logical palette for it.// Returns  - TRUE for success
// sBMPFile - Full path of the BMP file
// bitmap   - The bitmap object to initialize
// pPal     - Will hold the logical palette. Can be NULL

HRESULT LoadBMPFromHex( LPCTSTR pczHex , CBitmap & bitmap, CPalette * pPal )
{
    HRESULT hRes          = S_OK ;
    DWORD   nPackedDIBLen = 0L ;
    HGLOBAL hDIB          = (HGLOBAL)NULL ;
    LPVOID  lpDIBBits     = NULL ;
    // ----
#ifdef _DEBUG
    size_t _debug_len = strlen( pczHex ) ;
#endif

    if ( pczHex == NULL )
    {
        hRes = E_INVALIDARG ;
        goto CleanUp ;
    }

    nPackedDIBLen = (DWORD)HexToBinLength( pczHex ) ;
    hDIB          = ::GlobalAlloc(GMEM_FIXED, nPackedDIBLen);

    if ( hDIB == 0 )
    {
        hRes = E_OUTOFMEMORY ;
        goto CleanUp ;
    }

    {
        LPVOID pBin = GlobalLock( hDIB ) ;
        // ----
        hRes = HexToBin( pczHex , pBin , nPackedDIBLen ) ;
        GlobalUnlock( hDIB ) ;

        if ( FAILED( hRes ) )
        {
            goto CleanUp ;
        }
    }

    { // *
        BITMAPINFOHEADER & bmiHeader = *(LPBITMAPINFOHEADER)hDIB ;
        BITMAPINFO       & bmInfo    = *(LPBITMAPINFO)hDIB ;
        // ----
        // If bmiHeader.biClrUsed is zero we have to infer the number
        // of colors from the number of bits used to specify it.
        int    nColors = bmiHeader.biClrUsed ? bmiHeader.biClrUsed : 1 << bmiHeader.biBitCount;
        // ----
        if ( bmInfo.bmiHeader.biBitCount > 8 )
        {
            lpDIBBits = (LPVOID)((LPDWORD)(bmInfo.bmiColors + bmInfo.bmiHeader.biClrUsed) +
                                 ((bmInfo.bmiHeader.biCompression == BI_BITFIELDS) ? 3 : 0));
        }
        else
        {
            lpDIBBits = (LPVOID)(bmInfo.bmiColors + nColors) ;
        }

        // Create the logical palette
        if ( pPal != NULL )
        {     // Create the palette
            if ( nColors <= 256 )
            {
                try
                {
                    UINT         nSize  = sizeof(LOGPALETTE) + (sizeof(PALETTEENTRY) * nColors) ;
                    BYTE       * pBytes = new BYTE[nSize] ;
                    LOGPALETTE * pLP    = (LOGPALETTE *)pBytes ;
                    // ----
                    pLP->palVersion    = 0x300 ;
                    pLP->palNumEntries = nColors ;

                    for ( int i = 0 ; i < nColors ; i++ )
                    {
                        pLP->palPalEntry[i].peRed   = bmInfo.bmiColors[i].rgbRed ;
                        pLP->palPalEntry[i].peGreen = bmInfo.bmiColors[i].rgbGreen ;
                        pLP->palPalEntry[i].peBlue  = bmInfo.bmiColors[i].rgbBlue ;
                        pLP->palPalEntry[i].peFlags = 0 ;
                    }

                    pPal->CreatePalette( pLP );

                    delete [] pBytes ;
                }
                catch( ... )
                {
                    hRes = E_OUTOFMEMORY ;
                    goto CleanUp ;
                }
            }
        }

        { // **
            CClientDC  dc(NULL);
            CPalette * pOldPalette = NULL ;
            // ----
            if ( pPal != NULL )
            {
                pOldPalette = dc.SelectPalette( pPal, FALSE );
                dc.RealizePalette();
            }

            { // ***
                HBITMAP hBmp = CreateDIBitmap( dc.m_hDC,       // handle to device context
                                               &bmiHeader,      // pointer to bitmap size and format data
                                               CBM_INIT,       // initialization flag
                                               lpDIBBits,       // pointer to initialization data
                                               &bmInfo,         // pointer to bitmap color-format data
                                               DIB_RGB_COLORS);  // color-data usage
                // ----
                ASSERT( hBmp != (HBITMAP)NULL ) ;

                bitmap.Attach( hBmp ) ;
            } // ***

            if ( pOldPalette )
            {
                dc.SelectPalette( pOldPalette, FALSE );
            }
        } // **
    } // *


CleanUp: // -- !! --

    if ( hDIB != 0 )
    {
        ::GlobalFree(hDIB) ;
    }

    return hRes ;
}

//--------------------------------------------------------------------

CHexBmp::CHexBmp( LPCTSTR pczBMPFile /*=NULL*/ ) :
        m_pzHexBmp(NULL) ,
        m_pBitmap(NULL) ,
        m_wFlags(0)
{
    if ( pczBMPFile != NULL )
    {
        Load( pczBMPFile ) ;
    }
}

//--------------------------------------------------------------------

CHexBmp::~CHexBmp()
{
    free( m_pzHexBmp ) ;
    delete m_pBitmap ;
}

//--------------------------------------------------------------------

HRESULT CHexBmp::Load( LPCTSTR pczBMPFile )
{
    if ( pczBMPFile != NULL && *pczBMPFile != (TCHAR)0 )
    {
        free( m_pzHexBmp ) ; // Avoids leaks.

        return BMPFileToHex( pczBMPFile , &m_pzHexBmp ) ;
    }

    return E_INVALIDARG ;
}

//--------------------------------------------------------------------

static LPCSTR _GetSignature( size_t * pnLen = NULL )
{
    static char szSign[] = "[<HexBmp:1.0>]" ;
    // ----
    if ( pnLen )
    {
        *pnLen = sizeof( szSign ) - 1 ;
    }
    return szSign ;
}

//--------------------------------------------------------------------

HRESULT CHexBmp::SetHexString( LPCSTR _pczHex )
{
    if ( _pczHex != NULL )
    {
        size_t nLen    = 0 ;
        LPCSTR pczSign = _GetSignature( &nLen ) ;
        LPCSTR pczHex  = _pczHex ;
        // ----
        if ( memcmp( pczSign , pczHex , nLen ) == 0 )
        {
            pczHex += nLen ; // Signature accepted but not mandatory.
        }


        if ( _IsHexChar( *pczHex ) || _EscapeToHexChar( *pczHex ) ) // Filter possible errors.
        {
            LPSTR   pzHexDest = NULL ;
            HRESULT hRes ;
            // ----
            if ( SUCCEEDED( hRes = ::UnPack( pczHex , &pzHexDest ) ) )
            {
                m_wFlags &= ~flagPacked ;

                free( m_pzHexBmp ) ;
                delete m_pBitmap ;

                m_pzHexBmp = pzHexDest ;
            }
        }

        return E_FAIL ;
    }
    return E_INVALIDARG ;
}

//--------------------------------------------------------------------

HRESULT CHexBmp::GetHexString( LPCSTR * ppczHex ) const
{
    if ( ppczHex != NULL )
    {
        *ppczHex = m_pzHexBmp ;
        return S_OK ;
    }

    return E_INVALIDARG ;
}

//--------------------------------------------------------------------

HRESULT CHexBmp::GetBitmap( CBitmap * * ppBitmap ) const
{
    if ( ppBitmap != NULL )
    {
        *ppBitmap = m_pBitmap ;
        return ( *ppBitmap != NULL ? S_OK : S_FALSE ) ;
    }

    return E_INVALIDARG ;
}

//--------------------------------------------------------------------

HRESULT CHexBmp::LoadBitmap( CBitmap * pBitmap, CPalette * pPal /*=NULL*/ )
{
    HRESULT hRes = S_OK ;
    // ----
    if ( pBitmap != NULL )
    {
        if ( ( m_wFlags & flagPacked ) == 0 || SUCCEEDED( hRes = Unpack() ) )
        {
            if ( m_pzHexBmp != NULL && *m_pzHexBmp != (TCHAR)0 )
            {
                hRes = LoadBMPFromHex( m_pzHexBmp , *pBitmap , pPal ) ;
            }
            else
            {
                ASSERT(FALSE) ;
                hRes = E_UNEXPECTED ;
            }
        }
    }
    else
    {
        hRes = E_INVALIDARG ;
    }

    return hRes ;
}

//--------------------------------------------------------------------

HRESULT CHexBmp::LoadBitmap( CPalette * pPal /*=NULL*/ )
{
    HRESULT hRes = S_OK ;
    // ----
    if ( m_pBitmap != NULL )
    {
        delete m_pBitmap ;
        m_pBitmap = NULL ;
    }

    try
    {
        m_pBitmap = new CBitmap() ;
        hRes      = LoadBitmap( m_pBitmap , pPal ) ;
    }
    catch ( ... )
    {
        m_pBitmap = NULL ;
        hRes = E_OUTOFMEMORY ;
    }

    return hRes ;
}

//--------------------------------------------------------------------

HRESULT CHexBmp::Pack()
{
    if ( ( m_wFlags & flagPacked ) == flagPacked )
    {
        // Already packed :
        return S_FALSE ;
    }
    else
    {
        HRESULT hRes = ::Pack( m_pzHexBmp , m_pzHexBmp ) ;
        // ----
        if ( SUCCEEDED( hRes ) )
        {
            m_wFlags |= flagPacked ;
        }
        return hRes ;
    }
}

//--------------------------------------------------------------------

HRESULT CHexBmp::Unpack()
{
    if ( ( m_wFlags & flagPacked ) == 0 )
    {
        // Not packed :
        return S_FALSE ;
    }
    else
    {
        HRESULT hRes = ::UnPack( m_pzHexBmp , &m_pzHexBmp ) ;
        // ----
        if ( SUCCEEDED( hRes ) )
        {
            m_wFlags &= ~flagPacked ;
        }
        return hRes ;
    }
}

//--------------------------------------------------------------------

HRESULT CHexBmp::CopyHex()
{
    HRESULT hRes = E_FAIL ;
    // ----
    if ( m_pzHexBmp != NULL )
    {
        size_t nHexlen = strlen( m_pzHexBmp ) ;
        size_t nLen    = 0 ;
        LPCSTR pczSign = _GetSignature( &nLen ) ;
        HANDLE hClip   = ::GlobalAlloc( GHND , nLen + nHexlen + sizeof( "||" ) ) ;
        // ----
        if ( hClip != (HANDLE)NULL )
        {
            LPSTR pzClip = reinterpret_cast<LPSTR>( ::GlobalLock( hClip ) ) ;
            // ----
            ASSERT( pzClip != NULL ) ;

            memcpy( pzClip , pczSign , nLen ) ;
            memcpy( pzClip + nLen , m_pzHexBmp , nHexlen ) ;
            memcpy( pzClip + nHexlen + nLen , "||" , sizeof( "||" ) ) ;
            ::GlobalUnlock( hClip ) ;

            if ( ::OpenClipboard( NULL ) != FALSE )
            {
                ::EmptyClipboard() ;

                if ( ::SetClipboardData( CF_TEXT , hClip ) != (HANDLE)NULL )
                {
                    hRes = S_OK ;
                }

                ::CloseClipboard() ;
            }
        }
        else
        {
            hRes = E_OUTOFMEMORY ;
        }
    }

    return hRes ;
}

//--------------------------------------------------------------------

HRESULT CHexBmp::PasteHex()
{
    HRESULT hRes = E_FAIL ;
    // ----
    if ( ::OpenClipboard( NULL ) != FALSE )
    {
        HANDLE hClip = ::GetClipboardData(CF_TEXT) ;
        // ----
        if ( hClip != (HANDLE)NULL )
        {
            LPCSTR pzcClip = reinterpret_cast<LPCSTR>( ::GlobalLock( hClip ) ) ;
            LPSTR  pzHex   = NULL ;
            size_t nLen    = 0 ;
            LPCSTR pczSign = _GetSignature( &nLen ) ;
            // ----
            if ( memcmp( pczSign , pzcClip , nLen ) == 0 )
            {
                if ( SUCCEEDED( hRes = ::UnPack( pzcClip + nLen , &pzHex ) ) )
                {
                    m_wFlags &= ~flagPacked ;

                    free( m_pzHexBmp ) ;
                    delete m_pBitmap ;

                    m_pzHexBmp = pzHex ;
                }
            }
            else
            {
                hRes = E_FAIL ;
            }

            ::GlobalUnlock( hClip ) ;
        }

        ::CloseClipboard() ;
    }

    return hRes ;
}

//--------------------------------------------------------------------

BOOL CHexBmp::CanPaste()
{
    BOOL bRet = FALSE ;
    // ----
    if ( ::OpenClipboard( NULL ) != FALSE )
    {
        HANDLE hClip = ::GetClipboardData(CF_TEXT) ;
        // ----
        if ( hClip != (HANDLE)NULL )
        {
            LPCSTR pzcClip = reinterpret_cast<LPCSTR>( ::GlobalLock( hClip ) ) ;
            size_t nLen    = 0 ;
            LPCSTR pczSign = _GetSignature( &nLen ) ;
            // ----
            if ( memcmp( pczSign , pzcClip , nLen ) == 0 )
            {
                bRet = TRUE ;
            }
            ::GlobalUnlock( hClip ) ;
        }
        ::CloseClipboard() ;
    }

    return bRet ;
}

//--------------------------------------------------------------------
