/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CExtendedListBoxEditor.cpp: implementation of the CCExtendedListBoxEditor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CExtendedListBoxEditor.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//----------------------------------------------------------------------------

inline BOOL IsKeyDown(UINT ui)
{
    return GetKeyState(ui) & 0x8000;
}

//----------------------------------------------------------------------------

static void LoadButtonBitmap(UINT uiID, CBitmap& bmp)
{
    HINSTANCE hInstApp = AfxFindResourceHandle(MAKEINTRESOURCE(uiID),RT_BITMAP);
    HANDLE hBmp = LoadImage(hInstApp,
                            MAKEINTRESOURCE(uiID),
                            IMAGE_BITMAP,
                            0, 0, LR_LOADMAP3DCOLORS);

    ASSERT(hBmp != NULL);
    VERIFY(bmp.Attach(hBmp));

    // TODO:  Figure out why LoadImage doesn't correctly translate
    //        the highlight colors, then ditch this code.

    CDC DC;
    DC.CreateCompatibleDC(NULL);
    CBitmap* pbmpOld = DC.SelectObject(&bmp);
    CPen penHi(PS_SOLID, 0, GetSysColor(COLOR_3DHILIGHT));
    BITMAP bmpInfo;
    bmp.GetObject(sizeof(bmpInfo), &bmpInfo);
    CPen* ppenOld = DC.SelectObject(&penHi);
    DC.MoveTo(0, bmpInfo.bmHeight - 2);
    DC.LineTo(0, 0);
    DC.LineTo(bmpInfo.bmWidth - 1, 0);
    DC.SelectObject(ppenOld);
    DC.SelectObject(pbmpOld);
}

//----------------------------------------------------------------------------

static int GetBottomIndex(CListBox* pLB)
{
    CRect rcClient;
    pLB->GetClientRect(&rcClient);
    int iNumItems = pLB->GetCount();

    for (int i = pLB->GetTopIndex(); i < iNumItems; i++)
    {
        CRect rcItem;
        if (pLB->GetItemRect(i, &rcItem) == LB_ERR ||
                rcItem.top > rcClient.bottom)
        {
            return i - 1;
        }
    }

    return -1;
}

const int RHC_CLIENT    = 0x0;
const int RHC_LEFT      = 0x1;
const int RHC_ABOVE     = 0x2;
const int RHC_RIGHT     = 0x4;
const int RHC_BELOW     = 0x8;

//----------------------------------------------------------------------------

static int GetRectangleHitCode(const CRect& r, const CPoint& pt)
{
    int iRet = RHC_CLIENT;
    if ( pt.x < r.left )
        iRet |= RHC_LEFT;
    else if ( pt.x > r.right )
        iRet |= RHC_RIGHT;
    if ( pt.y < r.top )
        iRet |= RHC_ABOVE;
    else if ( pt.y > r.bottom )
        iRet |= RHC_BELOW;
    return iRet;
}

//----------------------------------------------------------------------------

static BOOL AfxRegisterNamedWndClass(LPCTSTR pcszName, UINT nClassStyle,
                                     HCURSOR hCursor, HBRUSH hbrBackground,
                                     HICON hIcon)
{
    WNDCLASS wndcls;
    if (::GetClassInfo(AfxGetInstanceHandle(), pcszName, &wndcls))
    {
        // NOTE: We have to trust that the hIcon, hbrBackground, and the
        // hCursor are semantically the same, because sometimes Windows does
        // some internal translation or copying of those handles before
        // storing them in the internal WNDCLASS retrieved by GetClassInfo.

        ASSERT(wndcls.style == nClassStyle);
        return TRUE;
    }

    wndcls.style = nClassStyle;
    wndcls.lpfnWndProc = DefWindowProc;
    wndcls.cbClsExtra = wndcls.cbWndExtra = 0;
    wndcls.hInstance = AfxGetInstanceHandle();
    wndcls.hIcon = hIcon;
    wndcls.hCursor = hCursor;
    wndcls.hbrBackground = hbrBackground;
    wndcls.lpszMenuName = NULL;
    wndcls.lpszClassName = pcszName;

    if (AfxRegisterClass(&wndcls))
        return TRUE;

    AfxThrowResourceException();
    return FALSE;
}

//----------------------------------------------------------------------------

static UINT FindUnusedCtrlID(CWnd* pWnd)
{
    const UINT MAX_DLGID = 32767;
    const UINT MIN_DLGID = 1;
    UINT uiID;
    for (uiID = MAX_DLGID; uiID != MIN_DLGID; uiID--)
        if (::GetDlgItem(pWnd->GetSafeHwnd(), uiID) == NULL)
            break;
    ASSERT(uiID != MIN_DLGID);
    return uiID;
}

//----------------------------------------------------------------------------

static BOOL ReplaceListBoxItem(CListBox* pLB, int iIndex,
                               LPCTSTR pcszString,
                               DWORD dwItemData)
{
    BOOL bReturn = FALSE;
    pLB->SetRedraw(FALSE);
    int iSel = pLB->GetCurSel();
    if (pLB->InsertString(iIndex, pcszString) > -1)
    {
        pLB->SetItemData(iIndex, dwItemData);
        pLB->DeleteString(iIndex + 1);
        if (pLB->GetStyle() & LBS_MULTIPLESEL)
        {
            pLB->SetSel(iSel);
        }
        else
        {
            pLB->SetCurSel(iSel);
        }
        bReturn = TRUE;
    }
    pLB->SetRedraw(TRUE);
    return bReturn;
}

//----------------------------------------------------------------------------

static BOOL DrawBitmap(CDC* pDC, CBitmap* pBmp, int x, int y)
{
    BOOL bReturn = FALSE;
    CDC dcMem;
    if (dcMem.CreateCompatibleDC(pDC))
    {
        BITMAP bmp;
        pBmp->GetObject(sizeof(bmp), &bmp);
        CBitmap* pbmpOld = dcMem.SelectObject(pBmp);
        bReturn = pDC->BitBlt(x, y, bmp.bmWidth, bmp.bmHeight,
                              &dcMem, 0, 0, SRCCOPY);
        dcMem.SelectObject(pbmpOld);
    }

    return bReturn;
}

//----------------------------------------------------------------------------

static void CreateDownBitmap(CBitmap* pSrc, CBitmap* pDst)
{
    CDC DC;
    DC.CreateCompatibleDC(NULL);
    BITMAP bmp;
    pSrc->GetObject(sizeof(bmp), &bmp);
    pDst->CreateBitmapIndirect(&bmp);
    CBitmap* pbmpOld = DC.SelectObject(pDst);
    DrawBitmap(&DC, pSrc, 1, 1);

    // Draw the outer shading.

    CPen penHi(PS_SOLID, 0, GetSysColor(COLOR_3DHILIGHT));
    CPen penLo(PS_SOLID, 0, GetSysColor(COLOR_3DDKSHADOW));
    CPen* ppenOld = DC.SelectObject(&penHi);
    DC.MoveTo(0, bmp.bmHeight - 1);
    DC.LineTo(bmp.bmWidth - 1, bmp.bmHeight - 1);
    DC.LineTo(bmp.bmWidth - 1, -1);
    DC.SelectObject(&penLo);
    DC.MoveTo(bmp.bmWidth - 1, 0);
    DC.LineTo(0, 0);
    DC.LineTo(0, bmp.bmHeight - 1);

    // Draw the inner shading.

    CPen penLt(PS_SOLID, 0, GetSysColor(COLOR_3DFACE));
    CPen penDk(PS_SOLID, 0, GetSysColor(COLOR_3DSHADOW));
    DC.SelectObject(&penLt);
    DC.MoveTo(1, bmp.bmHeight - 2);
    DC.LineTo(bmp.bmWidth - 2, bmp.bmHeight - 2);
    DC.LineTo(bmp.bmWidth - 2, 0);
    DC.SelectObject(&penDk);
    DC.MoveTo(bmp.bmWidth - 3, 1);
    DC.LineTo(1, 1);
    DC.LineTo(1, bmp.bmHeight - 2);

    DC.SelectObject(ppenOld);
    DC.SelectObject(pbmpOld);
}

////////////////////////////////////////////////////////////////////////////////
// CExtendedListBoxDragWnd implementation

class CExtendedListBoxDragWnd : public CWnd
{
protected:
    //{{AFX_MSG(CExtendedListBoxDragWnd)
    afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
    afx_msg void OnMouseMove(UINT nFlags, CPoint point);
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()

protected:
    BOOL m_bDragging;           // are we busy dragging?
    BOOL m_bAbort;              // has the drag operation been aborted?
    BOOL m_bMouseBeyond;        // has mouse moved beyond dblclk threshold?
    BOOL m_bTrailingBlank;      // does the control have a trailing "blank" item?
    DWORD m_dwLastScrollTick;   // last tick count when scroll message sent
    DWORD m_dwStartInterval;    // initial interval for scrolling (in ms.)
    DWORD m_dwScrollInterval;   // current interval for scrolling (in ms.)
    DWORD m_dwMinInterval;      // minimum interval for scrolling (in ms.)
    DWORD m_dwIntervalDec;      // amount by which to decrement interval
    DWORD m_dwStartTime;        // tick count at which dragging began
    DWORD m_dwStopTime;         // tick count at which dragging ended
    DWORD m_dwDragDelay;        // delay before actual drag begins
    HCURSOR m_hcurClient;       // cursor to use when in owner window client
    HCURSOR m_hcurOld;          // original cursor (to be replaced when done)
    int m_hitLast;              // last owner-related cursor hit code
    int m_iLastInsertBar;       // client-coordinates of last insert bar
    int m_iSrcIndex;            // source index for drag operation
    int m_iDstIndex;            // destination index for drag insert
    CPoint m_ptDown;            // point at which drag began
    CPoint m_ptUp;              // mouse release point (in owner-client units)
    CListBox* m_pLB;            // pointer to the listbox

    DWORD GetDragTime() const
    {
        return (m_bDragging ? GetTickCount() : m_dwStopTime) - m_dwStartTime;
    }
    DWORD GetDragDelay() const
    {
        return m_dwDragDelay;
    }
    void SetMouseBeyondThreshold(BOOL b)
    {
        m_bMouseBeyond = b;
    }
    BOOL Start();
    void Stop();
    int GetOwnerHitCode(const CPoint& ptScreen) const;
    BOOL ProcessMessage(MSG& msg);
    void PerformTimedScroll();
    void DecrementScrollInterval();
    void DrawInsertBar(int i);

    enum ScrollDir { up = -1, down = 1 };
    void DoVScroll(ScrollDir d);

public:
    CExtendedListBoxDragWnd(CListBox* pLB);
    virtual ~CExtendedListBoxDragWnd();
    BOOL IsDragging() const
    {
        return m_bDragging;
    }
    void Abort()
    {
        m_bAbort = TRUE;
    }
    BOOL PerformDragNDrop(int& nOldIndex,int& nNewIndex);
    BOOL SetScrollParameters(DWORD dwInitial, DWORD dwMin, DWORD dwDec);
    DWORD SetDragDelay(DWORD dw)
    {
        return m_dwDragDelay = dw;
    }
    BOOL IsMouseBeyondThreshold() const
    {
        return m_bMouseBeyond;
    }
    BOOL IsDragDelayOver() const
    {
        return GetDragTime() > GetDragDelay();
    }
    HCURSOR GetClientCursor() const
    {
        return m_hcurClient;
    }
    HCURSOR SetClientCursor(HCURSOR hcur)
    {
        return m_hcurClient = hcur;
    }
    void SetTrailingBlank(BOOL bTrailingBlank)
    {
        m_bTrailingBlank = bTrailingBlank;
    }
};

static TCHAR BASED_CODE PCSZ_CDRAGWND[] = _T("CExtendedListBoxDragWnd");

//------------------------------------------------------------------------------

BOOL CExtendedListBoxDragWnd::Start()
{
    // if no current selection, no drag operation can occur
    int iSrcIndex=m_pLB->GetCurSel();
    if(iSrcIndex==LB_ERR) return FALSE;

    // if source is the last blank entry, no drag can occur (unless OK flag is set)
    if ((m_bTrailingBlank) && (iSrcIndex==m_pLB->GetCount()-1)) return FALSE;

    AfxRegisterNamedWndClass(PCSZ_CDRAGWND, CS_HREDRAW | CS_VREDRAW,
                             AfxGetApp()->LoadStandardCursor(IDC_ARROW),
                             (HBRUSH)::GetStockObject(WHITE_BRUSH), 0);
    if (CreateEx(0L, PCSZ_CDRAGWND, PCSZ_CDRAGWND, WS_POPUP, 0, 0, 0, 0, 0, 0))
    {
        m_iSrcIndex = iSrcIndex;
        m_iDstIndex = -2;
        m_bAbort = FALSE;
        m_bDragging = TRUE;
        m_iLastInsertBar = -1;
        SetCapture();
        SetMouseBeyondThreshold(FALSE);
        GetCursorPos(&m_ptDown);
        ScreenToClient(&m_ptDown);
        m_dwStartTime = m_dwStopTime = GetTickCount();
        return TRUE;
    }

    ASSERT(0);
    return FALSE;
}

//------------------------------------------------------------------------------

void CExtendedListBoxDragWnd::Stop()
{
    if (m_iLastInsertBar > -1)
        DrawInsertBar(m_iLastInsertBar);
    m_dwStopTime = GetTickCount();
    ReleaseCapture();
    m_bDragging = FALSE;
    DestroyWindow();
}

//------------------------------------------------------------------------------

int CExtendedListBoxDragWnd::GetOwnerHitCode(const CPoint& ptScreen) const
{
    CRect r;
    m_pLB->GetClientRect(&r);
    m_pLB->ClientToScreen(&r);
    return GetRectangleHitCode(r, ptScreen);
}

//------------------------------------------------------------------------------

BOOL CExtendedListBoxDragWnd::ProcessMessage(MSG& msg)
{
    if (msg.message == WM_QUIT)
    {
        Abort();
        ::PostQuitMessage(0);
        return TRUE;
    }

    if (msg.message == WM_KEYDOWN || msg.message == WM_KEYUP)
    {
        if (msg.wParam == VK_ESCAPE)
            Abort();
        return TRUE;
    }

    return FALSE;
}

//------------------------------------------------------------------------------

void CExtendedListBoxDragWnd::DecrementScrollInterval()
{
    m_dwScrollInterval -= m_dwIntervalDec;
    if (m_dwScrollInterval < m_dwMinInterval)
        m_dwScrollInterval = m_dwMinInterval;
}

//------------------------------------------------------------------------------

void CExtendedListBoxDragWnd::DoVScroll(ScrollDir dir)
{
    // Make sure we haven't already scrolled as far as we
    // can go in the given direction.

    CRect rcLast, rcClient;
    int iTopIndex = m_pLB->GetTopIndex();
    int iBotIndex = GetBottomIndex(m_pLB);

    m_pLB->GetItemRect(m_pLB->GetCount() - 1, &rcLast);
    m_pLB->GetClientRect(&rcClient);
    if ((iTopIndex == 0 && dir == up) ||
            (rcLast.bottom < rcClient.bottom && dir == down))
    {
        return;
    }

    // Otherwise, undraw the old insert bar (if needed),
    // calculate and set the new top index, and redraw
    // the insertion bar.

    int iNewTopIndex = iTopIndex + dir;
    if (m_iLastInsertBar > -1)
        DrawInsertBar(m_iLastInsertBar);
    m_pLB->SetTopIndex(iNewTopIndex);

    CRect rc;
    int iy;
    if (dir == up)
    {
        m_iDstIndex = iNewTopIndex;
        m_pLB->GetItemRect(m_iDstIndex, &rc);
        iy = rc.top;
    }
    else
    {
        m_iDstIndex = iBotIndex;
        if (m_iDstIndex == -1)
            // Subtract 1 for count->index conversion, 1 for the last blank entry
            m_pLB->GetItemRect(m_pLB->GetCount() - 2, &rc);
        else
            m_pLB->GetItemRect(m_iDstIndex, &rc);

        iy = rc.bottom;
    }

    DrawInsertBar(++iy);
}

//------------------------------------------------------------------------------

void CExtendedListBoxDragWnd::PerformTimedScroll()
{
    DWORD dwTicks = GetTickCount();
    DWORD dwDelta = (dwTicks - m_dwLastScrollTick);
    if (dwDelta > m_dwScrollInterval)
    {
        if (m_hitLast & RHC_ABOVE)
            DoVScroll(up);
        else if (m_hitLast & RHC_BELOW)
            DoVScroll(down);

        m_dwLastScrollTick = dwTicks;
        DecrementScrollInterval();
    }
}

//------------------------------------------------------------------------------

BOOL CExtendedListBoxDragWnd::PerformDragNDrop(int& nOldIndex,int& nNewIndex)
{
    if (!Start())
        return FALSE;

    while (!m_bAbort && m_bDragging)
    {
        MSG msg;
        if (::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            // If we handle this message, avoid dispatching it.

            if (ProcessMessage(msg))
                continue;

            if (!AfxGetThread()->PreTranslateMessage(&msg))
            {
                ::TranslateMessage(&msg);
                ::DispatchMessage(&msg);
            }

            AfxGetThread()->OnIdle(0);   // updates user interface
            AfxGetThread()->OnIdle(1);   // frees temporary objects

            // * background processing may be performed here *
        }

        // Handle owner scrolling if conditions are right.


        if (!m_bAbort && (m_hitLast != RHC_CLIENT))
            PerformTimedScroll();
    }

    Stop();
    if (m_hcurOld != NULL)
        ::SetCursor(m_hcurOld);

    if (IsMouseBeyondThreshold() && IsDragDelayOver() &&
            !m_bAbort && (m_iSrcIndex != m_iDstIndex) &&
            (m_iDstIndex != -2))
    {
        CString str;
        int iTopIndex = m_pLB->GetTopIndex();
        m_pLB->GetText(m_iSrcIndex, str);
        DWORD dwData = m_pLB->GetItemData(m_iSrcIndex);
        m_pLB->SetRedraw(FALSE);
        m_pLB->DeleteString(m_iSrcIndex);

        // We've just deleted a string; take this into account
        // when moving the data.

        if (m_iDstIndex > m_iSrcIndex)
            m_iDstIndex--;

        // if destination is end of list, make sure to insert
        // before the last entry (unless flag indicates that it
        // it is 'ok' to do this)
        if ((m_iDstIndex==-1) && (m_bTrailingBlank))
            m_iDstIndex=m_pLB->GetCount()-1;

        int iNewIndex = m_pLB->InsertString(m_iDstIndex, str);
        nOldIndex=m_iSrcIndex;
        nNewIndex=iNewIndex;
        m_pLB->SetItemData(iNewIndex, dwData);
        if (m_pLB->GetStyle() & LBS_MULTIPLESEL)
        {
            m_pLB->SetSel(iNewIndex);
        }
        else
        {
            m_pLB->SetCurSel(iNewIndex);
        }
        m_pLB->SetTopIndex(iTopIndex);
        m_pLB->SetRedraw(TRUE);
        return TRUE;
    }

    return FALSE;
}

//------------------------------------------------------------------------------

CExtendedListBoxDragWnd::CExtendedListBoxDragWnd(CListBox* pLB) :
        m_pLB(pLB),
        m_bTrailingBlank(TRUE),
        m_bAbort(FALSE),
        m_bDragging(FALSE),
        m_bMouseBeyond(FALSE),
        m_dwLastScrollTick(0L),
        m_dwStartInterval(300L),
        m_dwScrollInterval(0L),
        m_dwMinInterval(50L),
        m_dwIntervalDec(50L),
        m_dwDragDelay(250L),
        m_hitLast(RHC_CLIENT),
        m_iLastInsertBar(-1),
        m_hcurClient(NULL),
        m_hcurOld(NULL)
{}

//------------------------------------------------------------------------------


CExtendedListBoxDragWnd::~CExtendedListBoxDragWnd()
{}

//------------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CExtendedListBoxDragWnd, CWnd)
//{{AFX_MSG_MAP(CExtendedListBoxDragWnd)
ON_WM_LBUTTONUP()
ON_WM_MOUSEMOVE()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//------------------------------------------------------------------------------

void CExtendedListBoxDragWnd::OnLButtonUp(UINT nFlags, CPoint point)
{
    m_bDragging = FALSE;
    CWnd::OnLButtonUp(nFlags, point);
}

//------------------------------------------------------------------------------

void CExtendedListBoxDragWnd::OnMouseMove(UINT nFlags, CPoint point)
{
    if (!IsMouseBeyondThreshold())
    {
        int cx = GetSystemMetrics(SM_CXDOUBLECLK);
        int cy = GetSystemMetrics(SM_CYDOUBLECLK);
        int dx = abs(point.x - m_ptDown.x);
        int dy = abs(point.y - m_ptDown.y);
        if ((dx * dx + dy * dy) > (cx * cx + cy * cy))
        {
            SetMouseBeyondThreshold(TRUE);
            if (GetClientCursor())
                m_hcurOld = ::SetCursor(GetClientCursor());
        }
    }

    BOOL bOutside;
    CPoint lbpt(point);
    ClientToScreen(&lbpt);
    m_pLB->ScreenToClient(&lbpt);
    int iIndex = m_pLB->ItemFromPoint(lbpt, bOutside);

    // If last item is empty space for easy double click add,
    // make sure this space is not considered valid during
    // a drag operation!

    if ((m_bTrailingBlank) && (iIndex==m_pLB->GetCount()-1))
    {
        // disregard the "blank index" for purposes of drag
        --iIndex;
        if(iIndex<-1)
            iIndex=-1;
    }

    if (!bOutside)
    {
        // It's possible for the cursor to be beneath the last item
        // in the list.  If the last item in the list is supposed to
        // be empty, decrement the index so items can only be dragged
        // in front of this item.

        if (m_bTrailingBlank)
            if (iIndex == -1)
                iIndex = m_pLB->GetCount() - 1;

        // Now get the rectangle and figue out on which side (top
        // or bottom) the insert bar should be drawn.

        CRect rc;
        m_pLB->GetItemRect(iIndex, &rc);
        int iCenter = rc.top + rc.Height() / 2, iy;
        if (lbpt.y < iCenter)
        {
            iy = rc.top;
            m_iDstIndex = iIndex;
        }
        else
        {
            iy = rc.bottom;
            m_iDstIndex = iIndex + 1;
            if (m_iDstIndex == m_pLB->GetCount())
                m_iDstIndex = -1;
        }

        // If the new position is any different than the last,
        // undraw the bar (if needed) and draw the new bar.

        if (++iy != m_iLastInsertBar)
        {
            if (m_iLastInsertBar > -1)
                DrawInsertBar(m_iLastInsertBar);
            DrawInsertBar(iy);
        }
    }

    // Finally, check to see if we need to start scrolling.

    if (IsMouseBeyondThreshold() && IsDragDelayOver())
    {
        CPoint pt(point);
        ClientToScreen(&pt);
        int hitNew = GetOwnerHitCode(pt);
        if (hitNew != m_hitLast && m_hitLast == RHC_CLIENT)
            m_dwScrollInterval = m_dwStartInterval;
        m_hitLast = hitNew;
    }

    CWnd::OnMouseMove(nFlags, point);
}

//------------------------------------------------------------------------------

BOOL CExtendedListBoxDragWnd::SetScrollParameters(DWORD dwInitial, DWORD dwMin, DWORD dwDec)
{
    if (dwMin <= dwInitial && dwDec <= (dwInitial - dwMin))
    {
        m_dwStartInterval = dwInitial;
        m_dwMinInterval = dwMin;
        m_dwIntervalDec = dwDec;
        return TRUE;
    }

    ASSERT(0);
    return FALSE;
}

//------------------------------------------------------------------------------

void CExtendedListBoxDragWnd::DrawInsertBar(int iy)
{
    CRect rc;
    m_pLB->GetClientRect(&rc);
    m_iLastInsertBar = iy;
    CPen pen(PS_SOLID, 0, RGB(128, 128, 128));
    CWindowDC DC(m_pLB);
    CPen* ppenOld = DC.SelectObject(&pen);
    DC.SetROP2(R2_NOTXORPEN);
    DC.MoveTo(rc.left + 2, iy);
    DC.LineTo(rc.right + 1, iy);
    DC.SelectObject(ppenOld);
}

////////////////////////////////////////////////////////////////////////////////
// CExtendedFakeButton implementation

class CExtendedFakeButton : public CWnd
{
    BOOL m_bDown;           // is the button currently down?
    BOOL m_bDone;           // are we done faking yet?
    CBitmap& m_bmpDown;     // bitmap to use for down drawing
    CBitmap& m_bmpUp;       // bitmap to use for up drawing
    CRect& m_rc;            // rectangle (in screen coordinates)

protected:
    // Generated message map functions
    //{{AFX_MSG(CExtendedFakeButton)
    afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
    afx_msg void OnMouseMove(UINT nFlags, CPoint point);
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()

    void DoEvents();        // provides a local message pump
    void Draw(BOOL bDown);  // draws the appropriate bitmap

public:

    // To imitate a button, instantiate an object providing the
    // bitmaps and screen rectangle; call DoFakeButton to start
    // the process.  Said method will return true if the button
    // release was within the specified rectangle (i.e. the fake
    // button was "clicked").

CExtendedFakeButton(CBitmap& bmpDown, CBitmap& bmpUp, CRect& rc) :
    m_bDown(FALSE), m_bDone(FALSE), m_bmpDown(bmpDown),
    m_bmpUp(bmpUp), m_rc(rc)
    {}

    BOOL DoFakeButton();
};

//------------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CExtendedFakeButton, CWnd)
//{{AFX_MSG_MAP(CExtendedFakeButton)
ON_WM_LBUTTONUP()
ON_WM_MOUSEMOVE()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//------------------------------------------------------------------------------

void CExtendedFakeButton::Draw(BOOL bDown)
{
    CDC DC;
    VERIFY(DC.CreateDC(_T("display"), NULL, NULL, NULL));
    VERIFY(DrawBitmap(&DC, bDown ? &m_bmpDown : &m_bmpUp,
                      m_rc.left, m_rc.top));
}

//------------------------------------------------------------------------------

void CExtendedFakeButton::DoEvents()
{
    CWinThread* pThread = AfxGetThread();
    MSG msg;

    while (::PeekMessage(&msg, GetSafeHwnd(), 0, 0, PM_REMOVE))
    {
        if (!pThread->PreTranslateMessage(&msg))
        {
            ::TranslateMessage(&msg);
            ::DispatchMessage(&msg);
        }

        pThread->OnIdle(0);   // updates user interface
        pThread->OnIdle(1);   // frees temporary objects

        // * background processing may be performed here *
    }

}

//------------------------------------------------------------------------------

BOOL CExtendedFakeButton::DoFakeButton()
{
    // Create the window and capture the mouse.

    VERIFY(CreateEx(0, _T("static"), _T(""), 0, m_rc.left, m_rc.top,
                    m_rc.right, m_rc.bottom, 0, 0));
    SetCapture();

    // Set status variables and draw the appropriate bitmap.

    CPoint pt;
    GetCursorPos(&pt);
    Draw(m_bDown = m_rc.PtInRect(pt));
    m_bDone = FALSE;

    // While the internal done flag is not set, process messages
    // as they arrive.

    while (!m_bDone)
        DoEvents();

    // Destroy the window and return the down member set by the
    // button up processing.

    DestroyWindow();
    return m_bDown;
}

//------------------------------------------------------------------------------

void CExtendedFakeButton::OnMouseMove(UINT nFlags, CPoint point)
{
    nFlags; // UNUSED

    // If the mouse movement has changedd the down status,
    // update the member and redraw.

    ClientToScreen(&point);
    BOOL bNewState = m_rc.PtInRect(point);
    if (bNewState != m_bDown)
        Draw(m_bDown = bNewState);
}

//------------------------------------------------------------------------------

void CExtendedFakeButton::OnLButtonUp(UINT nFlags, CPoint point)
{
    nFlags; // UNUSED

    // Release the capture and see if the point where the mouse
    // was released is within the rectangle; if it is, then the
    // button was clicked and the down member will be true.  Set
    // the done flag to exit the fake button handling.

    VERIFY(ReleaseCapture());
    ClientToScreen(&point);
    m_bDown = m_rc.PtInRect(point);
    Draw(FALSE);
    m_bDone = TRUE;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// CCExtendedListBoxEdit implementation
//------------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CCExtendedListBoxEdit, CEdit)
//{{AFX_MSG_MAP(CCExtendedListBoxEdit)
ON_WM_GETDLGCODE()
ON_WM_KEYDOWN()
ON_WM_KEYUP()
ON_WM_KILLFOCUS()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//------------------------------------------------------------------------------

CCExtendedListBoxEdit::CCExtendedListBoxEdit (CListBox* pLB, UINT uiID, int iIndex, BOOL bDelOnCancel,
        UINT uiBrowseID, LPCTSTR p /*=NULL*/,
        BOOL bQuick /*=FALSE*/, int nSpaces /*=0*/) :

        m_bDelOnCancel(bDelOnCancel),
        m_iIndex(iIndex),
        m_uiBrowseID(uiBrowseID),
        mbQuick(bQuick),
        mnSpaces(nSpaces)
{
    CRect rc;
    CString str;
    m_iOldSel = pLB->GetCurSel();
    if (pLB->GetStyle() & LBS_MULTIPLESEL)
    {
        pLB->SetSel(iIndex);
    }
    else
    {
        pLB->SetCurSel(iIndex);
    }
    pLB->GetItemRect(iIndex, &rc);
    if (p == NULL)
        pLB->GetText(m_iIndex, str);
    else
        str = p;
    rc.top -= 2;
    rc.bottom += 2;
    Create(WS_CHILD | WS_BORDER | ES_AUTOHSCROLL, rc, pLB, uiID);
    SetWindowText(str);
    if (p == NULL)
        SetSel(0, -1);
    else
        SetSel(str.GetLength(), str.GetLength());
    SetFont(pLB->GetFont());

    ShowWindow(SW_SHOW);
    SetFocus();
}

//------------------------------------------------------------------------------

UINT CCExtendedListBoxEdit::OnGetDlgCode()
{
    return DLGC_WANTALLKEYS;
}

//------------------------------------------------------------------------------

void CCExtendedListBoxEdit::OnKillFocus (CWnd* pNewWnd)
{
    CEdit::OnKillFocus (pNewWnd);

    // The edit box should be hidden by now.
    // If not, then the focus changed by the user clicking the mouse on a different
    // control/window or pressing alt-tab..  So we need to hide the edit box.
    if (IsWindowVisible() && pNewWnd &&
            (UINT(pNewWnd->GetDlgCtrlID()) != m_uiBrowseID) )
        CCExtendedListBoxEdit::OnKeyDown (VK_RETURN, 1, 0); // Close/save the edit
}

//------------------------------------------------------------------------------

// Original CCExtendedListBoxEdit::OnKeyDown implementation

void CCExtendedListBoxEdit::OnKeyDownBase(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    BOOL bDone = FALSE;
    CListBox* pLB = (CListBox*)GetParent();

    pLB->SendMessage(WM_SECLSTBXEDIT_ESCAPE,0,0);   // clear
    if (nChar == VK_RETURN)
    {
        CString str;
        DWORD dwItemData = pLB->GetItemData(m_iIndex);
        GetWindowText(str);
        ReplaceListBoxItem(pLB, m_iIndex, str, dwItemData);
        if (pLB->GetStyle() & LBS_MULTIPLESEL)
        {
            pLB->SetSel(m_iIndex);
        }
        else
        {
            pLB->SetCurSel(m_iIndex);
        }
        bDone = TRUE;
    }
    else if (nChar == VK_ESCAPE)
    {
        if (m_bDelOnCancel)
        {
            if (pLB->GetStyle() & LBS_MULTIPLESEL)
            {
                pLB->SetSel(m_iOldSel);
            }
            else
            {
                pLB->SetCurSel(m_iOldSel);
            }
            pLB->DeleteString(m_iIndex);
        }

        // signal the escape state
        pLB->SendMessage(WM_SECLSTBXEDIT_ESCAPE,1,0);   // set
        bDone = TRUE;
    }
    else if (nChar == VK_F2 && m_uiBrowseID != 0)
    {
        CWnd* p = GetParent();
        CWnd* pCtrl = p->GetDlgItem(m_uiBrowseID);

        //We must set focus to the button so that CCExtendedListBoxEdit::OnKillFocus won't ask us to
        // destroy ourselves.  This makes the button have focus on return...but its better than
        // crashing...
        pCtrl->SetFocus();
        p->PostMessage(WM_COMMAND, MAKEWPARAM(m_uiBrowseID, BN_CLICKED),
                       (LPARAM)pCtrl->GetSafeHwnd());
    }
    else
    {
        CEdit::OnKeyDown(nChar, nRepCnt, nFlags);
    }

    if (bDone)
    {
        NMHDR nm;
        nm.hwndFrom = GetSafeHwnd();
        nm.idFrom = GetDlgCtrlID();
        nm.code = NM_LBE_MY_DONE;
        pLB->SendMessage(WM_NOTIFY, nm.idFrom, (LPARAM)&nm);
    }
}

//------------------------------------------------------------------------------

void CCExtendedListBoxEdit::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    if ((nChar == VK_UP) || (nChar == VK_DOWN) ||
            (nChar == VK_PRIOR) || (nChar == VK_NEXT) ||
            (nChar == VK_HOME) || (nChar == VK_END))
    {
        CEdit::OnKeyDown(nChar, 1, nFlags); // Close/save the edit
        return;
    }

    CString strText ;
    int nLength ;
    //---
    GetWindowText( strText ) ;
    nLength = strText.GetLength() ;

    SetSel( nLength - 1 , nLength ) ;
    Clear() ;
    SetRedraw( TRUE ) ;
    Invalidate() ;
}

//------------------------------------------------------------------------------

void CCExtendedListBoxEdit::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    SetRedraw( FALSE ) ;

    if ((nChar == VK_UP) || (nChar == VK_DOWN) ||
            (nChar == VK_PRIOR) || (nChar == VK_NEXT) ||
            (nChar == VK_HOME) || (nChar == VK_END))
    {
        CWnd* pWnd = GetParent();
        CCExtendedListBoxEdit::OnKeyDownBase(VK_RETURN, 1, nFlags); // Close/save the edit
        if (pWnd != NULL)
            pWnd->PostMessage (WM_KEYDOWN, nChar, GetCurrentMessage()->lParam);
        return;
    }
    else
        CCExtendedListBoxEditor* pParent = (CCExtendedListBoxEditor*)GetParent();
    // Check if focus should change
    BOOL    bChangeFocus = FALSE;
    if (nChar == VK_TAB)
    {
        bChangeFocus = TRUE;
        if (GetKeyState(VK_SHIFT) < 0)
            nChar = VK_BACK;
    }
    else
    {
        DWORD   dwSel = GetSel();

        // Send the character to the edit box
        CCExtendedListBoxEdit::OnKeyDownBase(nChar, nRepCnt , nFlags);

        // If not quick mode then use default handler
        if (!mbQuick)
            return;

        // At this point, if the char is <Space> then we need to see if this caused at lease
        // mnSpaces at the end of the string.  If so, then tab
        if ((nChar == VK_SPACE) && (LOWORD(dwSel) == HIWORD(dwSel)))
        {
            CString sString;
            GetWindowText (sString);
            sString.TrimRight();
            bChangeFocus = ((LOWORD(dwSel) - sString.GetLength() + 1) >= mnSpaces);
        }
    }

    // move the focus to the next dialog item
    if (bChangeFocus)
    {
        CWnd* pWnd = GetParent();
        CCExtendedListBoxEdit::OnKeyDownBase(VK_RETURN, 1, nFlags); // Close/save the edit
        if (pWnd != NULL)
            pWnd = pWnd->GetParent()->GetNextDlgTabItem (pWnd, nChar == VK_BACK);
        if (pWnd != NULL)
        {
            pWnd->SetFocus();
            if (pWnd->IsKindOf (RUNTIME_CLASS (CEdit)))
                ((CEdit*)pWnd)->SetSel (0, -1, FALSE);
        }
    }
}

////////////////////////////////////////////////////////////////////////////////
// CCExtendedListBoxEditorHelper implementation

//------------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CCExtendedListBoxEditorHelper, CListBox)
//{{AFX_MSG_MAP(CCExtendedListBoxEditorHelper)
ON_COMMAND(CCExtendedListBoxEditor::IDC_BUTTON, OnBrowse)
ON_WM_CHAR()
ON_WM_KEYDOWN()
ON_WM_LBUTTONDBLCLK()
ON_WM_LBUTTONDOWN()
ON_WM_SYSKEYDOWN()
//}}AFX_MSG_MAP
ON_MESSAGE(WM_SECLSTBXEDIT_ESCAPE,OnEscapeState)
END_MESSAGE_MAP()

//------------------------------------------------------------------------------

CCExtendedListBoxEditorHelper::CCExtendedListBoxEditorHelper(CCExtendedListBoxEditor& lbe, CListBox* p):m_LBE(lbe)
{
    VERIFY(CWnd::SubclassWindow(p->GetSafeHwnd()));
}


//------------------------------------------------------------------------------

void CCExtendedListBoxEditorHelper::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    nFlags; // UNUSED
    nRepCnt; // UNUSED

    if (m_LBE.IsFlagSet(LBE_MY_AUTOEDIT) && !IsKeyDown(VK_CONTROL))
    {
        m_LBE.StartEditing((TCHAR)nChar);
        return;
    }

    Default();
}

//------------------------------------------------------------------------------

void CCExtendedListBoxEditorHelper::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    nFlags; // UNUSED
    nRepCnt; // UNUSED

    if ((nChar == VK_INSERT) && m_LBE.IsFlagSet(LBE_MY_SHORTCUTS))
        m_LBE.DoAdd();
    else if ((nChar == VK_DELETE) && m_LBE.IsFlagSet(LBE_MY_SHORTCUTS))
        m_LBE.DoDelete();
    else if (nChar == VK_F2 && m_LBE.IsFlagSet(LBE_MY_BROWSE))
        m_LBE.StartEditing();
    else
        Default();
}

//------------------------------------------------------------------------------

void CCExtendedListBoxEditorHelper::OnLButtonDblClk(UINT nFlags, CPoint point)
{
    nFlags; // UNUSED
    point; // UNUSED

    m_LBE.StartEditing();
}

//------------------------------------------------------------------------------

void CCExtendedListBoxEditorHelper::OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    nFlags; // UNUSED
    nRepCnt; // UNUSED

    if ((nChar == VK_UP) && m_LBE.IsFlagSet(LBE_MY_SHORTCUTS))
        m_LBE.DoMoveUp();
    else if ((nChar == VK_DOWN) && m_LBE.IsFlagSet(LBE_MY_SHORTCUTS))
        m_LBE.DoMoveDown();
    else
        Default();
}

//------------------------------------------------------------------------------

LRESULT CCExtendedListBoxEditorHelper::OnEscapeState(WPARAM wParam, LPARAM lParam)
{
    lParam; // unreferenced

    m_LBE.SetEscape((BOOL)wParam);
    return (LRESULT)0;
}


//------------------------------------------------------------------------------

void CCExtendedListBoxEditorHelper::OnLButtonDown(UINT uiFlags, CPoint point)
{
    uiFlags; // UNUSED
    point; // UNUSED

    // If we're editing, check to see if the user has entered something.
    // If so, keep it; otherwise discard it.  Note: we cancel editing by
    // posting a message; any other means seems to screw up the focus.
    int nOldSel=LB_ERR;

    if (m_LBE.IsEditing())
    {
        WPARAM wKey;
        if (m_LBE.m_pEdit->GetWindowTextLength() > 0)
            wKey = VK_RETURN;
        else
            wKey = VK_ESCAPE;
        m_LBE.m_pEdit->SendMessage(WM_KEYDOWN, wKey);
    }
    else nOldSel=GetCurSel();

    Default();

    if (m_LBE.IsFlagSet(LBE_MY_DRAGNDROP))
    {
        CPoint ptFinal;
        CExtendedListBoxDragWnd lbdw(this);
        lbdw.SetTrailingBlank(!m_LBE.IsFlagSet(LBE_MY_NOTRAILBLANK));
        int nOldIndex=-1;
        int nNewIndex=-1;

        if(lbdw.PerformDragNDrop(nOldIndex,nNewIndex))
        {
            // provide feedback on the success of this operation
            ASSERT(nOldIndex!=-1);
            ASSERT(nNewIndex!=-1);
            CString strItemText;
            CListBox* pLB=m_LBE.GetListBoxPtr();
            pLB->GetText(nNewIndex,strItemText);
            m_LBE.OnItemMoved(nOldIndex,nNewIndex,strItemText,TRUE);
        }
    }

    // Post a LBN_SELCHANGE?
    if(nOldSel!=LB_ERR)
    {
        int nNewSel=GetCurSel();
        if((nNewSel!=LB_ERR)&&(nNewSel!=nOldSel))
            GetParent()->PostMessage(WM_COMMAND,(WPARAM) MAKELONG (GetDlgCtrlID(), LBN_SELCHANGE), (LPARAM) m_hWnd);
    }
}

//------------------------------------------------------------------------------

BOOL CCExtendedListBoxEditorHelper::OnNotify(WPARAM wParam, LPARAM lParam,
        LRESULT* pResult)
{
    pResult; // UNUSED
    wParam; // UNUSED

    NMHDR* p = (NMHDR*)lParam;
    if (p->idFrom == CCExtendedListBoxEditor::IDC_EDIT &&
            p->code == NM_LBE_MY_DONE)
    {
        m_LBE.StopEditing();
        return TRUE;
    }

    return Default();
}

/////////////////////////////////////////////////////////////////////////////
// CCExtendedListBoxEditor

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Constructs an CCExtendedListBoxEditor object.
//@xref <c SECListBoxDirEditor>
CCExtendedListBoxEditor::CCExtendedListBoxEditor() :
        m_pEdit(NULL),
        m_pHelper(NULL),
        m_iFlags(0),
        mbQuick(FALSE),
        mnSpaces(1)
{
    m_bAddInProgress=FALSE;
    m_bEscape=FALSE;
    m_bInitialized=FALSE;
    m_calcdHeight=0;

    // By default, all buttons will be displayed.
    for(int nIndex=0;nIndex<NUM_BUTTONS;++nIndex)
        m_bUseButton[nIndex]=TRUE;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc CCExtendedListBoxEditor destructor.
//@xref <c SECListBoxDirEditor>
CCExtendedListBoxEditor::~CCExtendedListBoxEditor()
{
    delete m_pHelper;
    m_pHelper = NULL;

    if(m_pEdit)
    {
        m_pEdit->DestroyWindow();
        delete m_pEdit;
        m_pEdit = NULL;
    }

}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Returns a pointer to the listbox control bound to this control.
//@rdesc Pointer to the CListBox control.
//@xref <c SECListBoxDirEditor>
CListBox* CCExtendedListBoxEditor::GetListBoxPtr() const
{
    return m_pHelper;
}

//------------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CCExtendedListBoxEditor, CWnd)
//{{AFX_MSG_MAP(CCExtendedListBoxEditor)
ON_WM_PAINT()
ON_WM_LBUTTONDOWN()
ON_WM_NCHITTEST()
ON_WM_SIZE()
ON_WM_MOVE()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Calculates the height of the control.
//@rdesc The height in pixels.
//@xref <c SECListBoxDirEditor>
int CCExtendedListBoxEditor::CalculateHeight()
{
    CClientDC DC(this);
    CFont* pfontOld = DC.SelectObject(GetFont());
    TEXTMETRIC tm;
    DC.GetTextMetrics(&tm);
    DC.SelectObject(pfontOld);
    int cx = tm.tmHeight + 4 * GetSystemMetrics(SM_CYBORDER);

    for (int i = 0; i < NUM_BUTTONS; i++)
    {
        BITMAP bmpInfo;
        CBitmap bmp;
        if(!bmp.LoadBitmap(IDB_ELB_ADD + i))
            // If this ASSERT fires, make sure you have included the
            // Objective Toolkit resources (secres.rc,secres.h) in your
            // project. Consult the User's Guide for the proper method.
            ASSERT(FALSE);

        bmp.GetObject(sizeof(bmpInfo), &bmpInfo);
        bmpInfo.bmHeight += 2;
        if (bmpInfo.bmHeight > cx)
            cx = bmpInfo.bmHeight;
    }

    return cx;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Moves an item up in the listbox.
//@rdesc Nonzero if successful, otherwise 0.
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::DoMoveUp()
{
    CListBox* pLB = GetListBoxPtr();
    ASSERT(pLB);

    int iSel = pLB->GetCurSel();

    // if the control supports muliple selection,
    // let's do some checking to make sure that
    // there is a valid selection to move...
    if (pLB->GetStyle() & LBS_MULTIPLESEL)
    {
        int nSels = pLB->GetSelCount();

        // moving an multiple selections does
        // not make sense...
        if (nSels != 1)
            // no items to move!
            return FALSE;

        // GetCurSel returns the last item clicked on.
        // For list boxes that support multiple selection,
        // this isn't necessarily the currenly selected item.
        int iSels[1];
        nSels = pLB->GetSelItems(1, iSels);
        if (LB_ERR == nSels)
            return FALSE;
        iSel = iSels[0];

    }

    int iLast;
    if (AreFlagsSet(LBE_MY_NOTRAILBLANK))
    {
        // all items can be included in count
        iLast = pLB->GetCount();
    }
    else
    {
        // last item is blank and can be ignored
        iLast = pLB->GetCount() - 1;
    }

    if ((iSel < iLast)&&(iSel>0))
    {
        pLB->SetRedraw(FALSE);
        MoveUpCore(iSel);

        if (pLB->GetStyle() & LBS_MULTIPLESEL)
        {
            pLB->SetSel(iSel - 1);
        }
        else
        {
            pLB->SetCurSel(iSel - 1);
        }
        pLB->SetRedraw(TRUE);

        // virtual feedback call
        CString strTextMoved;
        pLB->GetText(iSel-1,strTextMoved);
        this->OnItemMoved(iSel,iSel-1,strTextMoved,FALSE);
        return TRUE;
    }

    return FALSE;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Moves an item down in the listbox.
//@rdesc Nonzero if successful, otherwise 0.
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::DoMoveDown()
{
    CListBox* pLB = GetListBoxPtr();
    ASSERT(pLB);

    int iSel = pLB->GetCurSel();

    // if the control supports muliple selection,
    // let's do some checking to make sure that
    // there is a valid selection to move...
    if (pLB->GetStyle() & LBS_MULTIPLESEL)
    {
        int nSels = pLB->GetSelCount();

        // moving an multiple selections does
        // not make sense...
        if (nSels != 1)
            // no items to move!
            return FALSE;

        // GetCurSel returns the last item clicked on.
        // For list boxes that support multiple selection,
        // this isn't necessarily the currenly selected item.
        int iSels[1];
        nSels = pLB->GetSelItems(1, iSels);
        if (LB_ERR == nSels)
            return FALSE;
        iSel = iSels[0];

    }

    int iLast;
    if (AreFlagsSet(LBE_MY_NOTRAILBLANK))
    {
        // all items can be included in count
        iLast = pLB->GetCount();
    }
    else
    {
        // last item is blank and can be ignored
        iLast = pLB->GetCount() - 1;
    }

    if (-1 < iSel && iSel < (iLast-1))
    {
        pLB->SetRedraw(FALSE);
        MoveUpCore(iSel + 1);
        if (pLB->GetStyle() & LBS_MULTIPLESEL)
        {
            pLB->SetSel(iSel + 1);
        }
        else
        {
            pLB->SetCurSel(iSel + 1);
        }
        pLB->SetRedraw(TRUE);

        // virtual feedback call
        CString strTextMoved;
        pLB->GetText(iSel+1,strTextMoved);
        this->OnItemMoved(iSel,iSel+1,strTextMoved,FALSE);
        return TRUE;
    }

    return FALSE;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Moves an item in the list box up one position.
//@rdesc void
//@parm int | iIndex | Index of item to move.
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::MoveUpCore(int iIndex)
{
    CString str;
    CListBox* pLB = GetListBoxPtr();
    pLB->GetText(iIndex, str);
    DWORD dwData = pLB->GetItemData(iIndex);
    pLB->DeleteString(iIndex);
    int iNewIndex = pLB->InsertString(--iIndex, str);
    pLB->SetItemData(iNewIndex, dwData);
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Deletes the currently selected item from the listbox.
//@comm For list boxes with the LBS_MULTIPLESEL style, this method
// will delete all selected items.  One difference in behavior for list
// boxes with the LBS_MULTIPLESEL flag if the delete operation
// completes and returns TRUE, there will be no selected items in the list
// box.  This differs from listboxes without the LBS_MULTIPLESEL flag, where
// the same index position will remain selected, if an item exists at that
// index after the deletion.
//@rdesc Nonzero if successful, otherwise 0.  For list boxes without the
// LBE_MY_NOTRAILBLANK flag set, this method will return 0 if the last (blank)
// item is selected when this method is called (it cannot be deleted).
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::DoDelete()
{
    CListBox* pLB = GetListBoxPtr();
    ASSERT(pLB);

    // we much handle multiple selection list boxes specially...
    if (pLB->GetStyle() & LBS_MULTIPLESEL)
    {
        // have to ask for the selections one at a time. DeleteItem
        // method requires an index, and getting a list of all
        // selected items from the listbox won't work because
        // the indices are going to change as each item gets deleted...
        BOOL rc = TRUE;
        while (pLB->GetSelCount())
        {
            int iSels[1];
            if (pLB->GetSelItems(1, iSels) != LB_ERR)
                rc = DeleteItem(iSels[0]);
            else
                rc = FALSE;

            if (FALSE == rc)
                break;
        }

        return rc;
    }
    else
        return DeleteItem(GetListBoxPtr()->GetCurSel());
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Deletes an item in the listbox.
//@rdesc Nonzero if successful, otherwise 0.
//@parm int | nItem | Index of item to delete.
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::DeleteItem(int nItem)
{
    CListBox* pLB = GetListBoxPtr();
    int nNum = pLB->GetCount();
    if ((nItem < 0) || (nItem >= nNum))
        return FALSE;

    CString strText;
    pLB->GetText (nItem, strText);

    // Don't allow deleting last blank item
    if ((nItem == nNum-1) && strText.IsEmpty())
        return FALSE;

    // virtual hook to bail out of delete
    if (!this->OnItemDelete (nItem, strText))
        return FALSE;

    pLB->SetRedraw (FALSE);
    pLB->DeleteString (nItem);
    if (nItem < nNum)
    {
        if (pLB->GetStyle() & LBS_MULTIPLESEL)
        {
            //pLB->SetSel(nItem);
        }

        else
        {
            pLB->SetCurSel (nItem);
        }
    }
    pLB->SetRedraw (TRUE);

    // Notify parent of item deleted
    mnChangedLine = nItem;
    CWnd* pWnd = GetParent();
    if (pWnd != NULL)
        pWnd->SendMessage (WM_COMMAND, (WPARAM) MAKELONG (pLB->GetDlgCtrlID(), LBN_LINEDELETED), (LPARAM) pLB->m_hWnd);
    return TRUE;
}

//------------------------------------------------------------------------------

// Overridable feedback functions
//@doc CCExtendedListBoxEditor
//@mfunc Called after an item has been renamed.
//@rdesc void
//@parm int | nIndexRenamed | Index of item to be renamed.
//@parm CString | strOldName | Old item text.
//@parm CString | strNewName | New item text.
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::OnItemRenamed(int nIndexRenamed,CString strOldName,CString strNewName)
{
    // suppress annoying compiler warning:
    strOldName;

    strNewName.TrimRight();
    if (strNewName.IsEmpty())
        DeleteItem(nIndexRenamed);
    else
    {
        CListBox* pLB = GetListBoxPtr();
        if (nIndexRenamed == pLB->GetCount()-1)
            // if this next statement is true, there is supposed to be a
            // "blank" line at the bottom of the list for the user to edit
            if (!IsFlagSet(LBE_MY_NOTRAILBLANK))
                pLB->InsertString (-1, _T(""));
    }
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Called after an item has been added.
//@rdesc void
//@parm int | nIndexNewItem | Index of newly added item.
//@parm CString | strTextInserted |  New item text.
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::OnItemAdded(int nIndexNewItem,CString strTextInserted)
{
    strTextInserted.TrimRight();
    if (strTextInserted.IsEmpty())
        DeleteItem (nIndexNewItem);
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Called after an item has been moved.
//@rdesc void
//@parm int | nOldIndex | Index item was at previously.
//@parm int | nNewIndex | New index for moved item.
//@parm CString | strTextMoved | Text of moved item.
//@parm BOOL | WasDragged |  TRUE if the item was moved by dragging.
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::OnItemMoved(int nOldIndex,int nNewIndex,CString strTextMoved,BOOL WasDragged)
{
    strTextMoved;
    WasDragged;

    mnChangedLine = nOldIndex;
    mnNewLine = nNewIndex;
    CListBox* pLB = GetListBoxPtr();
    CWnd* pWnd = GetParent();
    if (pWnd != NULL)
        pWnd->SendMessage (WM_COMMAND, (WPARAM) MAKELONG (pLB->GetDlgCtrlID(), LBN_LINEMOVED), (LPARAM) pLB->m_hWnd);

}

//------------------------------------------------------------------------------

//
//@doc CCExtendedListBoxEditor
//@mfunc Called before a delete operation has occurred.
//@comm Derived classes
// should return FALSE to prevent items from being deleted.
//@rdesc Nonzero if deletion is to proceed, 0 to halt delete operation.
//@parm int | nIndexToBeDeleted | Index of item to be deleted.
//@parm CString | strTextDeleted | Text of item to be deleted.
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::OnItemDelete(int nIndexToBeDeleted,CString strTextDeleted)
{
    // suppress annoying compiler warning:
    nIndexToBeDeleted;
    strTextDeleted;
    return TRUE;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Returns the index of an item before it changed.
//@rdesc Index of item.
//@xref <c SECListBoxDirEditor>
int CCExtendedListBoxEditor::GetChangedLine() const
{
    return mnChangedLine;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Returns the index of an item after being moved.
//@rdesc Index of item.
//@xref <c SECListBoxDirEditor>
int CCExtendedListBoxEditor::GetNewLine()const
{
    return mnNewLine;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Adds an item to the listbox.
//@rdesc Nonzero if successful, otherwise 0.
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::DoAdd()
{
    CListBox* pLB = GetListBoxPtr();

    // First make sure the last item is blank
    if (pLB->GetCount() <= 0)
        pLB->InsertString (-1, _T(""));
    else
    {
        CString sText;
        pLB->GetText (pLB->GetCount()-1, sText);
        if (!sText.IsEmpty())
            pLB->InsertString (-1, _T(""));
    }

    // Add a new blank item to the list and edit it the previous blank item
    int iIndex = 0;

    // if this next statement is true, there is supposed to be a
    // "blank" line at the bottom of the list for the user to edit
    if (!IsFlagSet(LBE_MY_NOTRAILBLANK))
        // InsertString returns a zero-based index of last string
        iIndex = pLB->InsertString (-1, _T(""));
    else
        // GetCount returns nonzero-based count
        iIndex = pLB->GetCount();

    EditCore(iIndex-1, TRUE);
    m_bAddInProgress = TRUE;
    return TRUE;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Called when editing has started.
//@rdesc void
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::OnEditingJustStarted()
{
    CListBox* pLB = GetListBoxPtr();
    CWnd* pWnd = GetParent();
    if (pWnd != NULL)
        pWnd->PostMessage (WM_COMMAND, (WPARAM) MAKELONG (pLB->GetDlgCtrlID(), LBN_SELCHANGE), (LPARAM) pLB->m_hWnd);
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Called after editing has stopped.
//@rdesc void
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::OnEditingStopped()
{
    CListBox* pLB = GetListBoxPtr();

    // Make sure the current selection has not been cleared
    if (pLB->GetCurSel() < 0)
    {
        if (pLB->GetStyle() & LBS_MULTIPLESEL)
        {
            pLB->SetSel (pLB->GetCount()-1);
        }
        else
        {
            pLB->SetCurSel (pLB->GetCount()-1);
        }
    }
    // Notify parent of selection changing
    CWnd* pWnd = GetParent();
    if (pWnd != NULL)
        pWnd->PostMessage (WM_COMMAND, (WPARAM) MAKELONG (pLB->GetDlgCtrlID(), LBN_SELCHANGE), (LPARAM) pLB->m_hWnd);
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Creates the edit control to initiate editing.
//@rdesc Nonzero if successful, otherwise 0.
//@parm int | iIndex |  Index of item to be edited.
//@parm  BOOL | bDelOnCancel | If true, item is deleted when
// a cancel occurs.
//@parm  LPCTSTR | p | Initial text to appear in edit control.
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::EditCore(int iIndex, BOOL bDelOnCancel, LPCTSTR p)
{
    // fire virtual callback
    OnEditingJustStarted();

    BOOL bBrowse = IsFlagSet(LBE_MY_BROWSE);
    CListBox* pLB = GetListBoxPtr();
    m_pEdit = new CCExtendedListBoxEdit (pLB, IDC_EDIT, iIndex, bDelOnCancel,
                                         bBrowse ? IDC_BUTTON : 0, p,
                                         mbQuick, mnSpaces);
    if (bBrowse)
    {
        //m_pEdit->SetReadOnly( TRUE ) ;
        OnPaint() ;
        CSize sizeBtn;
        CRect rcBtn;
        CString strText(_T("..."));
        //---
        { // BLOCK DC
            CWindowDC DC(this);
            CFont* pfontOld = DC.SelectObject(m_pEdit->GetFont());
            sizeBtn = DC.GetTextExtent(strText);
            CSize sizeSpace = DC.GetTextExtent(_T(" "), 1);
            sizeBtn.cx += 2 * sizeSpace.cx + 2 * GetSystemMetrics(SM_CXBORDER);
            DC.SelectObject(pfontOld);
            m_pEdit->GetWindowRect(&rcBtn);
        } // FREE DC



        int iDivider = rcBtn.Width() - sizeBtn.cx;
        m_pEdit->SetWindowPos(NULL, 0, 0, iDivider, rcBtn.Height(),
                              SWP_NOZORDER | SWP_NOMOVE);
        pLB->ScreenToClient(&rcBtn);
        rcBtn.left = iDivider;
        DWORD dwStyle = WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON;
        VERIFY(m_btnBrowse.Create(strText, dwStyle, rcBtn, pLB, IDC_BUTTON));

    }

    return TRUE;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Begins editing mode.
//@rdesc Nonzero if successful, otherwise 0.
//@parm TCHAR | c | Initial character typed by user.
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::StartEditing(TCHAR c)
{
    CListBox* pLB=GetListBoxPtr();
    if ((c == VK_SPACE) || (c == VK_BACK))
    {
        if (mbQuick)
        {
            CWnd* pWnd = pLB->GetParent()->GetNextDlgTabItem (pLB, c == VK_BACK);
            if (pWnd != NULL)
            {
                pWnd->SetFocus();
                if (pWnd->IsKindOf (RUNTIME_CLASS (CEdit)))
                    ((CEdit*)pWnd)->SetSel (0, -1, FALSE);
            }
            return FALSE;
        }
        c = NULL;
    }

    int iSel = pLB->GetCurSel();
    if (iSel > -1)
    {
        pLB->GetText(iSel,m_strRenameOldName);
        TCHAR szTmp[2] = { c, '\0' };
        return EditCore(iSel, FALSE, c != '\0' ? szTmp : NULL);
    }
    return FALSE;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Forces an end to active in-place editing.
//@rdesc void
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::StopEditing()
{
    if (IsFlagSet(LBE_MY_BROWSE))
        m_btnBrowse.DestroyWindow();
    if(!m_pEdit) return;
    m_pEdit->DestroyWindow();
    delete m_pEdit;
    m_pEdit = NULL;

    // pump our virtual callbacks
    CString strNewStr;
    CListBox* pLB=GetListBoxPtr();
    int iSel = pLB->GetCurSel();

    // If we were editing the trailing "blank" entry, make sure the
    // appropriate "Add" callbacks are called.
    BOOL bAddBlank=FALSE;
    BOOL bLastEntry=FALSE;
    if(iSel==pLB->GetCount()-1)
    {
        // if this next statement is true, there is supposed to be a
        // "blank" line at the bottom of the list for the user to edit
        bAddBlank = !IsFlagSet(LBE_MY_NOTRAILBLANK);
        bLastEntry=TRUE;
    }

    // Pump our callbacks only if operation successfully completed
    BOOL bDidEscape=GetEscape();
    if((iSel!=LB_ERR)&&!bDidEscape)
    {
        pLB->GetText(iSel,strNewStr);
        if((bLastEntry)&&(m_strRenameOldName!=strNewStr)&&
                !m_strRenameOldName.IsEmpty())
            // Last index was renamed, don't add new blank
            OnItemRenamed(iSel,m_strRenameOldName,strNewStr);
        else if((m_bAddInProgress)||(bAddBlank))
        {
            OnItemAdded(iSel,strNewStr);

            // trailing blank now valid. Create new trailing blank
            // (only if new item was not a blank, avoid duplicates!)
            CString strTest=strNewStr;
            strTest.TrimLeft();
            if((bAddBlank)&&(!strTest.IsEmpty()))
                pLB->InsertString (-1, _T(""));
        }
        else
        {
            // Don't bother if string didn't change (or user cancelled)
            if(m_strRenameOldName!=strNewStr)
                OnItemRenamed(iSel,m_strRenameOldName,strNewStr);
        }
    }
    m_strRenameOldName=(CHAR)NULL;
    m_bAddInProgress=FALSE;

    // fire virtual callback
    OnEditingStopped();
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Sets quick edit mode.
//@comm In this mode, the user can simulate
// typing a tab key by using the space key.
//@rdesc void CCExtendedListBoxEditor::SetQuick
//@parmopt BOOL | bQuick | TRUE | If TRUE, enables quick edit mode.
//@xref <c SECListBoxDirEditor> <mf CCExtendedListBoxEditor::GetQuick>
//@end
void CCExtendedListBoxEditor::SetQuick (BOOL bQuick /*=TRUE*/)
{
    mbQuick = bQuick;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Gets the quick edit mode state.
//@rdesc Nonzero if quick edit mode is enabled, otherwise 0.
//@xref <c SECListBoxDirEditor> <mf CCExtendedListBoxEditor::SetQuick>
BOOL CCExtendedListBoxEditor::GetQuick() const
{
    return mbQuick;
};

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Sets the number of spaces the user must press to
// simulate pressing a tab key.
//@comm Used with quick edit mode.
// See CCExtendedListBoxEditor::<mf CCExtendedListBoxEditor::SetQuick>.
//@rdesc void
//@parmopt int | nSpaces | 1  | Number of spaces to simulate
// a tab key.
//@xref <c SECListBoxDirEditor> <mf CCExtendedListBoxEditor::GetSpacesToTab> <mf CCExtendedListBoxEditor::SetQuick>
//@end
void CCExtendedListBoxEditor::SetSpacesToTab (int nSpaces /*=1*/)
{
    mnSpaces = nSpaces;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Returns the number of spaces that must be pressed
// to simulate a tab key.
//@comm Used with quick edit mode.
// See <mf CCExtendedListBoxEditor::SetQuick>.
//@rdesc Number of spaces for quick edit mode.
//@xref <c SECListBoxDirEditor> <mf CCExtendedListBoxEditor::SetSpacesToTab>
// <mf CCExtendedListBoxEditor::SetQuick>
int CCExtendedListBoxEditor::GetSpacesToTab() const
{
    return mnSpaces;
}

/////////////////////////////////////////////////////////////////////////////
// CCExtendedListBoxEditor message handlers

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Initializes the CCExtendedListBoxEditCCExtendedListBoxEditor object by attaching to the list
// box control identified by uiID.
//@comm This is done by subclassing the list
// box control and attaching it to the instance of CCExtendedListBoxEditor for
// which Initialize was called.
//
// The style flags, iFlags, indicate what editing features the list box
// will have.  To use these styles, logically OR the ones you require and
// pass the result via the iFlags parameter.  Available styles include:
//@flag LBE_MY_BROWSE   | Editable list box has a browse button for each item.
//@flag LBE_MY_AUTOEDIT | Alphanumeric key starts in-place edit of selected item.
//@flag LBE_MY_BUTTONS  | Dispaly buttons if individual button flags are also set.
//@flag LBE_MY_NEWBUTTON | Display the "New" button if LBE_MY_BUTTONS is also set.
//@flag LBE_MY_DELBUTTON | Display the "Delete" button if LBE_MY_BUTTONS is also set.
//@flag LBE_MY_UPBUTTON | Display the "Up" button if LBE_MY_BUTTONS is also set.
//@flag LBE_MY_DOWNBUTTON | Display the "Down" button if LBE_MY_BUTTONS is also set.
//@flag LBE_MY_NOTRAILBLANK | Do not display the editable "trailing" list item.
//@flag LBE_MY_TOOLTIPS | Display tooltips for the command buttons.
//@flag LBE_MY_SHORTCUTS | Allow keyboard shortcuts for New, Delete, Move, etc.
//@flag LBE_MY_DRAGNDROP | Allow items to be drag-n-drop to and from the list.
//@flag LBE_MY_DEFAULT | The default combination (ORing) of above style flags (except LBE_MY_NOTRAILBLANK).
//
//@rdesc Nonzero if successful, otherwise 0.
//@parm CWnd* | pwndParent | A pointer to the parent of the list box control.
//@parm  UINT | uiID | The ID of the list box control to attach to.
//@parm  int | iFlags | Flags indicating edit features the list box should have.
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::Initialize(CWnd* pwndParent, UINT uiID, int iFlags)
{
    // If we can't link to the listbox, bail out before we get in too deep.
    CListBox* pLB = (CListBox*)pwndParent->GetDlgItem(uiID);
    if(!pLB) return FALSE;

    // Set our flags, get an unused control ID, create our window, and
    // set the font based on the parent.

    m_iFlags = iFlags;
    UINT uiOurID = FindUnusedCtrlID(pwndParent);
    VERIFY(CreateEx(0, _T("static"), _T(""), SS_SUNKEN | WS_CHILD, 0, 0, 0, 0,
                    pwndParent->GetSafeHwnd(), (HMENU)uiOurID));
    SetFont(pwndParent->GetFont());

    // Calculate our rectangle, adjust
    // the listbox rectangle accordingly, convert to parent window
    // coorinates, get the previous window in the z-order, and finally
    // update both windows.

    CRect rcLB;
    pLB->GetWindowRect(&rcLB);
    CRect rcUs(rcLB);
    m_calcdHeight=CalculateHeight();
    rcUs.bottom = rcUs.top + m_calcdHeight;
    rcLB.top = rcUs.bottom + 2 * GetSystemMetrics(SM_CYBORDER);
    pwndParent->ScreenToClient(&rcLB);
    pwndParent->ScreenToClient(&rcUs);
    CWnd* pwndPrev = pLB->GetWindow(GW_HWNDPREV);
    SetWindowPos(pwndPrev,rcUs.left, rcUs.top, rcUs.Width(),
                 rcUs.Height(), 0);
    pLB->SetWindowPos(this, rcLB.left, rcLB.top,
                      rcLB.Width(), rcLB.Height(), 0);

    // Update button visibility boolean member based on flag settings.
    SetButtonVisibility(iFlags);

    // Instantiate the listbox helper object, calculate the button
    // rectangles, and configure the tooltip control as needed.

    m_pHelper = new CCExtendedListBoxEditorHelper(*this, pLB);
    CalculateButtonRectangles();
    if (AreFlagsSet(LBE_MY_BUTTONS | LBE_MY_TOOLTIPS))
    {
        VERIFY(m_ToolTip.Create(this, TTS_ALWAYSTIP));
        for (int i = 0; i < NUM_BUTTONS; i++)
        {
            if(m_bUseButton[i])
            {
                UINT uiID = IDS_ELB_ADD + i;
                VERIFY(m_ToolTip.AddTool(this, uiID, m_rcButtons[i], uiID));
            }
        }
        m_ToolTip.Activate(TRUE);
    }

    // if this next statement is true, there is supposed to be a
    // "blank" line at the bottom of the list for the user to edit,
    // so we need to ensure that it is present.
    if (!IsFlagSet(LBE_MY_NOTRAILBLANK))
    {
        if (pLB->GetCount() <= 0)
        {
            pLB->InsertString (-1, _T(""));
        }
        else
        {
            CString strText;
            pLB->GetText (pLB->GetCount()-1, strText);
            if (!strText.IsEmpty())
                pLB->InsertString (-1, _T(""));
        }
    }

    // Notify parent of selection changing
    CWnd* pWnd = GetParent();
    if (pWnd != NULL)
        pWnd->PostMessage (WM_COMMAND, (WPARAM) MAKELONG (pLB->GetDlgCtrlID(), LBN_SELCHANGE), (LPARAM) pLB->m_hWnd);

    ShowWindow(SW_SHOW);
    m_bInitialized=TRUE;

    return TRUE;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Resets the edit control size and position.
//@rdesc void
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::ResetEditorSizePos()
{

    CWnd* pParent=GetParent();
    ASSERT(pParent);

    // Since the listbox editor is nothing more than the header control
    // wrapped around the listbox, need to modify this window's size such
    // that the header and listbox are sized to fit the whole.
    CRect rectWnd;
    GetWindowRect(rectWnd);
    if(rectWnd.Height()<=m_calcdHeight) return;

    CRect rectHelper=rectWnd;
    rectWnd.bottom=rectWnd.top+m_calcdHeight;
    rectHelper.top=rectWnd.bottom + 2 * GetSystemMetrics(SM_CYBORDER);
    pParent->ScreenToClient(rectHelper);
    pParent->ScreenToClient(rectWnd);
    SetWindowPos(&wndTop,rectWnd.left,rectWnd.top,rectWnd.Width(),rectWnd.Height(),0);
    m_pHelper->SetWindowPos(&wndTop,rectHelper.left,rectHelper.top,rectHelper.Width(),rectHelper.Height(),0);

    // Adjust current rect info, then punt to helper
    CalculateButtonRectangles();
    RecycleTooltips();
    Invalidate();
    m_pHelper->Invalidate();
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Resets tooltips for the buttons.
//@rdesc void
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::RecycleTooltips()
{
    if (AreFlagsSet(LBE_MY_BUTTONS | LBE_MY_TOOLTIPS))
    {
        for (int i = 0; i < NUM_BUTTONS; i++)
        {
            UINT uiID = IDS_ELB_ADD + i;
            m_ToolTip.DelTool(this, uiID);
            m_ToolTip.AddTool(this, uiID, m_rcButtons[i], uiID);
        }
        m_ToolTip.Activate(TRUE);
    }
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Called when Windows or an application makes a request to repaint.
//@rdesc void
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::OnPaint()
{
    CPaintDC DC(this);
    CRect rc;
    GetClientRect(rc);
    rc.bottom=rc.top+m_calcdHeight;

    CString str;
    GetWindowText(str);
    CFont* pfontOld = DC.SelectObject(GetFont());
    CRect rectErase(rc);
    rectErase.bottom-=2;    // do not erase listbox border
    DC.FillSolidRect(&rectErase,GetSysColor(COLOR_BTNFACE));
    DC.SetBkMode(TRANSPARENT);
    rc.left += 2 * GetSystemMetrics(SM_CXBORDER);
    DC.SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
    DC.DrawText(str, rc, DT_VCENTER | DT_SINGLELINE);
    DC.SelectObject(pfontOld);

    if (IsFlagSet(LBE_MY_BUTTONS))
    {
        for (int i = 0; i < NUM_BUTTONS; i++)
        {
            if(m_bUseButton[i])
            {
                CBitmap bmp;
                CRect& r = m_rcButtons[i];
                LoadButtonBitmap(IDB_ELB_ADD + i, bmp);
                VERIFY(DrawBitmap(&DC, &bmp, r.left, r.top));
            }
        }
    }

    // Do not call CWnd::OnPaint() for painting messages
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Called when the user presses the left mouse button.
//@rdesc void
//@parm UINT | nFlags | Indicates whether various virtual keys are down.
//@parm  CPoint | point | Specifies the x- and y-coordinate of the cursor.
//@xref <c SECListBoxDirEditor>

void CCExtendedListBoxEditor::OnLButtonDown(UINT nFlags, CPoint point)
{
    nFlags; // UNUSED
    point; // UNUSED

    if (IsEditing() || !IsFlagSet(LBE_MY_BUTTONS))
        return;

    for (int i = 0; i < NUM_BUTTONS; i++)
    {
        if ((m_rcButtons[i].PtInRect(point))&&(m_bUseButton[i]))
        {
            CRect rc(m_rcButtons[i]);
            ClientToScreen(&rc);
            CBitmap bmpUp, bmpDn;
            LoadButtonBitmap(IDB_ELB_ADD + i, bmpUp);
            CreateDownBitmap(&bmpUp, &bmpDn);
            CExtendedFakeButton btn(bmpDn, bmpUp, rc);

            if (btn.DoFakeButton())
            {
                BOOL (CCExtendedListBoxEditor::*pfn[])() =
                    { DoAdd, DoDelete, DoMoveUp, DoMoveDown };
                (this->*pfn[i])();
                break;
            }
        }
    }

    //CWnd::OnLButtonDown(nFlags, point);
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Called for the CWnd object that contains the cursor (or the CWnd object
// that used the SetCapture member function to capture the mouse input)
// every time the mouse is moved.
//@parm CPoint | point | Contains the x- and y-coordinates of the cursor.
//@rdesc void
//@xref <c SECListBoxDirEditor>

UINT CCExtendedListBoxEditor::OnNcHitTest(CPoint point)
{
    point; // UNUSED
    return HTCLIENT;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Called after the window's size has changed.
//@rdesc void
//@parm UINT | nType | Specifies the type of resizing requested.
//@parm  int  | cx | Specifies the new width of the client area.
//@parm  int | cy | Specifies the new height of the client area.
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::OnSize(UINT nType, int cx, int cy)
{
    CWnd::OnSize(nType,cx,cy);

    if(m_bInitialized)
        ResetEditorSizePos();
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Called after the CWnd object has been moved.
//@rdesc void
//@parm int | x | Specifies the new x-coordinate location of the
// upper-left corner of the client area.
//@parm  int | y | Specifies the new y-coordinate location of the
// upper-left corner of the client area.
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::OnMove(int x, int y)
{
    CWnd::OnMove(x,y);

    if(m_bInitialized)
        ResetEditorSizePos();
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Calculates the button placements.
//@rdesc void
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::CalculateButtonRectangles()
{
    CRect rc;
    GetClientRect(&rc);
    rc.bottom=rc.top+m_calcdHeight;
    for (int i = NUM_BUTTONS - 1; i > -1; i--)
    {
        if(m_bUseButton[i])
        {
            BITMAP bmpInfo;
            CBitmap bmp;
            if(!bmp.LoadBitmap(IDB_ELB_ADD + i))
                // If this ASSERT fires, make sure you have included the
                // Objective Toolkit resources (secres.rc,secres.h) in your
                // project. Consult the User's Guide for the proper method.
                ASSERT(FALSE);

            bmp.GetObject(sizeof(bmpInfo), &bmpInfo);
            rc.left = rc.right - bmpInfo.bmWidth;
            m_rcButtons[i] = rc;
            rc.right = rc.left;
        }
    }
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Changes the visibility status of the buttons.
//@rdesc void
//@parm int iFlags
//@xref <c SECListBoxDirEditor>
void CCExtendedListBoxEditor::SetButtonVisibility(int iFlags)
{
    if(iFlags & LBE_MY_NEWBUTTON)
        m_bUseButton[0]=TRUE;
    else m_bUseButton[0]=FALSE;
    if(iFlags & LBE_MY_DELBUTTON)
        m_bUseButton[1]=TRUE;
    else m_bUseButton[1]=FALSE;
    if(iFlags & LBE_MY_UPBUTTON)
        m_bUseButton[2]=TRUE;
    else m_bUseButton[2]=FALSE;
    if(iFlags & LBE_MY_DOWNBUTTON)
        m_bUseButton[3]=TRUE;
    else m_bUseButton[3]=FALSE;
}

//------------------------------------------------------------------------------

//@doc CCExtendedListBoxEditor
//@mfunc Used to translate window messages before they are dispatched.
//@rdesc Nonzero if the message was translated and should not be dispatched.
//@parm MSG* | pMsg | Pointer to MSG structure.
//@xref <c SECListBoxDirEditor>
BOOL CCExtendedListBoxEditor::PreTranslateMessage(MSG* pMsg)
{
    BOOL bReturn = CWnd::PreTranslateMessage(pMsg);
    if (IsFlagSet(LBE_MY_TOOLTIPS) && (pMsg->message == WM_MOUSEMOVE))
        m_ToolTip.RelayEvent(pMsg);
    return bReturn;
}
