/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CExtendedListBoxEditor.h: interface for the CCExtendedListBoxEditor class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CEXTENDEDLISTBOXEDITOR_H__BD03C9A3_0F81_11D3_AC27_00105A483E3C__INCLUDED_)
#define AFX_CEXTENDEDLISTBOXEDITOR_H__BD03C9A3_0F81_11D3_AC27_00105A483E3C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

const int LBE_MY_BROWSE       = 0x0001; // edit has a browse button?
const int LBE_MY_AUTOEDIT     = 0x0002; // alphanumeric key starts edit?
const int LBE_MY_BUTTONS      = 0x0004;     // supply command buttons?
const int LBE_MY_TOOLTIPS     = 0x0008; // supply tooltips for buttons ?
const int LBE_MY_SHORTCUTS    = 0x0010; // add, delete, and move from keyboard?
const int LBE_MY_DRAGNDROP    = 0x0020; // drag and drop items in list?
const int LBE_MY_NOTRAILBLANK = 0x0040;    // do not add the trailing blank item
const int LBE_MY_NEWBUTTON    = 0x0100; // display "new" pushbutton
const int LBE_MY_DELBUTTON    = 0x0200; // display "delete" pushbutton
const int LBE_MY_UPBUTTON     = 0x0400; // display "Up" pushbutton
const int LBE_MY_DOWNBUTTON   = 0x0800; // display "Down" pushbutton

const int LBE_MY_DEFAULT = LBE_MY_AUTOEDIT | LBE_MY_BUTTONS |
                           LBE_MY_TOOLTIPS | LBE_MY_SHORTCUTS | LBE_MY_DRAGNDROP |
                           LBE_MY_NEWBUTTON | LBE_MY_DELBUTTON |
                           LBE_MY_UPBUTTON | LBE_MY_DOWNBUTTON;

const UINT NM_LBE_MY_DONE = 1000;

#define LBN_LINEDELETED     10
#define LBN_LINEMOVED       11
#define WM_SECLSTBXEDIT_ESCAPE  WM_USER+0x0100  // VK_ESCAPE notification

class CCExtendedListBoxEdit ;
class CCExtendedListBoxEditorHelper ;

class CCExtendedListBoxEditor : public CWnd
{
public:

    //@access Creation/Initialization

    //@cmember
    /* Constructs an CCExtendedListBoxEditor object.*/
    CCExtendedListBoxEditor();

    // CCExtendedListBoxEditor destructor.
    virtual ~CCExtendedListBoxEditor();

    //@cmember
    /* Attaches an CCExtendedListBoxEditor object to an existing list box control.*/
    virtual BOOL Initialize(CWnd* pwndParent, UINT uiID,
                            int iFlags = LBE_MY_DEFAULT);

    //@access Attributes

    //@cmember
    /* Determines whether in-place editing is occurring at the moment.*/
    BOOL IsEditing() const
    {
        return m_pEdit != NULL;
    }

    //@cmember
    /* Sets escape key handling.*/
    void SetEscape(BOOL bSet)
    {
        m_bEscape=bSet;
    }

    //@cmember
    /* Retrieves escape key handling state.*/
    BOOL GetEscape()
    {
        return m_bEscape;
    }

    //@cmember
    /* Returns a pointer to the listbox control bound to this control.*/
    CListBox* GetListBoxPtr() const;

    BOOL IsFlagSet(int i)
    {
        return m_iFlags & i;
    }

    BOOL AreFlagsSet(int i)
    {
        return (m_iFlags & i) == i;
    }

    //@cmember
    /* Sets "quick mode".*/
    virtual void SetQuick (BOOL bQuick = TRUE);

    //@cmember
    /* Retrieves the "quick mode" state.*/
    BOOL GetQuick() const;

    //@access Operations

    //@cmember
    /* Forces an end to active in-place editing.*/
    virtual void StopEditing();

    //@cmember
    /* Forces a repaint*/
    void ForcePaint()
    {
        OnPaint();
    }

protected:


public:

    //@access Overridable

    //@cmember
    /* Called when editing has started.*/
    virtual void OnEditingJustStarted();

    //@cmember
    /* Called when editing has stopped.*/
    virtual void OnEditingStopped();

    //@cmember
    /* Called after an item has been renamed.*/
    virtual void OnItemRenamed(int nIndexRenamed,CString strOldName,CString strNewName);

    //@cmember
    /* Called after an item has been added.*/
    virtual void OnItemAdded(int nIndexNewItem,CString strTextInserted);

    //@cmember
    /* Called after an item has been moved.*/
    virtual void OnItemMoved(int nOldIndex,int nNewIndex,CString strTextMoved,BOOL WasDragged);

    //@cmember
    /* Called before an item is deleted.*/
    virtual BOOL OnItemDelete(int nIndexToBeDeleted,CString strTextDeleted);

    // The following methods may be overridden in descendant classes.
    //@cmember
    /* Adds an item to the listbox.*/
    virtual BOOL DoAdd();

    //@cmember
    /* Deletes the currently selected item from the listbox.*/
    virtual BOOL DoDelete();

    //@cmember
    /* Moves an item down in the listbox.*/
    virtual BOOL DoMoveDown();

    //@cmember
    /* Moves an item up in the listbox.*/
    virtual BOOL DoMoveUp();

    //@cmember
    /* Begins editing mode.*/
    virtual BOOL StartEditing(TCHAR c = NULL);

    //@cmember
    /* Calculates the height of the control.*/
    virtual int CalculateHeight();

    //@cmember
    /* Calculates the button placements.*/
    virtual void CalculateButtonRectangles();

    //@cmember
    /* Resets tooltips for the buttons.*/
    virtual void RecycleTooltips();

    //@cmember
    /* Resets the edit control size and position.*/
    virtual void ResetEditorSizePos();

    //@cmember
    /* Changes the visibility status of the buttons.*/
    virtual void SetButtonVisibility(int iFlags);

    //@cmember
    /* Creates the edit control for editing*/
    virtual BOOL EditCore(int iIndex, BOOL bDelOnCancel, LPCTSTR p = NULL);

    //@cmember
    /* Moves an item in the list box up one position.*/
    virtual void MoveUpCore(int iIndex);

    //@cmember
    /* Deletes an item in the listbox.*/
    virtual BOOL DeleteItem (int nItem);

    //@cmember
    /* Sets the number of spaces to treat as a tab for "quick mode".*/
    virtual void SetSpacesToTab (int nSpaces = 1);

    //@cmember
    /* Called when the browse button is clicked.*/
    virtual void OnBrowse()
    { }

    // access Overrides

    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CCExtendedListBoxEditor)

    //Used to translate window messages before they are dispatched.

    virtual BOOL PreTranslateMessage(MSG* pMsg);

    //}}AFX_VIRTUAL

    //{{AFX_MSG(CCExtendedListBoxEditor)

    // Called when Windows or an application makes a request to repaint.
    afx_msg void OnPaint();

    // Called when the user presses the left mouse button.
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);

    // Called every time the mouse is moved.
    afx_msg UINT OnNcHitTest(CPoint point);

    // Called after the window's size has changed.
    afx_msg void OnSize(UINT nType, int cx, int cy);

    // Called after the CWnd object has been moved.
    afx_msg void OnMove(int x, int y);

    // Called after gaining the input focus.
    afx_msg void OnSetFocus(CWnd* pOldWnd);

    // Called immediately before losing the input focus.
    afx_msg void OnKillFocus(CWnd* pNewWnd);


    //}}AFX_MSG

    //@access Implementation

    // Useful constants -- hidden in class scope.
    enum { NUM_BUTTONS = 4, IDC_EDIT = 100, IDC_BUTTON = 101 };

    //@cmember
    /* Returns the index of an item before it changed.*/
    int  GetChangedLine() const;

    //@cmember
    /* Returns the index of an item after it moved.*/
    int  GetNewLine() const;

    //@cmember
    /* Gets the number of spaces to be treated as a tab for "quick mode".*/
    int  GetSpacesToTab() const;


    // Grant friendship to a class used in the implementation.
    friend class CCExtendedListBoxEditorHelper;

protected:

    //@access Protected data members.

    //@cmember
    /* Browse button (if used).*/
    CButton     m_btnBrowse;

#ifdef WIN32

    //@cmember
    /* Tooltip control.*/
    CToolTipCtrl m_ToolTip;

#endif

    //@cmember
    /* Pointer to editing control.*/
    CCExtendedListBoxEdit* m_pEdit;

    //@cmember
    /* Pointer to helper object.*/
    CCExtendedListBoxEditorHelper* m_pHelper;

    //@cmember
    /* Button rectangles*/
    CRect m_rcButtons[NUM_BUTTONS];

    //@cmember
    /* Button existance boolean flags*/
    BOOL  m_bUseButton[NUM_BUTTONS];

    //@cmember
    /* Flags controlling operation.*/
    int m_iFlags;

    //@cmember
    /* Initialization flag.*/
    BOOL m_bInitialized;

    //@cmember
    /* Calculated height.*/
    int m_calcdHeight;

    //@cmember
    /* TRUE if a new item is being created.*/
    BOOL m_bAddInProgress;

    //@cmember
    /* Item text before rename occurred.*/
    CString m_strRenameOldName;

    //@cmember
    /* TRUE if handling escape key*/
    BOOL m_bEscape;

    //@cmember
    /* The index of an item before it is moved.*/
    int     mnChangedLine;

    //@cmember
    /* The index of an item after it is moved.*/
    int     mnNewLine;

    //@cmember
    /* "Quick mode" flag.*/
    BOOL    mbQuick;

    //@cmember
    /* Number of spaces treated as a tab (quick mode).*/
    int     mnSpaces;

    DECLARE_MESSAGE_MAP()

};

class CCExtendedListBoxEditorHelper : public CListBox
{
public:
    CCExtendedListBoxEditorHelper(CCExtendedListBoxEditor& lbe, CListBox* p);

protected:
    CCExtendedListBoxEditor& m_LBE;
    void OnBrowse()
    {
        m_LBE.OnBrowse();
    }
    virtual BOOL OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult);

protected:

    //{{AFX_MSG(CCExtendedListBoxEditorHelper)
    afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
    afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
    afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
    afx_msg void OnLButtonDown(UINT uiFlags, CPoint point);
    afx_msg void OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
    //}}AFX_MSG
    afx_msg LRESULT OnEscapeState(WPARAM wParam, LPARAM lParam);
    DECLARE_MESSAGE_MAP()
};

class CCExtendedListBoxEdit : public CEdit
{
protected:
    enum {IDC_BUTTON = 100};
    BOOL    m_bDelOnCancel;
    int     m_iIndex;
    int     m_iOldSel;
    UINT    m_uiBrowseID;

    void DoNotify(BOOL bChanged);
    void OnKeyDownBase(UINT nChar, UINT nRepCnt, UINT nFlags);

    //{{AFX_MSG(CCExtendedListBoxEdit)
    afx_msg UINT OnGetDlgCode();
    afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
    afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
    afx_msg void OnKillFocus(CWnd* pNewWnd);
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
public:
    CCExtendedListBoxEdit (CListBox* pLB, UINT uiID, int iIndex,
                           BOOL bDelOnCancel, UINT uiBrowseID,
                           LPCTSTR p = NULL,
                           BOOL bQuick = FALSE, int nSpaces = 0);

protected:
    BOOL    mbQuick;
    int     mnSpaces;
};


#endif // !defined(AFX_CEXTENDEDLISTBOXEDITOR_H__BD03C9A3_0F81_11D3_AC27_00105A483E3C__INCLUDED_)
