/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CheckTreeCtrl.cpp: implementation of the CCheckTreeCtrl class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "CyrRes.h"
#include "resource.h"
#include "CheckTreeCtrl.h"


#define IDC_CHECK_TREE 1
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------

CCheckTreeCtrl::CCheckTreeCtrl()
{
}

//-------------------------------------------------------------------


CCheckTreeCtrl::~CCheckTreeCtrl()
{
}

//-------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CCheckTreeCtrl, CExtendedTreeCtrl)
//{{AFX_MSG_MAP(CCheckTreeCtrl)
ON_WM_LBUTTONDOWN()
ON_WM_KEYDOWN()
ON_UPDATE_COMMAND_UI(ID_MNU_SELECT, OnUpdateSelectItem)
ON_UPDATE_COMMAND_UI(ID_MNU_UNSELECT, OnUpdateUnSelectItem)
ON_COMMAND(ID_MNU_SELECT, OnSelectItem)
ON_COMMAND(ID_MNU_UNSELECT, OnUnSelectItem)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//-------------------------------------------------------------------
// state images as checked boxes
void CCheckTreeCtrl::InitStateImage()
{
    CBitmap  cBmpChkbox ;
    //--
    cBmpChkbox.LoadBitmap( _T("CHKBOXS") );
    m_cImageListState.Create(16,
                             16,
                             TRUE,
                             3,
                             0);
    HIMAGELIST hWndImageListState = m_cImageListState.m_hImageList;
    if(!hWndImageListState)
    {
        return ;
    }
    m_cImageListState.Add( &cBmpChkbox, RGB(255,0,0));

    SetImageList(&m_cImageListState , LVSIL_STATE);
}

//-------------------------------------------------------------------
// insert items and their states (checked boxes)
BOOL CCheckTreeCtrl::InsertItem( HTREEITEM & hItem ,
                                 LPCTSTR     lpszItem ,
                                 int         nImage ,
                                 int         nSelectedImage ,
                                 DWORD       dwID ,
                                 DWORD       dwParentID )
{

    BOOL bRet = CExtendedTreeCtrl::InsertItem( hItem ,
                lpszItem ,
                nImage ,
                nSelectedImage ,
                dwID ,
                dwParentID ) ;
    if( bRet )
        SetItemState( hItem , INDEXTOSTATEIMAGEMASK(2) , TVIS_STATEIMAGEMASK ) ;
    return bRet ;
}

//-------------------------------------------------------------------
// Change the state of the checkboxes
void CCheckTreeCtrl::OnLButtonDown( UINT nFlags , CPoint point )
{
    UINT uFlags = 0 ;
    HTREEITEM hItem = HitTest( point , &uFlags ) ;

    if( uFlags & TVHT_ONITEMSTATEICON )
    {

        int iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
        SetChildrenState( iImage , hItem ) ;
        SetParentState( iImage , hItem ) ;

        //Get if the image is the checked one or not
        SelectItem( hItem );
        return ;
    }
    else
    {
        if( uFlags & TVHT_ABOVE ||
                uFlags & TVHT_BELOW ||
                uFlags & TVHT_NOWHERE ||
                uFlags & TVHT_ONITEM ||
                uFlags & TVHT_ONITEMRIGHT ||
                uFlags & TVHT_ONITEMICON ||
                uFlags & TVHT_ONITEMINDENT ||
                uFlags & TVHT_ONITEMLABEL ||
                uFlags & TVHT_TOLEFT ||
                uFlags & TVHT_TORIGHT )
        {
            SelectItem( hItem );
            return ;
        }
        else
        {
            if( uFlags & TVHT_ONITEMBUTTON )
            {
                SelectItem( hItem );
                CExtendedTreeCtrl::OnLButtonDown( nFlags , point ) ;
            }
        }
    }

}

//-------------------------------------------------------------------
// Change the state of the checkboxes
void CCheckTreeCtrl::OnKeyDown( UINT nChar , UINT nRepCnt, UINT nFlags )
{
    if( nChar = VK_SPACE )
    {
        HTREEITEM hItem = GetSelectedItem() ;
        int iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
        SetChildrenState( iImage , hItem ) ;
        SetParentState( iImage , hItem ) ;

        //Get if the image is the checked one or not
        SelectItem( hItem );
        return ;
    }
    else
        CExtendedTreeCtrl::OnKeyDown(  nChar , nRepCnt, nFlags );
}

//-------------------------------------------------------------------
// return TRUE if hItem is checked else FALSE
BOOL CCheckTreeCtrl::IsItemChecked( HTREEITEM hItem )
{
    int iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
    if( iImage == 2 )
        return TRUE ;
    else
        return FALSE ;
}

//-------------------------------------------------------------------
// return TRUE if hItem is unchecked else FALSE
BOOL CCheckTreeCtrl::IsItemUnChecked( HTREEITEM hItem )
{
    int iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
    if( iImage == 3 )
        return TRUE ;
    else
        return FALSE ;
}

//-------------------------------------------------------------------
// Get the first item checked in the tree
HTREEITEM CCheckTreeCtrl::GetFirstCheckedItem( )
{
    HTREEITEM hItem = GetRootItem() ;
    //---
    while( !IsItemChecked( hItem ) && hItem != NULL )
    {
        GetNextItem( hItem , TVGN_NEXT ) ;
    }
    return hItem ;
}

//-------------------------------------------------------------------
// Get the next item checked in the tree
HTREEITEM CCheckTreeCtrl::GetNextCheckedItem( HTREEITEM hItem )
{
    GetNextItem( hItem , TVGN_NEXT ) ;
    //---
    while( !IsItemChecked( hItem ) && hItem != NULL )
    {
        GetNextItem( hItem , TVGN_NEXT ) ;
    }
    return hItem ;
}

//-------------------------------------------------------------------
// Get the previous item checked in the tree
HTREEITEM CCheckTreeCtrl::GetPreviousCheckedItem( HTREEITEM hItem )
{
    GetNextItem( hItem , TVGN_NEXT ) ;
    //---
    while( !IsItemChecked( hItem ) && hItem != NULL )
    {
        GetNextItem( hItem , TVGN_PREVIOUS ) ;
    }
    return hItem ;
}

//-------------------------------------------------------------------
// Get the previous item checked in the tree
void CCheckTreeCtrl::SetParentState( int iImage , HTREEITEM hItem )
{
    iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
    HTREEITEM hParentItem ;
    //---
    hParentItem = GetNextItem( hItem , TVGN_PARENT ) ;
    if( hParentItem )
    {
        if( iImage == 3 )
        {
            if( hParentItem && HasBrothers( hItem ) && !IsBrotherStateUncheck( hItem ) )
                SetCheckThreeState( hParentItem ) ;
            else
                UnCheck( hParentItem ) ;
        }
        else
        {
            if( iImage == 2 )
            {
                if( hParentItem && HasBrothers( hItem ) && !IsBrotherStateChecked( hItem ) )
                    SetCheckThreeState( hParentItem ) ;
                else
                    SetCheck( hParentItem ) ;
            }
            else
                SetCheckThreeState( hParentItem ) ; // 3
        }


        SetParentState( iImage , hParentItem ) ;

    }


}

//-------------------------------------------------------------------
// Get the previous item checked in the tree
void CCheckTreeCtrl::SetChildrenState( int iImage , HTREEITEM hParentItem )
{
    HTREEITEM hItem ;
    //---
    hItem = GetNextItem( hParentItem , TVGN_CHILD ) ;
    while( hItem )
    {
        SetChildrenState( iImage , hItem ) ;
        hItem = GetNextItem( hItem , TVGN_NEXT ) ;
    }

    SetItemState( hParentItem , INDEXTOSTATEIMAGEMASK( iImage == 3 ? 2 : 3 ) , TVIS_STATEIMAGEMASK ) ;
}

//-------------------------------------------------------------------
// check if all the brother of hItem are checked : 2 or 3. Ifyes ->check or checkThreeStatefather( = return TRUE )
// If no ->uncheck father ( = return FALSE )
BOOL CCheckTreeCtrl::IsBrotherStateChecked( HTREEITEM hBrotherItem )
{
    HTREEITEM hItem ;
    int iImage ;
    //---
    //Look before hBrotherItem
    hItem = GetNextItem( hBrotherItem , TVGN_PREVIOUS ) ;
    while( hItem )
    {
        iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
        if( iImage == 2  ) // checked
            hItem = GetNextItem( hItem , TVGN_PREVIOUS ) ;
        else  // unchecked
            return FALSE ;

    }
    //Look after hBrotherItem
    hItem = GetNextItem( hBrotherItem , TVGN_NEXT ) ;
    while( hItem )
    {
        iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
        if( iImage == 2 )
            hItem = GetNextItem( hItem , TVGN_NEXT ) ;
        else  // not checked
            return FALSE ;

    }

    return TRUE ;
}

//-------------------------------------------------------------------
// check if all the brother of hItem are unchecked. Ifyes ->uncheck father( = return TRUE )
// If no ->check father ( = return FALSE )
BOOL CCheckTreeCtrl::IsBrotherStateUncheck( HTREEITEM hBrotherItem )
{
    HTREEITEM hItem ;
    int iImage ;
    //---
    //Look before hBrotherItem
    hItem = GetNextItem( hBrotherItem , TVGN_PREVIOUS ) ;

    while( hItem )
    {
        iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
        if( iImage == 3 ) // not checked
            hItem = GetNextItem( hItem , TVGN_PREVIOUS ) ;
        else  // checked
            return FALSE ;
    }
    //Look after hBrotherItem
    hItem = GetNextItem( hBrotherItem , TVGN_NEXT ) ;
    while( hItem )
    {
        iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
        if( iImage == 3 )  // not checked
            hItem = GetNextItem( hItem , TVGN_NEXT ) ;
        else  // checked
            return FALSE ;

    }

    return TRUE ;
}

//-------------------------------------------------------------------
// check if all the brother of hItem are checked as image == 2.
BOOL CCheckTreeCtrl::IsAllBrothersChecked( HTREEITEM hBrotherItem )
{
    HTREEITEM hItem ;
    int iImage ;
    //---
    //Look before hBrotherItem
    hItem = GetNextItem( hBrotherItem , TVGN_PREVIOUS ) ;
    while( hItem )
    {
        iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
        if( iImage == 2 ) // checked
            hItem = GetNextItem( hItem , TVGN_PREVIOUS ) ;
        else  // unchecked
            return FALSE ;

    }
    //Look after hBrotherItem
    hItem = GetNextItem( hBrotherItem , TVGN_NEXT ) ;
    while( hItem )
    {
        iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
        if( iImage == 2 )
            hItem = GetNextItem( hItem , TVGN_NEXT ) ;
        else  // not checked
            return FALSE ;

    }

    return TRUE ;
}

//-------------------------------------------------------------------
// check if the item has a brother : return true if yes
// false otherwise
BOOL CCheckTreeCtrl::HasBrothers( HTREEITEM hItem )
{
    HTREEITEM hBrotherItem ;
    //---
    //Look before hItem
    hBrotherItem = GetNextItem( hItem , TVGN_PREVIOUS ) ;
    if( hBrotherItem )
        return TRUE ;
    //Look after hItem
    hBrotherItem = GetNextItem( hItem , TVGN_NEXT ) ;
    if( hBrotherItem )
        return TRUE ;

    return FALSE ;

}

//-------------------------------------------------------------------
// return TRUE if the user has clicked on the state image else FALSE
BOOL CCheckTreeCtrl::IsImageStateSelected( CPoint pt )
{
    UINT uFlags = 0 ;
    HTREEITEM hItem = HitTest( pt , &uFlags ) ;

    if( uFlags & TVHT_ONITEMBUTTON )
        return FALSE ;
    if( uFlags & TVHT_ONITEMSTATEICON )
        return TRUE ;
    else
        return FALSE ;
}

//-------------------------------------------------------------------
// Check the item and its children
void CCheckTreeCtrl::SetImageStateSelected( CPoint pt )
{
    UINT uFlags = 0 ;
    HTREEITEM hItem = HitTest( pt , &uFlags ) ;
    if( uFlags & TVHT_ONITEMSTATEICON )
    {
        int iImage = GetItemState( hItem , TVIS_STATEIMAGEMASK )>>12 ;
        SetChildrenState( iImage , hItem ) ;
    }
}

//-------------------------------------------------------------------
// enable or not to collapse or expand
BOOL CCheckTreeCtrl::IsCollapse_Expand_Enable()
{
    DWORD               dwpos;
    POINT               pt;
    UINT                nFlags ;

    // find out where the cursor was
    dwpos = GetMessagePos();
    pt.x = LOWORD(dwpos);
    pt.y = HIWORD(dwpos);

    ::MapWindowPoints(HWND_DESKTOP, m_hWnd, &pt, 1);

    HTREEITEM hItem = HitTest( pt , &nFlags ) ;
    SelectItem( hItem );

    //Test if the state image has been clicked
    // if clicked -> no expand and no collapse
    // update the check state
    if( IsImageStateSelected( pt ) )
    {
        SetImageStateSelected( pt ) ;
        return FALSE ;
    }
    else
        return TRUE ;
}

//-------------------------------------------------------------------
// Add menu select and unselect to the TreeCtrl
void CCheckTreeCtrl::DoOwnerContextMenu( CMenu * pSubMenu )
{
    pSubMenu->AppendMenu(MF_SEPARATOR, ID_MNU_SEPARATOR , "");

    pSubMenu->AppendMenu(MF_STRING, ID_MNU_SELECT, _T("Select") );

    pSubMenu->AppendMenu(MF_STRING, ID_MNU_UNSELECT, _T("UnSelect") );
}

//-------------------------------------------------------------------
// Enable or not to select item
void CCheckTreeCtrl::OnUpdateSelectItem(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( TRUE );
}

//-------------------------------------------------------------------
// Select item
void CCheckTreeCtrl::OnSelectItem()
{
    HTREEITEM hItem = GetSelectedItem() ;
    SetChildrenState( 3 , hItem ) ;
    SetParentState( 3 , hItem ) ;
    SelectItem( hItem ) ;
}

//-------------------------------------------------------------------
// Enable or not to UnSelect item
void CCheckTreeCtrl::OnUpdateUnSelectItem(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( TRUE );
}

//-------------------------------------------------------------------
// UnSelect item
void CCheckTreeCtrl::OnUnSelectItem()
{
    HTREEITEM hItem = GetSelectedItem() ;
    SetChildrenState( 2 , hItem ) ;
    SetParentState( 2 , hItem ) ;
    SelectItem( hItem ) ;
}

//-------------------------------------------------------------------
// UnSelect item
void CCheckTreeCtrl::SetCheck( HTREEITEM hItem )
{
    SetItemState( hItem , INDEXTOSTATEIMAGEMASK( 2 ) , TVIS_STATEIMAGEMASK ) ;
}

//-------------------------------------------------------------------
// UnSelect item
void CCheckTreeCtrl::UnCheck( HTREEITEM hItem )
{
    SetItemState( hItem , INDEXTOSTATEIMAGEMASK( 3 ) , TVIS_STATEIMAGEMASK ) ;
}

//-------------------------------------------------------------------
// Select item with three state
void CCheckTreeCtrl::SetCheckThreeState( HTREEITEM hItem )
{
    SetItemState( hItem , INDEXTOSTATEIMAGEMASK( 1 ) , TVIS_STATEIMAGEMASK ) ;
}

//-------------------------------------------------------------------
// Select item with three state
void CCheckTreeCtrl::UnCheckAll( HTREEITEM hItem )
{
    if( hItem != NULL )
    {
        UnCheck( hItem ) ;
        HTREEITEM hNextItem = GetNextItem( hItem , TVGN_CHILD ) ;
        if( hNextItem != NULL )
        {
            UnCheckAll( hNextItem ) ;
        }

        hNextItem  = GetNextItem( hItem , TVGN_NEXT ) ;
        if( hNextItem )
        {
            UnCheckAll( hNextItem ) ;
        }
    }
}

//---------------------------------------------------------------------------
