/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"
#include "ComCtrl32Ver.h"
#include <windows.h>

// #include <shlwapi.h>

//--------------------------------------------------------------------

DWORD CComCtl32Version::m_dwMajor = 0L ;
DWORD CComCtl32Version::m_dwMinor = 0L ;

//--------------------------------------------------------------------

void CComCtl32Version::Getversion()
{
    if ( FAILED( ::GetComCtlVersion( &m_dwMajor , &m_dwMinor ) ) )
    {
        m_dwMajor = 0L ;
        m_dwMinor = 0L ;
    }
}

//--------------------------------------------------------------------

DWORD CComCtl32Version::GetMajor() const
{
    if ( ! m_dwMajor ) // Supposed to be at least 4
    {
        const_cast<CComCtl32Version *>(this)->Getversion() ;
    }
    return m_dwMajor ;
}

//--------------------------------------------------------------------

DWORD CComCtl32Version::GetMinor() const
{
    if ( ! m_dwMajor ) // Supposed to be at least 4
    {
        const_cast<CComCtl32Version *>(this)->Getversion() ;
    }
    return m_dwMinor ;
}

//--------------------------------------------------------------------
// The following codes comes from :
// http://www.eu.microsoft.com/msdn/sdk/inetsdk/help/itt/CommCtls/Common/Common.htm#Common_Controls

HRESULT GetComCtlVersion(LPDWORD pdwMajor, LPDWORD pdwMinor)
{
    // DLLVERSIONINFO & DLLGETVERSIONPROC : I could not finf the proper header
    // so I had to copy these declarations from the doc :
    typedef struct _DllVersionInfo
    {
        DWORD cbSize;
        DWORD dwMajorVersion;
        DWORD dwMinorVersion;
        DWORD dwBuildNumber;
        DWORD dwPlatformID;
    }
    DLLVERSIONINFO ;

    typedef HRESULT (CALLBACK* DLLGETVERSIONPROC)(DLLVERSIONINFO *);

    HINSTANCE hComCtl ;
    // ----
    if( ::IsBadWritePtr(pdwMajor, sizeof(DWORD)) ||
            ::IsBadWritePtr(pdwMinor, sizeof(DWORD)))
    {
        return E_INVALIDARG;
    }

    //load the DLL
    hComCtl = LoadLibrary(TEXT("comctl32.dll"));

    if (hComCtl)
    {
        HRESULT           hr = S_OK;
        DLLGETVERSIONPROC pDllGetVersion;
        // You must get this function explicitly because earlier versions of the DLL
        // don't implement this function. That makes the lack of implementation of the
        // function a version marker in itself.

        pDllGetVersion = (DLLGETVERSIONPROC)GetProcAddress(hComCtl, TEXT("DllGetVersion"));

        if ( pDllGetVersion != NULL )
        {
            DLLVERSIONINFO dvi ;
            // ----
            ZeroMemory( &dvi , sizeof(dvi));
            dvi.cbSize = sizeof(dvi) ;
            hr = (*pDllGetVersion)(&dvi);

            if ( SUCCEEDED(hr) )
            {
                *pdwMajor = dvi.dwMajorVersion ;
                *pdwMinor = dvi.dwMinorVersion ;
            }
            else
            {
                hr = E_FAIL ;
            }
        }
        else
        {
            // If GetProcAddress failed, then the DLL is a version previous to the one
            // shipped with IE 3.x.
            *pdwMajor = 4 ;
            *pdwMinor = 0 ;
        }

        FreeLibrary( hComCtl ) ;
        return hr;
    }
    return E_FAIL;
}
