/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// DefaultSimpleMfcMail.cpp: implementation of the CDefaultSimpleMfcMail class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DefaultSimpleMfcMail.h"

#include <mapi.h>

/////////////////////////////////////////////////////////////////////////////
// MAPI implementation helpers and globals
// --------------------------------------------------------------------------

static BOOL _afxIsMailAvail = (BOOL)-1;    // start out not determined

/////////////////////////////////////////////////////////////////////////////
// _AFX_MAIL_STATE2
// --------------------------------------------------------------------------

class _AFX_MAIL_STATE2 : public CNoTrackObject
{
public:
    HINSTANCE m_hInstMail;      // handle to MAPI32.DLL
    virtual ~_AFX_MAIL_STATE2();
};

_AFX_MAIL_STATE2::~_AFX_MAIL_STATE2()
{
    if (m_hInstMail != NULL)
        ::FreeLibrary(m_hInstMail);
}

EXTERN_PROCESS_LOCAL(_AFX_MAIL_STATE2, _afxMailState)

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDefaultSimpleMfcMail::CDefaultSimpleMfcMail()
{}

// --------------------------------------------------------------------------


CDefaultSimpleMfcMail::~CDefaultSimpleMfcMail()
{}

// --------------------------------------------------------------------------


HRESULT CDefaultSimpleMfcMail::_OnUpdateFileSendMail(CCmdUI* pCmdUI)
{
    // ASSERT_VALID(this);

    if (_afxIsMailAvail == (BOOL)-1)
    {
        _afxIsMailAvail = ::GetProfileInt(_T("MAIL"), _T("MAPI"), 0) != 0 &&
                          SearchPath(NULL, _T("MAPI32.DLL"), NULL, 0, NULL, NULL) != 0;
    }

    // enable the Send... menu item if available
    pCmdUI->Enable(_afxIsMailAvail);
    CMenu* pMenu = pCmdUI->m_pMenu;
    if (!_afxIsMailAvail && pMenu != NULL)
    {
        // remove the Send... menu and surrounding separators
        UINT nStateAbove = pMenu->GetMenuState(pCmdUI->m_nIndex-1, MF_BYPOSITION);
        UINT nStateBelow = pMenu->GetMenuState(pCmdUI->m_nIndex+1, MF_BYPOSITION);
        pMenu->RemoveMenu(pCmdUI->m_nIndex, MF_BYPOSITION);
        if (nStateAbove & nStateBelow & MF_SEPARATOR)
        {
            // a separator must be removed since the Send... is gone
            if (nStateAbove != (UINT)-1)
                pMenu->RemoveMenu(pCmdUI->m_nIndex-1, MF_BYPOSITION);
            else if (nStateBelow != (UINT)-1)
                pMenu->RemoveMenu(pCmdUI->m_nIndex, MF_BYPOSITION);
        }
    }

    return S_OK ;
}

// --------------------------------------------------------------------------

HRESULT CDefaultSimpleMfcMail::_OnFileSendMail( LPSTR lpszSubject ,    // Message Subject
        LPSTR lpszNoteText ,   // Message Text
        LPSTR lpszAttachedFilePath  /*=NULL*/ ,
        LPSTR lpszAttachedFileTitle /*=NULL*/ )
{
    // ASSERT_VALID(this);
    ASSERT(_afxIsMailAvail);   // update handler always gets called first

    CWaitCursor wait;

    _AFX_MAIL_STATE2* pMailState = _afxMailState;
    if (pMailState->m_hInstMail == NULL)
        pMailState->m_hInstMail = ::LoadLibraryA("MAPI32.DLL");

    if (pMailState->m_hInstMail == NULL)
    {
        AfxMessageBox(AFX_IDP_FAILED_MAPI_LOAD);
        return E_FAIL ;
    }
    ASSERT(pMailState->m_hInstMail != NULL);

    ULONG (PASCAL *lpfnSendMail)(ULONG, ULONG, MapiMessage*, FLAGS, ULONG);
    (FARPROC&)lpfnSendMail = GetProcAddress(pMailState->m_hInstMail, "MAPISendMail");
    if (lpfnSendMail == NULL)
    {
        AfxMessageBox(AFX_IDP_INVALID_MAPI_DLL);
        return E_FAIL ;
    }
    ASSERT(lpfnSendMail != NULL);

#if 0 ////////////////////////////////////////////////////////////

    TCHAR szTempName[_MAX_PATH];
    TCHAR szPath[_MAX_PATH];
    BOOL bRemoveTemp = FALSE;
    if (m_strPathName.IsEmpty() || IsModified())
    {
        // save to temporary path
        VERIFY(GetTempPath(_countof(szPath), szPath) != 0);
        VERIFY(GetTempFileName(szPath, _T("afx"), 0, szTempName) != 0);

        // save it, but remember original modified flag
        BOOL bModified = IsModified();
        BOOL bResult = DoSave(szTempName, FALSE);
        SetModifiedFlag(bModified);
        if (!bResult)
        {
            TRACE0("Warning: file save failed during File.Send Mail.\n");
            return;
        }
        bRemoveTemp = TRUE;
    }
    else
    {
        // use actual file since it isn't modified
        lstrcpyn(szTempName, m_strPathName, _countof(szTempName));
    }
#ifdef _UNICODE
    char szTempNameA[_MAX_PATH];
    _wcstombsz(szTempNameA, szTempName, _countof(szTempNameA));
#endif

    // build an appropriate title for the attachment
    TCHAR szTitle[_MAX_PATH];
    if (!m_strPathName.IsEmpty())
        AfxGetFileName(m_strPathName, szTitle, _countof(szTitle));
    else
    {
        lstrcpyn(szTitle, m_strTitle, _countof(szTitle));
        if (m_strTitle.Find('.') == -1) // no extension
        {
            // append the default suffix if there is one
            CString strExt;
            CDocTemplate* pTemplate = GetDocTemplate();
            if (pTemplate != NULL &&
                    pTemplate->GetDocString(strExt, CDocTemplate::filterExt))
            {
                lstrcat(szTitle, strExt);
            }
        }
    }

#ifdef _UNICODE
    char szTitleA[_MAX_PATH];
    _wcstombsz(szTitleA, szTitle, _countof(szTitleA));
#endif

#endif ////////////////////////////////////////////////////////////

    // prepare the file description (for the attachment)
    MapiFileDesc fileDesc;
    memset(&fileDesc, 0, sizeof(fileDesc));
    fileDesc.nPosition = (ULONG)-1;
#ifdef _UNICODE
    ASSERT(FALSE) ; // Not yet implemented !
    fileDesc.lpszPathName = szTempNameA;
    fileDesc.lpszFileName = szTitleA;
#else
    if ( lpszAttachedFilePath != NULL && *lpszAttachedFilePath != 0 )
    {
        fileDesc.lpszPathName = lpszAttachedFilePath ;
        fileDesc.lpszFileName = lpszAttachedFileTitle ;
    }
#endif

#if 0  ///////////////////////////////////
    MapiRecipDesc recip ;
    // ----
    memset( &recip , 0 , sizeof( recip ) );

    // ULONG ulReserved;                       /* Reserved for future use                  */
    ulRecipClass      = MAPI_TO ;              /* Recipient class MAPI_TO, MAPI_CC, MAPI_BCC, MAPI_ORIG    */
    LPSTR lpszName    = "plachaise@cyrano.com;pvillame@cyrano.com" ;  /* Recipient name                           */
    LPSTR lpszAddress = "plachaise;pvillame" ; /* Recipient address (optional)             */
    ULONG ulEIDSize   = 0 ;            /* Count in bytes of size of pEntryID       */
    LPVOID lpEntryID  = NULL ;           /* System-specific recipient reference      */
#endif   ///////////////////////////////////

    // prepare the message (empty with 1 attachment)
    MapiMessage message;
    memset(&message, 0, sizeof(message));
    message.nFileCount   = ( lpszAttachedFilePath != NULL && *lpszAttachedFilePath != 0 ) ? 1 : 0 ;
    message.lpszSubject  = lpszSubject ;
    message.lpszNoteText = lpszNoteText ;
    message.lpFiles      = &fileDesc;
#if 0  ///////////////////////////////////
    message.nRecipCount  = 2 ;
    message.lpRecips     = &recip;
#endif     ///////////////////////////////////

    // prepare for modal dialog box
    AfxGetApp()->EnableModeless(FALSE);
    HWND hWndTop;
    CWnd* pParentWnd = CWnd::GetSafeOwner(NULL, &hWndTop);

    // some extra precautions are required to use MAPISendMail as it
    // tends to enable the parent window in between dialogs (after
    // the login dialog, but before the send note dialog).
    pParentWnd->SetCapture();
    ::SetFocus(NULL);
    pParentWnd->m_nFlags |= WF_STAYDISABLED;

    int nError = lpfnSendMail(0, (ULONG)pParentWnd->GetSafeHwnd(),
                              &message, MAPI_LOGON_UI|MAPI_DIALOG, 0);

    // after returning from the MAPISendMail call, the window must
    // be re-enabled and focus returned to the frame to undo the workaround
    // done before the MAPI call.
    ::ReleaseCapture();
    pParentWnd->m_nFlags &= ~WF_STAYDISABLED;

    pParentWnd->EnableWindow(TRUE);
    ::SetActiveWindow(NULL);
    pParentWnd->SetActiveWindow();
    pParentWnd->SetFocus();
    if (hWndTop != NULL)
        ::EnableWindow(hWndTop, TRUE);
    AfxGetApp()->EnableModeless(TRUE);

    if (nError != SUCCESS_SUCCESS &&
            nError != MAPI_USER_ABORT && nError != MAPI_E_LOGIN_FAILURE)
    {
        AfxMessageBox(AFX_IDP_FAILED_MAPI_SEND);
    }

#if 0 ////////////////////////////////////////////////////////////

    // remove temporary file, if temporary file was used
    if (bRemoveTemp)
        CFile::Remove(szTempName);

#endif ////////////////////////////////////////////////////////////

    return S_OK ;
}

// --------------------------------------------------------------------------

PROCESS_LOCAL(_AFX_MAIL_STATE2, _afxMailState)

// --------------------------------------------------------------------------
