/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ExtendedPropertySheet.cpp: implementation of the CExtendedPropertySheet class.
//
//////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "CyrRes.h"
#include "ExtendedPropertyPage.h"
#include "ExtendedPropertySheet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

IMPLEMENT_DYNAMIC(CExtendedPropertySheet, CPropertySheet)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CExtendedPropertySheet, CPropertySheet)
//{{AFX_MSG_MAP(CExtendedPropertySheet)
ON_BN_CLICKED(IDHELP, OnHelpFinder)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//CONSTRUCTOR
//---------------------------------------------------------------------------

CExtendedPropertySheet::CExtendedPropertySheet()
        :   CPropertySheet()
{
}

//---------------------------------------------------------------------------

// nIDCaption = Caption ID of the propertySheet
// pParentWnd = Parent Window : default NULL
// iSelectPage = Current Page : default 0

CExtendedPropertySheet::CExtendedPropertySheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
        :   CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{}

//---------------------------------------------------------------------------

// pszCaption = Caption of the propertySheet
// pParentWnd = Parent Window : default NULL
// iSelectPage = Current Page : default 0

CExtendedPropertySheet::CExtendedPropertySheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
        :   CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{}

//---------------------------------------------------------------------------


CExtendedPropertySheet::~CExtendedPropertySheet()
{
}

//---------------------------------------------------------------------------
//Add one page in the property sheet and set the propertySheet parent for the page

void CExtendedPropertySheet::AddPage( CExtendedPropertyPage *pPage )
{
    CPropertySheet::AddPage(pPage);
    pPage->m_pPropSheet = this;
}

//---------------------------------------------------------------------------
//Enable Pages from nStart to nEnd or not
void CExtendedPropertySheet::EnablePages(BOOL bEnable, int nStart, int nEnd /* = -1 */)
{
    if (nEnd == -1)
        nEnd = nStart;

    if (nStart > nEnd)
    {
        int tmp = nEnd;
        nEnd = nStart;
        nStart = tmp;
    }

    int count = GetPageCount();

    if (nEnd >= count)
        nEnd = count -1;

    CExtendedPropertyPage *pPage;


    for (int act=nStart; act <= nEnd;act++)
    {
        pPage = (CExtendedPropertyPage*) GetPage(act);
        pPage->Enable(bEnable);
    }
}

//---------------------------------------------------------------------------
//Show the PropertySheet
void CExtendedPropertySheet::Show(CWnd* _pParent, DWORD  _dwStyle)
{
    EnableStackedTabs(FALSE);

    if ( !Create(_pParent, _dwStyle) )
        ASSERT(FALSE);
    GetTabControl()->ModifyStyle( 0, TCS_FIXEDWIDTH, 0);

}

//---------------------------------------------------------------------------
//Get the width of the propertySheet
int CExtendedPropertySheet::Width() const
{
    CRect rc;
    GetWindowRect(&rc);
    return rc.Width();
}

//---------------------------------------------------------------------------
//Get the Height of the propertySheet
int CExtendedPropertySheet::Height() const
{
    CRect rc;
    GetWindowRect(&rc);
    return rc.Height();
}

//---------------------------------------------------------------------------
//Get the position of the propertySheet
void CExtendedPropertySheet::SetPosition(int _iLeft, int _iTop)
{
    SetWindowPos(NULL, _iLeft, _iTop, Width(), Height(), SWP_NOZORDER ) ;
}

//---------------------------------------------------------------------------
// cancel clicked
void CExtendedPropertySheet::OnCancel()
{}

//---------------------------------------------------------------------------
// ok clicked

BOOL CExtendedPropertySheet::OnOK()
{
    return TRUE;
}

//---------------------------------------------------------------------------
// next clicked
void CExtendedPropertySheet::OnWizardNext( CExtendedPropertyPage* _pPage )
{}

//---------------------------------------------------------------------------
// back clicked

void CExtendedPropertySheet::OnWizardBack( CExtendedPropertyPage* _pPage )
{}

//---------------------------------------------------------------------------
// activate focus on _pPage

void CExtendedPropertySheet::OnSetActive( CExtendedPropertyPage* _pPage )
{
    if ( ( m_psh.dwFlags & PSH_WIZARD ) == 0 )
        return;

    GetDlgItem(ID_WIZFINISH)->EnableWindow( GetPageIndex(_pPage) == GetPageCount()-1 ) ;
    GetDlgItem(ID_WIZNEXT)->EnableWindow( GetPageIndex(_pPage) != GetPageCount()-1 ) ;
    GetDlgItem(ID_WIZBACK)->EnableWindow( GetPageIndex(_pPage) != 0 ) ;
}

//---------------------------------------------------------------------------
//Kill focus of _pPage
void CExtendedPropertySheet::OnKillActive( CExtendedPropertyPage* _pPage )
{}

// --------------------------------------------------------------------------


BOOL CExtendedPropertySheet::EnablePage( CExtendedPropertyPage* _page , BOOL bEnable /*=TRUE*/ )
{
    int index = GetPageIndex( _page ) ;
    // ----
    return ( index < 0 ? FALSE : m_TabCtrlEx.EnableItem( index , bEnable ) ) ;
}

// --------------------------------------------------------------------------

BOOL CExtendedPropertySheet::IsPageEnabled( CExtendedPropertyPage* _page )
{
    int index = GetPageIndex( _page ) ;
    // ----
    return ( index < 0 ? FALSE : const_cast<CTabCtrlEx &>(m_TabCtrlEx).IsItemEnabled( index ) ) ;
}


//---------------------------------------------------------------------------
//Init the PropertySheet Dialog
//Init the button names and their positions
BOOL CExtendedPropertySheet::OnInitDialog()
{
    CPropertySheet::OnInitDialog();

    m_TabCtrlEx.Install( this ) ;

    CString strButtonName ;
    strButtonName.Empty() ;

    CWnd *buttonCancel  = GetDlgItem( IDCANCEL ) ;
    CWnd *buttonHelp    = GetDlgItem( IDHELP ) ;

    if ( ( m_psh.dwFlags & PSH_WIZARD ) != 0 )
    {
        CWnd *buttonBack    = GetDlgItem( ID_WIZBACK ) ;
        CWnd *buttonNext    = GetDlgItem( ID_WIZNEXT ) ;
        CWnd *buttonFinish  = GetDlgItem( ID_WIZFINISH ) ;

        //Move the button next near the button back
        CRect BackButton ;
        buttonBack->GetWindowRect( &BackButton ) ;

        ScreenToClient( &BackButton ) ;

        buttonBack->SetWindowPos( NULL , BackButton.left - 2 * BackButton.Width() - 15 ,
                                  BackButton.top, BackButton.Width(),  BackButton.Height(),
                                  SWP_NOACTIVATE || SWP_NOSIZE || SWP_NOZORDER || SWP_SHOWWINDOW);

        buttonNext->SetWindowPos(buttonBack, BackButton.left - BackButton.Width() - 15, BackButton.top,
                                 BackButton.Width(), BackButton.Height(), SWP_NOACTIVATE || SWP_SHOWWINDOW);

        buttonFinish->SetWindowPos(buttonBack, BackButton.left - 10 , BackButton.top,
                                   BackButton.Width(), BackButton.Height(), SWP_NOACTIVATE || SWP_SHOWWINDOW);

        buttonCancel->SetWindowPos(buttonBack, BackButton.left + BackButton.Width() - 5, BackButton.top,
                                   BackButton.Width(), BackButton.Height(), SWP_NOACTIVATE || SWP_SHOWWINDOW);

        buttonHelp->SetWindowPos(buttonBack, BackButton.left + 2 * BackButton.Width(), BackButton.top,
                                 BackButton.Width(), BackButton.Height(), SWP_NOACTIVATE || SWP_SHOWWINDOW);

        strButtonName.LoadString( IDS_BACK ) ;
        buttonBack->SetWindowText( strButtonName ) ;

        strButtonName.LoadString( IDS_NEXT ) ;
        buttonNext->SetWindowText( strButtonName ) ;

        strButtonName.LoadString( IDS_FINISH ) ;
        buttonFinish->SetWindowText( strButtonName ) ;
        buttonFinish->ShowWindow(SW_SHOW ) ;
    }
    else
    {
        CRect rectApply;
        CWnd *buttonApply    = GetDlgItem( ID_APPLY_NOW ) ;

        buttonApply->ShowWindow(SW_HIDE ) ;
        buttonApply->GetWindowRect( &rectApply ) ;
        ScreenToClient( &rectApply ) ;

        buttonHelp->SetWindowPos(buttonApply, rectApply.left , rectApply.top,
                                 rectApply.Width(), rectApply.Height(), SWP_NOACTIVATE || SWP_SHOWWINDOW);


    }

    strButtonName.LoadString( IDS_CANCEL ) ;
    buttonCancel->SetWindowText( strButtonName ) ;

    strButtonName.LoadString( IDS_HELP ) ;
    buttonHelp->ShowWindow(SW_SHOW ) ;
    buttonHelp->SetWindowText( strButtonName ) ;

    return TRUE;
}



