/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ExtendedTreeView.cpp : implementation file
//

#include "stdafx.h"
#include "StubDocument.h"
#include "CyrRes.h"
#include "ExtendedTreeView.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// CExtendedTreeView
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
IMPLEMENT_DYNCREATE(CExtendedTreeView, CTreeView)

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
BEGIN_MESSAGE_MAP(CExtendedTreeView, CTreeView)
//{{AFX_MSG_MAP(CExtendedTreeView)
ON_WM_RBUTTONDOWN()
// NOTE - the ClassWizard will add and remove mapping macros here.
//
ON_COMMAND(ID_EXTGRID_EXPAND_ALL, _OnExpandAll)
ON_UPDATE_COMMAND_UI(ID_EXTGRID_EXPAND_ALL, _OnUpdateExpandAll)
//
ON_COMMAND(ID_EXTGRID_COLLAPSE_ALL, _OnCollapseAll)
ON_UPDATE_COMMAND_UI(ID_EXTGRID_COLLAPSE_ALL, _OnUpdateCollapseAll)
//
ON_COMMAND(ID_EXTGRID_EXPAND_BRANCH, _OnExpand)
ON_UPDATE_COMMAND_UI(ID_EXTGRID_EXPAND_BRANCH, _OnUpdateExpand)
//
ON_COMMAND(ID_EXTGRID_COLLAPSE_BRANCH, _OnCollapse)
ON_UPDATE_COMMAND_UI(ID_EXTGRID_COLLAPSE_BRANCH, _OnUpdateCollapse)
//
ON_WM_INITMENUPOPUP()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//--------------------------------------------------------------------

CExtendedTreeView::CExtendedTreeView(BOOL bDestroyInfoForDisplay /*= TRUE*/ ) :
        CAbstractTree( bDestroyInfoForDisplay )
{}

//--------------------------------------------------------------------


CExtendedTreeView::~CExtendedTreeView()
{}

//--------------------------------------------------------------------

// Get the tree control for abstract tree

CTreeCtrl & CExtendedTreeView::GetTreeCtrl()
{
    return CTreeView::GetTreeCtrl() ;
}

//--------------------------------------------------------------------

void CExtendedTreeView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}


#ifdef _DEBUG

//--------------------------------------------------------------------

void CExtendedTreeView::AssertValid() const
{
    CTreeView::AssertValid();
}

//--------------------------------------------------------------------

void CExtendedTreeView::Dump(CDumpContext& dc) const
{
    CTreeView::Dump(dc);
}
#endif //_DEBUG

//--------------------------------------------------------------------

CStubDocument * CExtendedTreeView::GetStubDocument() const
{
    CStubDocument * pStubDoc = static_cast<CStubDocument *>(GetParentFrame()->GetActiveDocument()) ;

#ifdef _DEBUG
    static BYTE b = 0 ;
    if ( ! b )
    {
        ASSERT( pStubDoc != NULL && pStubDoc->IsKindOf(RUNTIME_CLASS(CStubDocument)) ) ;
        b = 1 ;
    }
#endif

    return pStubDoc ;
}

//--------------------------------------------------------------------
//Get the Tree Root
HTREEITEM CExtendedTreeView::GetAtLeastTheRoot( CPoint * pHitPoint ) const
{
    UINT      nFlags = 0 ;
    CTreeCtrl &pCtrl = CTreeView::GetTreeCtrl() ;
    HTREEITEM hItem  = pHitPoint != NULL ? pCtrl.HitTest( *pHitPoint , &nFlags ) : (HTREEITEM)NULL ;

    if ( hItem == (HTREEITEM)NULL || ! ( nFlags & TVHT_ONITEM|TVHT_ONITEMBUTTON ) )
    {
        if ( ( hItem = pCtrl.GetSelectedItem() ) == (HTREEITEM)NULL )
        {
            hItem = pCtrl.GetRootItem() ;
        }
    }

    return hItem ;
}

//--------------------------------------------------------------------

//you know the ordre of insertion
HTREEITEM CExtendedTreeView::AddItem( int         nDepth ,
                                      int       * pnLastDepth ,
                                      HTREEITEM * phLastItem ,//last hitem
                                      LPCTSTR     lpszItem  /*=LPSTR_TEXTCALLBACK*/ ,
                                      void      * pItemData /*=NULL to pass any argument you want*/ ,
                                      int         nImage         /*=I_IMAGECALLBACK*/ ,
                                      int         nSelectedImage /*=I_IMAGECALLBACK*/ )
{
    if ( pnLastDepth == NULL || phLastItem == NULL )
    {
        ASSERT(FALSE) ;
        return (HTREEITEM)NULL ;
    }

    HTREEITEM hParent       = TVI_ROOT ;
    HTREEITEM hInsertAfter  = TVI_LAST ;
    CTreeCtrl &pCtrl        = CTreeView::GetTreeCtrl() ;

    if ( pCtrl.GetCount() == 0 ) // Virgin state
    {
        ASSERT( nDepth == 1 ) ; // Only the root is acceptable.
        // Keep initial values.
    }

    else
    {
        ASSERT( phLastItem != NULL ) ;
        if ( nDepth == *pnLastDepth )
        {
            // Siblings are to share the same parent.
            hParent = pCtrl.GetParentItem( *phLastItem ) ;
            ASSERT( nDepth == 1 || hParent != (HTREEITEM)NULL ) ;
        }
        else if ( nDepth > *pnLastDepth )
        {
            ASSERT( nDepth - *pnLastDepth == 1 ) ; // Cannot increase by more than one to the right !
            hParent = *phLastItem ;
        }
        else
        {
            ASSERT( nDepth > 0 ) ;
            hParent = *phLastItem ;
            for ( int nDiff = *pnLastDepth - nDepth + 1 ; nDiff > 0 ; nDiff-- )
            {
                hParent = pCtrl.GetParentItem( hParent ) ;
            }
        }
    }

    HTREEITEM hNewItem = pCtrl.InsertItem( lpszItem , nImage , nSelectedImage , hParent , hInsertAfter ) ;

    if ( hNewItem != (HTREEITEM)NULL )
    {
        *phLastItem = hNewItem ;
        if ( pItemData != NULL )
        {
            if ( pCtrl.SetItemData( hNewItem , reinterpret_cast<DWORD>(pItemData) ) == FALSE )
            {
                pCtrl.DeleteItem( hNewItem ) ;     // It's got to be all
                hNewItem = (HTREEITEM)NULL ; // or nothing
            }

        }

        if ( hNewItem != (HTREEITEM)NULL ) // Still OK.
            *pnLastDepth = nDepth ;
    }

    return hNewItem ;
}


// --------------------------------------------------------------------------

//the window which will receive the item
HRESULT CExtendedTreeView::GetClientWindow( /*[out]*/ CWnd * * ppWnd )
{
    if ( ppWnd != NULL )
    {
        *ppWnd = this ;
        return S_OK ;
    }

    return E_INVALIDARG ;
}

// --------------------------------------------------------------------------

//the window whiwh will receive the messages
HRESULT CExtendedTreeView::GetCmdMsgTargetWindow( /*[out]*/ CWnd * * ppTargetWnd )
{
    if ( ppTargetWnd != NULL )
    {
        *ppTargetWnd = GetParentFrame() ;
        return S_OK ;
    }

    return E_INVALIDARG ;
}

// --------------------------------------------------------------------------

//when click with the right button of the mouse
void CExtendedTreeView::OnRButtonDown(UINT nFlags, CPoint point)
{
    UINT        nItemFlags      = 0 ;
    CTreeCtrl   &pCtrl          = GetTreeCtrl() ;
    HTREEITEM   hItem           = pCtrl.HitTest( point , &nItemFlags ) ;
    BOOL        bAncestorCalled = FALSE ;
    BOOL        bPostPopupMenu  = GetPostPopupMenu() ;

    if ( hItem != (HTREEITEM)NULL && ( nItemFlags & TVHT_ONITEM ) != 0 )
    {
        if ( bPostPopupMenu )
            SetStatus( CAbstractTree::status_no_selchange_timer , TRUE ) ;

        CTreeView::OnRButtonDown(nFlags, point) ;
        bAncestorCalled = true ;

        //Select Item
        pCtrl.Select( hItem , TVGN_CARET ) ;

        if ( bPostPopupMenu )
            SetStatus( CAbstractTree::status_no_selchange_timer , FALSE ) ;
    }

    //Display popup menu
    if ( CanDoPopupMenu() )
    {
        if ( bPostPopupMenu == FALSE )
        {
            ClientToScreen( &point ) ;
            DoPopupMenu( point ) ;
        }
    }
    else if ( bAncestorCalled == FALSE )
    {
        OnRButtonDown(nFlags, point);
    }


}

//--------------------------------------------------------------------
//Implement default popupmenu
BOOL CExtendedTreeView::DoPopupMenu( CPoint clientPoint )
{
    BOOL bOk = FALSE ;

    CMenu menu;

    VERIFY( menu.LoadMenu( IDM_TREE_CHOICE ) );

    CMenu* pPopup = menu.GetSubMenu(0);

    ASSERT(pPopup != NULL);

    DoOwnerContextMenu( pPopup );

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, clientPoint.x, clientPoint.y, AfxGetMainWnd());

    return bOk;
}

// --------------------------------------------------------------------------

//expand all
void CExtendedTreeView::_OnExpandAll()
{
    ExpandAll() ;
}

// --------------------------------------------------------------------------
//enable or not expand all
void CExtendedTreeView::_OnUpdateExpandAll(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CanExpandAll() ) ;
}

// --------------------------------------------------------------------------
//collapse all
void CExtendedTreeView::_OnCollapseAll()
{
    CollapseAll() ;
}

// --------------------------------------------------------------------------
//enable or not collapse all
void CExtendedTreeView::_OnUpdateCollapseAll(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CanCollapseAll() ) ;
}

// --------------------------------------------------------------------------
//expand one branch
void CExtendedTreeView::_OnExpand()
{
    ExpandBranch() ;
}

// --------------------------------------------------------------------------
//enable or not expand one branch
void CExtendedTreeView::_OnUpdateExpand(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CanExpandBranch() ) ;
}

// --------------------------------------------------------------------------
//collapse one branch
void CExtendedTreeView::_OnCollapse()
{
    CollapseBranch() ;
}

// --------------------------------------------------------------------------
//enable or not collapse one branch
void CExtendedTreeView::_OnUpdateCollapse(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CanCollapseBranch() ) ;
}

//--------------------------------------------------------------------

//InitMenu popup
void CExtendedTreeView::OnInitMenuPopup(CMenu* pMenu, UINT, BOOL bSysMenu)
{
    ::CyrLibOnInitMenuPopup( this , pMenu, bSysMenu ) ;
}

// --------------------------------------------------------------------------
// enable to change the default menu
//overload DoOwnerContextMenu
HRESULT CExtendedTreeView::OnPreMenuPopup( /*[in]*/ CMenu * pMenu )
{
    if ( pMenu != NULL )
    {
        ASSERT( ::IsMenu( pMenu->m_hMenu ) ) ;
        DoOwnerContextMenu( pMenu ) ;
        return S_OK ;
    }

    return E_INVALIDARG ;
}

// --------------------------------------------------------------------------

// if you want to have your own menu popup pSubmenu is a pointer
// of the menu tree. You have to overload this function

void CExtendedTreeView::DoOwnerContextMenu( CMenu * pSubMenu )
{
}

// --------------------------------------------------------------------------

//insert items in the tree. You don't need to know the order
// the item are identified by their id and their parent's id
// if the parent is the root. Its parent 's id = 0

BOOL CExtendedTreeView::InsertItem( HTREEITEM & hItem ,
                                    LPCTSTR     lpszItem ,
                                    int         nImage ,
                                    int         nSelectedImage ,
                                    DWORD       dwID ,
                                    DWORD       dwParentID )
{
    return CAbstractTree::InsertItem( hItem , // handle of the new item added
                                      lpszItem , // new item text
                                      nImage ,   // bitmap
                                      nSelectedImage , //selected bitmap
                                      dwID ,            //id associated with the item
                                      dwParentID ) ; // id associated with the parent item
}

//---------------------------------------------------------------------------

//init the style of the window

BOOL CExtendedTreeView::PreCreateWindow(CREATESTRUCT& cs)
{
    cs.style = WS_BORDER|WS_CHILD|WS_VISIBLE|TVS_LINESATROOT|TVS_HASLINES|TVS_HASBUTTONS|TVS_SHOWSELALWAYS ;
    return CTreeView::PreCreateWindow(cs);
}

