#ifndef _CYRALIB_INTERFACES_
#define _CYRALIB_INTERFACES_
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


//--------------------------------------------------------------------
// The Interfaces declared in this file represent a compromise between
// a full-fleged ActiveX approach and a more conservative plain C++
// one. It's purpose is also to gain experience in interface design
// without getting confused with COM-specific implementation code.
// Once proved as working and useful these interfaces should easyly port
// to proper COM if need be.
//--------------------------------------------------------------------
//--------------------------------------------------------------------
// N.B : values are exchanged via parameters that may be "in", "out" or
// "in-out" (IDL), all methodes return a HRESULT the value of witch can
// be found in "winerror.h"
// The SUCCEEDED(), FAILED() and IS_ERROR() should be used for testing
// the return codes.
//--------------------------------------------------------------------

interface IDataGridChild ;

interface ISortGridChild ;

interface ISearchGridChild ;
interface IFilterGridChild ;

interface IColGenInfo ;
interface IColGenInfoSet ;

interface IColSortInfo ;

//--------------------------------------------------------------------
// IGenGrid : Grid operator interface
// ----------

interface IGenGrid
{
    virtual HRESULT GetWnd( /*[out]*/ CWnd * * ppWnd ) const = 0 ;

    virtual HRESULT GetColCount( /*[out]*/ DWORD * pdwColCount ) const = 0 ;

    virtual HRESULT GetRowCount( /*[out]*/ DWORD * pdwRowCount ) const = 0 ;

    // Do the job methods :
    virtual HRESULT Sort( /*[in]*/ ISortGridChild * pSort ) = 0 ;
    // Sort info is a permanent attribute of grid cols :
    virtual HRESULT GetSortInfo( /*[in]*/ DWORD dwCol , /*[out]*/ IColSortInfo * * ppInfo ) = 0 ;
    // If not found, Search() must return S_FALSE otherwise S_OK
    virtual HRESULT Search( /*[in]*/     ISearchGridChild * pSearch ,
                                         /*[in-out]*/ DWORD            * pdwFind ,
                                         /*[in]*/     BOOL               bUp ) = 0 ;
    // 'pFilter == NULL' means "remove filter"
    virtual HRESULT Filter( /*[in]*/  IFilterGridChild * pFilter ,
                                      /*[out]*/ DWORD            * pdwRemains ) = 0 ;

    virtual HRESULT GetColName( /*[in]*/ DWORD dwCol , LPCTSTR * ppczColName ) = 0 ;

    // Length of the string content in cell
    virtual HRESULT GetLength( /*[in]*/  DWORD     dwRow ,
                                         /*[in]*/  DWORD     dwCol ,
                                         /*[out]*/ DWORD   * pdwLen ) const = 0 ;

    // String value in cell
    virtual HRESULT GetCellValue( /*[in]*/  DWORD     dwRow ,
                                            /*[in]*/  DWORD     dwCol ,
                                            /*[out]*/ LPCTSTR * ppczValue ) const = 0 ;

    // Get/set currently selected line
    virtual HRESULT GetCurLine( /*[out]*/ DWORD * pdwLine ) const = 0 ;
    virtual HRESULT SetCurLine( /*[in]*/  DWORD dwLine ) = 0 ;

    // Get supported search modes (so that search dlgs can avoid displaying unavailable options)
    virtual HRESULT GetSupportedSearch( /*[out]*/ BOOL * pbUp ,
            /*[out]*/ BOOL * pbCase ,
            /*[out]*/ BOOL * pbRegExp ,
            /*[out]*/ BOOL * pbMultiCol ) const = 0 ;

    // Operation was performed, display needs to be updated.
    virtual HRESULT Refresh() = 0 ;
} ;

//--------------------------------------------------------------------
// IDataGrid : Centralization interface
// -----------

interface IDataGrid
{
    // Enumerates cols by calling IDataGridChild::AddCol for each
    virtual HRESULT EnumCol( /*[in]*/ IDataGridChild * pDest , /*[out]*/ DWORD * pdwEnumCount ) = 0 ;

    // Gets the grid object interface associated to the data grid instance
    virtual HRESULT GetGrid( /*[out]*/ IGenGrid * * ppGrid ) const = 0 ;

    // Registers the child interface withe the datagrid and and initiates
    // the entry process :
    // MUST call IDataGridChild::RegisterOwner
    // The [in] parameter MAY be NULL, if such is the the case, the 'IDataGrid'
    // implementation MUST provide a default implementation of the child.
    virtual HRESULT OpenSort(   /*[in]*/ ISortGridChild   * pSort ) = 0 ;
    virtual HRESULT OpenSearch( /*[in]*/ ISearchGridChild * pSearch ) = 0 ;
    virtual HRESULT OpenFilter( /*[in]*/ IFilterGridChild * pFilter ) = 0 ;

    virtual HRESULT CloseSort() = 0 ;
    virtual HRESULT CloseSearch() = 0 ;
    virtual HRESULT CloseFilter() = 0 ;

    // If no corresopnding child interface is active E_FAIL is returned
    // and the out parameter-is set to NULL.
    virtual HRESULT GetSortChild(   /*[out]*/ ISortGridChild   * * ppSort ) const = 0 ;
    virtual HRESULT GetSearchChild( /*[out]*/ ISearchGridChild * * ppSearch ) const = 0 ;
    virtual HRESULT GetFilterChild( /*[out]*/ IFilterGridChild * * ppFilter ) const = 0 ;

    // Tells the data grid that a child is being removed from its dependancy.
    // (Usually by being destroyed)
    // IDataGridChild implementations MUST call their owner's (known though
    // RegisterOwner'()' when they are about to be destroyed.
    virtual HRESULT Forget( /*[in]*/ IDataGridChild * pChild ) = 0 ;
} ;

//--------------------------------------------------------------------

interface IDataGridChild
{
    // Called by the owner grid after child creation
    virtual HRESULT RegisterOwner( /*[in]*/ IDataGrid * pOwner ) = 0 ;

    // Adds one column to the list of parent cols
    virtual HRESULT AddCol( /*[in]*/ DWORD dwCol , /*[in]*/ LPCTSTR pczName ) = 0 ;
    // Sets current col (useful for modeless windows)
    virtual HRESULT SetCurCol( /*[in]*/ DWORD dwCol ) = 0 ;
    // Gets col count
    virtual HRESULT GetColCount( /*[in]*/ DWORD * pdwCol ) const = 0 ;


    // Selects col from available ones
    virtual HRESULT SelectCol( /*[in]*/ DWORD dwCol ) = 0 ;
    // Selects col from available ones
    virtual HRESULT UnSelectCol( /*[in]*/ DWORD dwCol ) = 0 ;

    // Gets interface to selected col set :
    virtual HRESULT GetSelectColSet( /*[out]*/ IColGenInfoSet * * ppColInfoSet ) = 0 ;

    virtual HRESULT ShowModal( /*[out]*/ int * pnResult ) = 0 ;
    virtual HRESULT ShowModeless() = 0 ;
    // Are we modal (S_OK) or modeless (S_FALSE) ?
    virtual HRESULT IsModal( /*[out]*/ BOOL * pbModal ) const = 0 ;

    // Modeless close
    virtual HRESULT Close() = 0 ;
} ;

//--------------------------------------------------------------------
// Be it for sort, search, or filter purposes columns related object always
// share a col number :

interface IColGenInfo
{
    virtual HRESULT SetColNum( /*[in]*/ DWORD dwCol ) = 0 ;
    virtual HRESULT GetColNum( /*[out]*/ DWORD * pdwCol ) const = 0 ;
} ;

//--------------------------------------------------------------------

interface IColGenInfoSet
{
    // Col related info are accessible through array-like semantics. Cols numbers
    // cannot be used since nothing prevents the cols to be mutiply represented in a set
    // (Like several search criterions for the same col, for example)
    // Cols can be enumerated inside a loop :
    virtual HRESULT GetAt( /*[in]*/ DWORD dwIndex , /*[out]*/ IColGenInfo * * ppInfo ) = 0 ;
    virtual HRESULT GetCount( /*[out]*/ DWORD * pdwCol ) const = 0 ;

    // Creates a new col info object (for simplicity's sake, individual col info objects
    // are created and managed during their entire lifetime by the container)
    // N.B : After successfull creation, the new object can be accessed via the 'GetAt()' method.
    // Implementation MUST enforce 'pdwPos != NULL', otherwise the position of the new item
    // could be lost.
    virtual HRESULT NewCol( DWORD * pdwPos ) = 0 ;
} ;

//--------------------------------------------------------------------

interface IColSortInfo : public IColGenInfo
{
    virtual HRESULT SetReverse( /*[in]*/  BOOL bReverse ) = 0 ;
    virtual HRESULT GetReverse( /*[out]*/ BOOL * pbReverse ) const = 0 ;
} ;

//--------------------------------------------------------------------

interface IColSortInfoSet : public IColGenInfoSet
{
    // Cols can be enumerated inside a loop :
    virtual HRESULT GetAt( /*[in]*/ DWORD dwIndex , /*[out]*/ IColSortInfo * * ppInfo ) = 0 ;
    // Reminder : virtual HRESULT GetCount( /*[out]*/ DWORD * pdwCol ) const = 0 ;
}
;

//--------------------------------------------------------------------

interface IColSearchInfo : public IColGenInfo
{
    virtual HRESULT SetRegExp( /*[in]*/ BOOL bOn ) = 0 ; // Search regular expression.
    virtual HRESULT SetCase( /*[in]*/ BOOL bOn ) = 0 ;   // Search case-sensitive.

    virtual HRESULT GetRegExp( /*[out]*/ BOOL * pbOn ) const = 0 ;
    virtual HRESULT GetCase( /*[out]*/ BOOL * pbOn ) const = 0 ;

    // Search (filter) arguments :
    // Comparison operators are strings : "<" ">" "<=" ">=" "!="
    virtual HRESULT SetRangeBottom( /*[in]*/ LPCTSTR pczArg , /*[in]*/ LPCTSTR pczCompOp ) = 0 ;
    virtual HRESULT SetRangeTop( /*[in]*/ LPCTSTR pczArg , /*[in]*/ LPCTSTR pczCompOp ) = 0 ;

    virtual HRESULT GetRangeBottom( /*[out]*/ LPCTSTR * ppczArg , /*[out]*/ LPCTSTR * ppczCompOp ) = 0 ;
    virtual HRESULT GetRangeTop( /*[out]*/ LPCTSTR * ppczArg , /*[out]*/ LPCTSTR * ppczCompOp ) = 0 ;

    // Next col info to evaluate through logical operator
    // Supported operators "&&" AND "||" OR and "!"
    virtual HRESULT SetLogOp( /*[in]*/ IColSearchInfo * pOperande , /*[in]*/ LPCTSTR pczLogOp ) = 0 ;
    // NULL arguments are ignored :
    virtual HRESULT GetLogOp( /*[out]*/ IColSearchInfo * * ppOperande , /*[out]*/ LPCTSTR * ppczLogOp ) = 0 ;
} ;

//--------------------------------------------------------------------

interface IColSearchInfoSet : public IColGenInfoSet
{
    // Cols can be enumerated inside a loop :
    virtual HRESULT GetAt( /*[in]*/ DWORD dwIndex , /*[out]*/ IColSearchInfo * * ppInfo ) = 0 ;
    // Reminder : virtual HRESULT GetCount( /*[out]*/ DWORD * pdwCol ) const = 0 ;

    virtual HRESULT SetSearchUp( /*[in]*/ BOOL bUp ) = 0 ;
    virtual HRESULT GetSearchUp( /*[out]*/ BOOL * pbUp ) const = 0 ;

    // IColSearchInfo are added via calls to 'IColSearchInfo::SetLogOp()'
    virtual HRESULT And( /*[out]*/ IColSearchInfo * * ppInfo ) = 0 ;
    virtual HRESULT Or(  /*[out]*/ IColSearchInfo * * ppInfo ) = 0 ;
    virtual HRESULT Not( /*[out]*/ IColSearchInfo * * ppInfo ) = 0 ;
} ;

//--------------------------------------------------------------------

interface ISortGridChild : public IDataGridChild
{
    // This interface is responsible of creating an destroying the 'IColSortInfoSet' object
    virtual HRESULT GetSortInfo( /*[out]*/ IColSortInfoSet * * ppColInfoSet ) = 0 ;

    virtual HRESULT SetSortCol( /*[in]*/ DWORD dwCol , /*[in]*/ BOOL bAscending ) = 0 ;
    virtual HRESULT GetSortCol( /*[in]*/ DWORD dwCol , /*[out]*/ BOOL * pbAscending ) = 0 ;
} ;

//--------------------------------------------------------------------

interface ISearchGridChild : public IDataGridChild
{
    // This interface is responsible of creating an destroying the 'IColSearchInfoSet' object
    virtual HRESULT GetSearchInfo( /*[out]*/ IColSearchInfoSet * * ppInfoSet ) = 0 ;


} ;

//--------------------------------------------------------------------

interface IFilterGridChild : public IDataGridChild
{
    // This interface is responsible of creating an destroying the 'IColSearchInfoSet' object
    virtual HRESULT GetSearchInfo( /*[out]*/ IColSearchInfoSet * * ppInfoSet ) = 0 ;
} ;

//--------------------------------------------------------------------

#endif // _CYRALIB_INTERFACES_
