/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//========================================================================
//
// CECServiceManager.cpp
// ---------------------
//
//    This is the implementation file for the CECServiceStartup class
//
//=========================================================================
#include "stdafx.h"
#include <winsvc.h>

#include "daemoncfg.h"
#include "MainFrm.h"
#include "CECServiceManager.h"
#include "CECServiceStartup.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECServiceManager dialog


CECServiceManager::CECServiceManager(const CString& szCyranoHome, CWnd* pParent /*=NULL*/)
        : CDialog(CECServiceManager::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECServiceManager)
    //}}AFX_DATA_INIT
    m_szCyranoHome = szCyranoHome;
    m_timer = 0;
}


void CECServiceManager::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECServiceManager)
    DDX_Control(pDX, IDC_SERVICES, m_Services);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECServiceManager, CDialog)
//{{AFX_MSG_MAP(CECServiceManager)
ON_BN_CLICKED(IDSTARTUP, OnStartup)
ON_BN_CLICKED(IDSTART, OnStart)
ON_BN_CLICKED(IDREMOVE, OnRemove)
ON_BN_CLICKED(IDINSTALL, OnInstall)
ON_BN_CLICKED(IDSTOP, OnStop)
ON_WM_DESTROY()
ON_NOTIFY(LVN_ITEMCHANGED, IDC_SERVICES, OnItemchanged)
ON_WM_TIMER()
ON_NOTIFY(NM_DBLCLK, IDC_SERVICES, OnDblclkServices)
//}}AFX_MSG_MAP
ON_UPDATE_COMMAND_UI(IDSTARTUP,OnUpdateStartup)
ON_UPDATE_COMMAND_UI(IDSTART,OnUpdateStart)
ON_UPDATE_COMMAND_UI(IDREMOVE,OnUpdateRemove)
ON_UPDATE_COMMAND_UI(IDINSTALL,OnUpdateInstall)
ON_UPDATE_COMMAND_UI(IDSTOP,OnUpdateStop)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECServiceManager message handlers

//
// OnStartup
// =========
//
// Display and Manipulate the startup parameters
void CECServiceManager::OnStartup()
{
    // Get the sel which is selected - if any
    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    ASSERT(pos != NULL);
    int nItem = m_Services.GetNextSelectedItem(pos);

    CECServiceStartup startup;

    startup.SetStartup(m_arServiceDetails[nItem].m_szStartup);
    startup.SetService(m_arServiceDetails[nItem].m_szImage);
    startup.SetName(m_arServiceDetails[nItem].m_szName);

    if (startup.DoModal() == IDOK)
    {
        // Retrieve new state
        m_arServiceDetails[nItem].m_szStartup = startup.GetStartup();
        m_Services.SetItemText(nItem,2,m_arServiceDetails[nItem].m_szStartup);
    }

    DisplayState();
}

//
// OnStart
// =======
//
// Start the service
void CECServiceManager::OnStart()
{
    // Get the sel which is selected - if any
    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    ASSERT(pos != NULL);
    int nItem = m_Services.GetNextSelectedItem(pos);

    // Start the Selected Service
    StartWDTService(nItem);

    // And refresh the List Control
    DisplayState();
}

//
// OnRemove
// ========
//
// Uninstall the service
void CECServiceManager::OnRemove()
{
    // Get the sel which is selected - if any
    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    ASSERT(pos != NULL);
    int nItem = m_Services.GetNextSelectedItem(pos);

    CMainFrame* pMainWnd = (CMainFrame*) AfxGetMainWnd();
    ASSERT(pMainWnd != NULL);

    CString szCommandLine;
    szCommandLine.Format ("%sServer\\%s" ,m_szCyranoHome, m_arServiceDetails[nItem].m_szImage);

    if (!pMainWnd->FileExists (szCommandLine)) return;

    szCommandLine += " -u";

    HANDLE handle;
    DWORD  pid;

    pMainWnd->StartProcess(szCommandLine, &handle, &pid);

    DisplayState();

    // And set the Service Startup field
    CString szOperation;
    VERIFY(szOperation.LoadString(IDS_STARTUP_REMOVED));

    m_arServiceDetails[nItem].m_szStartup = szOperation;
    m_Services.SetItemText(nItem,2,m_arServiceDetails[nItem].m_szStartup);
}

//
// OnInstall
// =========
//
// Install the service
void CECServiceManager::OnInstall()
{
    // Get the sel which is selected - if any
    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    ASSERT(pos != NULL);
    int nItem = m_Services.GetNextSelectedItem(pos);

    CMainFrame* pMainWnd = (CMainFrame*) AfxGetMainWnd();
    ASSERT(pMainWnd != NULL);

    CString szCommandLine;
    szCommandLine.Format ("%sServer\\%s", m_szCyranoHome, m_arServiceDetails[nItem].m_szImage);

    if (!pMainWnd->FileExists (szCommandLine)) return;

    szCommandLine += " -i";

    HANDLE handle;
    DWORD  pid;

    pMainWnd->StartProcess(szCommandLine, &handle, &pid);

    DisplayState();

    // Get Startup
    CString szStartup;
    GetStartup(m_arServiceDetails[nItem].m_szName, szStartup);

    // And update the startup for this service
    m_arServiceDetails[nItem].m_szStartup = szStartup;
    m_Services.SetItemText(nItem,2,m_arServiceDetails[nItem].m_szStartup);
}

//
// OnStop
// ======
//
// Stop the service
void CECServiceManager::OnStop()
{
    // Get the sel which is selected - if any
    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    ASSERT(pos != NULL);
    int nItem = m_Services.GetNextSelectedItem(pos);

    // Stop the service
    StopWDTService(nItem);

    // And refresh the List Box
    DisplayState();
}

//
// OnInitDialog
// ============
//
// Initialize the dialog with all available services
BOOL CECServiceManager::OnInitDialog()
{
    CDialog::OnInitDialog();
    CenterWindow();

    m_Services.InsertColumn(0,_T("Service"),LVCFMT_CENTER,150,0);
    m_Services.InsertColumn(1,_T("Status"),LVCFMT_CENTER,90,1);
    m_Services.InsertColumn(2,_T("Startup"),LVCFMT_CENTER,76,1);
    m_Services.SetExtendedStyle(LVS_EX_FULLROWSELECT);

    // Find all Available Services here
    FindServices();

    // And add them to the list box
    for (int i=0; i<m_arServiceDetails.GetSize(); i++)
    {
        m_Services.InsertItem(i,    m_arServiceDetails[i].m_szName);
        m_Services.SetItemText(i,1, m_arServiceDetails[i].m_szStatus);
        m_Services.SetItemText(i,2, m_arServiceDetails[i].m_szStartup);
    }

    // Start the auto-update timer
    m_timer = SetTimer(1, 1000, NULL);

    DisplayState();

    UpdateData(FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//
// FindServices
// ============
//
// Find all available services returning TRUE to indicate we have found at
// least one service


BOOL CECServiceManager::FindServices()
{
    // Get the path to the server directory
    CString szServerDir;
    szServerDir.Format ("%sServer", m_szCyranoHome);

    // Set up the search path
    CString wdtFiles = szServerDir + "\\CYRWDT*.exe";

    CFileFind wdtFinder;

    // And find the wdt (WHO DID THIS !!!) files
    int bWorking = wdtFinder.FindFile(wdtFiles);
    int nCount = 0;
    while (bWorking)
    {
        bWorking = wdtFinder.FindNextFile();

        // Extract the actual file name
        CString szService = TranslateService(wdtFinder.GetFileTitle());

        // And it it to the Internal Array
        ServiceDetails Service;
        Service.m_szName = szService;
        Service.m_szImage = wdtFinder.GetFileName();

        VERIFY(Service.m_szStatus.LoadString(IDS_SERVICE_UNKNOWN));
        VERIFY(Service.m_szStartup.LoadString(IDS_STARTUP_AUTO));

        m_arServiceDetails.Add(Service);

        nCount++;
    }

    return (!m_arServiceDetails.GetSize()) ? FALSE : TRUE;
}

//
// TranslateService
// ================
//
// This function simply translates any found services with a suitable description
CString CECServiceManager::TranslateService(const CString& szService)
{
    CString szID = szService.Right(3);
    CString szIDName;

    if (szID.Compare("001")==0)
    {
        VERIFY(szIDName.LoadString(IDS_SERVICE001));
        return szIDName;
    }

    VERIFY(szIDName.LoadString(IDS_SERVICEXXX));
    return "Unknown Service";
}

//
// ReportLastError
// ===============
//
// Simply display the last system error in a AfxMessageBox
void CECServiceManager::ReportLastError()
{
    LPTSTR lpMsgBuf;

    FormatMessage(  FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                    NULL,
                    GetLastError(),
                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                    (LPTSTR) &lpMsgBuf,
                    0,
                    NULL);

    AfxMessageBox (lpMsgBuf);
}

//
// OnDestroy
// =========
//
// Kill the timer when leaving the dialog
void CECServiceManager::OnDestroy()
{
    CDialog::OnDestroy();

    // Stop the timer
    if (m_timer != 0)
    {
        KillTimer(1);
    }
}

//
// OnTimer
// =======
//
// Refresh the List box when the timer fires
void CECServiceManager::OnTimer(UINT nIDEvent)
{
    // Update the status
    DisplayState();
}

//
// DisplayState
// ============
//
// Update all items in the Services List box with their current
// status
void CECServiceManager::DisplayState()
{
    // Do this for every item, so

    for (int i=0; i<m_arServiceDetails.GetSize(); i++)
    {
        // Get Status
        CString szStatus;
        GetStatus(m_arServiceDetails[i].m_szName, szStatus);

        // And update the status for this service
        m_arServiceDetails[i].m_szStatus = szStatus;
        m_Services.SetItemText(i,1,m_arServiceDetails[i].m_szStatus);

        // Get Startup
        CString szStartup;
        GetStartup(m_arServiceDetails[i].m_szName, szStartup);

        // And update the startup for this service
        m_arServiceDetails[i].m_szStartup = szStartup;
        m_Services.SetItemText(i,2,m_arServiceDetails[i].m_szStartup);

    }

    UpdateDialogControls(GetParent(),FALSE);
}

//
// OnUpdateStartup
// ===============
//
// Enable the Startup Options
void CECServiceManager::OnUpdateStartup(CCmdUI *pCmdUI)
{
    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    BOOL bState = FALSE;

    if (pos != NULL)
    {
        int nItem = m_Services.GetNextSelectedItem(pos);
        CString szItemStartup = m_arServiceDetails[nItem].m_szStartup;

        CString szRemoved;

        VERIFY (szRemoved.LoadString(IDS_STARTUP_REMOVED));

        // Make sure it is installed
        if ( (szItemStartup.CompareNoCase(szRemoved) != 0))
        {
            bState = TRUE;
        }
    }
    pCmdUI->Enable(bState);
}

//
// OnUpdateStart
// =============
//
// Enable the Start Option
void CECServiceManager::OnUpdateStart(CCmdUI *pCmdUI)
{
    BOOL bState = FALSE;

    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    if (pos != NULL)
    {
        int nItem = m_Services.GetNextSelectedItem(pos);

        CString szItemStatus = m_arServiceDetails[nItem].m_szStatus;
        CString szItemStartup = m_arServiceDetails[nItem].m_szStartup;

        CString szStopped, szPaused, szUnknown;
        CString szRemoved;

        // Identify the allowable states
        VERIFY (szStopped.LoadString(IDS_SERVICE_STOPPED));
        VERIFY (szPaused.LoadString(IDS_SERVICE_PAUSED));
        VERIFY (szUnknown.LoadString(IDS_SERVICE_UNKNOWN));
        VERIFY (szRemoved.LoadString(IDS_STARTUP_REMOVED));

        // First make sure it is not removed
        if ( (szItemStartup.CompareNoCase(szRemoved) != 0))
        {
            if ( (szItemStatus.CompareNoCase(szStopped) == 0)   ||
                    (szItemStatus.CompareNoCase(szPaused) == 0)    ||
                    (szItemStatus.CompareNoCase(szUnknown) == 0))
            {
                bState = TRUE;
            }
        }
    }

    pCmdUI->Enable(bState);
}

//
// OnUpdateRemove
// ==============
//
// Enable the Remove Option
void CECServiceManager::OnUpdateRemove(CCmdUI *pCmdUI)
{
    BOOL bState = FALSE;

    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    if (pos != NULL)
    {
        int nItem = m_Services.GetNextSelectedItem(pos);

        CString szItemStartup = m_arServiceDetails[nItem].m_szStartup;
        CString szItemStatus = m_arServiceDetails[nItem].m_szStatus;
        CString szRemoved;
        CString szStopped, szUnknown;

        // Identify the allowable states
        VERIFY (szRemoved.LoadString(IDS_STARTUP_REMOVED));
        VERIFY (szStopped.LoadString(IDS_SERVICE_STOPPED));
        VERIFY (szUnknown.LoadString(IDS_SERVICE_UNKNOWN));

        if (szItemStartup.CompareNoCase(szRemoved) != 0)
        {
            // However don't allow it if it's running
            if ( (szItemStatus.CompareNoCase(szStopped) == 0)       ||
                    (szItemStatus.CompareNoCase(szUnknown) == 0))
            {
                bState = TRUE;
            }
        }
    }

    pCmdUI->Enable(bState);
}

//
// OnUpdateInstall
// ===============
//
// Enable the Install Option
void CECServiceManager::OnUpdateInstall(CCmdUI *pCmdUI)
{
    BOOL bState = FALSE;

    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    if (pos != NULL)
    {
        int nItem = m_Services.GetNextSelectedItem(pos);

        CString szItemStartup = m_arServiceDetails[nItem].m_szStartup;
        CString szRemoved;

        // Identify the allowable states
        VERIFY (szRemoved.LoadString(IDS_STARTUP_REMOVED));

        if (szItemStartup.CompareNoCase(szRemoved) == 0)
        {
            bState = TRUE;
        }
    }

    pCmdUI->Enable(bState);
}

//
// OnUpdateStop
// ============
//
// Enable the Stop Option
void CECServiceManager::OnUpdateStop(CCmdUI *pCmdUI)
{
    BOOL bState = FALSE;

    POSITION pos = m_Services.GetFirstSelectedItemPosition();
    if (pos != NULL)
    {
        int nItem = m_Services.GetNextSelectedItem(pos);

        CString szItemStatus = m_arServiceDetails[nItem].m_szStatus;
        CString szItemStartup = m_arServiceDetails[nItem].m_szStartup;

        CString szRunning, szPaused, szUnknown;
        CString szRemoved;

        // Identify the allowable states
        VERIFY (szRunning.LoadString(IDS_SERVICE_RUNNING));
        VERIFY (szPaused.LoadString(IDS_SERVICE_PAUSED));
        VERIFY (szUnknown.LoadString(IDS_SERVICE_UNKNOWN));
        VERIFY (szRemoved.LoadString(IDS_STARTUP_REMOVED));

        // First make sure it is not disabled
        if ( (szItemStartup.CompareNoCase(szRemoved) != 0))
        {
            if ( (szItemStatus.CompareNoCase(szRunning) == 0)   ||
                    (szItemStatus.CompareNoCase(szPaused) == 0)    ||
                    (szItemStatus.CompareNoCase(szUnknown) == 0))
            {
                bState = TRUE;
            }
        }
    }

    pCmdUI->Enable(bState);
}

//
// OnItemchanged
// =============
//
// List box selection has changed
void CECServiceManager::OnItemchanged(NMHDR* pNMHDR, LRESULT* pResult)
{
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
    // TODO: Add your control notification handler code here

    *pResult = 0;

    UpdateDialogControls(GetParent(), FALSE);
}

void CECServiceManager::OnDblclkServices(NMHDR* pNMHDR, LRESULT* pResult)
{
    // TODO: Add your control notification handler code here
    *pResult = 0;

    // ONLY Allow if the service is installed
    POSITION pos = m_Services.GetFirstSelectedItemPosition();

    if (pos != NULL)
    {
        int nItem = m_Services.GetNextSelectedItem(pos);
        CString szItemStartup = m_arServiceDetails[nItem].m_szStartup;

        CString szRemoved;

        VERIFY (szRemoved.LoadString(IDS_STARTUP_REMOVED));

        // Make sure it is installed
        if ( (szItemStartup.CompareNoCase(szRemoved) != 0))
        {
            OnStartup();
        }
    }
}

//
// IsRunning
// =========
//
// Searches the service manager for a service and determines if it is
// currently running
BOOL CECServiceManager::IsRunning(const CString& szService)
{
    BOOL bIsRunning = FALSE;

    FindServices();

    for (int i=0; i<m_arServiceDetails.GetSize(); i++)
    {
        if (m_arServiceDetails[i].m_szName.CompareNoCase(szService) == 0)
        {
            CString szStatus;
            return GetStatus(szService, szStatus);
        }
    }

    // Not found so it cannot be running
    return FALSE;
}

//
// GetStatus
// =========
//
// Get the status of a named service. The return value indicates whether or not
// the service is actually in one of the running states
BOOL CECServiceManager::GetStatus(const CString &szService, CString& szStatus)
{
    DWORD dwState = 0xFFFFFFFF;

    // Open the service control manager
    SC_HANDLE hServiceManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
    ASSERT(hServiceManager);

    // And Open the actual service
    SC_HANDLE hActualService = OpenService( hServiceManager,
                                            szService,
                                            SERVICE_ALL_ACCESS);
    if (hActualService)
    {
        // Get the current status
        SERVICE_STATUS status;
        memset(&status, 0, sizeof(status));

        // And perform the query
        BOOL bQuery = QueryServiceStatus(hActualService, &status);

        if(!bQuery)
        {
            ReportLastError();
        }
        else
        {
            dwState = status.dwCurrentState;

            // If the service is running, send a control request
            // to get its current status
            if (dwState == SERVICE_RUNNING)
            {
                bQuery = ::ControlService(  hActualService,
                                            SERVICE_CONTROL_INTERROGATE,
                                            &status);
                if (bQuery)
                {
                    dwState = status.dwCurrentState;
                }
            }
        }

        // Close the service handle
        CloseServiceHandle(hActualService);
    }

    // And Close the service control manager handle
    CloseServiceHandle(hServiceManager);

    // Now see exactly what state the Service is in and set it up
    // in the list box
    BOOL bIsRunning;

    switch (dwState)
    {
    case SERVICE_STOPPED:
        {
            // Set the running flag
            bIsRunning = FALSE;

            VERIFY (szStatus.LoadString(IDS_SERVICE_STOPPED));
            break;
        }
    case SERVICE_START_PENDING:
        {
            // Set the running flag
            bIsRunning = TRUE;

            VERIFY (szStatus.LoadString(IDS_SERVICE_START_PENDING));
            break;
        }
    case SERVICE_STOP_PENDING:
        {
            // Set the running flag
            bIsRunning = TRUE;

            VERIFY (szStatus.LoadString(IDS_SERVICE_STOP_PENDING));
            break;
        }
    case SERVICE_RUNNING:
        {
            // Set the running flag
            bIsRunning = TRUE;

            VERIFY (szStatus.LoadString(IDS_SERVICE_RUNNING));
            break;
        }
    case SERVICE_CONTINUE_PENDING:
        {
            // Set the running flag
            bIsRunning = TRUE;

            VERIFY (szStatus.LoadString(IDS_SERVICE_CONTINUE_PENDING));
            break;
        }
    case SERVICE_PAUSE_PENDING:
        {
            // Set the running flag
            bIsRunning = TRUE;

            VERIFY (szStatus.LoadString(IDS_SERVICE_PAUSE_PENDING));
            break;
        }
    case SERVICE_PAUSED:
        {
            // Set the running flag
            bIsRunning = TRUE;

            VERIFY (szStatus.LoadString(IDS_SERVICE_PAUSED));
            break;
        }
    default:
        {
            // Set the running flag
            bIsRunning = FALSE;

            VERIFY (szStatus.LoadString(IDS_SERVICE_UNKNOWN));
            break;
        }
    }

    return bIsRunning;
}

//
// GetStartup
// ==========
//
// Get the startup status of a named service. The return value indicates whether or not
// the service is manual, automatic or disabled
BOOL CECServiceManager::GetStartup(const CString &szService, CString& szStartup)
{
    DWORD dwStartup = 0xFFFFFFFF;
    DWORD dwBytesNeeded;
    BOOL bKnowStartup = FALSE;
    BOOL bQuery = FALSE;

    // Open the service control manager
    SC_HANDLE hServiceManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
    ASSERT(hServiceManager);

    // And Open the actual service
    SC_HANDLE hActualService = OpenService( hServiceManager,
                                            szService,
                                            SERVICE_QUERY_CONFIG);
    if (hActualService)
    {
        // Get the current status
        LPQUERY_SERVICE_CONFIG startup;

        startup = (LPQUERY_SERVICE_CONFIG) LocalAlloc(LPTR, 4096);

        // And perform the query
        bQuery = QueryServiceConfig(hActualService, startup, 4096, &dwBytesNeeded);

        if(!bQuery)
        {
            ReportLastError();

            // Assume it's removed
            VERIFY (szStartup.LoadString(IDS_STARTUP_REMOVED));
        }
        else
        {
            dwStartup = startup->dwStartType;

        }

        LocalFree(startup);

        // Close the service handle
        CloseServiceHandle(hActualService);
    }

    // And Close the service control manager handle
    CloseServiceHandle(hServiceManager);

    // Now see exactly what startup the Service is in and set it up
    // in the list box
    if(bQuery)
    {
        switch (dwStartup)
        {
        case SERVICE_AUTO_START:
            {
                // Service is Automatic
                bKnowStartup = TRUE;

                VERIFY (szStartup.LoadString(IDS_STARTUP_AUTO));
                break;
            }
        case SERVICE_DEMAND_START:
            {
                // Service is Manual
                bKnowStartup = TRUE;

                VERIFY (szStartup.LoadString(IDS_STARTUP_MANUAL));
                break;
            }
        case SERVICE_DISABLED:
            {
                // Service is Disabled
                bKnowStartup = TRUE;

                VERIFY (szStartup.LoadString(IDS_STARTUP_DISABLED));
                break;
            }
        default:
            {
                // Must be BOOT Start or System Start
                VERIFY (szStartup.LoadString(IDS_STARTUP_REMOVED));
                break;
            }
        }
    }

    return bKnowStartup;
}

//
// StopWDTService
// ==============
//
// Stop a specified service via its name
void CECServiceManager::StopWDTService(const CString &szService)
{
    FindServices();

    for (int i=0; i<m_arServiceDetails.GetSize(); i++)
    {
        if (m_arServiceDetails[i].m_szName.CompareNoCase(szService) == 0)
        {
            StopWDTService(i);
        }
    }
}


//
// StopWDTService
// ==============
//
// Stop a specified service
void CECServiceManager::StopWDTService(int nItem)
{
    // Open the service control manager
    SC_HANDLE hServiceManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
    ASSERT(hServiceManager);

    // Open the service
    SC_HANDLE hActualService = OpenService( hServiceManager,
                                            m_arServiceDetails[nItem].m_szName,
                                            SERVICE_ALL_ACCESS);
    ASSERT(hActualService);

    // And stop the service
    SERVICE_STATUS status;

    BOOL bStop = ControlService(hActualService,
                                SERVICE_CONTROL_STOP,
                                &status);
    if(!bStop)
    {
        // Report last error
        ReportLastError();
    }

    // Close the service handle
    CloseServiceHandle(hActualService);

    // And Close the service control manager handle
    CloseServiceHandle(hServiceManager);
}

//
// StartWDTService
// ===============
//
// Start a specified service via its name
void CECServiceManager::StartWDTService(const CString &szService)
{
    FindServices();

    for (int i=0; i<m_arServiceDetails.GetSize(); i++)
    {
        if (m_arServiceDetails[i].m_szName.CompareNoCase(szService) == 0)
        {
            CString szStatus;
            if (!GetStatus(szService, szStatus))
            {
                // Start the requested service - if it is not already running
                StartWDTService(i);
            }
        }
    }
}

//
// StartWDTService
// ===============
//
// Start a specified service
void CECServiceManager::StartWDTService(int nItem)
{
    // Open the service control manager
    SC_HANDLE hServiceManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
    ASSERT(hServiceManager);

    // Open the service
    SC_HANDLE hActualService = OpenService( hServiceManager,
                                            m_arServiceDetails[nItem].m_szName,
                                            SERVICE_ALL_ACCESS);
    ASSERT(hActualService);

    // Start the service
    BOOL bStart = StartService(hActualService, 0, NULL);
    if(!bStart)
    {
        // Report last error
        ReportLastError();
    }

    // Close the service handle
    CloseServiceHandle(hActualService);

    // Close the service control manager handle
    CloseServiceHandle(hServiceManager);
}
