/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "DaemonCfg.h"
#include "..\Studio\Common\CECRegistryTools.h"
#include "..\include\Solutions.h"
#include "CECServerConfigDlg.h"
#include "CECRegListDlg.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define WM_ICON_NOTIFY          WM_USER+10
#define TWENTY_MEG                      20

#define WMU_PROCESS_CHECK       WM_USER+11
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
//{{AFX_MSG_MAP(CMainFrame)
ON_WM_CREATE()
ON_COMMAND(ID_POPUP_STARTDAEMON, OnPopupStart)
ON_COMMAND(ID_POPUP_STOPDAEMON, OnPopupStop)
ON_UPDATE_COMMAND_UI(ID_POPUP_STARTDAEMON, OnUpdateStart)
ON_UPDATE_COMMAND_UI(ID_POPUP_STOPDAEMON, OnUpdateStop)
ON_COMMAND(ID_APP_EXIT, OnAppExit)
ON_COMMAND(ID_POPUP_CONFIGURE, OnPopupConfigure)
ON_COMMAND(ID_POPUP_REGISTEREDOBJECTS, OnRegisteredObjects)
ON_WM_TIMER()
//}}AFX_MSG_MAP
ON_MESSAGE(WM_ICON_NOTIFY, OnTrayNotification)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
    m_hDataServer = 0;
    m_DataServerPid = 0;
    m_AutoStart = false;
    m_bRunNamesService = FALSE;
    m_bTrace = false;
    m_bNotify = false;
    m_bProcessCheck = false;
    m_bRegObjectsVisible = false;
    m_bConfigureVisible = false;
}

CMainFrame::~CMainFrame()
{}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics


#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
    CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
    CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
        return -1;

    // Create the tray icon
    if (!m_TrayIcon.Create(this, WM_ICON_NOTIFY, _T("OpenSTA Name Server"), NULL, IDR_POPUP_MENU))
        return -1;

    m_TrayIcon.SetIcon(IDI_INACTIVE);

    //
    //    Translate CYRANO_HOME and format the executable name
    //
    char cCyranoHome[256];
    if (GetEnvironmentVariable ("CYRANO_HOME" ,cCyranoHome ,sizeof (cCyranoHome)) <= 0)
    {
        AfxMessageBox (IDS_NOCYRANOHOME);
        return -1;
    }
    m_CyranoHome = cCyranoHome;
    if (m_CyranoHome.Right(1) != '\\') m_CyranoHome += "\\";

    //
    //    Determine if this node is a runtime installation only or not.
    //    If we do not have the FRAMEWORK key and cannot find VDK2 then we are runtime only
    //
    CString DataServerVDK;
    DataServerVDK.Format ("%sServer\\CyrVDK002.dll" ,m_CyranoHome);

    //
    // If we have a VDK002 AND there is a valid registry key then we check to see if are going
    // to start the Omninames server as well as the daemon
    if (FileExists (DataServerVDK))
    {
        //
        // Compare the OmniORB hostname to the machine name. If they are the same, this machine is
        // to be started as the server so "omninames.exe" will be started
        HKEY hRegKey = NULL;
        CString orbHost = "";
        if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, SOLUTIONS_OMNIORB_KEY, 0, KEY_QUERY_VALUE, &hRegKey) == ERROR_SUCCESS)
        {
            DWORD dwType = REG_SZ;
            unsigned char szHost[32];
            unsigned long keySize = 32;

            // Find the key size then get the value
            if ((RegQueryValueEx(hRegKey, _T("ORBInitialHost"), NULL, &dwType, NULL, &keySize) == ERROR_SUCCESS) &&
                    (RegQueryValueEx(hRegKey, _T("ORBInitialHost"), NULL, &dwType, szHost, &keySize) == ERROR_SUCCESS))
            {
                orbHost = szHost;
            }
        }
        RegCloseKey(hRegKey);

        LPCTSTR key = "SYSTEM\\CurrentControlSet\\Control\\ComputerName\\ActiveComputerName";
        hRegKey = NULL;
        CString machineName = "";
        if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, key, 0, KEY_QUERY_VALUE, &hRegKey) == ERROR_SUCCESS)
        {
            DWORD dwType = REG_SZ;
            unsigned char szName[32];
            unsigned long keySize = 32;

            // Find the key size then get the value
            if ((RegQueryValueEx(hRegKey, _T("ComputerName"), NULL, &dwType, NULL, &keySize) == ERROR_SUCCESS) &&
                    (RegQueryValueEx(hRegKey, _T("ComputerName"), NULL, &dwType, szName, &keySize) == ERROR_SUCCESS))
            {
                machineName = szName;
            }
        }
        RegCloseKey(hRegKey);

        if (machineName.CompareNoCase(orbHost) == 0)
        {
            m_bRunNamesService = TRUE;
        }
    }


    // Check for AutStart of the daemon and do it if required
    AutoStart();

    return 0;
}

LRESULT CMainFrame::OnTrayNotification(WPARAM wParam, LPARAM lParam)
{
    // Delegate all the work back to the default implementation in
    // CTrayIcon.
    return m_TrayIcon.OnTrayNotification(wParam, lParam);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame/CTrayIcon menu message handlers

void CMainFrame::OnPopupStart()
{
    m_Stopping = FALSE;


    //
    //    Check the configuration on this machine and allow the user to change it/set it up.
    //
    if (!CheckConfiguration()) return;

    //
    //    Start up the name server if it is installed on this host.
    //
    if (m_bRunNamesService)
    {
        StartNamesService();
    }
    //
    //    ...then start up the dataserve daemon
    //
    if (StartDaemon () != 0) return;

    if (!m_AutoStart) AfxMessageBox(IDS_SERVER_UP);

    //
    //    Set an ACTIVE icon
    //
    m_TrayIcon.SetIcon(IDI_ACTIVE);

    //
    //    Start process checker timer - if requested
    //
    if (m_bNotify)
    {
        SetTimer(WMU_PROCESS_CHECK, 5000, NULL);
    }

}


void CMainFrame::OnPopupStop()
{
    if (AfxMessageBox (IDS_CONFIRM_STOP ,MB_YESNOCANCEL) == IDYES)
    {
        m_Stopping = TRUE;
        StopDaemon();
        AfxMessageBox (IDS_SERVER_DOWN);
        m_TrayIcon.SetIcon(IDI_INACTIVE);
        if (m_bNotify)
        {
            KillTimer(WMU_PROCESS_CHECK);
        }
    }
}

void CMainFrame::OnUpdateStart(CCmdUI* pCmdUI)
{
    pCmdUI->Enable (m_DataServerPid == 0);
}

void CMainFrame::OnUpdateStop(CCmdUI* pCmdUI)
{
    //
    //    Stop is only available if the process is running.
    //
    pCmdUI->Enable (m_DataServerPid != 0);
}

//
// StartNamesService
// =================
//
// Start the CYRANO Names service if Installed
//
void CMainFrame::StartNamesService()
{
    m_OmniOrb.Run();
}

//
// StartDaemon
// ===========
//
// Start the data server daemon names service if present
//
int CMainFrame::StartDaemon()
{
    //
    // Get the execuable name for the dataserve daemon.
    CString ImageName;
    ImageName.Format ("%sServer\\cyrdmn.exe" ,m_CyranoHome);


    // If tracing is on then redirect output to a local trace file
    if (m_bTrace)
    {
        //
        // And start a new log with maximum tracing
        ImageName += " --trace 1000";
    }

    //
    // ...and run it.
    return StartProcess (ImageName ,&m_hDataServer ,&m_DataServerPid);
}

//
// StartProcess
// ============
//
// Start up the specified process and return its pid and handle
//
int CMainFrame::StartProcess (CString& Command ,HANDLE* pReturnHandle ,DWORD* pReturnPid)
{
    CString     ErrorBuffer;
    STARTUPINFO StartInfo = {sizeof(StartInfo)};

    STARTUPINFO si;
    ZeroMemory(&si,sizeof(STARTUPINFO));
    si.cb = sizeof(STARTUPINFO);
    si.dwFlags = STARTF_USESTDHANDLES|STARTF_USESHOWWINDOW;
    si.hStdOutput = NULL;
    si.hStdInput  = NULL;
    si.hStdError  = NULL;
    si.wShowWindow = SW_HIDE;

    // Change the current directory to the server directory
    CString strServerDir;
    strServerDir.Format ("%sServer" ,m_CyranoHome);

    ::SetCurrentDirectory(strServerDir.GetBuffer(0));

    // Create the process to execute the command
    PROCESS_INFORMATION pi;
    if (!CreateProcess(NULL,
                       Command.GetBuffer(0), // command line
                       NULL,                // process security attributes
                       NULL,                // primary thread security attributes
                       TRUE,                // handles are inherited
                       CREATE_NEW_CONSOLE,  // creation flags
                       NULL,                // use parent's environment
                       strServerDir.GetBuffer(0),       // Current directory
                       &si,                 // STARTUPINFO pointer
                       &pi))                // receives PROCESS_INFORMATION
    {
        ErrorBuffer.Format ("Error invoking the image %s" ,Command);
        LPTSTR lpMsgBuf;

        FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                       NULL,
                       GetLastError(),
                       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                       (LPTSTR) &lpMsgBuf,
                       0,
                       NULL);
        AfxMessageBox (lpMsgBuf);

        //
        //    Free the buffer.
        //
        LocalFree( lpMsgBuf );
        return -1;
    }

    //
    //   ...and save the process details.
    //
    *pReturnHandle = pi.hProcess;
    *pReturnPid = pi.dwProcessId;
    return 0;
}

//
// StopNamesService
// ================
//
// Stop OmniNames
void CMainFrame::StopNamesService ()
{
    m_OmniOrb.OnStop();
}

//
// StopDaemon
// ==========
//
// Stop the Data Server Daemon
void CMainFrame::StopDaemon ()
{
    if (m_hDataServer != 0)
    {
        TerminateProcess (m_hDataServer ,0);
        m_hDataServer = 0;
        m_DataServerPid = 0;
    }

    // And delete any old logs - if we are finishing correctly then their is no point
    // keeping these files anyway.

    // If the daemon has fallen over then we will never reach this point and our log
    // files will remain in the server directory
    DeleteUsedLogs ();
}

void CMainFrame::OnAppExit()
{
    // First we see if VDK002 is on the system and the names service is running
    if (!m_bRunNamesService)
    {
     	if (m_hDataServer != 0)
        {
            if (AfxMessageBox (IDS_EXIT_UTILITY ,MB_YESNOCANCEL) != IDYES) return;
        }
    }
    else
    {
        CString szNamesService;
        VERIFY (szNamesService.LoadString(IDS_SERVICE001));

		if(m_OmniOrb.IsRunning())
		{
		    if (AfxMessageBox (IDS_SHUTDOWN_UTILITY ,MB_YESNOCANCEL) != IDYES) return;

            // So stop the Names Service
            StopNamesService();
        }

    }

    // And always stop the daemon
    StopDaemon ();

    m_TrayIcon.RemoveIcon();

    ::exit(0);
}
/*
int CMainFrame::DeleteLogs()
{
    char cOmniLogDir[256];
    if (GetEnvironmentVariable ("OMNINAMES_LOGDIR" ,cOmniLogDir ,sizeof (cOmniLogDir)) <= 0)
    {
        AfxMessageBox (IDS_NOOMNILOGDIR);
        return -1;
    }
    CString OmniLogDir = cOmniLogDir;
    if (OmniLogDir.Right(1) != '\\') OmniLogDir += "\\";
 
//
//    As a sanity check - ensure that the log directory exists and if it does not create it.
//
    DirExists (OmniLogDir ,TRUE);
 
//
//    Get the machine name as the log files use this.
//
    char ComputerName[MAX_COMPUTERNAME_LENGTH + 1];
    DWORD ComputerNameLength = MAX_COMPUTERNAME_LENGTH + 1;
    GetComputerName (ComputerName, &ComputerNameLength);
 
//
//    ...and format the names of the log files and delete them if they exist.
//
    CString LogFileName;
    LogFileName.Format("%somninames-%s.bak" ,OmniLogDir ,ComputerName);
    if (FileExists (LogFileName))
    {
        TRY
        {
            CFile::Remove( LogFileName );
        }
        
        CATCH( CFileException, e )
        {
            #ifdef _DEBUG
                afxDump << "File " << LogFileName << " cannot be removed\n";
            #endif
        }
        END_CATCH
    }
 
    LogFileName.Format("%somninames-%s.log" ,OmniLogDir ,ComputerName);
    if (FileExists (LogFileName))
    {
        TRY
        {
            CFile::Remove( LogFileName );
        }
        
        CATCH( CFileException, e )
        {
            #ifdef _DEBUG
                afxDump << "File " << LogFileName << " cannot be removed\n";
                #endif
        }
        END_CATCH
    }
    return 0;
}
*/
//
// DeleteUsedLogs
// ==============
//
// Delete old Daemon and TP Logs
//
void CMainFrame::DeleteUsedLogs()
{
    CString filename;

    try
    {
        CString strServerDir;
        strServerDir.Format ("%sServer" ,m_CyranoHome);

        //
        //    ...and format the names of the log files and delete them if they exist.
        //
        CString LogFiles = strServerDir + "\\cyrdmn_*.log";

        CFileFind Logfinder;
        int bWorking = Logfinder.FindFile(LogFiles);
        while (bWorking)
        {
            bWorking = Logfinder.FindNextFile();

            // Extract the actual file name
            filename=Logfinder.GetFileName();

            // And blitz it
            DeleteFile( filename );
        }

        //
        //    ...and format the names of the tplog files and delete them if they exist.
        //
        CString ErrFiles;
        ErrFiles = strServerDir + "\\TP_*.err";

        CFileFind Errfinder;
        bWorking = Errfinder.FindFile(ErrFiles);

        while (bWorking)
        {
            bWorking = Errfinder.FindNextFile();

            // Extract the actual file name
            filename=Errfinder.GetFileName();

            // And blitz it
            DeleteFile( filename );
        }
    }
    catch( ... )
    {
#ifdef _DEBUG
        afxDump << "File " << filename << " cannot be removed\n";
#endif
    }
}


void    CMainFrame::AutoStart ()
{
    CString KeyName = SOLUTIONS_SERVER_KEY;
    CString ValueName = "AutoStart";

    CECRegistryTools reg;
    CString AutoStart = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
    if (AutoStart == "Yes")
    {
        m_AutoStart = true;
        OnPopupStart();
    }
}


bool CMainFrame::CheckConfiguration ()
{
    //    First we must check to see if we have enough disk space on this machine

    //    This is obviously dependent on the usage of the daemon but as a sanity
    //    check we must ensure we have at least 20 Meg of free space before attempting
    //    to do anything meaningful
    if (!CheckDiskSpace())
        return false;
    //
    //    The server configuration depends on whether or not we have the dataserve daemon on this
    //    machine (CyrVDK002.dll). If this present we must have the server key set up in the registry
    //    If we are not a dataserver, then all we are interested in is the name service port
    //
    bool ReturnStatus = true;

    //
    //    If we have a dataserver - check for the following keys in the server key
    //
    //      ProjectDir - Work directory for the project
    //      ServerAddress
    //      ServerPort
    //
    //    If any of these are missing we will ask the user to specify them.
    //
    CECRegistryTools reg;
    CString ConfigError;
    bool ConfigurationOK = true;
    CString KeyName ,ValueName ,KeyValue;

    do
    {

        ConfigError = "";

        //
        //    First check ProjectDir
        //
        KeyName = SOLUTIONS_SERVER_KEY;
        ValueName = "ProjectDir";
        KeyValue = reg.GetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName);
        if (KeyValue.IsEmpty())
        {
            char ProjectDir[256];
            GetEnvironmentVariable ("CYRANO_HOME" ,ProjectDir ,sizeof (ProjectDir));

            (void) strcat (ProjectDir, "Projects\\");

            reg.SetStringValue(HKEY_CURRENT_USER ,KeyName ,"ProjectDir", ProjectDir);
        }
        //
        //    Now the tracing
        //
        KeyName = SOLUTIONS_SERVER_KEY;
        ValueName = "Tracing";
        CString szTrace = reg.GetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName);
        m_bTrace = (szTrace.CompareNoCase("1") == 0) ? 1 : 0;
        //
        //    Now the notification
        //
        KeyName = SOLUTIONS_SERVER_KEY;
        ValueName = "Notify";
        CString szNotify = reg.GetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName);
        m_bNotify = (szNotify.CompareNoCase("1") == 0) ? 1 : 0;

        //
        //    ...then OmniOrb Port
        //
        KeyName = SOLUTIONS_OMNIORB_KEY;
        ValueName = "ORBInitialPort";
        KeyValue = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
        if (KeyValue.IsEmpty())
        {
            // Try to set a default
            reg.SetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName, "1250", HKEY_LOCAL_MACHINE);
        }

        //
        //    ...then OmniOrb Host
        //
        // First declare a default which we can use if none of the other host details are set up.
        CString szDefaultHost;

        KeyName = SOLUTIONS_OMNIORB_KEY;
        ValueName = "ORBInitialHost";
        szDefaultHost = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
        if (szDefaultHost.IsEmpty())
        {
            // Try to set a default - the local machine
            CString szKeyName = "SYSTEM\\CurrentControlSet\\Control\\ComputerName\\ActiveComputerName";

            szDefaultHost = reg.GetStringValue(HKEY_LOCAL_MACHINE ,szKeyName ,"ComputerName");

            reg.SetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName, szDefaultHost.GetBuffer(0), HKEY_LOCAL_MACHINE);
        }

        //
        //    ...then ServerAddress
        //
        ValueName = "ServerAddress";
        KeyName = SOLUTIONS_SERVER_KEY;
        KeyValue = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
        if (KeyValue.IsEmpty())
        {
            // Try to set a default - the default
            reg.SetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName, szDefaultHost.GetBuffer(0), HKEY_LOCAL_MACHINE);
        }
        //
        //    ...then ServerName
        //
        ValueName = "ServerName";
        KeyValue = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
        if (KeyValue.IsEmpty())
        {
            // Try to set a default - the default
            reg.SetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName, szDefaultHost.GetBuffer(0), HKEY_LOCAL_MACHINE);
        }

        //
        //    ...then ServerPort
        //
        ValueName = "ServerPort";
        KeyValue = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
        if (KeyValue.IsEmpty())
        {
            // Try to set a default
            reg.SetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName, "49201", HKEY_LOCAL_MACHINE);
        }

        //
        //    If we have had any configuration errors then throw up the configuration dialog
        //
        if (!ConfigError.IsEmpty())
        {
            AfxMessageBox(ConfigError);
            CECServerConfigDlg ConfigureDlg(m_bRunNamesService);
            if (ConfigureDlg.DoModal() == IDOK)
            {
                m_bTrace = ConfigureDlg.IsTraceOn();
                m_bNotify = ConfigureDlg.IsNotifyOn();
            }
            else
            {
                ReturnStatus = false;
            }
        }
    }

    //
    //    Loop again if we have configuration errors and have not canceled from the configuration dialog
    //
    while ((!ConfigurationOK) && (ReturnStatus));

    return ReturnStatus;
}

//
// CheckDiskSpace
// ==============
//
// This function checks to see if we have a basic amount of space available to use for
// logging etc. As this is totally dependent on the tests to be run all we can do is ensure
// we have at least 20 Meg of space availble which should be sufficient for most tests
bool CMainFrame::CheckDiskSpace()
{
    // the only real way of finding out free disk space is calling
    // GetDiskFreeSpaceExA, but it doesn't exist on Win95 - which we dont yet support
    HINSTANCE hInstance = LoadLibraryA("kernel32.dll");
    if (hInstance)
    {
        ULARGE_INTEGER ulA, ulB, ulFreeBytes;

        typedef BOOL (WINAPI *MyFunc)(LPCSTR RootName, PULARGE_INTEGER pulA, PULARGE_INTEGER pulB, PULARGE_INTEGER pulFreeBytes);
        MyFunc pfnGetDiskFreeSpaceEx = (MyFunc)GetProcAddress(hInstance, "GetDiskFreeSpaceExA");
        FreeLibrary(hInstance);

        if (pfnGetDiskFreeSpaceEx)
        {
            if (!pfnGetDiskFreeSpaceEx(NULL, &ulA, &ulB, &ulFreeBytes))
                return false;

            long lFreeSize = ulFreeBytes.QuadPart / (1024*1024);

            if (lFreeSize < TWENTY_MEG)
            {
                CString szQuestion;
                szQuestion.Format("You only have %d Megabytes of disk space left on this drive.\n\n"
                                  "The CYRANO Daemon requires at least %d Megabytes of free disk space "
                                  "before you can execute tests on this injector.\n\n"
                                  "Do you wish to continue ?", lFreeSize, TWENTY_MEG);
                if (AfxMessageBox(szQuestion, MB_YESNO) == IDNO)
                {
                    return FALSE;
                }
            }

        }
    }

    return TRUE;
}

BOOL CMainFrame::FileExists(CString &FileName)
{
    CFileStatus FileStatus;
    return CFile::GetStatus(FileName, FileStatus);
}

BOOL CMainFrame::DirExists(CString &DirPath ,BOOL CreateIfNot)
{
    CFileStatus fs;
    BOOL bExists = CFile::GetStatus(DirPath+".", fs);
    if(bExists && (CreateIfNot))
    {
        return CreateDirectory(DirPath, NULL);
    }
    return bExists;
}

void CMainFrame::OnPopupConfigure()
{
    if (!m_bConfigureVisible)
    {
        m_bConfigureVisible = TRUE;
        CECServerConfigDlg ConfigureDlg(m_bRunNamesService);
        if (ConfigureDlg.DoModal())
        {
            m_bTrace = ConfigureDlg.IsTraceOn();
            m_bNotify = ConfigureDlg.IsNotifyOn();
        }
        m_bConfigureVisible = FALSE;
    }
}

void CMainFrame::OnRegisteredObjects()
{
    if (!m_bRegObjectsVisible)
    {
        m_bRegObjectsVisible = TRUE;
        CECRegListDlg dlg;
        dlg.DoModal();
        m_bRegObjectsVisible = FALSE;
    }
}

void CMainFrame::OnTimer(UINT nIDEvent)
{
    if (m_Stopping) return;

    BOOL NSFallOver = FALSE;
    BOOL DmnFallOver = FALSE;

    // Check to see if the Names Service has fallen over
    if (m_bRunNamesService)
    {
		NSFallOver = !m_OmniOrb.IsRunning();
    }

    // Check to see if the Daemon has fallen over
    if (m_hDataServer != 0)
    {
        if (WaitForSingleObject(m_hDataServer, 0) == WAIT_OBJECT_0)
            DmnFallOver = TRUE;
    }

    //
    //    If either of the processes that should be running but have fallen over, ensure both
    //    are stopped and change our icon.
    //
    if (NSFallOver || DmnFallOver)
    {
        KillTimer(WMU_PROCESS_CHECK);
        StopDaemon();
        if(NSFallOver)
        {
            AfxMessageBox (IDS_NSFAIL, MB_SYSTEMMODAL);
        }
        else
        {
            AfxMessageBox (IDS_DAEMONFAIL, MB_SYSTEMMODAL);
        }
        m_TrayIcon.SetIcon(IDI_FALLOVER);
    }
    CFrameWnd::OnTimer(nIDEvent);
}
