/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef __GATEWAY_CONSOLE_COMMAND__
#define __GATEWAY_CONSOLE_COMMAND__


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __MTSAFE__
#  include <mtsafe.hpp>
#endif  // __MTSAFE__

#ifndef __SOCKET__
#  include <socket.hpp>
#endif  // __SOCKET__


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////// Class CGWho //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCONSCMD CGWho : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGWho ()
: lTaskId_ (0), lParentTaskId_ (0), lType_ (0)
    { }

    CGWho (const long lTaskId, const long lParentTaskId, const long lType, const string& clientAddress, const string& gatewayAddress, const string& targetAddress, const string& library)
: lTaskId_ (lTaskId), lParentTaskId_ (lParentTaskId), lType_ (lType), clientAddress_ (clientAddress), gatewayAddress_ (gatewayAddress), targetAddress_ (targetAddress), library_ (library)
    { }

    CGWho (const CGWho& who)
: lTaskId_ (who.lTaskId_), lParentTaskId_ (who.lParentTaskId_), lType_ (who.lType_), clientAddress_ (who.clientAddress_), gatewayAddress_ (who.gatewayAddress_), targetAddress_ (who.targetAddress_), library_ (who.library_)
    { }

    ~CGWho ()
    { }


public:                         // Operators.
    CGWho&                      operator = (const CGWho& who)
    {
        lTaskId_ = who.lTaskId_ ;
        lParentTaskId_ = who.lParentTaskId_ ;
        lType_ = who.lType_ ;
        clientAddress_ = who.clientAddress_ ;
        gatewayAddress_ = who.gatewayAddress_ ;
        targetAddress_ = who.targetAddress_ ;
        library_ = who.library_ ;
        return *this ;
    }
    bool                        operator == (const CGWho& who)
    {
        return (lTaskId_ == who.lTaskId_) ;
    }

public:                         // Get methodes.
    long&                       getTaskId (void)
    {
        return lTaskId_ ;
    }
    long&                       getParentTaskId (void)
    {
        return lParentTaskId_ ;
    }
    long&                       getType (void)
    {
        return lType_ ;
    }
    string&                 getClientAddress (void)
    {
        return clientAddress_ ;
    }
    string&                 getGatewayAddress (void)
    {
        return gatewayAddress_ ;
    }
    string&                 getTargetAddress (void)
    {
        return targetAddress_ ;
    }
    string&                 getLibrary (void)
    {
        return library_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    long                        lTaskId_ ;
    long                        lParentTaskId_ ;
    long                        lType_ ;
    string                  clientAddress_ ;
    string                  gatewayAddress_ ;
    string                  targetAddress_ ;
    string                  library_ ;

    //-----------------------------------------------------------------------------
}

; // CGWho

inline CTSocket& operator<< (CTSocket& socket, CGWho& who)
{
    socket << who.getTaskId () << who.getParentTaskId () << who.getType () << who.getClientAddress () << who.getGatewayAddress () << who.getTargetAddress () << who.getLibrary () ;
    return socket ;
} // operator<<

inline CTSocket& operator>> (CTSocket& socket, CGWho& who)
{
    socket >> who.getTaskId () >> who.getParentTaskId () >> who.getType () >> who.getClientAddress () >> who.getGatewayAddress () >> who.getTargetAddress () >> who.getLibrary () ;
    return socket ;
} // operator>>

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////// End of Class CGWho //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CGWhoList //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCONSCMD CGWhoList : public vector<CGWho>, virtual public CTMTSafe
    {}

; // CGWhoList

inline CTSocket& operator<< (CTSocket& socket, CGWhoList& whoList)
{
    socket << (long) whoList.size () ;
    for (size_t i = 0 ; i  < whoList.size () ; ++ i)
    {
        socket << whoList[i] ;
    }
    return socket ;
} // operator<<

inline CTSocket& operator>> (CTSocket& socket, CGWhoList& whoList)
{
    long    lEntries ;
    socket >> lEntries ;
    for (size_t i = 0 ; i  < (size_t) lEntries ; ++ i)
    {
        CGWho   who ;
        socket >> who ;
        whoList.push_back (who) ;
    }
    return socket ;
} // operator>>

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CGWhoList //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CGListen //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCONSCMD CGListen : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGListen ()
: bAdministration_ (false), bTargetAddressAutoDetection_ (true)
    { }

    CGListen (const bool bAdministration, const string& targetAddress, const string& listenPort, const string& library, const bool bTargetAddressAutoDetection)
: bAdministration_ (bAdministration), targetAddress_ (targetAddress), listenPort_ (listenPort), library_ (library), bTargetAddressAutoDetection_ (bTargetAddressAutoDetection || targetAddress.empty () )
    { }

    CGListen (const CGListen& listen)
: bAdministration_ (listen.bAdministration_), targetAddress_ (listen.targetAddress_), listenPort_ (listen.listenPort_), library_ (listen.library_), bTargetAddressAutoDetection_ (listen.bTargetAddressAutoDetection_)
    { }

    ~CGListen ()
    { }


public:                         // Operators.
    CGListen&                   operator = (const CGListen& listen)
    {
        bAdministration_ = listen.bAdministration_ ;
        targetAddress_ = listen.targetAddress_ ;
        listenPort_ = listen.listenPort_ ;
        library_ = listen.library_ ;
        bTargetAddressAutoDetection_ = listen.bTargetAddressAutoDetection_ ;
        return *this ;
    }
    bool                        operator == (const CGListen& listen)
    {
        return (listenPort_ == listen.listenPort_) ;
    }

public:                         // Get methodes.
    bool&                       getIsAdministrationListen (void)
    {
        return bAdministration_ ;
    }
    string&                 getTargetAddress (void)
    {
        return targetAddress_ ;
    }
    string&                 getPort (void)
    {
        return listenPort_ ;
    }
    string&                 getLibrary (void)
    {
        return library_ ;
    }
    bool&                       getTargetAddressAutoDetectionMode (void)
    {
        return bTargetAddressAutoDetection_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    bool                        bAdministration_ ;
    string                  targetAddress_ ;
    string                  listenPort_ ;
    string                  library_ ;
    bool                        bTargetAddressAutoDetection_ ;

    //-----------------------------------------------------------------------------
}

; // CGListen

inline CTSocket& operator<< (CTSocket& socket, CGListen& listen)
{
    socket << listen.getIsAdministrationListen () << listen.getTargetAddress () << listen.getPort () << listen.getLibrary () << listen.getTargetAddressAutoDetectionMode () ;
    return socket ;
} // operator<<

inline CTSocket& operator>> (CTSocket& socket, CGListen& listen)
{
    socket >> listen.getIsAdministrationListen () >> listen.getTargetAddress () >> listen.getPort () >> listen.getLibrary () >> listen.getTargetAddressAutoDetectionMode () ;
    return socket ;
} // operator>>

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// End of Class CGListen //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////// Class CGKill //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCONSCMD CGKill : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGKill ()
: lTaskId_ (0), bChildrenAlso_ (false)
    { }

    CGKill (const long lTaskId, const bool bChildrenAlso = false)
: lTaskId_ (lTaskId), bChildrenAlso_ (bChildrenAlso)
    { }

    CGKill (const CGKill& kill)
: lTaskId_ (kill.lTaskId_), bChildrenAlso_ (kill.bChildrenAlso_)
    { }

    ~CGKill ()
    { }


public:                         // Operators.
    CGKill&                     operator = (const CGKill& kill)
    {
        lTaskId_ = kill.lTaskId_ ;
        bChildrenAlso_ = kill.bChildrenAlso_ ;
        return *this ;
    }
    bool                        operator == (const CGKill& kill)
    {
        return (lTaskId_ == kill.lTaskId_) ;
    }

public:                         // Get methodes.
    long&                       getTaskId (void)
    {
        return lTaskId_ ;
    }
    bool&                       getChildrenAlso (void)
    {
        return bChildrenAlso_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    long                        lTaskId_ ;
    bool                        bChildrenAlso_ ;

    //-----------------------------------------------------------------------------
}

; // CGKill

inline CTSocket& operator<< (CTSocket& socket, CGKill& kill)
{
    socket << kill.getTaskId () << kill.getChildrenAlso () ;
    return socket ;
} // operator<<

inline CTSocket& operator>> (CTSocket& socket, CGKill& kill)
{
    socket >> kill.getTaskId () >> kill.getChildrenAlso () ;
    return socket ;
} // operator>>

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////// End of Class CGKill //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////// Class CGShutdown //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCONSCMD CGShutdown : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGShutdown ()
: bWithNoWait_ (false)
    { }

    CGShutdown (const bool bWithNoWait)
: bWithNoWait_ (bWithNoWait)
    { }

    CGShutdown (const CGShutdown& shutdown)
: bWithNoWait_ (shutdown.bWithNoWait_)
    { }

    ~CGShutdown ()
    { }


public:                         // Operators.
    CGShutdown&                 operator = (const CGShutdown& shutdown)
    {
        bWithNoWait_ = shutdown.bWithNoWait_ ;
        return *this ;
    }

public:                         // Get methodes.
    bool&                       getWithNoWait (void)
    {
        return bWithNoWait_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    bool                        bWithNoWait_ ;

    //-----------------------------------------------------------------------------
}

; // CGShutdown

inline CTSocket& operator<< (CTSocket& socket, CGShutdown& shutdown)
{
    socket << shutdown.getWithNoWait () ;
    return socket ;
} // operator<<

inline CTSocket& operator>> (CTSocket& socket, CGShutdown& shutdown)
{
    socket >> shutdown.getWithNoWait () ;
    return socket ;
} // operator>>

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////// End of Class CGShutdown //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////// Class CGTrace //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCONSCMD CGTrace : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGTrace ()
: bOn_ (false)
    { }

    CGTrace (const bool bOn)
: bOn_ (bOn)
    { }

    CGTrace (const CGTrace& trace)
: bOn_ (trace.bOn_)
    { }

    ~CGTrace ()
    { }


public:                         // Operators.
    CGTrace&                    operator = (const CGTrace& trace)
    {
        bOn_ = trace.bOn_ ;
        return *this ;
    }

public:                         // Get methodes.
    bool&                       getOn (void)
    {
        return bOn_ ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.
    bool                        bOn_ ;

    //-----------------------------------------------------------------------------
}

; // CGTrace

inline CTSocket& operator<< (CTSocket& socket, CGTrace& trace)
{
    socket << trace.getOn () ;
    return socket ;
} // operator<<

inline CTSocket& operator>> (CTSocket& socket, CGTrace& trace)
{
    socket >> trace.getOn () ;
    return socket ;
} // operator>>

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////// End of Class CGTrace //
///////////////////////////////////////////////////////////////////////////////

#endif  // __GATEWAY_CONSOLE_COMMAND__
