/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// OutputFindView.cpp : implementation file
//

#include "stdafx.h"
#include "Modeller.h"
#include "VariousObject.h"
#include "OutputFindView.h"

#include "Constantes.h"
#include "ModellerDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CModellerApp theApp ;


//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(COutputFindExtendedEditCtrl, COSEditCtrl)
//{{AFX_MSG_MAP(COutputFindExtendedEditCtrl)
ON_WM_LBUTTONDBLCLK()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

COutputFindExtendedEditCtrl::COutputFindExtendedEditCtrl(CView* pParent ,LPCSTR pczRegistryPath  ):
        COSEditCtrl( pczRegistryPath ),
        m_pParent( pParent )
{}

//---------------------------------------------------------------------------


COutputFindExtendedEditCtrl::~COutputFindExtendedEditCtrl()
{}



//---------------------------------------------------------------------------


void COutputFindExtendedEditCtrl::DoOwnerContextMenu( CMenu *pMenu )
{
}

//---------------------------------------------------------------------------

void COutputFindExtendedEditCtrl::OnLButtonDblClk(UINT nFlags, CPoint point)
{
    int nCurLine;

    nCurLine = GetCurrentLine();
    SelectLine( nCurLine -1);

    CString strText = GetString( nCurLine, nCurLine );
	if(strText.Left(13) == "Searching for")
		return;

	int nPos = strText.Find("):");
	if(nPos == -1)
		return;

	char *szFile = strText.GetBufferSetLength(nPos+1);
	szFile[nPos] = NULL;

	char *szLine = strrchr(szFile,'(');
	*szLine = NULL;
	int nLine = atoi(++szLine);

	CModellerApp *pApp = (CModellerApp*)AfxGetApp();
	pApp->OpenFile(szFile);

	CMDIFrameWnd *pParent = (CMDIFrameWnd*)AfxGetMainWnd();
	CWnd *pMDIClient = CWnd::FromHandle(pParent->m_hWndMDIClient);

	CWnd *pChild = pMDIClient->GetWindow(GW_CHILD);
	while(pChild)
	{
		CBasedDocument* pDoc = (CBasedDocument*)((CFrameWnd*)pChild)->GetActiveDocument();
        if(pDoc->GetSCLDocInfo()->GetOriginalPath().CompareNoCase(szFile)==0)
		{
			//Set the line
			pDoc->SetErrorLine( nLine-1 ) ;
			return;
		}
		
		pChild = pChild->GetWindow(GW_HWNDNEXT);
	}
	
    COSEditCtrl::OnLButtonDblClk(nFlags, point);
}


//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(COutputFindView, CView)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(COutputFindView, CView)
//{{AFX_MSG_MAP(COutputFindView)
ON_WM_CREATE()
ON_WM_SIZE()
ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
ON_COMMAND(ID_EDIT_CUT, OnEditCut)
ON_COMMAND(ID_EDIT_FIND, OnEditFind)
ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
ON_COMMAND(ID_EDIT_REPLACE, OnEditReplace)
ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
ON_UPDATE_COMMAND_UI(ID_EDIT_FIND, OnUpdateEditFind)
ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
ON_UPDATE_COMMAND_UI(ID_EDIT_REPLACE, OnUpdateEditReplace)
ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

COutputFindView::COutputFindView()
        :m_Editor( this , theApp.GetRegistryPath() )
{}

//---------------------------------------------------------------------------


COutputFindView::~COutputFindView()
{}

/////////////////////////////////////////////////////////////////////////////
// COutputFindView drawing
//---------------------------------------------------------------------------


void COutputFindView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// COutputFindView diagnostics
//---------------------------------------------------------------------------


#ifdef _DEBUG
void COutputFindView::AssertValid() const
{
    CView::AssertValid();
}

//---------------------------------------------------------------------------

void COutputFindView::Dump(CDumpContext& dc) const
{
    CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COutputFindView message handlers
//---------------------------------------------------------------------------

void COutputFindView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
    if( lHint == ID_FIND_VIEW )
	{
		CString str(((CStringObject*)pHint)->m_strObject);
		if(!str.IsEmpty())
			str = m_Editor.GetWindowText() + "\n" + str;

		m_Editor.SetWindowText(str) ;
	}
}

//---------------------------------------------------------------------------

COSEditCtrl* COutputFindView::GetEditCtrl()
{
    return &m_Editor ;
}

//---------------------------------------------------------------------------

BOOL COutputFindView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
    if(m_Editor.OnCmdMsg(nID,nCode,pExtra,pHandlerInfo))
        return true;

    return CView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

//---------------------------------------------------------------------------

int COutputFindView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CView::OnCreate(lpCreateStruct) == -1)
        return -1;

    if ( m_Editor.Create( WS_CHILD | WS_VISIBLE | WS_VSCROLL, CRect( 0, 0, 100, 100 ), this, 100 ) == -1 )
        return -1;

    m_Editor.SetReadOnly(TRUE);

    return 0;
}

//---------------------------------------------------------------------------

void COutputFindView::OnSize(UINT nType, int cx, int cy)
{
    CView::OnSize(nType, cx, cy);

    if ( m_Editor.m_hWnd )
        m_Editor.SetWindowPos( NULL, -1, -1, cx, cy, SWP_NOZORDER | SWP_NOMOVE );
}

//---------------------------------------------------------------------------

void COutputFindView::OnEditCopy()
{
    m_Editor.Copy();

}

//---------------------------------------------------------------------------

void COutputFindView::OnEditCut()
{
    m_Editor.Cut();
}

//---------------------------------------------------------------------------

void COutputFindView::OnEditFind()
{
    m_Editor.SendMessage(CMM_EXECUTECMD, CMD_FIND, 0 );
}

//---------------------------------------------------------------------------

void COutputFindView::OnEditPaste()
{
    m_Editor.Paste();
}

//---------------------------------------------------------------------------

void COutputFindView::OnEditRedo()
{
    m_Editor.Redo();
}

//---------------------------------------------------------------------------

void COutputFindView::OnEditReplace()
{
    ::SendMessage( m_Editor.m_hWnd, CMM_EXECUTECMD, CMD_FINDREPLACE, 0 );
}

//---------------------------------------------------------------------------

void COutputFindView::OnEditUndo()
{
    m_Editor.Undo();

}

//---------------------------------------------------------------------------

void COutputFindView::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCopy());
}

//---------------------------------------------------------------------------

void COutputFindView::OnUpdateEditCut(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCut());
}

//---------------------------------------------------------------------------

void COutputFindView::OnUpdateEditFind(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void COutputFindView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanPaste());

}

//---------------------------------------------------------------------------

void COutputFindView::OnUpdateEditRedo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanRedo());
}

//---------------------------------------------------------------------------

void COutputFindView::OnUpdateEditReplace(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(!m_Editor.IsReadOnly());
}

//---------------------------------------------------------------------------

void COutputFindView::OnUpdateEditUndo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanUndo());
}
