/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECCommander.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "CECCommander.h"
#include "CECAboutDlg.h"
#include "CECPluginFrame.h"
#include "CECChildFrm.h"
#include "CECCommanderDoc.h"
#include "CECCommanderView.h"
#include "../Common/CECRegistryTools.h"
#include "Solutions.h"
#include "../Common/CECXServerError.h"
#include "CECMainFrame.h"
#include "../Common/CECStyleFinder.h"
#include "CECProjectOpenDlg.h"
#include "CECProjectDeleteDlg.h"

//
//  Externalized 'C' functions and other 'PSL' includes.
//
extern "C"
{
#include <psl_macros.h>

    //
    //  Function Prototypes.
    //
#include <psl_rtndef.h>         // PSL Routines 
#include <vtest_rtndef.h>       // VTEST routines

    //
    //    Structures.
    //
}

//
//    Static Data
//

CECWorkspace*    CECCommanderApp::m_pWorkspace  = NULL;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECCommanderApp

BEGIN_MESSAGE_MAP(CECCommanderApp, CWinApp)
//{{AFX_MSG_MAP(CECCommanderApp)
ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
ON_COMMAND(ID_FILE_OPENPROJECT, OnFileOpenproject)
ON_UPDATE_COMMAND_UI(ID_FILE_OPENPROJECT, OnUpdateFileOpenproject)
ON_COMMAND(ID_FILE_CLOSEPROJECT, OnFileCloseproject)
ON_UPDATE_COMMAND_UI(ID_FILE_CLOSEPROJECT, OnUpdateFileCloseproject)
ON_UPDATE_COMMAND_UI(ID_FILE_MRU_PROJECT, OnUpdateFileMruProject)
ON_COMMAND(ID_PROJECT_DELETE, OnProjectDelete)
ON_UPDATE_COMMAND_UI(ID_PROJECT_DELETE, OnUpdateProjectDelete)
//}}AFX_MSG_MAP
// Standard file based document commands
ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
// Standard print setup command
ON_COMMAND_EX_RANGE(ID_FILE_MRU_PROJECT,ID_FILE_MRU_PROJECT+4, OnOpenRecentProject)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECCommanderApp construction

CECCommanderApp::CECCommanderApp()
{
    //
    //    Recover and save the installation directory.
    //
    PSL_ADDROOT ("" ,m_InstallationDirectory ,sizeof (m_InstallationDirectory));
    //
    //    And create a trace log object immediately so we can log everything
    //
    m_pTrace = NULL;

    //
    //    First see if we have a trace parameter
    //
    CString szValue;
    if (GetCommandLineParameter("-trace", szValue))
    {
        //
        //    And if we do then we should see if tracing is to be turned on or off
        //
        if (szValue.CompareNoCase("on") == 0)
        {
            //
            //    It's on so create our trace object
            //
            m_pTrace = new CECTrace(m_InstallationDirectory);
        }
    }

    //
    //    Initialize the product info array
    //
    for (int i=0;   i<MAX_PRODUCTS;     i++)    m_pProductInfo[i]       = NULL;
    for (    i=0;   i<MAX_PRODUCTS;     i++)    m_hResourceHandles[i]   = NULL;

    m_pDLLLoader        = NULL;
    m_pServer = new CYRServer;

    m_KnownFileExtensions = new CStringArray;

    m_szCoreProductTitle = "CYRANO Framework";
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CECCommanderApp object

CECCommanderApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CECCommanderApp initialization

BOOL CECCommanderApp::InitInstance()
{
    // If we want any tracing - then create a trace file
    if (m_pTrace != NULL)
    {
        if (!m_pTrace->Create())
            return FALSE;
    }

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "InitInstance", "Entering");

    AfxEnableControlContainer();

    //
    //    Standard initialization
    //
    //    If you are not using these features and wish to reduce the size of your final executable,
    //    you should remove from the following the specific initialization routines you do not need.
    //
    //    Must now initialise the OLE libraries
    //
    if (!AfxOleInit())
    {
        AfxMessageBox("OLE Initilaization Failed.");

        if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "InitInstance", "OLE Initialiation Failed");
        return FALSE;
    }

#ifdef _AFXDLL
    Enable3dControls();         // Call this when using MFC in a shared DLL
#else
Enable3dControlsStatic();   // Call this when linking to MFC statically
#endif

    SetRegistryKey(ConstructUserKey());

    m_pRecentProjectList = new CRecentFileList(0,"Commander\\MRU","Project%d",5);
    m_pRecentProjectList->ReadList();

    //
    //    Check the licencing if applicable
    //
    unsigned long options = 0;
#if !defined(NOPROTECTION)
    if (!CheckAuthorization(options))
        return FALSE;
#endif

    //
    //    And get the licenced options
    //
    GetLicenceOptions (options);

    //
    //    Load in the product info. This call scans the registry for all CATID's which are
    //    TestCommander Compliant - then loads in their product names. We need to do this
    //    now so that we can load the title and product icon
    //
    if (!LoadProductInfo())
    {
        return FALSE;
    }

    //
    //    Allocate a new project workspace object
    //
    m_pWorkspace = new CECWorkspace ();
    m_pWorkspace->RestoreWorkspace();

    // Initialize all Managers for usage. They are automatically constructed
    // if not yet present
    InitContextMenuManager();
    // Enable user-defined tools. If you want allow more than 10 tools,
    // add tools entry to resources (ID_USER_TOOL11, ID_USER_TOOL12,...)
    EnableUserTools (ID_TOOLS_ENTRY, ID_USER_TOOL1, ID_USER_TOOL10);


    //
    //    Set up the path for invoking external images.
    //
    SetPath();

    // create main MDI Frame window
    CECMainFrame* pMainFrame = new CECMainFrame;
    m_pMainWnd = pMainFrame;

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "InitInstance", "Attempting to initialize MDI Frame Window");

    if (!pMainFrame->LoadFrame(IDR_MAINFRAME, WS_OVERLAPPEDWINDOW))
        return FALSE;

    // And set up the Core Products Icon
    HICON hIcon = LoadIcon(m_nCoreProductIcon);
    pMainFrame->SetIcon(hIcon, FALSE);
    pMainFrame->SetIcon(hIcon, TRUE);

    // Parse command line for standard shell commands, DDE, file open
    CCommandLineInfo cmdInfo;
    ParseCommandLine(cmdInfo);

    cmdInfo.m_nShellCommand = CCommandLineInfo::FileNothing;

    // try to launch as an OLE server
    if (cmdInfo.m_bRunEmbedded)
    {
        // application was run with /Embedding flag.  Instead of showing
        //  the window, the application waits to receive OLE requests.
        return TRUE;
    }

    // Dispatch commands specified on the command line
    if (!ProcessShellCommand(cmdInfo))
        return FALSE;

    LoadCustomState();

    // Set the Window title
	CString appName = "";
	appName.LoadString(IDS_CYRANO_STUDIO_NAME);
    pMainFrame->SetWindowText(GetCoreProductTitle()+" "+appName);

    // Restore the Workspace to the left it was left the last time it was ran
    if (!RestoreWorkspace())
    {
        return FALSE;
    }

    //
    // And add in the menu items - if visible
    //
    if (!m_pWorkspace->IsFullScreen())
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "InitInstance", "Adding Menus");
        pMainFrame->AddMenus();
        CBCGToolBar::SetBasicCommands (pMainFrame->m_lstBasicCommands);
    }

    pMainFrame->Invalidate(FALSE);

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "InitInstance", "Leaving");
    return TRUE;
}


// App command to run the dialog
void CECCommanderApp::OnAppAbout()
{
    CECAboutDlg aboutDlg;
    aboutDlg.DoModal();
}


void CECCommanderApp::PreLoadState ()
{

    GetContextMenuManager()->AddMenu (_T("My menu"), IDR_CONTEXT_MENU);

    // TODO: add another context menus here
}

//
//
// GetCommandLineParameter
// =======================
//
// Function to obtain a command line parameter
//

BOOL CECCommanderApp::GetCommandLineParameter(const CString& szArgument, CString& szValue)
{
    BOOL bFound = FALSE;

    // We will have to search the command line for the correct argument ...
    int i = 1;
    while (__argc > i)
    {
        CString szField = __argv[i];

        if (szField.CompareNoCase(szArgument) == 0)
        {
            if (__argc > i)
            {
                szValue = __argv[i+1];
                bFound = TRUE;
            }
            break;
        }
        i++;
    }
    return bFound;
}

/////////////////////////////////////////////////////////////////////////////
// CECCommanderApp message handlers
//
// ConstructUserKey
// ================
//
// Construct the Registry Key that we will store all the Workspace options under
//
CString CECCommanderApp::ConstructUserKey ()
{
    //
    // Change the registry key under which our settings are stored to use the Framework key
    CString szKey = CYRANO_SOLUTIONS;
    szKey += "\\Framework";

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ConstructUserKey", "Constructing Registry Key <" + szKey + ">");

    return szKey;
}

//
// LoadProductInfo
// ===============
//
// Finds and loads in all framework compliant product info structures so that we know
// a). what we are running and
// b). what we have to licence
//
BOOL CECCommanderApp::LoadProductInfo()
{
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "LoadProductInfo", "Loading the Product Information");

    UINT index = 0;
    LPPRODUCTFUNC   lpfnGetProductInfo;
    LPRESFUNC       lpfnGetResourceInfo;
    BOOL  bReturn;

    // Now ensure that we have an ini file
    CString szIniFileName;
    if (!GetInitializationFileName(szIniFileName))
    {
        return FALSE;
    }

    if (m_pDLLLoader == NULL)
    {
        m_pDLLLoader = new CECDLLFunctions(m_pTrace);
    }

    int DLLLoadStatus = m_pDLLLoader->Initialize(szIniFileName, "");
    if(DLLLoadStatus != 0)
    {
        return false;
    }

    CStringArray& arStudioDlls = m_pDLLLoader->GetExtensionDLLs();

    for (int i = 0; i < arStudioDlls.GetSize(); i++)
    {
        CString szDll = arStudioDlls.GetAt(i);

        if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "LoadProductInfo", "Loading <" + szDll + ">");

        // And see if GetProductInfo is supported
        lpfnGetProductInfo = (LPPRODUCTFUNC) m_pDLLLoader->GetFunctionPtr("GetProductInfo", szDll);

        if (lpfnGetProductInfo)
        {
            // Obtain (and allocate memory) the next product ID
            UINT ProductID = GetNextProductID();

            // Get the product info
            bReturn = lpfnGetProductInfo(m_pProductInfo[ProductID]);

            // And check if we got a valid Product Info
            ValidateProductInfo(ProductID);
        }

        // Now see if GetResourceInfo is supported as well - in which case we store the resource handle
        lpfnGetResourceInfo = (LPRESFUNC) m_pDLLLoader->GetFunctionPtr("GetResourceInfo", szDll);

        // And load in all the resource handles as well
        if (lpfnGetResourceInfo)
        {
            if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "LoadProductInfo", "Saving Resource Handle for <" + szDll + ">");

            SaveResourceHandle(index, m_pDLLLoader->GetDLL(szDll));
            index++;
        }
        else
        {
            AfxFreeLibrary(m_pDLLLoader->GetDLL(szDll));
        }

    }

    // And set up the Core Product details right away so that we know
    // exactly what we are trying to run
    SetCoreProductDetails();

    return TRUE;
}
//
// RestoreWorkspace
// ================
//
// Attempt to restore the last used workspace settings (if any)
// and initialize the workspace accordingly
//
BOOL CECCommanderApp::RestoreWorkspace ()
{

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "RestoreWorkspace", "Restoring Workspace ...");

    // Ensure that we have been passed in a valid mainframe
    CECMainFrame* pMainFrame = (CECMainFrame*) AfxGetMainWnd();
    ASSERT (pMainFrame != NULL);

    // Attempt to restore the workspace.
    m_pWorkspace->RestoreWorkspace();

    // Now we position the window
    CRect rect = m_pWorkspace->StringToRect(m_pWorkspace->GetMainWindowRect());
    pMainFrame->MoveWindow(rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);

    //
    //    ...and activate the window, restoring it to its original size and state
    //
    if (m_pWorkspace->IsZoomed())
    {
        pMainFrame->ActivateFrame(SW_SHOWMAXIMIZED);
    }
    else
    {
        pMainFrame->ActivateFrame(m_nCmdShow);
    }

    // Display it in Full Screen Moode
    pMainFrame->ShowFullScreen(m_pWorkspace->IsFullScreen());

    //Use project from last session if it exists still...
    if(AreWeUsingProjects())
    {
        CString strProjectName = m_pWorkspace->GetCurrentProject();

        // And set up the new Project Name by firing an event to all known Plug-ins
        OnSwitchProjects(strProjectName);
    }

    return TRUE;
}

//
// @mfunc SetPath
//        =======
//
//    This function is responsible for modifying the path as known to this process
//    to include the installation directory.
//
// @parm None
//
void CECCommanderApp::SetPath   () const
{
    //
    //    First lets recover the urrent value for the PATH environment variable.
    //
    char    Path[4096];
    GetEnvironmentVariable("PATH" ,Path ,sizeof (Path));

    //
    //    ...and append on to this the installation directory.
    //
    (void) sprintf (Path ,"%s;%s" ,Path ,m_InstallationDirectory);
    if (!SetEnvironmentVariable ("PATH" ,Path))
    {
        AfxMessageBox   ("Failed to set the PATH Environment Variable"
                         ,MB_ICONSTOP | MB_OK);
    }
    return;
}

//
// AreWeUsingAServer
// =================
//
// See if any of the product DLLs require a server
//
BOOL CECCommanderApp::AreWeUsingAServer()
{
    BOOL        bAnswer = FALSE;
    register    i       = 0;

    // And loop round all our available products
    while (m_pProductInfo[i] != NULL)
    {
        // And examine the product info styles variable
        CECStyleFinder styles;
        if (styles.HasServer(m_pProductInfo[i]->dwStyles))
        {
            // And break out as we have found a product that does need a server
            bAnswer = TRUE;
            break;
        }
        i++;
    }
    return bAnswer;
}

//
// AreWeUsingProjects
// ==================
//
// See if any of the product DLLs need a project
//
BOOL CECCommanderApp::AreWeUsingProjects()
{
    BOOL bAnswer = FALSE;

    // And automatically return if we have no server
    if (AreWeUsingAServer())
    {
        register i = 0;

        // And loop round all our available products
        while (m_pProductInfo[i] != NULL)
        {
            // And examine the product info styles variable
            CECStyleFinder styles;
            if (styles.HasProject(m_pProductInfo[i]->dwStyles))
            {
                // And break out as we have found a product that does need a project
                bAnswer = TRUE;
                break;
            }
            i++;
        }
    }
    return bAnswer;
}

//
//  @mfunc CheckAuthorization
//
//  This function checks the status of the authorization key
//
//  @parm licenced options returned
//  @rdesc Returns TRUE if we are licenced, FALSE otherwise
//
BOOL CECCommanderApp::CheckAuthorization (unsigned long& /*options*/)
{
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "CheckAuthorization", "Checking Authorisation");

#pragma message ("CheckAuthorization not yet implemented")
    return TRUE;
}



//
//  @mfunc GetLicenceOptions
//
//  This function determines what options we are licenced for
//
//  @parm bitmask of options returned from CheckAuthorization
//
void CECCommanderApp::GetLicenceOptions     (unsigned long /*options*/)
{
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "GetLicenceOptions", "Obtaining Licence Options");

#pragma message ("GetLicenceOptions not yet implemented")
}

//
// SetCoreProductDetails
// =====================
//
// Returns the main product title and splash screen
//
void CECCommanderApp::SetCoreProductDetails()
{
    register i = 0;
    m_szCoreProductTitle.Empty();
    m_nCoreProductIcon = 0;
    m_nCoreProductLogo = 0;

    // We always provide the Core Product title
    while (m_pProductInfo[i] != NULL)
    {
        // And examine the product info styles variable
        CECStyleFinder styles;
        if (styles.IsSubProduct(m_pProductInfo[i]->dwStyles))
        {
            // If we have a subproduct then check to see if it can be
            // promoted to the main core product
            if (m_szCoreProductTitle.IsEmpty())
            {
                // It can so set the product display title
                m_szCoreProductTitle    = m_pProductInfo[i]->strDisplayName;

                // And the product internal name
                m_szCoreProductName = m_pProductInfo[i]->strInternalName;

                // And the product project prefix
                m_szCoreProductProjectPrefix = m_pProductInfo[i]->strProjectPrefix;

                // And the Background Logo
                if (m_pProductInfo[i]->nBackground != 0)
                {
                    m_nCoreProductLogo = m_pProductInfo[i]->nBackground;
                }

                // And the Product Icon
                if (m_pProductInfo[i]->nIcon != 0)
                {
                    m_nCoreProductIcon = m_pProductInfo[i]->nIcon;
                }

                // And the Product Styles
                if (m_pProductInfo[i]->dwStyles != 0)
                {
                    m_dwCoreProductStyles = m_pProductInfo[i]->dwStyles;
                }
            }
        }
        else
        {
            // This can only be a main product. Relegate any previously loaded
            // products and replace it with this one instead
            m_szCoreProductTitle    = m_pProductInfo[i]->strDisplayName;

            // And the product internal name
            m_szCoreProductName = m_pProductInfo[i]->strInternalName;

            // products project name prefix
            m_szCoreProductProjectPrefix = m_pProductInfo[i]->strProjectPrefix;

            if (m_pProductInfo[i]->nBackground != 0)
            {
                m_nCoreProductLogo = m_pProductInfo[i]->nBackground;
            }

            if (m_pProductInfo[i]->nIcon != 0)
            {
                m_nCoreProductIcon = m_pProductInfo[i]->nIcon;
            }

            if (m_pProductInfo[i]->dwStyles != 0)
            {
                m_dwCoreProductStyles = m_pProductInfo[i]->dwStyles;
            }
        }
        i++;
    }

    // And set some defaults if no products/splash screens were found at all
    if (m_szCoreProductTitle.IsEmpty())
    {
        VERIFY (m_szCoreProductTitle.LoadString(IDS_CYRANO_STUDIO_NAME));
    }

    if (m_nCoreProductLogo == 0)
    {
        m_nCoreProductLogo = IDB_STUDIO_BACKGROUND;
    }

    if (m_nCoreProductIcon == 0)
    {
        m_nCoreProductIcon = IDR_MAINFRAME;
    }

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "SetCoreProductDetails", "Attempting to run <" + m_szCoreProductTitle + "> as Core Product");
}

//
// SaveResourceHandle
// ==================
//
// When loading in the DLL's we have to store a handle to their
// resources so that we can switch to them when adding in any requested buttons. Not all DLL's
// need to do this so we only do it for those which need buttons adding.
void CECCommanderApp::SaveResourceHandle(UINT index, HINSTANCE hDLL)
{
    // And load in it's resources
    if (hDLL != NULL)
    {
        m_hResourceHandles[index] = hDLL;
    }
}

//
// ValidateProductInfo
// ===================
//
// Confirms whether or not we actually got a valid product (core or sub project that is)
//
BOOL CECCommanderApp::ValidateProductInfo(UINT index)
{
    // This check makes sure we got a valid Product Info

    // To do this we check the product name which is mandatory.
    if (m_pProductInfo[index]->strDisplayName[0] == '\0')
    {
        // And delete the item as the next DLL will want to use this slot instead.
        delete m_pProductInfo[index];
        m_pProductInfo[index] = NULL;
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ValidateProductInfo", "This DLL is a Generic DLL and contains no Product info");

        return false;
    }
    else
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ValidateProductInfo", "Confirmed as a Product DLL");
    }
    return true;
}

//
// GetNextProductID
// ================
//
// Finds the next available product ID and allocates memory for it
//
UINT CECCommanderApp::GetNextProductID()
{
    UINT index = 0;

    // Find the next available slot in the array
    while ((m_pProductInfo[index] != NULL)) index++;

    // And create a new element to populate
    m_pProductInfo[index] = new PRODUCTINFO;

    // And set the Product Name to null which we check afterward to see if we got anything
    m_pProductInfo[index]->strDisplayName[0] = '\0';

    // As well as the Product Splash screen
    m_pProductInfo[index]->nSplashScreen = 0;

    return index;
}

//
//
// GetInitializationFileName
// =========================
//
// Function to see if we are overiding the default CYRStudio.ini file name
//
BOOL CECCommanderApp::GetInitializationFileName(CString& szIniFile)
{
    // First we get the product name from the command line - or use the default
    if (!GetCommandLineParameter("-i", szIniFile))
    {
        //No product specified, so use first in registry

        CString KeyName = SOLUTIONS_FRAMEWORK_KEY;

        CECRegistryTools reg;
        if(reg.Open(HKEY_LOCAL_MACHINE, KeyName + "\\Products",KEY_READ) != ERROR_SUCCESS)
            return false;

        TCHAR szBuffer[256];
        DWORD dwSize = 256;
        DWORD Index = 0;
        HKEY Key = reg.GetHKey();

        if(RegEnumKeyEx(Key, Index, szBuffer, &dwSize, NULL, NULL, NULL, NULL) == ERROR_SUCCESS)
        {
            szIniFile = szBuffer;
        }
    }

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "GetInitializationFileName", "Using the ini file <" + szIniFile + ">");

    return TRUE;
}

//
// OnSwitchProjects
// ================
//
// This routine handles a change of project in the framework by firing an event
// to all the connected plug-ins.
//
BOOL CECCommanderApp::OnSwitchProjects (CString strProject)
{
    // To access our projects we need a valid server so ...
    if(AreWeUsingProjects())
    {
        if(strProject == m_pServer->GetOpenProjectName())
            return true;


        //Close all windows
        AfxGetMainWnd()->SendMessage(STUDIO_BEGIN_WAIT);
        AfxGetMainWnd()->SendMessage(WM_COMMAND,MAKEWPARAM(ID_WINDOW_CLOSE_ALL,0),0);

        BOOL bResult = m_pServer->OpenProject(strProject);
		CString appName = "";
		appName.LoadString(IDS_CYRANO_STUDIO_NAME);
        if(bResult)
        {
            //Set window title
            CString strTitle = GetCoreProductTitle() + " " + appName +" - " + strProject;
            AfxGetMainWnd()->SetWindowText(strTitle);

            if (strProject != "")
            {
                m_pRecentProjectList->Add(strProject);
                m_pRecentProjectList->WriteList();
            }
        }
        else
        {
            AfxGetMainWnd()->SetWindowText(GetCoreProductTitle() + " " + appName);
        }

        // As this is a pretty hefty change to the Framework we will probably
        // need to inform everyone about it so that they can tidy up and display
        // different buttons if needs be. We shall fire a SetActiveProject event
        // to all DLL's and it's up to them to do the biz.
        GlobalFireEvent(STUDIO_SWITCH_PROJECTS);
        AfxGetMainWnd()->SendMessage(STUDIO_END_WAIT);

        return bResult;
    }
    return false;
}

//    SetOutputWindow
//
//
void CECCommanderApp::SetOutputWindow   (char* pWindowName ,char* pText)
{

    //
    //    ...the first task is to see if we actually have an output tab with the specified name.
    //
    int nCount = m_OutputWndArray.GetSize();
    for (int isub = 0; isub<nCount; isub++)
    {
        if (m_OutputWndArray[isub]->GetName() == pWindowName) break;
    }

    if (isub >= nCount)
    {
        CECOutputWnd* pOutputWnd = new CECOutputWnd(pWindowName, NULL);
        isub = m_OutputWndArray.Add(pOutputWnd);

        CECMainFrame* pMainFrame = (CECMainFrame*)AfxGetMainWnd();
        pMainFrame->m_wndOutputView.InsertTab(pOutputWnd);
    }

    //
    //    If the text supplied is NULL then we simply clear the window, otherwise we append the
    //    text supplied to the existing edit control.
    //
    CECOutputWnd* pCompileWnd = m_OutputWndArray[isub];
    if (pText == NULL)
    {
        pCompileWnd->Clear();
    }

    else
    {
        pCompileWnd->Output(pText);
    }

    //
    //    ...and activate the appropriate tab
    //
    m_OutputWndArray[isub]->Activate();

}

LRESULT CECCommanderApp::SpecificFireEvent (UINT nID, LONG lParam)
{
    if (m_pDLLLoader != NULL)
    {
        return m_pDLLLoader->SpecificFireEvent(nID, lParam);
    }
    else
    {
        return 0;
    }
}

BOOL CECCommanderApp::CanCreateProjects()
{
    //See if any of the products support project creation
    //Get the number of products
    int nProductCount = m_pDLLLoader->GetStudioProductCount();

    // Check each product to see if it supports project creation
    for ( int i = 0; i<nProductCount; i++)
    {
        // Firstly we get it's name
        CString szProductName = m_pDLLLoader->GetProduct(i);

        //If this product cannot create a project, don't add it
        LPCREATEPROJECTFUNC Create = (LPCREATEPROJECTFUNC)m_pDLLLoader->GetFunctionPtr("CreateProject" ,szProductName);
        if(Create)
            return true;
    }

    return false;
}

void CECCommanderApp::OnFileOpenproject()
{
    CECProjectOpenDlg dlg;
    if(dlg.DoModal() == IDOK)
    {
        CString strProject = dlg.GetProjectName();
        OnSwitchProjects(strProject);
    }
}

void CECCommanderApp::OnUpdateFileOpenproject(CCmdUI* pCmdUI)
{
    int nCount = m_pServer->GetProjectCount();

    //If there is an open project ignore it
    if(m_pServer->IsThereAnOpenProject())
        nCount--;

    //if there are no available projects disable command
    pCmdUI->Enable(nCount);

}

void CECCommanderApp::OnFileCloseproject()
{
    //Close all windows
    AfxGetMainWnd()->SendMessage(WM_COMMAND,MAKEWPARAM(ID_WINDOW_CLOSE_ALL,0),0);

    // then set the active project to null
    OnSwitchProjects ("");

}

void CECCommanderApp::OnUpdateFileCloseproject(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_pServer->IsThereAnOpenProject());
}

void CECCommanderApp::AddKnownFileExtension(CString strExt)
{
    m_KnownFileExtensions->Add(strExt);
}

//
// FindDocumentTemplate
// ====================
//
// Function to find a requested Document Template
//
POSITION CECCommanderApp::FindDocumentTemplate(const CString& szSearchString)
{
    ASSERT (!szSearchString.IsEmpty());

    BOOL bFound = FALSE;

    POSITION nCurrentPosition = GetFirstDocTemplatePosition();
    POSITION nDocumentPosition = NULL;

    while (nCurrentPosition != NULL)
    {
        CString szExistingType;

        nDocumentPosition = nCurrentPosition;

        // Get the next Template
        CMultiDocTemplate *pExistingTemplate = (CMultiDocTemplate*) GetNextDocTemplate(nCurrentPosition);

        // And get the associated document type
        pExistingTemplate->GetDocString(szExistingType, CDocTemplate::regFileTypeName);

        if (szSearchString.Compare(szExistingType) == 0)
        {
            // BINGO - We found a match
            bFound = TRUE;
            break;
        }
    }

    return (bFound) ? nDocumentPosition : NULL;
}

int CECCommanderApp::OnOpenRecentProject( UINT nID )
{
    int index = nID-ID_FILE_MRU_PROJECT;

    CString proj = m_pRecentProjectList->operator [](index);
    int nLen = proj.GetLength();
    int nPos = proj.ReverseFind('\\');
    proj = proj.Right(nLen-nPos-1);

    OnSwitchProjects(proj);

    return 0;
}

void CECCommanderApp::OnUpdateFileMruProject(CCmdUI* pCmdUI)
{
    if (m_pRecentProjectList == NULL) // no MRU files
        pCmdUI->Enable(FALSE);
    else
    {
        if (pCmdUI->m_pMenu && !pCmdUI->m_pSubMenu) // Only do something if we're the submenu
        {
            // Insert a separator so that CRecentFileList::UpdateMenu() will behave
            pCmdUI->m_pMenu->InsertMenu(0, MF_SEPARATOR | MF_BYPOSITION);
            pCmdUI->m_nIndex++;
            pCmdUI->m_nIndexMax++;

            m_pRecentProjectList->ReadList();
            m_pRecentProjectList->UpdateMenu(pCmdUI); // Update the menu

            // Get rid of the separator we just inerted
            pCmdUI->m_pMenu->DeleteMenu(0, MF_BYPOSITION);
            pCmdUI->m_nIndex--;
            pCmdUI->m_nIndexMax--;
        }
    }
}


void CECCommanderApp::OnProjectDelete()
{
    CECProjectDeleteDlg deleteDlg;
    deleteDlg.DoModal();
}

void CECCommanderApp::OnUpdateProjectDelete(CCmdUI* pCmdUI)
{
    int nCount = m_pServer->GetProjectCount();

    //If there is an open project ignore it
    if(m_pServer->IsThereAnOpenProject())
        nCount--;

    //if there are no available projects disable command
    pCmdUI->Enable(nCount);
}

int CECCommanderApp::ExitInstance()
{
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ExitInstance", "Entering");

    if (m_pRecentProjectList != NULL)
    {
        m_pRecentProjectList->WriteList();
        delete m_pRecentProjectList;
    }

    // Delete the Studio Workspace object
    if (m_pWorkspace != NULL)
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ExitInstance", "Deleting Workspace Object");

        delete m_pWorkspace;
        m_pWorkspace = NULL;
    }

    while(m_FloorWndArray.GetSize())
    {
        delete m_FloorWndArray[0];
        m_FloorWndArray.RemoveAt(0);
    }

    while(m_OutputWndArray.GetSize())
    {
        delete m_OutputWndArray[0];
        m_OutputWndArray.RemoveAt(0);
    }

    // Delete all the ProductInfo structures
    for (int i=0; i<MAX_PRODUCTS; i++)
    {
        if (m_pProductInfo[i] != NULL)
        {
            CString szName = m_pProductInfo[i]->strDisplayName;
            if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ExitInstance", "Deleting ProductInfo for " + szName);

            delete m_pProductInfo[i];
            m_pProductInfo[i] = NULL;
        }
    }

    if (m_pDLLLoader != NULL)
    {
        CStringArray& arDLLs = m_pDLLLoader->GetExtensionDLLs();

        // And the Destructors
        for (i=0; i< arDLLs.GetSize(); i++)
        {
            if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ExitInstance", "Destroying the Plug-In resource objects");

            // Destroy any objects created within the DLL
            LPONDESTROYFUNC     lpfnOnDestroy;

            HINSTANCE hDLL = GetResourceHandle(i);

            lpfnOnDestroy = (LPONDESTROYFUNC)::GetProcAddress(hDLL,"OnDestroy");

            if (lpfnOnDestroy)
            {
                lpfnOnDestroy();
            }
        }

        // And the resource handles
        for (i=0; i<MAX_PRODUCTS; i++)
        {

            if (m_hResourceHandles[i] != NULL)
            {
                if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ExitInstance", "Deleteing a Plug-In resource handle");

                // And free the library from the framework
                AfxFreeLibrary(m_hResourceHandles[i]);
                m_hResourceHandles[i] = NULL;
            }
        }

        if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ExitInstance", "Deleteing the DLL Loader Object");

        delete m_pDLLLoader;
        m_pDLLLoader = NULL;
    }

    if (m_pServer != NULL)
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ExitInstance", "Deleting the Server Object");

        delete m_pServer;
        m_pServer = NULL;
    }

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECCommanderApp", "ExitInstance", "Leaving");

    if (m_pTrace != NULL)
    {
        delete m_pTrace;
        m_pTrace = NULL;
    }

    if(m_KnownFileExtensions != NULL)
    {
        m_KnownFileExtensions->RemoveAll();
        delete m_KnownFileExtensions;
        m_KnownFileExtensions=NULL;
    }

    return CWinApp::ExitInstance();
}


