/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECCommander.h : main header file for the CECCOMMANDER application
//

#if !defined(AFX_CECCOMMANDER_H__FF67E806_67AF_11D4_A6DA_00500447FC27__INCLUDED_)
#define AFX_CECCOMMANDER_H__FF67E806_67AF_11D4_A6DA_00500447FC27__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
#error include 'stdafx.h' before including this file for PCH
#endif
#include <bcgcb.h>          // BCG Control Bar

#include "resource.h"       // main symbols
#include "../Common/CECTrace.h"
#include "../interfaces/CyrPlugins.h"
#include "../Common/CECDllFunctions.h"
#include "Server.h"
#include "CECWorkSpace.h"
#include "CECFloorWnd.h"
#include "CECOutputView.h"
////////////////////////////////////////////////////////////////
// CECCommanderApp:
// See CECCommander.cpp for the implementation of this class
//

class CECCommanderApp : public CWinApp,
            public CBCGWorkspace
{
public:

    // Static pointer to the Workspace
    static CECWorkspace*   m_pWorkspace;

    // Routine to load in the product info from the plug-ins
    BOOL LoadProductInfo();

    CECCommanderApp();                  // @cmember Constructor.

    static CECWorkspace*    GetWorkspace ()         // @cmember Get a reference to the current workspace
    {
        return m_pWorkspace;
    }

    CPtrArray&      GetToolBarArray ()
    {
        return m_ToolBarArray;
    }                  // @cmember Return a reference to the Toolbar array

    CPtrArray&      GetFloorWndArray ()
    {
        return m_FloorWndArray;
    }                 // @cmember Return a reference to the TreeView array

    CPtrArray&      GetOutputWndArray ()
    {
        return m_OutputWndArray;
    }                // @cmember Return a reference to the OutputView array

    CBCGTabWnd*     GetHandleOutputWnd () const     // @cmember Return a handle to the output window
    {
        return m_pOutputWnd;
    }

    void            SetHandleOutputWnd (CBCGTabWnd* pWnd)
    {
        m_pOutputWnd = pWnd;
    }                    // @cmember Set the handle to the output window

    //  SEC3DTabWnd*    GetHandleWorkspaceWnd () const  // @cmember Return a handle to the workspace window
    //      { return m_pWorkspaceWnd; }

    //  void            SetHandleWorkspaceWnd (SEC3DTabWnd* pWnd)
    //      { m_pWorkspaceWnd = pWnd; }                 // @cmember Set the handle to the workspace window

    CYRServer*  GetServer ()                    // @cmember Get the Main server object
    {
        return m_pServer;
    }

    CECTrace*       GetTrace ()                     // @cmember Get the Trace object
    {
        return m_pTrace;
    }

    // Get the index of the next product to be added
    UINT GetNextProductID();

    // Save an instance to the resources
    void SaveResourceHandle(UINT index, HINSTANCE hDLL);

    // Save an instance to the resources
    HINSTANCE GetResourceHandle(UINT index)
    {
        return m_hResourceHandles[index];
    };

    // @cmember Get the core product title
    void SetCoreProductDetails();

    // @cmember And the Core Product Details
    CString GetCoreProductTitle()
    {
        return m_szCoreProductTitle;
    }
    CString GetCoreProductName()
    {
        return m_szCoreProductName;
    }
    CString GetCoreProductProjectPrefix()
    {
        return m_szCoreProductProjectPrefix;
    }
    UINT    GetCoreProductIcon()
    {
        return m_nCoreProductIcon;
    }
    UINT    GetCoreProductLogo()
    {
        return m_nCoreProductLogo;
    }

    // @cmember Get a Product Icon
    UINT    GetProductIcon(int index)
    {
        return m_pProductInfo[index]->nIcon;
    }

    // @cmember Is the Server up and running
    BOOL IsServerRunning();

    // @cmember Check the validity of the Product Info
    BOOL ValidateProductInfo(UINT index);

    // @cmember Find an Existing Document Template
    POSITION FindDocumentTemplate(const CString& szSearchString);

    // @cmember Get the DLL Loader
    CECDLLFunctions* GetDLLLoader()
    {
        return m_pDLLLoader;
    }

    // @cmember Get the allowed file extensions added by the dll's
    CStringArray*    GetKnownFileExtensions()
    {
        return m_KnownFileExtensions;
    };

    // @cmember Add an allowed file extension
    void             AddKnownFileExtension(CString strExt);

    // @cmember Restore the Workspace to the previous run
    BOOL RestoreWorkspace();

    // @cmember Switch projects
    BOOL OnSwitchProjects (CString szNewProject);

    // @cmember Simple Wrapper function to access the DLLLoader's GlobalFireEvent
    void GlobalFireEvent (UINT nID, LONG lParam=NULL)
    {
        if (m_pDLLLoader != NULL) m_pDLLLoader->GlobalFireEvent(nID, lParam);
    }

    // @cmember Simple Wrapper function to access the DLLLoader's GlobalFireEvent
    LRESULT SpecificFireEvent (UINT nID, LONG lParam=NULL);

    // @cmember See if any of the product DLLs has CYR_HAS_PROJECT style set
    BOOL AreWeUsingProjects();

    // @cmember See if any of the product DLLs has CYR_HAS_SERVER style set
    BOOL AreWeUsingAServer();

    BOOL    AreWeUsingContext   () const
    {
        return ((m_dwCoreProductStyles & CYR_HAS_CONTEXT) != 0);
    }

    // @cmember Routine to obtain the ini file name from the command line
    BOOL GetInitializationFileName(CString& szIniFile);

    // @cmember Routine to obtain a command line argument
    BOOL GetCommandLineParameter(const CString& szArgument, CString& szValue);

    // @cmember Function to output text onto the named output tab
    void SetOutputWindow    (char* pWindowName ,char* pText);

    // Get the Installation Directory
    char* GetInstallationDirectory()
    {
        return m_InstallationDirectory;
    }

    CString ConstructUserKey ();                // @cmember Construct a registry key for Workspace options

    BOOL CanCreateProjects();               //returns true if there is a plugin that can create a project

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CECCommanderApp)
public:
    virtual BOOL InitInstance();
    virtual int ExitInstance();
    //}}AFX_VIRTUAL

    // Implementation
    //{{AFX_MSG(CECCommanderApp)
    afx_msg void OnAppAbout();
    afx_msg void OnFileOpenproject();
    afx_msg void OnUpdateFileOpenproject(CCmdUI* pCmdUI);
    afx_msg void OnFileCloseproject();
    afx_msg void OnUpdateFileCloseproject(CCmdUI* pCmdUI);
    afx_msg void OnUpdateFileMruProject(CCmdUI* pCmdUI);
    afx_msg void OnProjectDelete();
    afx_msg void OnUpdateProjectDelete(CCmdUI* pCmdUI);
    //}}AFX_MSG
    afx_msg int OnOpenRecentProject(UINT nID);
    DECLARE_MESSAGE_MAP()

private:
    BOOL            CheckAuthorization (unsigned long& options);
    // @cmember Ensure authorized to use this product

    void            GetLicenceOptions   (unsigned long options);
    // @cmember Flag licenced options

    void            SetPath         () const;       // @cmember Set-up the PATH environment variable

    CBCGTabWnd*     m_pOutputWnd;                   // @cmember pointer to the output TAB window

    //  SEC3DTabWnd*    m_pWorkspaceWnd;                // @cmember pointer to the workspace window

    char            m_InstallationDirectory[_MAX_PATH];// @cmember The path to the installation directory

    CString         m_szCoreProductTitle;           // @cmember The current core product
    CString         m_szCoreProductName;            // @cmemer The current products internal name
    CString         m_szCoreProductProjectPrefix;       // @cmember The prefix used by this product for project names

    UINT            m_nCoreProductIcon;             // @cmember The current framework icon

    UINT            m_nCoreProductLogo;             // @cmember The current logo to be used

    DWORD           m_dwCoreProductStyles;          // @cmember The core product styles

    // Struct which contains all the product ID info
    struct PRODUCTINFO*     m_pProductInfo[MAX_PRODUCTS];

    // Array of file extensions added by the product dll's
    CStringArray*           m_KnownFileExtensions;


    // Array of HINSTANCES to the Resources of each Snap In
    HINSTANCE               m_hResourceHandles[MAX_PRODUCTS];

    // DLL Loader functions
    CECDLLFunctions* m_pDLLLoader;
    //
    // @cmember ToolBar array - to keep track of all the toolbars that may be added.
    //     MainFrame toolbar is stored first.
    //
    CTypedPtrArray<CPtrArray, CToolBar*> m_ToolBarArray;
    CTypedPtrArray<CPtrArray, CECFloorWnd*> m_FloorWndArray;
    CTypedPtrArray<CPtrArray, CECOutputWnd*> m_OutputWndArray;

    CYRServer* m_pServer;

    // Trace Info
    CECTrace*       m_pTrace;

    CRecentFileList* m_pRecentProjectList;

    void CECCommanderApp::PreLoadState ();

};


extern CECCommanderApp theApp;

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CECCOMMANDER_H__FF67E806_67AF_11D4_A6DA_00500447FC27__INCLUDED_)
