/* Copyright (C) 2000 Cyrano SA

 

   This file is part of OpenSTA

 

   OpenSTA is free software; you can redistribute it and/or modify it under

   the terms of the GNU General Public License as published by the Free

   Software Foundation; either version 2, or (at your option) any later

   version.

 

   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY

   WARRANTY; without even the implied warranty of MERCHANTABILITY or

   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License

   for more details.

 

   You should have received a copy of the GNU General Public License along

   with OpenSTA; see the file COPYING.  If not, write to the Free Software

   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 

   For more information about the GNU Public license see:

   http://www.gnu.org/copyleft/gpl.html

 

   OpenSTA is at:

   http://www.opensta.org/

   http://opensta.sourceforge.net/

*/



#include "stdafx.h"



#include "CECCommander.h"

#include "CECOutputView.h"

#include "CECMainFrame.h"

#include "../Common/CECTargetEnvironment.h"



#ifdef _DEBUG

#define new DEBUG_NEW

#undef THIS_FILE

static char THIS_FILE[] = __FILE__;

#endif



/////////////////////////////////////////////////////////////////////////////

// CECOutputView

//



IMPLEMENT_DYNAMIC(CECOutputView, CBCGSizingControlBar);



CECOutputView::CECOutputView() : CBCGSizingControlBar()

{}





CECOutputView::~CECOutputView()

{}





BEGIN_MESSAGE_MAP(CECOutputView, CBCGSizingControlBar)

//{{AFX_MSG_MAP(CECOutputView)

ON_WM_CREATE()

ON_WM_SIZE()

ON_WM_ERASEBKGND()

//}}AFX_MSG_MAP

END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////

// CECOutputView message handlers



int CECOutputView::OnCreate(LPCREATESTRUCT lpCreateStruct)

{

    if (CBCGSizingControlBar::OnCreate(lpCreateStruct) == -1)

        return -1;



    CRect rectDummy;

    rectDummy.SetRectEmpty ();



    // Create tabs window:

    if (!m_wndTab.Create (CBCGTabWnd::STYLE_FLAT, rectDummy, this, 1))

    {

        TRACE0("Failed to create output tab window\n");

        return -1;      // fail to create

    }



    // save the pointer to the app so this can be retrieved at a later date



    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();

    ASSERT(pApp != NULL);



    pApp->SetHandleOutputWnd (&m_wndTab);



    CPtrArray& OutputWndArray = pApp->GetOutputWndArray ();

    if (OutputWndArray.GetSize() > 0)

    {



        for (int OutputId=0; OutputId<OutputWndArray.GetSize(); OutputId++)

        {

            CECOutputWnd* pNewOutputWnd = (CECOutputWnd*) OutputWndArray.GetAt(OutputId);



            pNewOutputWnd->Create(NULL, NULL, WS_CHILD | WS_VISIBLE, CRect(0,0,0,0), &m_wndTab, 3093);

            m_wndTab.AddTab(pNewOutputWnd, pNewOutputWnd->GetName());

            pNewOutputWnd->SetIndex(OutputId);

        }



        // Set up selected tab

        m_wndTab.SetActiveTab(0);

    }



    return 0;

}





void CECOutputView::OnUpdateCmdUI(CFrameWnd* /*pTarget*/, BOOL /*bDisableIfNoHndler*/)

{}





void CECOutputView::OnSize(UINT nType, int cx, int cy)

{

    CRect rectInside;

    GetClientRect(rectInside);

    //  rectInside.InflateRect(-1,-1);

    ::SetWindowPos(m_wndTab.m_hWnd, NULL, rectInside.left, rectInside.top,

                   rectInside.Width(), rectInside.Height(),

                   SWP_NOZORDER|SWP_NOACTIVATE|SWP_NOREDRAW);

    CBCGSizingControlBar::OnSize(nType, cx, cy);

}





void CECOutputView::ActivateTab(CWnd *pTabWnd)

{

    CECOutputWnd* pOutWnd  = (CECOutputWnd*)pTabWnd;



    m_wndTab.SetActiveTab(pOutWnd->GetIndex());

}



BOOL CECOutputView::OnEraseBkgnd(CDC* pDC)

{

    return CBCGSizingControlBar::OnEraseBkgnd(pDC);

}



void CECOutputView::InsertTab(CECOutputWnd* pOutWnd)

{

    pOutWnd->Create(NULL, NULL, WS_CHILD | WS_VISIBLE, CRect(0,0,0,0), &m_wndTab, 3093);

    m_wndTab.AddTab(pOutWnd, pOutWnd->GetName());

}



/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////



// @doc

//

// @module CECOutputWnd.h

//

// Implementation file for the <c CECOutputWnd> Class

// Base class for the Output Window

//



/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////



CECOutputWnd::CECOutputWnd()

{

    m_pEditBox = NULL;

    m_nNumVisibleLines =0;

    m_nFontSize = 100;

    m_nIndex = 0;

}



CECOutputWnd::CECOutputWnd(const CString& szName, long hWnd)

{

    m_pEditBox = NULL;

    SetName         (szName);

    SetWindowHandle (hWnd);

    m_nNumVisibleLines =0;

    m_nFontSize = 100;

}



CECOutputWnd::~CECOutputWnd()

{

    if(m_pEditBox !=NULL)

    {

        delete m_pEditBox;

        m_pEditBox = NULL;

    }

}





BEGIN_MESSAGE_MAP(CECOutputWnd, CWnd)

//{{AFX_MSG_MAP(CECOutputWnd)

ON_WM_ERASEBKGND()

ON_WM_CREATE()

ON_WM_SIZE()

ON_WM_HSCROLL()

//}}AFX_MSG_MAP

END_MESSAGE_MAP()





/////////////////////////////////////////////////////////////////////////////

// BuildWnd message handlers



BOOL CECOutputWnd::OnEraseBkgnd(CDC* pDC)

{

    ASSERT_VALID(pDC);



    // Fill background with APPWORKSPACE

    CBrush backBrush(GetSysColor(COLOR_WINDOW));

    CBrush* pOldBrush = pDC->SelectObject(&backBrush);

    CRect rect;

    pDC->GetClipBox(&rect);     // Erase the area needed



    //  pDC->PatBlt(rect.left, rect.top, rect.Width(), rect.Height(), PATCOPY);

    pDC->SelectObject(pOldBrush);

    return TRUE;

}



int CECOutputWnd::OnCreate(LPCREATESTRUCT lpCreateStruct)

{

    if (CWnd::OnCreate(lpCreateStruct) == -1)

        return -1;



    m_pEditBox = new CRichEditCtrl;



    // Create a multi-row edit box in this window setting it to be read-only



    m_pEditBox->Create (WS_CHILD|WS_VISIBLE|WS_VSCROLL|ES_LEFT|ES_MULTILINE|ES_READONLY|ES_AUTOHSCROLL

                        ,CRect(0,0,0,0)

                        ,this

                        ,ID_OUTPUTEDIT);



    m_pEditBox->SetEventMask(m_pEditBox->GetEventMask() | ENM_MOUSEEVENTS|ENM_REQUESTRESIZE);



    m_Font.CreatePointFont(m_nFontSize,"Courier");

    m_pEditBox->SetFont(&m_Font);



    return 0;

}



void CECOutputWnd::OnSize(UINT nType, int cx, int cy)

{

    CWnd::OnSize(nType, cx, cy);

    if (m_pEditBox->m_hWnd != NULL)

    {

        m_pEditBox->SetWindowPos(NULL ,0, 0 ,cx ,cy ,SWP_NOMOVE);



        //7.5 is the number obtained when you divide cy by the font size

        //and then divide the number of visible lines (counted by hand)

        //by the result. There's probably a better way to do this,

        //but I don't know it.

        m_nNumVisibleLines = (7.5*cy)/m_nFontSize;

    }

}



void CECOutputWnd::Output   (const CString& szBuffer)

{

    CString WorkBuffer = szBuffer;



    //

    //    Format the new output first.

    //

    FormatOutput(WorkBuffer);



    //

    //    Recover the current contents of the window and append this to it,

    //    inserting a CR/LF before we do so.

    //

    CString s;

    m_pEditBox->GetWindowText(s);

    if (!s.IsEmpty()) s = s + "\r\n\r\n";

    s = s + WorkBuffer;

    m_pEditBox->SetSel (0 ,-1);

    m_pEditBox->ReplaceSel (s);



    //Workout how far to scroll

    int nNumLines = m_pEditBox->GetLineCount();

    m_pEditBox->LineScroll(nNumLines);

    m_pEditBox->LineScroll(-(m_nNumVisibleLines-3));



    //This allows the edit box to be updated with the new text

    MSG msg;

    while ( ::PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )

    {

        if ( !AfxGetApp()->PumpMessage( ) )

            break;

    }

}





//

//    FormatOutput

//    ============

//

//    Re-format output recovered from a remote system so that it is readable by ourselves.

//    If we find a carriage return ('\r') without a following linefeed ('\n') then we add the

//    line feed to the buffer.

//

void    CECOutputWnd::FormatOutput  (CString& Buffer)

{

    CString NewOutput;

    char* pChar;

    char* pEndChar;

    char* pNewLine;

    int StartIndex = 0;

    int EndIndex = 0;

    int BufferLength;

    char SearchChar = '\r';



    NewOutput.Empty();



    pChar = Buffer.GetBuffer(0);

    BufferLength = Buffer.GetLength();

    pEndChar = pChar + BufferLength;



    pNewLine = strchr (pChar ,SearchChar);          // Search for first newline in buffer

    if (pNewLine == NULL)

    {

        SearchChar = '\n';

        pNewLine = strchr (pChar ,SearchChar);

    }



    while ((pNewLine != NULL) && (pNewLine < pEndChar))

    {

        if (!NewOutput.IsEmpty())

        {

            NewOutput += "\r\n";

        }

        // ...and add on newline/carriage return pair if another line

        EndIndex = pNewLine - Buffer.GetBuffer(0);

        // Get index of character BEFORE newline found

        if (EndIndex < 0) EndIndex = 0;         // Sanity check.



        NewOutput = NewOutput + Buffer.Mid (StartIndex ,(EndIndex - StartIndex));

        // Copy chunk into the buffer

        pNewLine += 1;                          // Step past the new line

        if (*pNewLine == '\n') pNewLine += 1;   // ...and any trailing carriage return.



        //

        //    Calculate the starting index of the next chunk of output in the buffer.

        //

        StartIndex = pNewLine - Buffer.GetBuffer(0);



        pNewLine = strchr (pNewLine ,SearchChar);       // Search for NEXT newline in buffer

        if (pNewLine == pEndChar)

        {

            *pNewLine = '\0';

            pNewLine = NULL;

        }

    }



    //

    //    Copy the remaninder of the buffer from the end of the last chunk copied to the end into

    //    the new buffer and then copy back into the input buffer.

    //

    NewOutput += Buffer.Right(BufferLength - StartIndex);

    Buffer = NewOutput;

}





void CECOutputWnd::Activate()

{

    CECMainFrame* pMainFrame = (CECMainFrame*)AfxGetMainWnd();

    pMainFrame->ActivateOutputWnd(this);

}



void CECOutputWnd::Clear()

{

    m_pEditBox->SetWindowText("");

}





BOOL CECOutputWnd::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)

{

    NMHDR *pHdr = (NMHDR*)lParam;



    switch(pHdr->code)

    {

    case EN_MSGFILTER:  //Test for message notification from edit box

        {

            //Find out which message was sent

            MSGFILTER* pMsgFilter = (MSGFILTER*)lParam;

            switch(pMsgFilter->msg)

            {

            case WM_LBUTTONDBLCLK:

                //Double click occured on the edit box

                OnEditBoxLButtonDblClk();

                break;

            }

        }

        break;

    case EN_REQUESTRESIZE:

        {

            CRect rcClient;

            m_pEditBox->GetClientRect(rcClient);



            CBCGTabWnd *pParent = (CBCGTabWnd*)GetParent();



            //Get a pointer to scroll bar

            CScrollBar *pBar = pParent->GetScrollBar();

            if (pBar != NULL && pBar->m_hWnd != NULL)

            {

                SCROLLINFO SI;

                SI.cbSize = sizeof(SCROLLINFO);

                SI.nMax = 32768;

                SI.nMin = 0;

                SI.nPage = rcClient.Width();

                SI.fMask = SIF_RANGE|SIF_PAGE;

                pBar->SetScrollInfo(&SI,TRUE);

            }

        }

        break;

    }

    return CWnd::OnNotify(wParam, lParam, pResult);

}



//

//  OnEditBoxLButtonDblClk

//  ======================

//

//  Aim is to read in the line in the edit box which was double clicked

//  and if it contains a file name, open the file and select the line

//

void CECOutputWnd::OnEditBoxLButtonDblClk()

{

    //return;

    //Find out which char was clicked on...

    long lSelStart,lSelEnd;

    m_pEditBox->GetSel(lSelStart,lSelEnd);



    CString strLine;

    m_pEditBox->GetWindowText(strLine);



    //Look for the next new line to the right of the double click

    //and trim off everything after

    int nNewLinePos = strLine.Find("\r\n",lSelStart);

    if(nNewLinePos != -1)

    {

        strLine = strLine.Left(nNewLinePos);

    }



    //Look for end of previous line and trim of everything before it

    nNewLinePos = strLine.ReverseFind('\n');

    if(nNewLinePos != -1)

    {

        strLine = strLine.Right(strLine.GetLength()-nNewLinePos-1);

    }



    // If the line contains a filename it will be organised

    // as  "C:\FOLDER\FILENAME.EXT(200) : other text"

    // where 200 is the line number



    //Look for : separator after position 2 because this may be the : in c:

    int nColonPos = strLine.Find(':',2);

    if(nColonPos !=-1)

    {

        strLine = strLine.Left(nColonPos);



        //Look for ( from line number

        int nBracketPos = strLine.ReverseFind('(');

        if(nBracketPos != -1)

        {

            CString strFile = strLine.Left(nBracketPos);

            int nLineNumber = atoi( strLine.Right(strLine.GetLength()-nBracketPos-1));



            //Test that the file exists...

            CFileStatus fs;

            if(CFile::GetStatus(strFile,fs))

            {

                if(strFile.Right(4).CompareNoCase(".scl")==0)

                {

                    //Strip the path

                    int nPos = strFile.ReverseFind('\\');

                    if(nPos != -1)

                        strFile.Delete(0,nPos+1);



                    //Start the modeller

                    CECTargetEnvironment te;

                    te.InvokeModeller (TARGET_UNKNOWN ,strFile);

                }

            }

        }

    }

}





void CECOutputWnd::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)

{

    if(pScrollBar != NULL)

    {

        SCROLLINFO SI;

        pScrollBar->GetScrollInfo(&SI);

        switch(nSBCode)

        {

        case SB_LEFT:

            SI.nPos = 0;

            break;

        case SB_RIGHT:

            SI.nPos = SI.nMax;

            break;

        case SB_LINELEFT:

            SI.nPos -= 10;

            break;

        case SB_LINERIGHT:

            SI.nPos += 10;

            break;

        case SB_PAGELEFT:

            SI.nPos -= SI.nPage;

            break;

        case SB_PAGERIGHT:

            SI.nPos += SI.nPage;

            break;

        case SB_THUMBPOSITION:

        case SB_THUMBTRACK:

            SI.nPos = SI.nTrackPos;

            break;

        case SB_ENDSCROLL:

            break;

        }



        if(SI.nPos < 0)

            SI.nPos = 0;



        bool bMaxSize = false;

        if(SI.nPos > SI.nMax)

        {

            bMaxSize = true;

            SI.nPos = SI.nMax;

        }



        SI.nTrackPos = SI.nPos;

        SI.fMask = SIF_POS|SIF_TRACKPOS;

        pScrollBar->SetScrollInfo(&SI,TRUE);



        if(!bMaxSize)

        {

            //Tell the edit box to scroll

            m_pEditBox->SendMessage(WM_HSCROLL,MAKEWPARAM(SB_THUMBPOSITION,SI.nPos),(LPARAM)NULL);

        }

    }



    //  CWnd::OnHScroll(nSBCode, nPos, pScrollBar);

}







void CECOutputWnd::SetIndex(int nIndex)

{

    m_nIndex = nIndex;

}



int CECOutputWnd::GetIndex()

{

    return m_nIndex;

}

