/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECProjectDeleteDlg.cpp : implementation file
//

#include "stdafx.h"

//
#include "CECCommander.h"
#include "CECProjectDeleteDlg.h"
#include "..\..\HTMLHelp\Commander\Delete_Project.inl"
#include "htmlhelp.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECProjectDeleteDlg dialog


CECProjectDeleteDlg::CECProjectDeleteDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECProjectDeleteDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECProjectDeleteDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    //
    //    Recover the server
    //
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);
    m_pServer = pApp->GetServer();

    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}


void CECProjectDeleteDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECProjectDeleteDlg)
    // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECProjectDeleteDlg, CDialog)
//{{AFX_MSG_MAP(CECProjectDeleteDlg)
ON_BN_CLICKED(IDC_DELETE, OnDelete)
ON_BN_CLICKED(IDC_CLOSE, OnClose)
ON_LBN_SELCHANGE(IDC_PROJ_LIST, OnSelchangeProjectList)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
ON_UPDATE_COMMAND_UI(IDC_DELETE, OnUpdateDeleteButton)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECProjectDeleteDlg message handlers

BOOL CECProjectDeleteDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    FillProjectList();

    UpdateDialogControls(GetParent(), FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//
// OnDelete
// ========
//
// Delete a project folder from disk.
//

void CECProjectDeleteDlg::OnDelete()
{
    CListBox* pList = (CListBox*)GetDlgItem(IDC_PROJ_LIST);
    ASSERT (pList != NULL);

    // Get the selected item
    CString strProjectToDelete;

    int nSelection = pList->GetCurSel();
    if (nSelection != LB_ERR)
    {
        pList->GetText(nSelection, strProjectToDelete);
        // Get the project and delete it.
        if(!m_pServer->DeleteProject(strProjectToDelete))
        {
            CString strErr = m_pServer->GetLastError();
            AfxMessageBox(strErr);
            return;
        }

        // Refresh project list
        FillProjectList();
    }

    UpdateDialogControls(GetParent(), FALSE);
}

void CECProjectDeleteDlg::OnClose()
{
    EndDialog(0);
}

void CECProjectDeleteDlg::FillProjectList()
{
    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();
    CListBox* pList = (CListBox*)GetDlgItem(IDC_PROJ_LIST);
    pList->ResetContent();

    //Get a list of projects from the server
    CStringArray arstrProjectList;
    m_pServer->GetProjectList(arstrProjectList);

    CString strOpenProjectName = m_pServer->GetOpenProjectName();

    //  Add the project names to the project combo box.
    for(int nNdx = 0; nNdx < arstrProjectList.GetSize();nNdx++)
    {
        // only add projects for this product but omit the current project
        if(arstrProjectList[nNdx]!=strOpenProjectName)
        {
            pList->AddString(arstrProjectList[nNdx]);
        }
    }
}

//
// OnUpdateDeleteButton
// ====================
//
// Enable/Disable the delete button
void CECProjectDeleteDlg::OnUpdateDeleteButton(CCmdUI* pCmdUI)
{
    CListBox* pList = (CListBox*)GetDlgItem(IDC_PROJ_LIST);
    ASSERT (pList != NULL);

    BOOL bState = (pList->GetCurSel() != LB_ERR) ? TRUE : FALSE;

    pCmdUI->Enable(bState);
}

//
// OnSelchangeProjectList
// ======================
//
// Enable/Disable the delete button
void CECProjectDeleteDlg::OnSelchangeProjectList()
{
    // Simple enable/disable the Project List
    UpdateDialogControls(GetParent(), FALSE);

}

//
// PreTranslateMessage
// ===================
//
// Catch the Delete Key and perform the delete
BOOL CECProjectDeleteDlg::PreTranslateMessage(MSG* pMsg)
{
    if(pMsg->wParam == VK_DELETE)
    {
        // Simply call the Delete function
        OnDelete();
    }

    return CDialog::PreTranslateMessage(pMsg);
}

BOOL CECProjectDeleteDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
