/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECProjectNewDlg.cpp : implementation file
//

#include "stdafx.h"
#include "CECCommander.h"
#include "CECProjectNewDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "..\..\HTMLHelp\Commander\New_Project.inl"
#include "htmlhelp.h"
#include "Server.h"

/////////////////////////////////////////////////////////////////////////////
// CECProjectNewDlg property page

IMPLEMENT_DYNCREATE(CECProjectNewDlg, CDialog)

CECProjectNewDlg::CECProjectNewDlg() : CDialog(CECProjectNewDlg::IDD)
{
    //{{AFX_DATA_INIT(CECProjectNewDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}

CECProjectNewDlg::~CECProjectNewDlg()
{}


void CECProjectNewDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECProjectNewDlg)
    // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP

}



BEGIN_MESSAGE_MAP(CECProjectNewDlg, CDialog)
//{{AFX_MSG_MAP(CECProjectNewDlg)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECProjectNewDlg message handlers

BOOL CECProjectNewDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}



//
// OnOK
// =======
//
// Apply the changes - i.e. create the project

void CECProjectNewDlg::OnOK()
{
    // Get hold of the dialog info
    CEdit* pProjName = (CEdit*)GetDlgItem(IDC_PROJECT);
    ASSERT (pProjName != NULL);

    CString strName;

    // Insist on a project name
    pProjName->GetWindowText(strName);
    strName.TrimLeft();
    strName.TrimRight();

    if (!ValidateProjectName(strName))
    {
        return;
    }

    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();
    CECDLLFunctions *pDllLoader = pApp->GetDLLLoader();

    // And create it
    LPCREATEPROJECTFUNC Create = (LPCREATEPROJECTFUNC)pDllLoader->GetFunctionPtr("CreateProject" ,"OpenSTA");
    if (!Create(strName, ""))
    {
        return;
    }

    CDialog::OnOK();
}

//
// ValidateProjectName
// ===================
//
// Ensure that the project name is not a duplicate
BOOL CECProjectNewDlg::ValidateProjectName(CString strNewProjectName)
{
    // Don't do anything unless we have a name
    if (strNewProjectName.IsEmpty())
    {
        AfxMessageBox("You must enter a project name.");
        return FALSE;
    }

    if (strNewProjectName.GetAt(0) == '.')
    {
        AfxMessageBox("The project name must not start with a \".\"");
        return FALSE;
    }

    //Recover the server
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);
    CYRServer *pServer = pApp->GetServer();

    //Get a list of projects from the server
    CStringArray arstrProjectList;
    pServer->GetProjectList(arstrProjectList);

    for(int nNdx = 0; nNdx < arstrProjectList.GetSize();nNdx++)
    {
        if(arstrProjectList[nNdx].CompareNoCase(strNewProjectName)==0)
        {
            AfxMessageBox("A project of this name already exists.");
            return FALSE;
        }
    }

    return TRUE;
}

BOOL CECProjectNewDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}


