/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CECDLLFunctions.h
// -----------------
//
//    This is the include file for the CECDLLFunctions class
//
//=========================================================================


#if !defined(AFX_CECDLLFUNCTIONS_H__97F10279_F959_11D2_8A98_0060084D8878__INCLUDED_)
#define AFX_CECDLLFUNCTIONS_H__97F10279_F959_11D2_8A98_0060084D8878__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CECPlugInDLL.h"
#include "CECTrace.h"
#include "AfxTempl.h"

class CECDLLFunctions
{
public:
    CECDLLFunctions();
    CECDLLFunctions(CECTrace* pTrace);

    virtual ~CECDLLFunctions();

    // @cmember Initialize the DLL structures
    int     Initialize                  (const CString& strIniFileName, const CString& strType);

    // @cmember Return a pointer to the specified function
    FARPROC GetFunctionPtr              (const CString& szFunctionName ,const CString& szSearchObject, BOOL bByType = TRUE) const;

    // @cmember Get the number of plugins for the studio
    int     GetStudioComponentCount     ()
    {
        return m_nComponentCount;
    }

    // @cmember Get the number of product plugins for the studio
    int     GetStudioProductCount       ()
    {
        return m_arProductArray.GetSize();
    }

    // @cmember Set the number of plugins for the studio
    void    SetStudioComponentCount     (int count)
    {
        m_nComponentCount = count;
    }

    // @cmember Get the array of CYRANO Studio Supported Products
    CStringArray& GetAvailableProducts ()
    {
        return m_arProductArray;
    }

    // @cmember Get a requested CYRANO Studio Supported Product
    CString GetProduct (int i)
    {
        return m_arProductArray.GetAt(i);
    }

    // @cmember Get the array of CYRANO Studio Supported Products
    CStringArray& GetAvailableControls ()
    {
        return m_arControlsArray;
    }

    // @cmember Get the array of dlls relating to the extension DLLS passed in
    CStringArray& GetExtensionDLLs ()
    {
        return m_arStudioDLLs;
    }

    // @cmember Get the array of dlls relating to the product passed in
    CStringArray& GetProductDLLs ()
    {
        return m_arProductArray;
    }

    // @cmember Get the Resource Handle of a particular DLL Type
    HINSTANCE   GetDLL              (const CString& szSearchObject, BOOL bByType = TRUE) const;

    // @cmember Fire an event to ALL connected DLL's
    LRESULT     GlobalFireEvent     (UINT nID, LONG lParam);

    // @cmember Fire an event to a specified DLL
    LRESULT     SpecificFireEvent   (UINT nID, LONG lParam);

protected:


    // @cmember Load the DLL's into the array
    int     LoadProductInfo     (const CString strType ,const CString strSubType);

    // @cmember Recover all plugin DLL's of the specified type
    int     LoadAllPlugIns      (const CString& strType ,const CString& strSubType);

    // @cmember Load a specific plugin DLL
    int     LoadPlugIn          (const CString& RegisteredName ,const CString& DLLName);


private:

    FARPROC     GetFunction     (HINSTANCE hDLL ,const CString& FunctionName) const;

    // @cmember The name of the ini file used
    CString         m_IniFileName;

    // @cmember Translation of the CYRANO_HOME environment variable.
    CString         m_CyranoHome;

    // @cmember The array of DLL objects.
    CTypedPtrArray<CPtrArray, CECPlugInDLL*> m_PlugInDLLs;

    // @cmember The count of CYRANO Studio components found
    int             m_nComponentCount;

    // @cmember The array of available CYRANO Studio Compiant Products
    CStringArray    m_arProductArray;

    // @cmember The array of available Controls DLLs
    CStringArray    m_arControlsArray;

    // @cmember The array of dlls making up this product
    CStringArray    m_arStudioDLLs;

    CECTrace* m_pTrace;
};

#endif // !defined(AFX_CECDLLFUNCTIONS_H__97F10279_F959_11D2_8A98_0060084D8878__INCLUDED_)
