/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//    CECDataNameComboCombo                                                 //
//    =====================                                                 //
//                                                                          //
//    This is the cpp file for the CECDataNameCombo Class.                  //
//    Specialise the ***SECMaskEdit class by extending the allowable masks  //
//    to include the new character 'V' to specify a VTEST name              //
//    A valid VTEST dataname comprises between 1 and 16 alphanumeric        //
//    underscore or hyphen characters. The first character must be          //
//    alphabetic; spaces are not allowed; two adjacent underscores          //
//    or hyphens are not allowed; and neither is a trailing underscore      //
//    or hyphen.                                                            //
//                                                                          //
//      The last part of the requirement (ie trailing underscore or hyphen) //
//      will have to be implemented later.                                  //
//                                                                          //
//    This interpretation is implemented by over-riding the virtual function //
//    IsCharValid.                                                          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "CECDataNameCombo.h"

//
//  PSL Macros.
//
extern "C"
{
#include <psl_macros.h>
}
//
//  Function Prototypes.
//
extern "C"
{
#include <psl_rtndef.h>         // PSL Routines 
#include <trc_rtndef.h>         // Trace Routines 
#include <vtest_rtndef.h>       // V-TEST Routines 

    //
    //  Structure Definitions.
    //
#include <shrmsg.h>             // PSL Messages 
#include <psldiag_prmdef.h>     // Diagnostic Routine Parameters 
#include <pslerr_prmdef.h>      // Error Constants 
#include <psl_trc_prmdef.h>     // Trace Routine Parameters 
}


//
//    Default constructor.
//

CECDataNameCombo::CECDataNameCombo( )
        : m_bSubClassed( FALSE ),
        m_bInsert( FALSE )
{
    m_nDataNameLength = 16;
    m_strSymbols = "-_";
    m_strValidChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-";

}


//
// Destructor
//
CECDataNameCombo::~CECDataNameCombo()
{}


BEGIN_MESSAGE_MAP(CECDataNameCombo, CComboBox)
//{{AFX_MSG_MAP(CECDataNameCombo)
ON_CONTROL_REFLECT(CBN_EDITCHANGE, OnEditchange)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//
// ValidDataName
// =============
//
// Evaluate whether the dataname 'strDataName' is a valid VTEST dataname.
//
// Returns TRUE to indicate a valid dataname
//          FALSE to indicate an invalid dataname
BOOL CECDataNameCombo::ValidDataName(const CString&  strDataName)
{
    char                chDataName[20];
    short               nSize;
    CString             strLocal;
    short               bRes;

    //
    // Make a local copy of dataname
    strLocal = strDataName;
    strcpy( chDataName, strLocal.GetBuffer(0));

    //
    // Call routine
    nSize = (short) strlen(chDataName);
    CHK_DATANAME( chDataName, &nSize, &bRes);

    //
    // decode the result
    if ( bRes == 0)
    {
        //
        // valid
        return TRUE;
    }
    else
    {
        // invalid
        return FALSE;
    }
}

/////////////////////////////////////////////////////////////////////////////
// CECDataNameCombo message handlers
//

//
// SubclassCombo
// =============
//
// This function subclasses the dialog control nID to CEColourEditBox.
//
// Here we over-ride SubclassEdit to set a data name mask.
//
BOOL CECDataNameCombo::SubclassCombo (UINT nID, CWnd* pParent, WORD wParseStyle)
{
    //
    // Update internal state
    m_bSubClassed = TRUE;

    wParseStyle; //unused
    SubclassDlgItem( nID, pParent);
    return TRUE;
}

//
// IsCharValid
// ===========
//
// The user has typed the character 'ch' at position 'nPos'
// Is the dataname still valid AND is 'ch' allowable at position 'nPos'
BOOL CECDataNameCombo::IsCharValid(int nPos,  TCHAR& ch)
{
    ASSERT_VALID(this);
    BOOL bRet = FALSE;
    TCHAR chSpread = ('a' - 'A');
    BOOL bPrevCharSymbol;       // is previous char a symbol
    BOOL bNextCharSymbol;       // is next char a symbol
    char chNextChar;
    char chPrevChar;

    CString strCurrentText;
    GetWindowText(strCurrentText);

    //
    // If attempting to enter more than allowed characters, SCREAM
    if( strCurrentText.GetLength()-1 == m_nDataNameLength)
        return FALSE;

    //
    // Validate VTEST data names
    if ( m_strValidChars.Find(ch) != -1 )
    {
        //
        // force upper-case
        if(ch >= 'a' && ch <= 'z')
            ch = (TCHAR) (ch - chSpread);
        //
        // if first character ensure only alphabetical
        if (nPos == 0)
            bRet = (ch >= 'A' && ch <= 'Z');
        else
        {
            if ( m_strSymbols.Find( ch) != -1)
            {
                //
                // If at the last character position of the mask
                // then definitley not OK - current position zero based index
                if( (int)nPos+1 == m_nDataNameLength)
                    return FALSE;
                //
                // if character is symobl, ensure previous/next
                // character is NOT a symbol.
                // if we are in over-write mode the next character
                // is the character at cursor location +1
                // if in insert mode the next character is the
                // character currently under the cursor.
                chPrevChar = strCurrentText .GetAt( nPos-1);
                if (m_bInsert)
                    chNextChar = strCurrentText .GetAt( nPos);
                else
                {
                    if ((nPos+1) < strCurrentText.GetLength())
                        chNextChar = strCurrentText.GetAt( nPos+1);
                    else
                        chNextChar = ' ';
                }
                bPrevCharSymbol = m_strSymbols.Find( chPrevChar) != -1;
                bNextCharSymbol = m_strSymbols.Find( chNextChar) != -1;
                bRet = ((!bPrevCharSymbol) && (!bNextCharSymbol));
            }
            else
                bRet = TRUE;
        } // else clause in nPos==0
    } // if character is allowable

    else
    {
        bRet = FALSE;
    } // else clause of if char allowable.

    return bRet;
}


//
// SetLength
// =========
//
// Set/Reset the length of the edit field.
void CECDataNameCombo::SetLength(const int nLength)
{
    m_nDataNameLength = nLength;
}

//
// GetData
// =======
//
// Function to get the data name.
// TODO - Because we are no longer deriving from SECMaskEdit we can remove
// these two Get/Set data functions and use the UpdateData mechanism.
void CECDataNameCombo::GetData(CString& strDataName, BOOL bUnused)
{
    bUnused;

    GetWindowText( strDataName );
}

//
// SetData
// =======
//
// Function to set the data name.
// TODO - Because we are no longer deriving from SECMaskEdit we can remove
// these two Get/Set data functions and use the UpdateData mechanism.
void CECDataNameCombo::SetData(CString& strDataName)
{
    SetWindowText( strDataName );
}

//
// PreTranslateMessage
// ===================
//
// Used by class CWinApp to translate window messages before they are dispatched to
// the TranslateMessage and DispatchMessage Windows functions.
//
// If the function returns non-zero it indicates the message has been dealt with and need
// not be dispatched.
//
// We use this mechanism to enforce the requirement that the user can not enter
// a dataname where the last character is a symbol, ('-' or '_')

BOOL CECDataNameCombo::PreTranslateMessage(MSG* pMsg)
{
    if ((pMsg->message == WM_KEYDOWN) &&        // message is keypress AND
            (pMsg->wParam == 16))                   // key pressed is a CR
    {
        //
        // Retrieve the dataname
        CString strDataName;
        GetWindowText( strDataName );

        //
        // Check if last char is a symbol
        BOOL     bLastCharSymbol = FALSE;
        if (!strDataName.IsEmpty() )
        {
            CString chLastChar = strDataName.Right( 1 );
            bLastCharSymbol = (m_strSymbols.Find(chLastChar) != -1);
        }

        //
        // Validate dataname not empty AND last char not a symbol.
        if ((strDataName.IsEmpty()) ||
                (bLastCharSymbol))
        {
            MessageBeep( MB_ICONEXCLAMATION );
            return TRUE;
        }
    }

    return CComboBox::PreTranslateMessage(pMsg);
}



void CECDataNameCombo::OnEditchange()
{
    // Retrieve position of cursor.
    DWORD dwStartPos = GetEditSel();

    // If at zero then there is no text left in the combo box to update
    if (dwStartPos == 0) return;

    // Which is contained in the low order word returned by GetSel
    WORD wStartPos = (WORD)LOWORD(dwStartPos) - 1;

    // Now we obtain the new character
    CString szText;
    GetWindowText(szText);
    TCHAR ch = szText.GetAt(wStartPos);

    // And save the old text value in case of an illegal entry
    CString szOldEntry = szText.Left(wStartPos) + szText.Mid(wStartPos+1);

    // So check the new character
    if (IsCharValid(wStartPos, ch))
    {
        // And display it
        //
        // force upper-case
        ch = (char) toupper(ch);
        szText.SetAt(wStartPos, ch);
        SetWindowText(szText);
    }
    else
    {
        // Revert back to the old entry
        SetWindowText(szOldEntry);

        // And reposition the cursor
        if (dwStartPos) dwStartPos--;

        MessageBeep( MB_ICONEXCLAMATION );
    }

    // And set the cursor location to the next character position
    SetEditSel(dwStartPos, dwStartPos);
}
