/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//    CECDataNameCombo                                                      //
//    ================                                                      //
//                                                                          //
//    This is the header file for the CECDataNameCombo Class.               //
//    Specialise the ***SECMaskEdit class by extending the allowable masks  //
//    to include the new character 'V' to specify a VTEST name              //
//    A valid VTEST dataname comprises between 1 and 16 alphanumeric        //
//    underscore or hyphen characters. The first character must be          //
//    alphabetic; spaces are not allowed; two adjacent underscores          //
//    or hyphens are not allowed; and neither is a trailing underscore      //
//    or hyphen.                                                            //
//                                                                          //
//      The last part of the requirement (ie trailing underscore or hyphen) //
//      will have to be implemented later.                                  //
//                                                                          //
//    This interpretation is implemented by over-riding the virtual function //
//    IsCharValid.                                                          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

//
//    Only load if not already loaded
//
#ifndef CECDataNameCombo_Loaded
#define CECDataNameCombo_Loaded

/////////////////////////////////////////////////////////////////////////////
// CECDataNameCombo window

class AFX_EXT_CLASS  CECDataNameCombo : public CComboBox
{
    // Construction
public:
    void SetData(CString& strDataName);
    void GetData(CString& strDataName, BOOL bUnused = TRUE);
    void SetLength(const int nLength);

    CECDataNameCombo();

    // Attributes
public:

    //
    // Over-rides
public:

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CECDataNameCombo)
public:
    virtual BOOL PreTranslateMessage(MSG* pMsg);
    //}}AFX_VIRTUAL

    // Implementation
public:
    virtual ~CECDataNameCombo();
    BOOL SubclassCombo (UINT nID, CWnd* pParent, WORD wParseStyle);

    //
    //    Static member function which can be called just to validate a dataname.
    //
    static ValidDataName(const CString&  strDataName);
    // Generated message map functions
protected:
    //{{AFX_MSG(CECDataNameCombo)
    afx_msg void OnEditchange();
    //}}AFX_MSG

    DECLARE_MESSAGE_MAP()
private:
    BOOL IsCharValid(int nPos,  TCHAR& ch);


    CString m_strValidChars;
    CString m_strSymbols;
    BOOL m_bKeyProcessed;
    BOOL m_bInsert;
    int m_nDataNameLength;
    BOOL m_bSubClassed;
};
#endif
/////////////////////////////////////////////////////////////////////////////


