/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CECDefaultCodeWriter.cpp
// ------------------------
//
//    This is the implementation file for the CECDefaultCodeWriter class
//
//////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CECDefaultCodeWriter.h"
#include "..\Interfaces\CyrPlugIns.h"
#include "CYRTestProperties.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECDefaultCodeWriter

CECDefaultCodeWriter::CECDefaultCodeWriter  ()
{}


CECDefaultCodeWriter::~CECDefaultCodeWriter()
{}

//
// GetSectionTitle
// ===============
//
// This Function returns a section Title


CString CECDefaultCodeWriter::GetSectionTitle(SECTION nType)
{
    CString szBuffer("");

    if      ((nType == SCL_ENVIRONMENT) || (nType == TSS_ENVIRONMENT) || (nType == THS_ENVIRONMENT))    szBuffer = "\nEnvironment\n\n";
    else if ((nType == SCL_DEFINITIONS) || (nType == TSS_DEFINITIONS) || (nType == THS_ENVIRONMENT))    szBuffer = "\nDefinitions\n\n";
    else if ((nType == SCL_CODE)        || (nType == TSS_CODE)        || (nType == THS_ENVIRONMENT))    szBuffer = "\nCode\n\n";

    return szBuffer;
}

//
// GetDefaultExitText
// ==================
//
// Get the default Exit Statement
//
CString CECDefaultCodeWriter::GetDefaultExitText()
{
    return "Exit";
}

//
// GetDefaultHeaderText
// ====================
//
// Get the Header Default Text
//
CString CECDefaultCodeWriter::GetDefaultHeaderText(CString szFileName)
{
    CString szBuffer("");

    szBuffer += "!/////////////////////////////////////////////////////////////////////////\n";
    szBuffer += "!//                                                                     //\n";
    szBuffer += "!// OpenSTA http://www.opensta.org/                                     //\n";
    szBuffer += "!//                                                                     //\n";
    szBuffer += "!//  This code has been generated for you as a starting template.       //\n";
    szBuffer += "!//  Please refer to the SCL Documentation if you wish to modify it     //\n";
    szBuffer += "!//  to provide further functionality to your script.                   //\n";
    szBuffer += "!//                                                                     //\n";
    szBuffer += "!//  File Name:  ";
    szBuffer += szFileName;
    szBuffer += "\n";
    szBuffer += "!//                                                                     //\n";
    szBuffer += "!/////////////////////////////////////////////////////////////////////////\n";

    return szBuffer;
}

//
// GetDefaultEnvironmentText
// =========================
//
// Get the Environment Default Text
//
CString CECDefaultCodeWriter::GetDefaultEnvironmentText()
{
    CString szBuffer("");

    return szBuffer;
}

//
// GetDefaultDefinitionText
// ========================
//
// Get any Stress Test Specific Text
//
CString CECDefaultCodeWriter::GetDefaultDefinitionText()
{
    CString szBuffer("");

    szBuffer += "\tInteger \tCOUNTER\n";
    szBuffer += "\tInteger \tTOTAL_REQUESTS\n";
    szBuffer += "\n";
    szBuffer += "\tCharacter*8 \tTIME\n";

    return szBuffer;
}

//
// GetDefaultCodeText
// ==================
//
// Get any Stress Test Specific Text
//
CString CECDefaultCodeWriter::GetDefaultCodeText()
{
    CString szBuffer("");

    szBuffer += "\tLoad time into TIME\n\n";
    szBuffer += "\tNote \" COMPLETED AT \", TIME,\" WITH \", &\n";
    szBuffer += "\tTOTAL_REQUESTS, \" REQUESTS \"\n\n";

    return szBuffer;
}

//
// GetDefaultText
// ==============
//
// This Function returns a section Title

CString CECDefaultCodeWriter::GetDefaultText(SECTION nType, CString szFileName /*=""*/)
{
    CString szBuffer("");

    if      ((nType == SCL_HEADER)      ||  (nType == TSS_HEADER)       ||  (nType == THS_HEADER))      szBuffer = GetDefaultHeaderText(szFileName);
    else if ((nType == SCL_ENVIRONMENT) ||  (nType == TSS_ENVIRONMENT)  ||  (nType == THS_ENVIRONMENT)) szBuffer = GetDefaultEnvironmentText();
    else if ((nType == SCL_DEFINITIONS))                                                                szBuffer = GetDefaultDefinitionText();
    else if ((nType == SCL_CODE))                                                                       szBuffer = GetDefaultCodeText();
    else if ((nType == SCL_EXIT)        ||  (nType == TSS_EXIT)         ||  (nType == THS_EXIT))        szBuffer = GetDefaultExitText();

    return szBuffer;
}

//
// DoesTemplateExist
// =================
//
// This function determines whether a template exists

BOOL CECDefaultCodeWriter::DoesTemplateExist(SECTION nType)
{
    // Now obtain the template that we are interested in
    CString szTemplate = GetTemplateFileName(nType);

    // Then we see if our template file exists
    CFileStatus FileStatus;
    return (CFile::GetStatus(szTemplate, FileStatus));
}

//
// UseTemplate
// ===========
//
// This function uses a template file to overide the default text which the
// wizard will generate

CString CECDefaultCodeWriter::UseTemplate(SECTION nType)
{
    CString String;
    CString szBuffer("");

    // Open the file ...
    try
    {
        CStdioFile file(GetTemplateFileName(nType), CFile::modeRead | CFile::typeText);

        while (file.ReadString(String) != NULL)
        {
            szBuffer += String;
            szBuffer += "\n";
        }

        szBuffer += "\n";
    }
    catch (CFileException* e)
    {
        e->ReportError();
        e->Delete();
    }

    return szBuffer;
}

//
// GetTemplateFileName
// ===================
//
// This function returns the expected template file name for the section requested

CString CECDefaultCodeWriter::GetTemplateFileName(SECTION nType)
{
    // First we get hold of the installation directory ...

    // Get a handle to the Mainframe
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    // Now get the User Key from the framework
    LPCSTR lpszInstallDir = (char*) pMainWnd->SendMessage (WMU_GET_INSTALLDIR);
    CString szInstallDir(lpszInstallDir);

    if (szInstallDir.IsEmpty())
        return "";

    StripOffTrailingSlash(szInstallDir);

    // Now find out the template name
    CString szFile;

    if      (nType == SCL_HEADER)       szFile = "scl_header.inc";
    else if (nType == SCL_ENVIRONMENT)  szFile = "scl_environment.inc";
    else if (nType == SCL_DEFINITIONS)  szFile = "scl_definitions.inc";
    else if (nType == SCL_CODE)         szFile = "scl_code.inc";
    else if (nType == SCL_EXIT)         szFile = "scl_exit.inc";
    else if (nType == TSS_HEADER)       szFile = "tss_header.inc";
    else if (nType == TSS_ENVIRONMENT)  szFile = "tss_environment.inc";
    else if (nType == TSS_DEFINITIONS)  szFile = "tss_definitions.inc";
    else if (nType == TSS_CODE)         szFile = "tss_code.inc";
    else if (nType == TSS_EXIT)         szFile = "tss_exit.inc";
    else if (nType == THS_HEADER)       szFile = "ths_header.inc";
    else if (nType == THS_ENVIRONMENT)  szFile = "ths_environment.inc";
    else if (nType == THS_DEFINITIONS)  szFile = "ths_definitions.inc";
    else if (nType == THS_CODE)         szFile = "ths_code.inc";
    else if (nType == THS_EXIT)         szFile = "ths_exit.inc";

    if (szFile.IsEmpty())
        return "";

    return szInstallDir + "\\" + szFile;
}

//
// StripOffTrailingSlash
// =====================
//
// Ensure we do not have a trailing slash at the end of our path
//
void CECDefaultCodeWriter::StripOffTrailingSlash(CString& szPath)
{
    CString szLastChar = szPath.Right(1);

    if (szLastChar.CompareNoCase("\\") == 0)
    {
        // OK so strip it off
        szPath = szPath.Left(szPath.GetLength() - 1);
    }
}

void CECDefaultCodeWriter::GenerateDefaultOtaTof(const CString& strOTAPath,const CString& strPhysicalNode)
{
    //get the current date and time
    CTime time(CTime::GetCurrentTime());
    CString strDate = time.Format("%d-%b-%Y %H:%M:%S");

    //Generate TOF text
    CString strTof;
    strTof += "tof.6\n";
    strTof += "date ";
    strTof += strDate +"\n";
    strTof += "lastedit ";
    strTof += strDate +"\n";
    strTof += "name\n";
    strTof += "description\n";

    strTof += "\n";
    strTof += "SimpleQuery\n";
    strTof += "enabled 1\n";
    strTof += "type 2\n";
    strTof += "tgtname\n";
    strTof += "delta 0\n";
    strTof += "frequency 5\n";
    strTof += "heading % CPU Usage\n";
    strTof += "text \"\n";
    strTof += "\\\\" + strPhysicalNode + "\\System\\% Total Processor Time\"\n";

    strTof += "\n";
    strTof += "SimpleQuery\n";
    strTof += "enabled 1\n";
    strTof += "type 2\n";
    strTof += "tgtname\n";
    strTof += "delta 0\n";
    strTof += "frequency 5\n";
    strTof += "heading PAGE SWAPPING\n";
    strTof += "text \"\n";
    strTof += "\\\\" + strPhysicalNode + "\\Memory\\Pages/Sec\"\n";

    strTof += "\n";
    strTof += "SimpleQuery\n";
    strTof += "enabled 1\n";
    strTof += "type 2\n";
    strTof += "tgtname\n";
    strTof += "delta 0\n";
    strTof += "frequency 5\n";
    strTof += "heading % NET USE\n";
    strTof += "text \"\n";
    strTof += "\\\\" + strPhysicalNode + "\\Server\\Bytes Total/sec\"";

    try
    {
        CStdioFile file(strOTAPath,CFile::modeCreate|CFile::modeWrite);
        file.WriteString(strTof);
        file.Close();
    }
    catch(...)
    {
        ASSERT(FALSE);
    }
}
