/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CECSclWriter.cpp
// ----------------
//
//    This is the implementation file for the CECSclWriter class
//
//////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CECSclWriter.h"
#include "CECTssWriter.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECSclWriter

CECSclWriter::CECSclWriter  ()
{}


CECSclWriter::~CECSclWriter()
{}

//
// GetNameDefinitionText
// =====================
//

CString CECSclWriter::GetNameDefinitionText(SCL_SECTION nType, CString szDescription /*=""*/)
{
    if (nType == ENVIRONMENT) return "\tDescription \t\"" + szDescription + "\"\n";
    else                      return "";
}

//
// GetScriptDependencyText
// =======================
//
CString CECSclWriter::GetScriptDependencyText(SCL_SECTION nType, CString szScriptName /*=""*/,
        int nPercent /*=0*/, int nDependencyIndex/*=0*/,int nTotalDependencyCount /*=0*/)
{
    CString szBuffer("");
    static CString szReport = "";
    static CString szTotals = "";
    if(nDependencyIndex==1)
    {
        szReport.Empty();
        szTotals.Empty();
    }

    // Must have a script name to be able to code anything
    CString strScript;
    strScript.Format("TOTAL_SCRIPT_%d",nDependencyIndex);

    if (nType == DEFINITIONS)
    {
        szBuffer += "\tInteger \t"+ strScript + "\n";
    }
    else if (nType == CODE)
    {
        // Conver the percentage field
        CString szPercent;
        szPercent.Format("%d", nPercent);

        // Add the variable generation code first time in
        if (nDependencyIndex == 1)
        {
            szBuffer += "\t\tGenerate RANDOM\n\n";
        }

        // Now a suitable comment for each script
        szBuffer += "! This Should Generate " + szScriptName + " scripts " + szPercent + "% of the time\n\n";

        // And the initial IF statement
        if (nDependencyIndex == 1)
        {
            szBuffer += "\t\tIf\t(RANDOM <=" + szPercent + ") then\n\n";
        }
        else if (nDependencyIndex < nTotalDependencyCount)
        {
            // Whereas the rest get else if's
            szBuffer += "\t\tElseif\t(RANDOM <=" + szPercent + ") then\n\n";
        }
        else
        {
            // or else's
            szBuffer += "\t\tElse\n\n";
        }

        // Always Call the actual script
        //      CECTssWriter Writer;
        //      if (Writer.GetCurrentTargetEnvironment() == "COMMAND")
        {
            szBuffer += "\t\t\tCall Script \"" + szScriptName + "\"\n";
        }
        //      else
        //      {
        //          szBuffer += "\t\t\tCall Tof \"" + szScriptName + "\"\n";
        //      }

        // Increment it's counter
        szBuffer += "\t\t\tSet " + strScript + " = " + strScript + "+1\n\n";

        // And add it to the report text ...

        // If the script count is greater than one tab all scripts into line
        if (nDependencyIndex > 1) szReport += "\t\t ";
        if (nDependencyIndex > 1) szTotals += "\t\t\t\t";

        // And add in a report string
        szReport += "\t\"" + szScriptName + "s = \", " +strScript;
        szTotals += "\t" + strScript;

        // Add in a continuation if there's more to follow
        if (nDependencyIndex < nTotalDependencyCount) szReport += ", &";
        if (nDependencyIndex < nTotalDependencyCount) szTotals += "\t+ &";

        // And throw in a new line
        szReport += "\n";
        szTotals += "\n";

        if (nDependencyIndex == nTotalDependencyCount)
        {
            // Finally we end the If statement
            szBuffer += "\t\tEndif\n\n";

            // And report the Script Totals
            szBuffer += "\t\tNote " + szReport + "\n";

            // And set the TOTAL_REQUESTS Variable
            szBuffer += "\t\tSet TOTAL_REQUESTS = " + szTotals + "\n\n";

            // And issue a second delay
            szBuffer += "\t\tWait 1\n\n";
        }
    }

    return szBuffer;
}

//
// GetDurationText
// ===============
//
// Get any Duration Specific Text
//
CString CECSclWriter::GetDurationText(SCL_SECTION nType, BOOL bSpecifyAtRuntime /*=FALSE*/, LOOP_MODE nMode/*=ENDDO*/, int nIterations /*=0*/)
{
    CString szBuffer("");

    if (nType == DEFINITIONS)
    {
        szBuffer += "\tInteger \tITERATIONS";

        // And see if we are obtaining the value at runtime or hardcoding it now
        if (bSpecifyAtRuntime) szBuffer += ", prompt = 'Enter the number of Iterations required'";

        szBuffer += "\n";
    }
    else if (nType == CODE)
    {
        if (nMode == DO)
        {
            if (!bSpecifyAtRuntime)
            {
                szBuffer += "! Set up the Iterations field\n\n";

                CString szNumber;
                szNumber.Format("%d", nIterations);

                szBuffer += "\tSet ITERATIONS = " + szNumber + "\n\n";
            }
            szBuffer += "\tDo COUNTER = 1,ITERATIONS\n\n";
        }
        else
        {
            szBuffer += "\t\tNote \"ITERATION \",COUNTER\n\n";
            szBuffer += "\tEndDo\n\n";
        }
    }

    return szBuffer;
}

//
//  GetSetupDurationText
//  ====================
//
//  Ask for number of iterations
//  (must be first line in code section, which is why its not in GetDurationText())
//
CString CECSclWriter::GetSetupDurationText()
{
    CString szBuffer("");

    // Set up At runtime
    szBuffer += "! Set up the Iterations field\n\n";
    szBuffer += "\tEntry [ITERATIONS]\n\n";

    return szBuffer;
}

//
// GetAuxillaryScriptText
// ======================
//
// Get any Auxillary Script Specific Text
//
CString CECSclWriter::GetAuxillaryScriptText(SCL_SECTION nType, AUXILLARY_SCRIPT nUsage /*=SETUP*/, CString szScriptName /*=""*/)
{
    CString szBuffer("");

    // Don't bother if we dont have a script name at all
    if ((szScriptName.CompareNoCase("<None>") == 0) || (szScriptName.IsEmpty()))
        return "";

    szScriptName.MakeUpper();

    if (nType == CODE)
    {
        if (nUsage == CONNECT)
        {
            szBuffer += "! Call the prolog Script\n\n";
            szBuffer += "\tNote \"Calling the prolog script\"\n\n";
        }
        else
        {
            szBuffer += "! Call the epilog Script\n\n";
            szBuffer += "\tNote \"Calling the epilog script\"\n\n";
        }

        //
        //    Always Call Script
        //
        //      CECTssWriter Writer;
        //      if (Writer.GetCurrentTargetEnvironment() == "COMMAND")
        //      {
        szBuffer += "\tCall Script \"" + szScriptName + "\" [BROWSER_TYPE,USE_PAGE_TIMERS]\n\n";
        //      }
        //      else
        //      {
        //          szBuffer += "\tCall Tof \"" + szScriptName + "\"\n\n";
        //      }

        if (nUsage == CONNECT)
        {
            szBuffer += "\tNote \"Completed prolog script\"\n\n";
        }
        else
        {
            szBuffer += "\tNote \"Completed epilog script\"\n\n";
        }

    }
    return szBuffer;
}

//
// GetTemplateText
// ===============
//
// If a template file does not exist this function will create some default
// text for the desired section.

CString CECSclWriter::GetTemplateText(SCL_SECTION nType, CString szFileName /*=""*/)
{
    // Check to see if we have a template ...
    if (DoesTemplateExist((SECTION) nType))
    {
        // If so - use it
        return UseTemplate((SECTION) nType);
    }

    // Else create some default test
    return CECDefaultCodeWriter::GetDefaultText((SECTION) nType, szFileName);
}

//
// GetSectionTitle
// ===============
//
// Simply use the default

CString CECSclWriter::GetSectionTitle(SCL_SECTION nType)
{
    return CECDefaultCodeWriter::GetSectionTitle((SECTION) nType);
}








