/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CECStyleFinder.cpp
// ------------------
//
//    This is the implementation file for the CECStyleFinder class
//
//=========================================================================


#include "stdafx.h"
#include "CECStyleFinder.h"
#include "..\Interfaces\CyrPlugIns.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CECStyleFinder::CECStyleFinder()
{
}


CECStyleFinder::~CECStyleFinder()
{
}

//
// IsSubProduct
// ============
//
// Determines whether a mask has the CYR_IS_SUBPRODUCT style set
//

BOOL CECStyleFinder::IsSubProduct(DWORD dwStyles)
{
    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}

//
// HasServer
// =========
//
// Determines whether a mask has the CYR_HAS_SERVER style set
//
BOOL CECStyleFinder::HasServer(DWORD dwStyles)
{
    // Shift to the bit in question
    dwStyles = dwStyles>>1;

    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}

//
// HasProject
// ==========
//
// Determines whether a mask has the CYR_HAS_PROJECT style set
//
BOOL CECStyleFinder::HasProject(DWORD dwStyles)
{
    // Shift to the bit in question
    dwStyles = dwStyles>>1;
    dwStyles = dwStyles>>1;

    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}

//
// HasSeparator
// ============
//
// Determines whether a mask has the CYR_NEEDS_SEPARATOR style set
//
BOOL CECStyleFinder::NeedsSeparator(DWORD dwStyles)
{
    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}

//
// IsRemoveable
// ============
//
// Determines whether a mask has the CYR_IS_REMOVEABLE style set
//
BOOL CECStyleFinder::IsRemoveable(DWORD dwStyles)
{
    // Shift to the bit in question
    dwStyles = dwStyles>>1;

    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}

//
// HasExtendedUIUpdate
// =================
//
// Determines whether a mask has the CYR_EXTENDED_UI_UPDATE style set
//
BOOL CECStyleFinder::HasExtendedUIUpdate(DWORD dwStyles)
{
    // Shift to the bit in question
    dwStyles = dwStyles>>1;
    dwStyles = dwStyles>>1;
    dwStyles = dwStyles>>1;
    dwStyles = dwStyles>>1;

    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}

//
// NeedsServer
// ===========
//
// Determines whether a mask has the CYR_NEEDS_SERVER style set
//
BOOL CECStyleFinder::NeedsServer(DWORD dwStyles)
{
    // Shift to the bit in question
    dwStyles = dwStyles>>1;
    dwStyles = dwStyles>>1;

    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}

//
// NeedsProject
// ============
//
// Determines whether a mask has the CYR_NEEDS_PROJECT style set
//
BOOL CECStyleFinder::NeedsProject(DWORD dwStyles)
{
    // Shift to the bit in question
    dwStyles = dwStyles>>1;
    dwStyles = dwStyles>>1;
    dwStyles = dwStyles>>1;

    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}

BOOL CECStyleFinder::HasContext(DWORD dwStyles)
{
    return ((dwStyles & CYR_HAS_CONTEXT) != 0);
}
//
// IsEditable
// ==========
//
// Determines whether a mask has the CYR_EDITABLE_DOCUMENT style set
//
BOOL CECStyleFinder::IsEditable(DWORD dwStyles)
{
    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}

//
// HasTemplate
// ===========
//
// Determines whether a mask has the CYR_EDITABLE_DOCUMENT style set
//
BOOL CECStyleFinder::HasTemplate(DWORD dwStyles)
{
    // Shift to the bit in question
    dwStyles = dwStyles>>1;
    dwStyles = dwStyles>>1;

    // And see if it is set
    return (dwStyles%2) ? TRUE : FALSE;
}
