/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECGraphSelect.cpp : implementation file
//

#include "stdafx.h"
#include "CECDynTestGraphs.h"
#include "CECGraphSelect.h"
#include "xchartctrl.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECGraphSelect dialog


CECGraphSelect::CECGraphSelect(CWnd* pParent /*=NULL*/)
        : CDialog(CECGraphSelect::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECGraphSelect)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pParent = NULL;

    m_nLastAvailable = -1;
    m_nLastSelected = -1;
}


void CECGraphSelect::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECGraphSelect)
    DDX_Control(pDX, IDC_SELECTED, m_ListSelected);
    DDX_Control(pDX, IDC_AVAILABLE, m_ListAvailable);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECGraphSelect, CDialog)
//{{AFX_MSG_MAP(CECGraphSelect)
ON_LBN_SELCHANGE(IDC_AVAILABLE, OnSelchangeAvailable)
ON_LBN_DBLCLK(IDC_AVAILABLE, OnDblclkAvailable)
ON_LBN_SELCHANGE(IDC_SELECTED, OnSelchangeSelected)
ON_LBN_DBLCLK(IDC_SELECTED, OnDblclkSelected)
ON_BN_CLICKED(IDC_ADD, OnAdd)
ON_BN_CLICKED(IDC_REMOVE, OnRemove)
ON_BN_CLICKED(IDC_ADD_ALL, OnAddAll)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECGraphSelect message handlers

void CECGraphSelect::SetAccess(CECDynTestGraphs *pParent)
{
    m_pParent = pParent;
}

BOOL CECGraphSelect::OnInitDialog()
{
    CDialog::OnInitDialog();

    ASSERT(m_pParent != NULL);

    UpdateAvailableList();
    UpdateSelectedList();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECGraphSelect::UpdateAvailableList()
{
    int nCount = m_pParent->m_availableTypes.GetSize();
    for (int i = 0; i<nCount; i++)
    {
        GRAPH_TYPE type = m_pParent->m_availableTypes.GetAt(i);
        m_ListAvailable.AddString(type.szName);
    }

}

void CECGraphSelect::UpdateSelectedList()
{
    m_ListSelected.ResetContent();

    int nCount = m_pParent->m_selectedTypes.GetSize();
    for (int i = 0; i<nCount; i++)
    {
        GRAPH_TYPE type = m_pParent->m_selectedTypes.GetAt(i);
        m_ListSelected.AddString(type.szName);
    }

}

void CECGraphSelect::OnSelchangeAvailable()
{
    m_nLastAvailable = m_ListAvailable.GetCurSel();
}

void CECGraphSelect::OnDblclkAvailable()
{
    m_nLastAvailable = m_ListAvailable.GetCurSel();
    AddGraph(m_nLastAvailable);
}

void CECGraphSelect::OnSelchangeSelected()
{
    m_nLastSelected = m_ListSelected.GetCurSel();

}

void CECGraphSelect::OnDblclkSelected()
{
    if (m_ListSelected.GetCount() == 1)
    {
        MessageBox("At least one graph must be selected!", "OpenSTA", MB_OK);
        return;
    }

    m_nLastSelected = m_ListSelected.GetCurSel();
    RemoveGraph(m_nLastSelected);
}

void CECGraphSelect::OnAdd()
{
    if (m_nLastAvailable >= 0)
    {
        AddGraph(m_nLastAvailable);
    }
}

void CECGraphSelect::OnRemove()
{
    if (m_ListSelected.GetCount() == 1)
    {
        MessageBox("At least one graph must be selected!", "OpenSTA", MB_OK);
        return;
    }

    if (m_nLastSelected >= 0)
    {
        RemoveGraph(m_nLastSelected);
    }
}

void CECGraphSelect::OnOK()
{
    CDialog::OnOK();
}

void CECGraphSelect::AddGraph(int nListIndex)
{
    GRAPH_TYPE type = m_pParent->m_availableTypes.GetAt(nListIndex);
    type.dataGroup = -1;

    int nCount = m_pParent->m_selectedTypes.GetSize();
    BOOL bFound = FALSE;
    for (int i = 0; i<nCount; i++)
    {
        GRAPH_TYPE t = m_pParent->m_selectedTypes.GetAt(i);
        if (t.szName == type.szName)
        {
            bFound = TRUE;
        }

    }

    if (!bFound)
    {
        m_pParent->m_selectedTypes.Add(type);
    }
    UpdateSelectedList();
}

void CECGraphSelect::RemoveGraph(int nListIndex)
{
    GRAPH_TYPE typ = m_pParent->m_selectedTypes[nListIndex];
    if (typ.pChart != NULL)
        delete typ.pChart;

    m_pParent->m_selectedTypes.RemoveAt(nListIndex);
    UpdateSelectedList();
}

void CECGraphSelect::OnAddAll()
{
    m_pParent->m_selectedTypes.RemoveAll();

    for (int i = 0; i<m_pParent->m_availableTypes.GetSize(); i++)
    {
        AddGraph(i);
    }
}

