/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#if !defined(AFX_CECINJECTORVIEW_H__A2825E43_0ED6_11D4_BD8D_00C04F89E392__INCLUDED_)
#define AFX_CECINJECTORVIEW_H__A2825E43_0ED6_11D4_BD8D_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// CECInjectorView.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CECInjectorView form view
#include "OSTAORB\OSTAORB.h"
#include "TestManager.h"
#include "Repository/CyrStRepository.h"
#include "CYRWrapLib\CECTestOutput.h"

#ifndef __AFXEXT_H__
#include <afxext.h>
#endif

#include "resource.h"

class CInjectorData;

class CECInjectorView : public CFormView
{
protected:
    CECInjectorView();           // protected constructor used by dynamic creation
    DECLARE_DYNCREATE(CECInjectorView)

	typedef enum {enInjectorBmp, enAboveThresholdBmp,enBelowThresholdBmp} InjectorListImages;

    typedef enum { enNameCol,enIPCol,enCPUCol,enSwapCol,enNetCol}InjectorListColumns;
    //      enTestCol,enStartCol,enPIDCol,enVUsCol,enTotalVUsCol}

public:
    typedef enum {enCPUType,enNetType,enSwapType,enNumDefQueryTypes} DefQueryTypes;

    void StartMonitor(CString strRegName);
    void SetRefreshDelay(int nDelay);

    // Form Data
public:
    //{{AFX_DATA(CECInjectorView)
    enum { IDD = IDD_INJECTOR_VIEW };
    CListCtrl   m_ctlInjectorList;
    //}}AFX_DATA

    // Attributes
public:

    // Operations
public:
    void SetRefreshRate(int nRate);

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CECInjectorView)
public:
    virtual void OnInitialUpdate();
protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    //}}AFX_VIRTUAL

    // Implementation
protected:
    virtual ~CECInjectorView();
#ifdef _DEBUG
    virtual void AssertValid() const;
    virtual void Dump(CDumpContext& dc) const;
#endif

    // Generated message map functions
    //{{AFX_MSG(CECInjectorView)
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg void OnSetFocus(CWnd* pNewWnd);
    afx_msg BOOL OnEraseBkgnd(CDC *pDC);
    afx_msg void OnInjectorsSelectColumns();
    afx_msg void OnDestroy();
    afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
    afx_msg void OnPopupShowgraphs();
    afx_msg void OnUpdatePopupShowgraphs(CCmdUI* pCmdUI);
    afx_msg BOOL OnHelpInfo(HELPINFO* pHelpInfo);
    afx_msg void OnFullHelp();
    afx_msg LRESULT OnDataReady(WPARAM,LPARAM);
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()

private:
    CString m_szTitle;

private:
    CString m_szHtmlHelpFile;

    CORBAServer* m_pCorba;
    CyrStTestRun_var m_TestRun;
    int m_nOTADataIndex;
    int m_nOTAHeaderIndex;
    CImageList m_InjectorImages;
    CString m_strManagerName;

    volatile BOOL m_bQuitting;
    volatile int m_nRefreshDelay;

    CMapStringToPtr m_mapIPtoInjector;
    CMapStringToPtr m_mapNdxToInjector;
    CMapStringToString m_mapIPToName;

    void InitOTADataIndexes();
    BOOL IsOTADescriptionLine(CString strOTALine);
    void AddOTAQueryType(const CString& strInjectorIP,const CString& strQueryType,int nDataNdx);
    CString GetComputerNameFromIP(const CString& strIP);
    void UpdateInjectorList();
    void FilterColumns(CListCtrl *pListCtrl);
    void LoadThresholds();
    BOOL IsShuttingDown();

    void SaveSettings();
    void RestoreSettings();

    static UINT DataRefreshThread(LPVOID lpParam);
    CWinThread *m_pRefreshThread;

    void GetNodeList(TestLevel_ptr pTestLevel);
};

class CInjectorData
{
public:
    typedef struct
    {
        int nNdx;
        double dValue;
        double dThreshold;
    }
    OtaData;

public:
    CInjectorData()
    {
        for(int nNdx=0;nNdx<CECInjectorView::enNumDefQueryTypes;nNdx++)
        {
            m_arOTAData[nNdx].nNdx=-1;
            m_arOTAData[nNdx].dValue=-1;
            m_arOTAData[nNdx].dThreshold = -1;
        }
        m_nListItem=-1;
    }

    CString strName;
    CString strIPAddress;
    int m_nListItem;
    OtaData m_arOTAData[CECInjectorView::enNumDefQueryTypes];
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CECINJECTORVIEW_H__A2825E43_0ED6_11D4_BD8D_00C04F89E392__INCLUDED_)
