/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECLoggingView.cpp : implementation file
//

#include "stdafx.h"
#include "CECLoggingView.h"
#include "..\..\Interfaces\CyrPlugins.h"
#include "server.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECLoggingView

IMPLEMENT_DYNCREATE(CECLoggingView, CFormView)

CECLoggingView::CECLoggingView()
        : CFormView(CECLoggingView::IDD)
{
    //{{AFX_DATA_INIT(CECLoggingView)
	//}}AFX_DATA_INIT

	m_pCorba = new CORBAServer(0, NULL);
	m_pServer = NULL;
}


CECLoggingView::~CECLoggingView()
{}


void CECLoggingView::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECLoggingView)
	DDX_Control(pDX, IDC_SAVE, m_Save);
	DDX_Control(pDX, IDC_GROUP, m_Group);
	DDX_Control(pDX, IDC_CORBA_REFRESH, m_Refresh);
	DDX_Control(pDX, IDC_LEVELS, m_lcLevels);
	//}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECLoggingView, CFormView)
//{{AFX_MSG_MAP(CECLoggingView)
ON_WM_SIZE()
	ON_BN_CLICKED(IDC_SAVE, OnSave)
	ON_BN_CLICKED(IDC_CORBA_REFRESH, OnCorbaRefresh)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECLoggingView diagnostics

#ifdef _DEBUG
void CECLoggingView::AssertValid() const
{
    CFormView::AssertValid();
}

void CECLoggingView::Dump(CDumpContext& dc) const
{
    CFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CECLoggingView message handlers

void CECLoggingView::OnInitialUpdate()
{
    CFormView::OnInitialUpdate();

    GetDocument()->SetTitle("System Tracing");

    // Get a handle to the Mainframe
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);


    // Now get the server login information from the framework
    m_pServer = (CYRServer *)AfxGetMainWnd()->SendMessage (WMU_GET_SERVERPTR);
    CString szUser = m_pServer->GetUserName();
    CString szPass = m_pServer->GetPassword();

    //
    // And also get the project name.
    //
    CString ProjName = m_pServer->GetOpenProjectName();

    //
    // Now we have all the information we need we can initialize the control
    //
    m_lcLevels.SetCorbaHelper( m_pCorba );
	InitListControl();
	PopulateLevels();
    CRect r;
    GetClientRect(r);

    OnSize(NULL, r.Width(), r.Height());
}

void CECLoggingView::OnSize(UINT nType, int cx, int cy)
{
    CFormView::OnSize(nType, cx, cy);

	if (m_Group.m_hWnd)
	{
		m_Group.MoveWindow(CRect(5,5,cx-70, cy-5));
	}
	if (m_Save.m_hWnd)
	{
		m_Save.MoveWindow(CRect(cx-60,20,cx-5,40));
	}
	if (m_Refresh.m_hWnd)
	{
		m_Refresh.MoveWindow(CRect(cx-60,50,cx-5,70));
	}
	if (m_lcLevels.m_hWnd)
	{
		m_lcLevels.MoveWindow(CRect(15,20,cx-75,cy-15));
	}
}

BOOL CECLoggingView::PreCreateWindow(CREATESTRUCT& cs)
{
    cs.style &= ~(ES_AUTOHSCROLL|WS_HSCROLL);
    return CFormView::PreCreateWindow(cs);
}

void CECLoggingView::InitListControl()
{
    const int nNumColumns = 3;
    // Get column header information and add columns to list control
    for ( int i = 0; i < nNumColumns; i++ )
	{
        m_lcLevels.CreateColumn( i, IDS_LEVELNAME_COL + i );
		m_lcLevels.SetColumnWidth(i, 100);
	}

}

void CECLoggingView::PopulateLevels()
{
    // Set cursor to be hourglass
    CWaitCursor cursor;

    // Empty any existing entries
    m_lcLevels.DeleteAllItems();

    // Get log settings from server
    if(m_pServer->CheckOutFile(CYRServer::enRootFolder,"",LOG_SETTINGS_FILE))
    {
        CString strPath = m_pServer->GetFolderPath(CYRServer::enRootFolder)+LOG_SETTINGS_FILE;

        m_lcLevels.ImportFile( strPath );
        m_lcLevels.ImportCorba();
    }

    // Just populated list control so cannot be dirty
    m_lcLevels.SetDirtyFlag( false );
}

void CECLoggingView::OnSave() 
{
    // Display wait cursor
    CWaitCursor cursor;

    // If there have been no changes to the entries within the list control
    // then there is no need to save and we can exit this function.
    if ( m_lcLevels.GetDirtyFlag() == false )
        return;

    // Get log settings file location from OpenSTARep
    CString strPath = m_pServer->GetFolderPath(CYRServer::enRootFolder) + LOG_SETTINGS_FILE;

    m_lcLevels.ExportFile( strPath );
    m_lcLevels.ExportCorba();

    // Check in new file
    m_pServer->CheckInFile(CYRServer::enRootFolder,"",LOG_SETTINGS_FILE,LOG_SETTINGS_FILE);

    // Details have been saved so entries are now clean
    m_lcLevels.SetDirtyFlag( false );
}

void CECLoggingView::OnCorbaRefresh() 
{
    if ( m_lcLevels.ImportCorba() == false )
    {
        // Some kind of exception or error occurred so show error message.
        CString strTitle, strError;
        strError.LoadString( IDS_FACTORY_NOT_FOUND );
        strTitle= "OpenSTA" ;
        MessageBox( strError, strTitle, MB_ICONEXCLAMATION | MB_OK );
    }	
}

void CECLoggingView::OnDestroy() 
{
	CFormView::OnDestroy();
	
    if (m_pCorba != NULL)
    {
        delete m_pCorba;
    }	
}
