/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYREventSink.cpp
// ----------------
//
//    This is the implementation file for the CYREventSink class.
//
//=========================================================================

#include "stdafx.h"
#include "CYREventSink.h"
#include "resource.h"
#include "CECInjectorView.h"
#include "InjectorViewRefreshRateDlg.h"

//
//    Product includes
//
#include "..\..\Interfaces\CyrCtrls.h"
#include "..\..\Interfaces\CyrPlugins.h"
#include "..\..\Common\CECDLLFunctions.h"
#include "..\..\common\CECClipboard.h"
#include "..\..\common\CECRegistryTools.h"
#include "solutions.h"
#include "server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif


#include "..\..\CECCommander\CECCommander.h"

extern  CECDLLFunctions*    pDllFunctions;
extern  CMultiDocTemplate*      pStatusScreenDocTemplate;
extern  CMultiDocTemplate*      pResViewDocTemplate;
extern  CMultiDocTemplate*      pLevelMonDocTemplate;
extern  CMultiDocTemplate*      pDynGraphsTemplate;
extern  CMultiDocTemplate*      pInjectorViewTemplate;
extern  CMultiDocTemplate*      pHttpStatsTemplate;
extern  CMultiDocTemplate*      pLoggingDocTemplate;

/////////////////////////////////////////////////////////////////////////////
// CYREventSink

CYREventSink::CYREventSink()
{
    // In release we have discovered that our extension DLL's behave in a different
    // manner to the debug builds. It appears that any member variables are not
    // persistant for the entirety of this class and unless static they lose their data
    // after leaving the constructor.

    // As a workaround we have found that if we set up a 16-byte char value ALL other
    // member variables then retain their values during this objects lifetime.

    // Don't ask me why - it's an MFC Release Build Memory Management thing ...
    (void) strcpy (EventSink, "CYRGenericCtrls");

    // Get and store the parent window for this event sink
    CWnd* pParentWnd = AfxGetMainWnd();
    if (pParentWnd != NULL)
    {
        m_hParent = pParentWnd->GetSafeHwnd();

        // Create a default window size
        if (m_hParent != NULL)
        {
            // Create a default window size
            CRect rect(0,0,0,0);
            Create(NULL, NULL, WS_CHILD, rect, pParentWnd, ID_GCTRLS_EVENT_SINK);
        }
    }

    m_pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);
}

CYREventSink::~CYREventSink()
{}



BEGIN_MESSAGE_MAP(CYREventSink, CWnd)
//{{AFX_MSG_MAP(CYREventSink)
ON_COMMAND(IDB_MONITOR_TEST, OnMonitorTest)
ON_COMMAND(IDB_VIEW_RESULTS, OnViewResults)
ON_MESSAGE(WMU_LEVEL_MONITOR, OnLevelMonitor)
ON_MESSAGE(WMU_DYN_TEST_GRAPH, OnDynTestGraphs)
ON_MESSAGE(WMU_INJECTOR_VIEW, OnInjectorView)
ON_COMMAND(IDB_LOGGING, OnSetLogging)
ON_MESSAGE(WMU_SET_LOGGING, OnSetLogging)
//}}AFX_MSG_MAP
ON_COMMAND(ID_INJECTOR_REFRESH_RATE, OnSetInjectorsRefreshRate)
ON_MESSAGE(WMU_MONITOR_TEST, OnMonitorTest)
ON_MESSAGE(WMU_VIEW_RESULTS, OnViewSelResults)
ON_MESSAGE(WMU_VIEW_HTTP_STATS, OnViewHttpStats)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CYREventSink message handlers

BOOL CYREventSink::IsEnabled(UINT nID)
{
    BOOL bState = TRUE;
    switch(nID)
    {
    case IDB_LOGGING:
        {
            bState = m_pServer->IsThereAnOpenProject();
            break;
        }
    }

    return bState;
}

afx_msg void CYREventSink::OnMonitorTest()
{
    //
    // If an MDI window is already open for this filename activate it and bring it
    // to the front instead of creating a new frame
    //
    if (!ActivateMDIWnd("Active Tests"))
    {
		try
		{
	        pStatusScreenDocTemplate->OpenDocumentFile(NULL);
		}
		catch(...)
		{
			AfxMessageBox("A problem occured opening the Active Test Monitor\n\nPlease check that the Name server is running.");
		}
    }
}

LRESULT CYREventSink::OnMonitorTest(WPARAM,LPARAM)
{
    // If an MDI window is already open for this filename activate it and bring it
    // to the front instead of creating a new frame
    //
    if (!ActivateMDIWnd("Active Tests"))
    {
        pStatusScreenDocTemplate->OpenDocumentFile(NULL);
    }

    return 0;
}

afx_msg void CYREventSink::OnViewResults()
{
    CECClipboard cb;
    cb.SetClipboardText("");

    if(m_pServer->IsThereAnOpenProject())
        pResViewDocTemplate->OpenDocumentFile(NULL);
}

LRESULT CYREventSink::OnViewSelResults(WPARAM,LPARAM)
{
    if(m_pServer->IsThereAnOpenProject())
        pResViewDocTemplate->OpenDocumentFile(NULL);
    return 0;
}

LRESULT CYREventSink::OnLevelMonitor(WPARAM ptrInfo, LPARAM )
{
    LPCTSTR strInfo = (LPCTSTR)ptrInfo;
    CString regNameInfo = (CString)strInfo;

    CECClipboard cb;
    cb.SetClipboardText(regNameInfo);
    pLevelMonDocTemplate->OpenDocumentFile(NULL);
    return 0;
}

LRESULT CYREventSink::OnDynTestGraphs(WPARAM ptrInfo, LPARAM )
{
    LPCTSTR strInfo = (LPCTSTR)ptrInfo;
    CString regNameInfo = (CString)strInfo;

    CECClipboard cb;
    cb.SetClipboardText(regNameInfo);
    pDynGraphsTemplate->OpenDocumentFile(NULL);
    return 0;
}

LRESULT CYREventSink::OnInjectorView(WPARAM ptrInfo, LPARAM)
{
    LPCTSTR strInfo = (LPCTSTR)ptrInfo;
    CString regNameInfo = (CString)strInfo;

    CECClipboard cb;
    cb.SetClipboardText(regNameInfo);
    pInjectorViewTemplate->OpenDocumentFile(NULL);
    return 0;
}

void CYREventSink::OnSetInjectorsRefreshRate()
{
    //Get the current rate from the registry
    CECRegistryTools RegTools;
    CString strRefreshRate = RegTools.GetStringValue(HKEY_CURRENT_USER,SOLUTIONS_FRAMEWORK_KEY,"Injector View Refresh");

    int nRefreshRate = 5;
    if(!strRefreshRate.IsEmpty())
        nRefreshRate = atoi(strRefreshRate);

    //Ask the user to enter a new rate
    CInjectorViewRefreshRateDlg dlg(nRefreshRate);
    if(dlg.DoModal() != IDOK)
        return;

    //set the new rate in the registry
    nRefreshRate = dlg.GetRefreshRate();

    strRefreshRate.Format("%d",nRefreshRate);
    RegTools.SetStringValue(HKEY_CURRENT_USER,SOLUTIONS_FRAMEWORK_KEY,"Injector View Refresh",strRefreshRate.GetBuffer(0));

    //Tell all the open injector views to change refresh rate
    if(pInjectorViewTemplate != NULL)
    {
        POSITION pos = pInjectorViewTemplate->GetFirstDocPosition();
        while(pos != NULL)
        {
            CDocument *pDoc = pInjectorViewTemplate->GetNextDoc(pos);

            POSITION posView = pDoc->GetFirstViewPosition();
            while(posView != NULL)
            {
                CECInjectorView *pView = (CECInjectorView*)pDoc->GetNextView(posView);
                pView->SetRefreshRate(nRefreshRate);
            }
        }
    }
}

LRESULT CYREventSink::OnViewHttpStats(WPARAM,LPARAM)
{
    pHttpStatsTemplate->OpenDocumentFile(NULL);
    return 0;
}

LRESULT CYREventSink::OnSetLogging(WPARAM,LPARAM)
{
    // If an MDI window is already open for this filename activate it and bring it
    // to the front instead of creating a new frame
    //
    if (!ActivateMDIWnd("System Tracing"))
    {
		try
		{
			pLoggingDocTemplate->OpenDocumentFile(NULL);
		}
		catch(...)
		{
			AfxMessageBox("A problem occured opening the System Tracing Tool\n\nPlease check that the Name server is running.");
		}
    }

    return 0;
}

void CYREventSink::OnSetLogging()
{
    // If an MDI window is already open for this filename activate it and bring it
    // to the front instead of creating a new frame
    //
    if (!ActivateMDIWnd("System Tracing"))
    {
		try
		{
			pLoggingDocTemplate->OpenDocumentFile(NULL);
		}
		catch(...)
		{
			AfxMessageBox("A problem occured opening the System Tracing Tool\n\nPlease check that the Name server is running.");
		}
    }
}

BOOL CYREventSink::ActivateMDIWnd(CString strTitle)
{
    CMDIFrameWnd* m_pMDIFrame = (CMDIFrameWnd*)AfxGetMainWnd();

    ASSERT(m_pMDIFrame);

    HWND hwndT;
    hwndT=::GetWindow(m_pMDIFrame->m_hWndMDIClient, GW_CHILD);
    while (hwndT != NULL)
    {
        CString wndName = "";

        ::GetWindowText(hwndT,wndName.GetBuffer(256),256);

        if (wndName == strTitle)
        {
            CWnd* pWnd = CWnd::FromHandle(hwndT);

            m_pMDIFrame->MDIActivate(pWnd);
            return TRUE;
        }

        hwndT=::GetWindow(hwndT,GW_HWNDNEXT);
    }

    return FALSE;
}
