/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VUserDetails.cpp : implementation file
//

#include "stdafx.h"
#include "CORBAObjects\TestLevel.h"
#include "ctime.hpp"
#include "VUserDetailsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVUserDetailsDlg dialog
#define WMU_UPDATED (WM_APP+10000)

CVUserDetailsDlg::CVUserDetailsDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CVUserDetailsDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CVUserDetailsDlg)
    m_strInjector = _T("");
    m_strIteration = _T("");
    m_strScript = _T("");
    m_strStatus = _T("");
    m_strTest = _T("");
    m_strInformation = _T("");
    m_strStart = _T("");
    m_strEnd = _T("");
    //}}AFX_DATA_INIT
    m_CurrentThread.nIteration = 0;
    m_CurrentThread.nStatus = SUSPENDED;
    m_CurrentThread.uiIdent = 0;
}


void CVUserDetailsDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CVUserDetailsDlg)
    DDX_Text(pDX, IDC_INJECTOR, m_strInjector);
    DDX_Text(pDX, IDC_ITERATION, m_strIteration);
    DDX_Text(pDX, IDC_SCRIPT, m_strScript);
    DDX_Text(pDX, IDC_STATUS, m_strStatus);
    DDX_Text(pDX, IDC_TEST, m_strTest);
    DDX_Text(pDX, IDC_INFORMATION, m_strInformation);
    DDX_Text(pDX, IDC_START, m_strStart);
    DDX_Text(pDX, IDC_END, m_strEnd);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CVUserDetailsDlg, CDialog)
//{{AFX_MSG_MAP(CVUserDetailsDlg)
ON_WM_CLOSE()
ON_WM_SIZE()
//}}AFX_MSG_MAP
ON_MESSAGE(WMU_UPDATED,OnUpdated)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVUserDetailsDlg message handlers

void CVUserDetailsDlg::SetThreadInfo(const CThreadInfo& threadInfo)
{
    m_CurrentThread = threadInfo;
    if(GetSafeHwnd())
        SendMessage(WMU_UPDATED);
}

LRESULT CVUserDetailsDlg::OnUpdated(WPARAM,LPARAM)
{
    if(m_CurrentThread.uiIdent)
    {
        m_strInjector = m_CurrentThread.strInjector;
        if(m_CurrentThread.nIteration == -1)
            m_strIteration = "-";
        else
            m_strIteration.Format("%d",m_CurrentThread.nIteration);
        m_strScript = m_CurrentThread.strScript;
        m_strTest = m_CurrentThread.strTestName;
        m_strInformation = m_CurrentThread.strInformation;

        switch(m_CurrentThread.nStatus)
        {
        case ACTIVE:
            m_strStatus = "Active";
            break;
        case SUSPENDED:
            m_strStatus = "Suspended";
            break;
        }

        string strTemp = m_CurrentThread.rwStartTime.Format("%H:%M:%S %d/%m/%Y" );
        m_strStart = strTemp.c_str();
        strTemp = m_CurrentThread.rwDuration.Format("%H:%M:%S");
        m_strEnd = strTemp.c_str();
    }
    else
    {
        m_strInjector = _T("-");
        m_strIteration = _T("-");
        m_strScript = _T("-");
        m_strStatus = _T("-");
        m_strTest = _T("-");
        m_strInformation = _T("-");
        m_strStart = _T("-");
        m_strEnd = _T("-");
    }

    if(GetSafeHwnd())
        UpdateData(FALSE);

    return 0;
}

BOOL CVUserDetailsDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    CPoint pt;
    CSize sz;
    pt.x = AfxGetApp()->GetProfileInt("VU Details","Window Pos Left",-1);
    pt.y = AfxGetApp()->GetProfileInt("VU Details","Window Pos Top",-1);
    sz.cx = AfxGetApp()->GetProfileInt("VU Details","Window Width",-1);
    sz.cy = AfxGetApp()->GetProfileInt("VU Details","Window Height",-1);

    if((pt.x!=-1) && (pt.y!=-1) && (sz.cx!=-1) && (sz.cy!=-1))
        SetWindowPos(NULL,pt.x,pt.y,sz.cx,sz.cy,SWP_NOZORDER);

    SendMessage(WMU_UPDATED);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CVUserDetailsDlg::OnClose()
{
    CRect rc;
    GetWindowRect(&rc);
    AfxGetApp()->WriteProfileInt("VU Details","Window Pos Left",rc.left);
    AfxGetApp()->WriteProfileInt("VU Details","Window Pos Top",rc.top);
    AfxGetApp()->WriteProfileInt("VU Details","Window Width",rc.Width());
    AfxGetApp()->WriteProfileInt("VU Details","Window Height",rc.Height());
    DestroyWindow();
}

void CVUserDetailsDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    const int nNumCtrls = 8;
    const UINT uiCtrlIds[nNumCtrls] =
        {
            IDC_TEST,IDC_SCRIPT,IDC_ITERATION,
            IDC_INJECTOR,IDC_STATUS,IDC_START,IDC_END,IDC_INFORMATION
        };


    CRect rcCtrl;

    CWnd *pWnd = GetDlgItem(IDC_TEST);
    if(!pWnd)
        return;

    pWnd->GetWindowRect(&rcCtrl);
    ScreenToClient(&rcCtrl);

    rcCtrl.right = cx - 10;

    if(rcCtrl.Width() <= 0)
        return;

    for(int nCtrl=0;nCtrl<nNumCtrls;nCtrl++)
    {
        pWnd = GetDlgItem(uiCtrlIds[nCtrl]);
        pWnd->SetWindowPos(NULL,0,0,rcCtrl.Width(),rcCtrl.Height(),SWP_NOZORDER|SWP_NOMOVE);
    }

    pWnd->GetWindowRect(&rcCtrl);
    ScreenToClient(&rcCtrl);
    rcCtrl.bottom = cy - 10;

    if(rcCtrl.Height() <= 0)
        return;

    pWnd->SetWindowPos(NULL,0,0,rcCtrl.Width(),rcCtrl.Height(),SWP_NOZORDER|SWP_NOMOVE);
}

void CVUserDetailsDlg::Clear()
{
    m_CurrentThread.uiIdent = 0;
}
