/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CyrPlugIns.h
// ------------
//
//    This is the interface file for activating the plugins
//
//=========================================================================

#if !defined CYRPLUGINS_H
#define CYRPLUGINS_H

// Define the studio limits
#define MAX_PRODUCTS        10
#define MAX_PLUGINS         100
#define MAX_DOCKINGWINDOWS  5
#define MAX_OUTPUTVIEWS     10
#define MAX_DOCUMENTS       100
#define MAX_WIZARDS         10

#define WMU_SET_PROJECT             WM_USER+201
#define WMU_FIRE_GLOBAL_EVENT       WM_USER+202
#define WMU_GET_KNOWN_EXTENSIONS    WM_USER+203
#define WMU_GET_USERKEY             WM_USER+204
#define WMU_GET_INSTALLDIR          WM_USER+205
#define WMU_WIZ_DISPLAYPROPS        WM_USER+206
#define WMU_WIZ_SETSELECTION        WM_USER+207
#define WMU_GET_SERVERPTR           WM_USER+208
#define WMU_SET_CLIPBOARD_TEXT      WM_USER+209
#define WMU_GET_SRCCTRL_OBJNAME     WM_USER+210
#define WMU_GET_PRODUCT_TYPE        WM_USER+212
#define WMU_FIRE_SPECIFIC_EVENT     WM_USER+213
#define WMU_EDIT_NODE               WM_USER+214
#define WMU_MONITOR_TEST            WM_USER+215
#define WMU_VIEW_RESULTS            WM_USER+216
#define WMU_REFRESH_TREE_VIEWS      WM_USER+217
#define WMU_LEVEL_MONITOR           WM_USER+218
#define WMU_EDIT_OTA_TOF            WM_USER+219
#define WMU_DYN_TEST_GRAPH          WM_USER+220
#define WMU_SET_LOGGING             WM_USER+221
#define WMU_GET_PRODUCT_PREFIX      WM_USER+222
#define WMU_OPEN_TEST_RUN           WM_USER+223
#define WMU_WIZARD_START            WM_USER+224
#define WMU_INJECTOR_VIEW           WM_USER+225
#define WMU_EXECUTE_WEB             WM_USER+226
#define WMU_VIEW_HTTP_STATS         WM_USER+227

//
//    Source control messages
//
#define WMU_SC_MSGSTART         WM_USER+300
#define WMU_SC_CHECKIN              WM_USER+301
#define WMU_SC_CHECKOUT             WM_USER+302
#define WMU_SC_UPDATE               WM_USER+303
#define WMU_SC_DELETE               WM_USER+304
#define WMU_SC_CANCEL               WM_USER+305
#define WMU_SC_ISINSCC              WM_USER+306
#define WMU_SC_ISCHECKEDOUT         WM_USER+307
#define WMU_SC_ADDFILE              WM_USER+308
#define WMU_SC_SETWORKFOLDER        WM_USER+309
#define WMU_SC_ENABLED              WM_USER+310
#define WMU_SC_SETLOCK              WM_USER+311
#define WMU_SC_DEL_TEST             WM_USER+312
#define WMU_SC_DEL_TESTRUN          WM_USER+313
#define WMU_SC_DEL_ALL_RESULTS      WM_USER+314
#define WMU_EDIT_TOF                WM_USER+315
#define WMU_SC_MSGEND           WM_USER+320


// Define Studio Global Events
#define STUDIO_SWITCH_PROJECTS  WM_USER+3000
#define STUDIO_EDIT_PROJECTS    WM_USER+3001
#define STUDIO_BEGIN_WAIT       WM_USER+3002
#define STUDIO_END_WAIT         WM_USER+3003
#define STUDIO_STATUSBAR_TEXT   WM_USER+3004

// Reserved for Build Settings
#define STUDIO_CUSTOM_LINK_1   WM_USER+3101
// Reserved for the Output Window
#define STUDIO_CUSTOM_LINK_2   WM_USER+3102
// Reserved for the Node Definitions
#define STUDIO_CUSTOM_LINK_3   WM_USER+3103
// Reserved for the Test Properties
#define STUDIO_CUSTOM_LINK_4   WM_USER+3104
// Reserved for the Thread Properties
#define STUDIO_CUSTOM_LINK_5   WM_USER+3105
// Reserved for the Delay Properties
#define STUDIO_CUSTOM_LINK_6   WM_USER+3106

// And the supported Additional Window Types
#define FLOOR_WINDOW    0
#define PROJECT_WINDOW  1

// Define some general constants
#define TEXT_LEN        64

// PRODUCTINFO
// ===========

// This structure is used to add in the Product details to the framework

// This structure should only be supported for product DLL's to load in
// the main product details. The product is capable of becoming a
// subproduct if the SubProduct flag (CYR_IS_SUBPRODUCT) is set.

// Note: Subproducts become Main products when no main product has been loaded

// Each product can specify certain styles of which more framework functionality
// will become available. Currently these are

// CYR_HAS_PROJECT - Will give all the options to manage a project
// CYR_HAS_SERVER - Will give all the options to connect to a server
// CYR_HAS_CONTEXT  - Will have the context (target) toolbar
//
// A product may also supply a splash screen. If none is supplied then the Studio
// will use a default. If the product is a subproduct then the main product's splash
// screen will be used and not the one supplied here.

// Finally the main product can provide a new background to overide that provided
// by the Studio. The Background will be displayed in the MDI client area of the Framework.
typedef struct PRODUCTINFO
{
    char strDisplayName[TEXT_LEN];          // Product Name         ("CYRANO Impact")
    char strTitle[TEXT_LEN];                // Product Title        ("CYRANO Impact Enterprise")
    char strInternalName[TEXT_LEN];         // Product Internal Name
    char strProjectPrefix[TEXT_LEN];        // Products project name prefix
    UINT nIcon;                             // Product Icon         (IDB_IMPACT_ICON)
    char strVersion[TEXT_LEN];              // Product Version      ("2.2.3.1")
    char strAboutText[TEXT_LEN];            // Product Description  ("This version of Impact ...")
    DWORD dwStyles;                         // Product Styles       (CYR_IS_SUBPRODUCT | CYR_HAS_PROJECT)
    UINT nSplashScreen;                     // Product Splash Screen(IDB_BETA_SPLASH)
    UINT nBackground;                       // Product Background   (IDB_IMPACT_BACKGROUND)

}

PRODUCTINFO;

// Customizable Styles - These must be specified to enable the various
// Options they provide
#define CYR_IS_SUBPRODUCT       1
#define CYR_HAS_SERVER          2
#define CYR_HAS_PROJECT         4
#define CYR_HAS_CONTEXT         8

// RESOURCEINFO
// ============

// This structure is used to add in any resources to the framework

// Note: The resource must provide a Menu Label to be added and will be
// ignored otherwise. This restriction is imposed so that we comply to general
// windows standards and do not add in buttons without any manu handlers

// If the preferred menu is absent two choices are made.
// a). The Generic DLL will create and add it into the "CYRANO Studio" menu
// b). A Product DLL will create and add it to a menu item named after the product itself

// The Separator Style will add in a seperator after the item

// The Removeable Style is used for advanced circumstances and defines a button as having
// the ability to be removed from the framework if it becomes disabled.

// The Needs Server Style indicates that the button will only be added if we are
// using a server

// The Needs Project Style indicates that the button will only be added if we are
// using projects

// A Command ID is Mandatory and provides the link back to the DLL when the resource
// is invoked. In the case where no button is supplied this is simply a unique ID else
// it is the resource ID which identifies the Bitmap

// If the preferred toolbar is absent two choices are made.
// a). The Generic DLL will create and add it onto the "CYRANO Studio" toolbar
// b). A Product DLL will create and add it in to a toolbar named after the product itself
typedef struct RESOURCEINFO
{
    char strLabel[TEXT_LEN];                // Menu Text            ("&Execute")
    char strPreferredMenu[TEXT_LEN];        // Preffered Menu       ("&Settings")
    UINT nCommandID;                        // Button/Command ID    (IDB_EXECUTE_TEST)
    char strPreferredToolbar[TEXT_LEN];     // Preferred Toolbar    ("&Warp Toolkit")
    DWORD dwStyles;

}
RESOURCEINFO;

// Customizable Styles - These must be specified to enable the various
// Options they provide
#define CYR_NEEDS_SEPARATOR     1
#define CYR_IS_REMOVEABLE       2
#define CYR_NEEDS_SERVER        4
#define CYR_NEEDS_PROJECT       8
#define CYR_EXTENDED_UI_UPDATE  16
#define CYR_BASIC_CMD           32

// WINDOWINFO
// ==========

// This WINDOWINFO info adds information concerning an extra dockable window

// This structure is used to create more dockable windows inside the studio.
// It must be a type of one of the supported styles defined at the top of this
// file and is typically used for displaying the contents of a project for example.

// The Window will assume default styles if not specified in the structure
typedef struct WINDOWINFO
{
    char    strName[TEXT_LEN];                  // Window Name          ("Project Viewer")
    DWORD   dwStyles;                           // Window Styles        (WS_VISIBLE | ...)
    UINT    nType;                              // Window Type          ("FLOOR_WINDOW")
    long    hWnd;                               // Window Handle        (0)

}

WINDOWINFO;

// OUTPUTINFO
// ==========

// This OUTPUTINFO info adds a tab to the output view
typedef struct OUTPUTINFO
{
    char strName[TEXT_LEN];                     // Tab Name             ("Debug")
    long hWnd;                                  // Window Handle        (0)

}

OUTPUTINFO;

// DOCUMENTINFO
// ============

// This DOCUMENTINFO info adds a document template to the framework
//
// The Editable Style will allow the Framework to treat the template as editable
//
typedef struct DOCUMENTINFO
{
    char strName[TEXT_LEN];
    char strFileExt[3];
    DWORD dwStyles;
    CMultiDocTemplate *pDocTemplate;
    GUID BASED_CODE clsid;

}
DOCUMENTINFO;

// Customizable Styles - These must be specified to enable the various
// Options they provide
#define CYR_EDITABLE_DOCUMENT   1
#define CYR_HAS_TEMPLATE        4

// WIZARDINFO
// ==========

// This WIZARDINFO info adds a wizard to the framework - it is only
// used when pressing the wizard button to optimize the initial loading
// time of the framework.
//
// The structure accepts a suitable Wizard Name along with the wizard styles
//
typedef struct WIZARDINFO
{
    char        strName[TEXT_LEN];
    DWORD       dwStyles;
    char        strPlugin[TEXT_LEN];
}
WIZARDINFO;

// And now we declare the function prototypes which will obtain the structure
typedef BOOL  ( * LPPRODUCTFUNC)        (PRODUCTINFO    *pProductInfo);
typedef BOOL  ( * LPRESFUNC)            (RESOURCEINFO   *pResourceInfo[]);
typedef BOOL  ( * LPWINDOWFUNC)         (WINDOWINFO     *pWindowInfo[]);
typedef BOOL  ( * LPOUTPUTFUNC)         (OUTPUTINFO     *pOutputInfo[]);
typedef BOOL  ( * LPDOCFUNC)            (DOCUMENTINFO   *pDocInfo[]);
typedef BOOL  ( * LPWIZARDFUNC)         (WIZARDINFO     *pWizardInfo[]);
typedef BOOL  ( * LPCOMMANDUIFUNC)      (UINT           *nID);
typedef BOOL  ( * LPONEVENTFUNC)        (UINT           *nID, LONG          *lParam);
typedef BOOL  ( * LPONDESTROYFUNC)      ();
typedef CWnd* ( * LPCREATEDOCWNDFUNC)   (CWnd           *pParent);
typedef BOOL  ( * LPCREATEPROJECTFUNC)  (CString strName, CString strDesc);
typedef BOOL  ( * LPCOMMANDUIFUNCEX)    (UINT           *nID, CCmdUI *pCmdUI);
typedef BOOL  ( * LPEXECUTEWIZARD)      (const CString& strWizName,const CString& strTestName = "");
#endif
