/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECCustomizeGraph.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "CECGlobalSettings.h"
#include "CECCustomizeGraph.h"
#include "CECVirtGridView.h"
#include "CECGraphCustomize.h"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ISO_VIRTBAR 0
#define AREA        1
#define LINEONLY    2
#define LINE        3
#define VIRTBAR     4
#include "..\..\..\HTMLHelp\CYRAnalyzer\Customize_Graph_Display.inl"

extern CECGlobalSettings* g_pGlobalSettings;
/////////////////////////////////////////////////////////////////////////////
// CECCustomizeGraph property page

IMPLEMENT_DYNCREATE(CECCustomizeGraph, CPropertyPage)

CECCustomizeGraph::CECCustomizeGraph() : CPropertyPage(CECCustomizeGraph::IDD)
{
    //{{AFX_DATA_INIT(CECCustomizeGraph)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_LabelCol.currentcolor = g_pGlobalSettings->m_LabelCol;
    m_BGCol.currentcolor = g_pGlobalSettings->m_BGCol;
    m_ATCol.currentcolor = g_pGlobalSettings->m_ATCol;
    m_GridCol.currentcolor = g_pGlobalSettings->m_GridCol;
    m_DefCol0.currentcolor = g_pGlobalSettings->m_nDefLineColours[0];
    m_DefCol1.currentcolor = g_pGlobalSettings->m_nDefLineColours[1];
    m_DefCol2.currentcolor = g_pGlobalSettings->m_nDefLineColours[2];
    m_DefCol3.currentcolor = g_pGlobalSettings->m_nDefLineColours[3];
    m_DefCol4.currentcolor = g_pGlobalSettings->m_nDefLineColours[4];
    m_DefCol5.currentcolor = g_pGlobalSettings->m_nDefLineColours[5];
    m_DefCol6.currentcolor = g_pGlobalSettings->m_nDefLineColours[6];
    m_DefCol7.currentcolor = g_pGlobalSettings->m_nDefLineColours[7];
    m_DefCol8.currentcolor = g_pGlobalSettings->m_nDefLineColours[8];
    m_DefCol9.currentcolor = g_pGlobalSettings->m_nDefLineColours[9];
}

CECCustomizeGraph::~CECCustomizeGraph()
{}


void CECCustomizeGraph::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECCustomizeGraph)
    DDX_Control(pDX, IDC_LABEL_COL, m_LabelCol);
    DDX_Control(pDX, IDC_BG_COL, m_BGCol);
    DDX_Control(pDX, IDC_AT_COL, m_ATCol);
    DDX_Control(pDX, IDC_GRID_COL, m_GridCol);
    DDX_Control(pDX, IDC_LINE_COL0, m_DefCol0);
    DDX_Control(pDX, IDC_LINE_COL1, m_DefCol1);
    DDX_Control(pDX, IDC_LINE_COL2, m_DefCol2);
    DDX_Control(pDX, IDC_LINE_COL3, m_DefCol3);
    DDX_Control(pDX, IDC_LINE_COL4, m_DefCol4);
    DDX_Control(pDX, IDC_LINE_COL5, m_DefCol5);
    DDX_Control(pDX, IDC_LINE_COL6, m_DefCol6);
    DDX_Control(pDX, IDC_LINE_COL7, m_DefCol7);
    DDX_Control(pDX, IDC_LINE_COL8, m_DefCol8);
    DDX_Control(pDX, IDC_LINE_COL9, m_DefCol9);
    DDX_Control(pDX, IDC_LABEL_FONT, m_LabelFontCombo);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECCustomizeGraph, CPropertyPage)
//{{AFX_MSG_MAP(CECCustomizeGraph)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECCustomizeGraph message handlers

BOOL CECCustomizeGraph::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    m_LabelFontCombo.Initialize();
    int sel = m_LabelFontCombo.FindStringExact(0, g_pGlobalSettings->m_LabelFont);
    m_LabelFontCombo.SetCurSel(sel);

    CComboBox* pGraphType = (CComboBox*)GetDlgItem(IDC_GRAPH_TYPE);

    CECGraphCustomize* pCust = (CECGraphCustomize*)GetParent();
    int nGraphStyle = pCust->GetGraphStyle();

    pGraphType->SetCurSel(nGraphStyle);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECCustomizeGraph::OnOK()
{
    g_pGlobalSettings->m_LabelCol = m_LabelCol.currentcolor;
    g_pGlobalSettings->m_BGCol = m_BGCol.currentcolor;
    g_pGlobalSettings->m_GridCol = m_GridCol.currentcolor;
    g_pGlobalSettings->m_ATCol = m_ATCol.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[0] = m_DefCol0.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[1] = m_DefCol1.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[2] = m_DefCol2.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[3] = m_DefCol3.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[4] = m_DefCol4.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[5] = m_DefCol5.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[6] = m_DefCol6.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[7] = m_DefCol7.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[8] = m_DefCol8.currentcolor;
    g_pGlobalSettings->m_nDefLineColours[9] = m_DefCol9.currentcolor;

    int sel = m_LabelFontCombo.GetCurSel();
    if(sel != CB_ERR)
        m_LabelFontCombo.GetLBText(sel, g_pGlobalSettings->m_LabelFont);

    CComboBox* pGraphType = (CComboBox*)GetDlgItem(IDC_GRAPH_TYPE);
    CECGraphCustomize* pCust = (CECGraphCustomize*)GetParent();

    pCust->SetGraphStyle(pGraphType->GetCurSel());

    CPropertyPage::OnOK();
}

BOOL CECCustomizeGraph::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
