/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECSortedListView.cpp: implementation of the CECSortedListView class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "CECGlobalSettings.h"
#include "CECSortedListView.h"
#include "CYREventSink.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern CYREventSink*        pEventSink;
extern CECGlobalSettings* g_pGlobalSettings;

IMPLEMENT_DYNCREATE(CECSortedListView, CFormView)

BEGIN_MESSAGE_MAP(CECSortedListView, CFormView)
//{{AFX_MSG_MAP(CECSortedListView)
ON_WM_CONTEXTMENU()
ON_COMMAND(ID_CSV_EXPORT, OnCsvExport)
ON_WM_SIZE()
ON_WM_ERASEBKGND()
ON_NOTIFY(LVN_GETDISPINFO, IDC_LIST, OnGetdispinfoList)
//}}AFX_MSG_MAP
// Standard printing commands
END_MESSAGE_MAP()

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CECSortedListView::CECSortedListView()
        : CFormView(CECSortedListView::IDD)
{
    //{{AFX_DATA_INIT(CECSummaryView)
    //}}AFX_DATA_INIT
}


CECSortedListView::~CECSortedListView()
{
}


void CECSortedListView::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECSortedListView)
    DDX_Control(pDX,IDC_LIST,m_ListCtrl);
    //}}AFX_DATA_MAP
}


void CECSortedListView::OnInitialUpdate()
{
    GetParentFrame()->SetTitle("Timer List");
    CFormView::OnInitialUpdate();

    // Get the current window style.
    m_ListCtrl.SetExtendedStyle(LVS_EX_FULLROWSELECT);

    // Set the column headings
    m_ListCtrl.InsertColumn(0,"Test Name",LVCFMT_LEFT,70,0);
    m_ListCtrl.InsertColumn(1,"Thread Name",LVCFMT_LEFT,100,1);
    m_ListCtrl.InsertColumn(2,"Script Name",LVCFMT_LEFT,70,2);
    m_ListCtrl.InsertColumn(3,"Timer Name",LVCFMT_LEFT,70,3);
    m_ListCtrl.InsertColumn(4,"Active Users",LVCFMT_LEFT,70,4);
    m_ListCtrl.InsertColumn(5,"Elapsed Time",LVCFMT_LEFT,70,5);
    m_ListCtrl.InsertColumn(6,"Start Date",LVCFMT_LEFT,70,6);
    m_ListCtrl.InsertColumn(7,"Start Time",LVCFMT_LEFT,70,7);

}

void CECSortedListView::Initialize(int nRunIndex)
{
    CECVirtGridView *pGrid = pEventSink->m_pVirtGrid;

    CString strTestRun;
    g_pGlobalSettings->GetRunInfo(nRunIndex, strTestRun);

    TimerData *pTimerData = pGrid->GetTimerData(strTestRun);

    int nNumTimers = pGrid->GetNumTimers(strTestRun);

    m_ListCtrl.SetRedraw(FALSE);
    m_ListCtrl.SetItemCount(nNumTimers);
    m_arTimers.SetSize(nNumTimers);

    CString strNames;
    int nRow = 0;
    int nItem = 0;
    POSITION pos = pTimerData->listTimers.GetHeadPosition();
    while(pos)
    {
        Timer *pTimer = pTimerData->listTimers.GetNext(pos);
        if(!pTimer->bInFilter)
            continue;

        m_arTimers[nRow] = pTimer;

        nItem = m_ListCtrl.InsertItem(nRow++,LPSTR_TEXTCALLBACK);
    }

    m_ListCtrl.SetRedraw(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Implements sorting when a column header is double-clicked
// @rdesc   Sorting complete
// @parm    ROWCOL | nRow | Row position clicked on.
// @parm    ROWCOL | nCol | Column position clicked on.
// @parm    UINT | nFlags | Flags
// @parm    CPoint | pt | Point clicked on.
/*BOOL CECSortedListView::OnLButtonDblClkRowCol(ROWCOL nRow, ROWCOL nCol, UINT nFlags, CPoint pt)
{
    BOOL bOldLock = LockUpdate(TRUE);
    if (nRow==0)
    {
        CGXSortInfoArray sortInfo;
        sortInfo.SetSize(1);        // 1 key only (you can also have more keys)
        sortInfo[0].nRC = nCol; // column nCol is the key
        sortInfo[0].sortType = CGXSortInfo::autodetect; // the grid
            // will determine if the key is a date, numeric or 
            // alphanumeric value
        sortInfo[0].sortOrder = CGXSortInfo::ascending; 
        SortRows(CGXRange().SetTable(), sortInfo);  
    }
    CFormView::OnLButtonDblClkRowCol(nRow,nCol,nFlags,pt);
 
    LockUpdate(bOldLock);
    Invalidate();
    return TRUE;
}
 
*/

void CECSortedListView::OnContextMenu(CWnd* pWnd, CPoint point)
{
    //This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;
    VERIFY(menu.LoadMenu(IDR_TIMERLIST_MENU));

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    CWnd* pWndPopupOwner = this;

    while (pWndPopupOwner->GetStyle() & WS_CHILD)
        pWndPopupOwner = pWndPopupOwner->GetParent();

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,
                           pWndPopupOwner);
}

void CECSortedListView::OnCsvExport()
{
    CFileDialog dlg(FALSE,"csv","Timers.csv",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"Comma Separated Values (*.csv)|*.csv||",this);
    if(dlg.DoModal() != IDOK)
        return;

    CString strPath(dlg.GetPathName());

    CStdioFile f;
    if(!f.Open(strPath,CFile::modeWrite|CFile::modeCreate))
    {
        CString strMsg;
        strMsg.Format("Failed to open the export file \"%s\"",strPath);
        MessageBox(strMsg,"Timer List",MB_ICONEXCLAMATION|MB_OK);
        return;
    }

    //Create a header line in the csv file
    CHeaderCtrl* pHeader = m_ListCtrl.GetHeaderCtrl();
    int nNumColumns = pHeader->GetItemCount();

    char szText[256];
    HDITEM hdItem;
    hdItem.mask = HDI_TEXT;
    hdItem.pszText = szText;
    hdItem.cchTextMax = 256;

    CString strLine;
    for(int nCol=0;nCol<nNumColumns;nCol++)
    {
        pHeader->GetItem(nCol,&hdItem);
        strLine += hdItem.pszText;
        strLine += ",,";
    }
    strLine += "\n";

    int nNumItems = m_ListCtrl.GetItemCount();
    for(int nItem=0;nItem<nNumItems;nItem++)
    {
        for(int nColumn=0;nColumn<nNumColumns;nColumn++)
        {
            strLine += m_ListCtrl.GetItemText(nItem,nColumn);
            strLine += ",,";
        }
        strLine += "\n";
    }
    f.WriteString(strLine);
    f.Close();
    MessageBox("Export complete","Timer List",MB_ICONINFORMATION|MB_OK);
}


void CECSortedListView::OnSize(UINT nType, int cx, int cy)
{
    CFormView::OnSize(nType, cx, cy);

    if(m_ListCtrl.GetSafeHwnd())
        m_ListCtrl.MoveWindow(0,0,cx,cy);
}

BOOL CECSortedListView::OnEraseBkgnd(CDC* pDC)
{
    return true;
    //  return CFormView::OnEraseBkgnd(pDC);
}


void CECSortedListView::OnGetdispinfoList(NMHDR* pNMHDR, LRESULT* pResult)
{
    LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;

    LV_ITEM &lvItem = pDispInfo->item;
    if(lvItem.mask & LVIF_TEXT)
    {
        int nItem = lvItem.iItem;
        Timer *pTimer = m_arTimers[nItem];
        switch(lvItem.iSubItem)
        {
        case 0:
            lvItem.pszText = pTimer->strTestName.GetBuffer(0);
            break;
        case 1:
            lvItem.pszText = pTimer->strThreadName.GetBuffer(0);
            break;
        case 2:
            lvItem.pszText = pTimer->strScriptName.GetBuffer(0);
            break;
        case 3:
            lvItem.pszText = pTimer->strTimerName.GetBuffer(0);
            break;
        case 4:
            m_strTemp.Format("%u",pTimer->dwActiveThreads);
            lvItem.pszText = m_strTemp.GetBuffer(0);
            break;
        case 5:
            m_strTemp.Format("%f",pTimer->dValue);
            lvItem.pszText = m_strTemp.GetBuffer(0);
            break;
        case 6:
            {
                CTime time(pTimer->dwTimeStamp);
                m_strTemp = time.Format("%d/%m/%y");
                lvItem.pszText = m_strTemp.GetBuffer(0);
            }
            break;
        case 7:
            {
                CTime time(pTimer->dwTimeStamp);
                m_strTemp = time.Format("%H:%M:%S");
                lvItem.pszText = m_strTemp.GetBuffer(0);
            }
            break;
        }
    }

    *pResult = 0;
}
