/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// @doc     Document
// @module  CECSummaryView | Statistical Summary screen to provide a statistical
//          analysis of the test run data.
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_CECSUMMARYVIEW_H__0BC06A73_4342_11D3_BE11_00A02456D506__INCLUDED_)
#define AFX_CECSUMMARYVIEW_H__0BC06A73_4342_11D3_BE11_00A02456D506__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "stdafx.h"
#include "AnalyzerDoc.h"
#include "CECVirtGridView.h"
#include "resource.h"

// @class Statistcal Summary Screen
// @base public | CFormView
class CECSummaryView : public CFormView
{

    // @access Protected Members
protected: // create from serialization only

    // @cmember Constructor
    CECSummaryView();
    DECLARE_DYNCREATE(CECSummaryView)


    // @access Public Members
public:
    //{{AFX_DATA(CECSummaryView)
    enum { IDD = IDD_SUMMARY };
    double  m_LowerQ;
    double  m_Max;
    double  m_Mean;
    double  m_Median;
    double  m_Min;
    double  m_Range;
    int     m_Sample;
    double  m_StdDev;
    double  m_UpperQ;
    //}}AFX_DATA

    // Attributes

    // @access Public Members
public:

    // @cmember Get reference to the document class
    CAnalyzerDoc* GetDocument();


    // @access Protected Members
protected:
    // Operations

    // @access Public Members
public:

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CECSummaryView)
public:
    virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    virtual void OnInitialUpdate(); // called first time after construct
    virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
    virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
    virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
    virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo);
    //}}AFX_VIRTUAL

    // Implementation

    // @access Public Members
public:

    // @cmember Initialize the summary screen with the given data index
    void Initialize(int RunIndex);

    // @cmember Destructor
    virtual ~CECSummaryView();
#ifdef _DEBUG
    virtual void AssertValid() const;
    virtual void Dump(CDumpContext& dc) const;
#endif

protected:

    // Generated message map functions
protected:
    //{{AFX_MSG(CECSummaryView)
    afx_msg BOOL OnHelpInfo(HELPINFO* pHelpInfo);
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()


    // @access Private Members
private:
    CString m_szHtmlHelpFile;

    // @cmember Start Row of the data set in the grid
    int m_nStartRow;
    // @cmember Pointer to the hidden grid
    CECVirtGridView* m_pGrid;

};

#ifndef _DEBUG  // debug version in CECSummaryView.cpp
inline CAnalyzerDoc* CECSummaryView::GetDocument()
{
    return (CAnalyzerDoc*)m_pDocument;
}
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CECSUMMARYVIEW_H__0BC06A73_4342_11D3_BE11_00A02456D506__INCLUDED_)
