// CompareRunWizard.cpp : implementation file
//

#include "stdafx.h"
#include "CECGlobalSettings.h"
#include "CYREventSink.h"
#include "CECVirtGridView.h"
#include "CompareRunWizard.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CECGlobalSettings* g_pGlobalSettings;
extern CYREventSink* pEventSink;
/////////////////////////////////////////////////////////////////////////////
// CCompareRunWizard

IMPLEMENT_DYNAMIC(CCompareRunWizard, CPropertySheet)

CCompareRunWizard::CCompareRunWizard(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
	FillPerfQueryList();
	if(m_arstrPerfQueries.GetSize())
		AddPage(&m_PerfQueryPage);

	FillTimerList();
	if(m_arstrTimers.GetSize())
		AddPage(&m_TimerSelPage);

	SetWizardMode();
}

CCompareRunWizard::~CCompareRunWizard()
{
}


BEGIN_MESSAGE_MAP(CCompareRunWizard, CPropertySheet)
	//{{AFX_MSG_MAP(CCompareRunWizard)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCompareRunWizard message handlers
void CCompareRunWizard::FillPerfQueryList()
{
	//Obtain a list of the performance queries 
	CMap <CString,LPCTSTR,bool,bool> mapQueries;
	
	CString strSelRuns = pEventSink->GetSelectedRuns();
    char *pszToken = strtok(strSelRuns.GetBuffer(0),"|");
    while(pszToken != NULL)
    {
        int nNdx = atoi(pszToken);

		CString strRun;
		if(g_pGlobalSettings->GetRunInfo(nNdx,strRun))
		{
			OtaQueryArray* pOtaQueries = pEventSink->m_pVirtGrid->GetOtaData(strRun);
			for(int nQuery=0;nQuery<pOtaQueries->GetSize();nQuery++)
			{
				mapQueries.SetAt((*pOtaQueries)[nQuery]->strQuery +"|"+ (*pOtaQueries)[nQuery]->strNode,false);
			}
		}
		pszToken = strtok(NULL,"|");
    }

	bool bDummy;
	CString strQuery;
	POSITION pos = mapQueries.GetStartPosition();
	while(pos)
	{
		mapQueries.GetNextAssoc(pos,strQuery,bDummy);
		m_arstrPerfQueries.Add(strQuery);
	}
}

void CCompareRunWizard::FillTimerList()
{
	//Obtain a list of timers 
	CMap <CString,LPCTSTR,bool,bool> mapTimers;
	
	CString strSelRuns = pEventSink->GetSelectedRuns();
    char *pszToken = strtok(strSelRuns.GetBuffer(0),"|");
    while(pszToken != NULL)
    {
        int nNdx = atoi(pszToken);

		CString strRun;
		if(g_pGlobalSettings->GetRunInfo(nNdx,strRun))
		{
			TimerData *pTimerData = pEventSink->m_pVirtGrid->GetTimerData(strRun);
			
			POSITION pos = pTimerData->listTimers.GetHeadPosition();
			while(pos)
			{
				Timer *pTimer = pTimerData->listTimers.GetNext(pos);
				
				mapTimers.SetAt(pTimer->strTimerName,false);
			}
		}
		pszToken = strtok(NULL,"|");
    }

	bool bDummy;
	CString strTimer;
	POSITION pos = mapTimers.GetStartPosition();
	while(pos)
	{
		mapTimers.GetNextAssoc(pos,strTimer,bDummy);
		m_arstrTimers.Add(strTimer);
	}
}

const CStringArray& CCompareRunWizard::GetPerfQueryList()
{
	return m_arstrPerfQueries;
}

const CStringArray& CCompareRunWizard::GetTimerList()
{
	return m_arstrTimers;
}

void CCompareRunWizard::SetSelectedQueries(CStringArray& arstrQueries)
{
	m_arstrSelPerfQueries.Copy(arstrQueries);

}

void CCompareRunWizard::SetSelectedTimers(CStringArray& arstrTimers)
{
	m_arstrSelTimers.Copy(arstrTimers);
}

int CCompareRunWizard::GetSelectedQueries(CStringArray& arstrQueries)
{
	arstrQueries.Copy(m_arstrSelPerfQueries);

	return arstrQueries.GetSize();
}

int CCompareRunWizard::GetSelectedTimers(CStringArray& arstrTimers)
{
	arstrTimers.Copy(m_arstrSelTimers);
	return arstrTimers.GetSize();
}


/////////////////////////////////////////////////////////////////////////////
// CTimerSelPage property page

IMPLEMENT_DYNCREATE(CTimerSelPage, CPropertyPage)

CTimerSelPage::CTimerSelPage() : CPropertyPage(CTimerSelPage::IDD)
{
	//{{AFX_DATA_INIT(CTimerSelPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CTimerSelPage::~CTimerSelPage()
{
}

void CTimerSelPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTimerSelPage)
	DDX_Control(pDX, IDC_TIMER_LIST, m_ctlTimerList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTimerSelPage, CPropertyPage)
	//{{AFX_MSG_MAP(CTimerSelPage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTimerSelPage message handlers
BOOL CTimerSelPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	FillTimerList();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CTimerSelPage::FillTimerList()
{
    CCompareRunWizard* pSheet = (CCompareRunWizard*) GetParent();

	const CStringArray& arstrTimers = pSheet->GetTimerList();

	m_ctlTimerList.SetRedraw(FALSE);
	
	for(int nTimer=0;nTimer<arstrTimers.GetSize();nTimer++)
		m_ctlTimerList.AddString(arstrTimers[nTimer]);

	m_ctlTimerList.SetRedraw();
}

BOOL CTimerSelPage::OnWizardFinish() 
{
	int nSelCount = m_ctlTimerList.GetSelCount();
	int *pSel = new int[nSelCount];
	m_ctlTimerList.GetSelItems(nSelCount,pSel);
	
	CStringArray arstrTimers;
	CString str;
	for(int nItem=0;nItem<nSelCount;nItem++)
	{
		m_ctlTimerList.GetText(pSel[nItem],str);
		arstrTimers.Add(str);			
	}
	delete []pSel;

    CCompareRunWizard* pSheet = (CCompareRunWizard*) GetParent();
	pSheet->SetSelectedTimers(arstrTimers);
	
	return CPropertyPage::OnWizardFinish();
}

BOOL CTimerSelPage::OnSetActive() 
{
    CCompareRunWizard* pSheet = (CCompareRunWizard*) GetParent();
	pSheet->SetWizardButtons(PSWIZB_BACK|PSWIZB_FINISH);
	return CPropertyPage::OnSetActive();
}

/////////////////////////////////////////////////////////////////////////////
// CPerformanceQueryPage property page

IMPLEMENT_DYNCREATE(CPerformanceQueryPage, CPropertyPage)

CPerformanceQueryPage::CPerformanceQueryPage() : CPropertyPage(CPerformanceQueryPage::IDD)
{
	//{{AFX_DATA_INIT(CPerformanceQueryPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CPerformanceQueryPage::~CPerformanceQueryPage()
{
}

void CPerformanceQueryPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPerformanceQueryPage)
	DDX_Control(pDX, IDC_QUERY_LIST, m_ctlQueryList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPerformanceQueryPage, CPropertyPage)
	//{{AFX_MSG_MAP(CPerformanceQueryPage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPerformanceQueryPage message handlers

BOOL CPerformanceQueryPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	FillQueryList();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CPerformanceQueryPage::FillQueryList()
{
    CCompareRunWizard* pSheet = (CCompareRunWizard*) GetParent();

	const CStringArray& arstrQueries = pSheet->GetPerfQueryList();

	m_ctlQueryList.SetRedraw(FALSE);
	
	for(int nQuery=0;nQuery<arstrQueries.GetSize();nQuery++)
		m_ctlQueryList.AddString(arstrQueries[nQuery]);

	m_ctlQueryList.SetRedraw();
}

LRESULT CPerformanceQueryPage::OnWizardNext() 
{
	int nSelCount = m_ctlQueryList.GetSelCount();
	int *pSel = new int[nSelCount];
	m_ctlQueryList.GetSelItems(nSelCount,pSel);
	
	CStringArray arstrQueries;
	CString str;
	for(int nItem=0;nItem<nSelCount;nItem++)
	{
		m_ctlQueryList.GetText(pSel[nItem],str);
		arstrQueries.Add(str);			
	}
	delete []pSel;

    CCompareRunWizard* pSheet = (CCompareRunWizard*) GetParent();
	pSheet->SetSelectedQueries(arstrQueries);
	
	return CPropertyPage::OnWizardNext();
}

BOOL CPerformanceQueryPage::OnSetActive() 
{
    CCompareRunWizard* pSheet = (CCompareRunWizard*) GetParent();
	pSheet->SetWizardButtons(PSWIZB_NEXT);
	return CPropertyPage::OnSetActive();
}
