/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CustomOTADlg.cpp : implementation file
//

#include "stdafx.h"
#include "CustomOTADlg.h"
#include "CECGlobalSettings.h"
#include "CYREventSink.h"
#include "CECVirtGridView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CYREventSink *pEventSink;
extern CECGlobalSettings* g_pGlobalSettings;
/////////////////////////////////////////////////////////////////////////////
// CCustomOTADlg dialog


CCustomOTADlg::CCustomOTADlg(int nRunIndex,CWnd* pParent /*=NULL*/)
        : CDialog(CCustomOTADlg::IDD, pParent)
{
	m_bShowVUs = FALSE;
    m_nRunIndex = nRunIndex;
    //{{AFX_DATA_INIT(CCustomOTADlg)
	//}}AFX_DATA_INIT
}



void CCustomOTADlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CCustomOTADlg)
    DDX_Control(pDX, IDC_TIMERS, m_ctlTimerList);
    DDX_Control(pDX, IDC_QUERIES, m_ctlQueryList);
	//}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CCustomOTADlg, CDialog)
//{{AFX_MSG_MAP(CCustomOTADlg)
	ON_BN_CLICKED(IDC_RESP_TIMERS, OnRespTimers)
	ON_BN_CLICKED(IDC_VIRTUAL_USERS, OnVirtualUsers)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCustomOTADlg message handlers

void CCustomOTADlg::OnOK()
{
	UpdateData(TRUE);

    m_arstrQueries.RemoveAll();
    m_arstrTimers.RemoveAll();


    int nSelCount = m_ctlQueryList.GetSelCount();
    int *pSel = new int[nSelCount];

    m_ctlQueryList.GetSelItems(nSelCount,pSel);
    m_arstrQueries.SetSize(nSelCount);
    for(int nNdx=0;nNdx<nSelCount;nNdx++)
        m_ctlQueryList.GetText(pSel[nNdx],m_arstrQueries[nNdx]);
    delete []pSel;

    nSelCount = m_ctlTimerList.GetSelCount();
    pSel = new int[nSelCount];
    m_arstrTimers.SetSize(nSelCount);
    m_ctlTimerList.GetSelItems(nSelCount,pSel);
    for(nNdx=0;nNdx<nSelCount;nNdx++)
        m_ctlTimerList.GetText(pSel[nNdx],m_arstrTimers[nNdx]);
    delete []pSel;

    if(!m_arstrTimers.GetSize() && !m_arstrQueries.GetSize())
    {
        CDialog::OnCancel();
        return;
    }

    CDialog::OnOK();
}

BOOL CCustomOTADlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Gain access to the grid for retrieval of data
    CECVirtGridView *pGrid = pEventSink->m_pVirtGrid;

    CString strQuery,strNode;

    //Check whether each test run needs a tab or not
    CString strTestRun;
    g_pGlobalSettings->GetRunInfo(m_nRunIndex,strTestRun);

    //If there is no OTA data, we do not need to display this test run
    OtaQueryArray *pQueryData = pGrid->GetOtaData(strTestRun);

    for(int nNdx=0;nNdx<pQueryData->GetSize();nNdx++)
    {
        strNode = pQueryData->GetAt(nNdx)->strNode;
        strQuery.Format("%s (%s)", pQueryData->GetAt(nNdx)->strQuery,strNode);

        if(m_ctlQueryList.FindStringExact(-1,strQuery) == LB_ERR)
        {
            m_ctlQueryList.AddString(strQuery);
        }
    }

    TimerData* pTimerData = pGrid->GetTimerData(strTestRun);

    POSITION pos = pTimerData->listTimers.GetHeadPosition();
    while(pos)
    {
        Timer *pTimer = pTimerData->listTimers.GetNext(pos);
        if(!pTimer->bInFilter)
            continue;

        if(m_ctlTimerList.FindStringExact(-1,pTimer->strTimerName)==LB_ERR)
            m_ctlTimerList.AddString(pTimer->strTimerName);
    }

	BOOL bVUData = (pGrid->GetVUData(strTestRun) ? TRUE : FALSE);
	GetDlgItem(IDC_VIRTUAL_USERS)->EnableWindow(bVUData);
	((CButton*)GetDlgItem(IDC_VIRTUAL_USERS))->SetCheck(m_bShowVUs && bVUData);
	((CButton*)GetDlgItem(IDC_RESP_TIMERS))->SetCheck(!m_bShowVUs || !bVUData);

	if(m_bShowVUs && bVUData)
		OnVirtualUsers();
	else
		OnRespTimers();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

void CCustomOTADlg::OnRespTimers() 
{
	m_bShowVUs=false;
	m_ctlTimerList.EnableWindow(TRUE);
}

void CCustomOTADlg::OnVirtualUsers() 
{
	m_bShowVUs = true;
	m_ctlTimerList.EnableWindow(FALSE);
}
