/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CYRGroupTestWizard.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "CYRGroupTestWizard.h"
#include "..\..\common\CYRTestProperties.h"
#include "..\..\common\cecTssWriter.h"
#include "..\..\interfaces\cyrplugins.h"
#include "htmlhelp.h"
#include "Server.h"
#include "..\..\common\CECSourceFileUtils.h"
#include "..\..\Common\TestWizardDefs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "..\..\..\HTMLHelp\CYRGeneric\Group_Wizard_Summary.inl"
#include "..\..\..\HTMLHelp\CYRGeneric\Group_Wizard_Name_Def.inl"
#include "..\..\..\HTMLHelp\CYRGeneric\Group_Wizard.Depends.inl"

IMPLEMENT_DYNCREATE(CYRWizardBasePage, CPropertyPage)

/////////////////////////////////////////////////////////////////////////////
// CYRGroupTestWizardSheet

IMPLEMENT_DYNAMIC(CYRGroupTestWizardSheet, CPropertySheet)

CYRGroupTestWizardSheet::CYRGroupTestWizardSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage,const CString& strTestName)
        :CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
    SetupWizard(strTestName);
}

CYRGroupTestWizardSheet::CYRGroupTestWizardSheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage,const CString& strTestName)
        :CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
    SetupWizard(strTestName);
}

BOOL CYRGroupTestWizardSheet::SetupWizard(const CString& strTestName)
{
    m_bWizardMode = true;
    m_pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);

    CString strTestPath;
    if(!strTestName.IsEmpty())
    {
        strTestPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder) + strTestName + "\\" + strTestName + ".WIZ";
        m_bWizardMode = true;

        CString strTestFile(strTestName + ".WIZ");
        if(m_pServer->CheckOutFile(CYRServer::enTestsFolder,strTestName,strTestFile))
        {
            DWORD dwMaxBufferSize = 200;
            CString strType;
            GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_TYPE,"",strType.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strTestPath);
            strType.ReleaseBuffer();
            m_bWizardMode = (strType != GROUPTEST);
        }
    }

    for(int nPage=0;nPage<enMaxPageNdx;nPage++)
    {
        switch(nPage)
        {
        case enNameDefPage:
            AddPage(&m_NameDefPage);
            m_NameDefPage.LoadPageSettings(strTestPath);
            break;
        case enGroupDepPage:
            AddPage(&m_GroupDepPage);
            m_GroupDepPage.LoadPageSettings(strTestPath);
            break;
        case enSummaryPage:
            if(m_bWizardMode)
            {
                AddPage(&m_SummaryPage);
                m_SummaryPage.LoadPageSettings(strTestPath);
            }
            break;
        }
    }

    if(m_bWizardMode)
        SetWizardMode();

    return true;
}

CYRGroupTestWizardSheet::~CYRGroupTestWizardSheet()
{}



BEGIN_MESSAGE_MAP(CYRGroupTestWizardSheet, CPropertySheet)
//{{AFX_MSG_MAP(CYRGroupTestWizardSheet)
// NOTE - the ClassWizard will add and remove mapping macros here.
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CYRGroupTestWizardSheet::OnInitDialog()
{
    BOOL bResult = CPropertySheet::OnInitDialog();

    //Show the context sensitive help ? button
    LONG style = ::GetWindowLong(m_hWnd, GWL_EXSTYLE);
    style |= WS_EX_CONTEXTHELP;
    ::SetWindowLong(m_hWnd, GWL_EXSTYLE, style);

    return bResult;
}
/////////////////////////////////////////////////////////////////////////////
// CYRGroupTestWizardSheet message handlers
void CYRGroupTestWizardSheet::OnWizardFinish()
{
    CString strTest(m_NameDefPage.m_strTestName);
    CString strTestFile(strTest + ".WIZ");
    CString strTestPath(m_pServer->GetFolderPath(CYRServer::enTestsFolder));

    //Create folders;
    BOOL b = CreateDirectory(strTestPath,NULL);
    strTestPath +=  strTest;
    b = CreateDirectory(strTestPath,NULL);
    strTestPath += "\\" + strTestFile;

    //Delete the Dependencies for the existing test
    WritePrivateProfileSection(WIZARD_DEPENDENCIES,"",strTestPath);

    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_TYPE,GROUPTEST,strTestPath);

    int nNumPages = GetPageCount();
    for(int nPage=0;nPage<nNumPages;nPage++)
    {
        CYRWizardBasePage* pPage = (CYRWizardBasePage*)GetPage(nPage);
        pPage->SavePageSettings(strTestPath);
    }

    m_pServer->CheckInFile(CYRServer::enTestsFolder,strTest,strTestFile);

    GenerateScripts();
}

void CYRGroupTestWizardSheet::GenerateScripts()
{
    WriteTSS();
}

void CYRGroupTestWizardSheet::WriteTSS()
{
    // If so get the title
    CString strTestName = m_NameDefPage.m_strTestName;

    CString strTSSName = strTestName + ".TSS";
    m_pServer->CheckOutFile(CYRServer::enSourceFolder,"",strTSSName);

    // szFileName = <Drive>:\\ ... \\Source\\szTestName
    CString strFileName = m_pServer->GetFolderPath(CYRServer::enSourceFolder) + strTSSName;
    strFileName.MakeUpper();

    // Now declare a file object
    CStdioFile file;

    // And a writer
    CECTssWriter writer;

    CString strBuffer;
    // And away we go ...
    try
    {
        CStdioFile file(strFileName, CFile::modeCreate | CFile::modeWrite | CFile::typeText);

        // Header Section
        // ==============

        // Formalate the Header Text
        strBuffer = writer.GetTemplateText(CECTssWriter::HEADER, strTestName);
        file.WriteString(strBuffer);


        // Environment section
        // ===================

        strBuffer = writer.GetSectionTitle(CECTssWriter::ENVIRONMENT);
        file.WriteString(strBuffer);

        // And add in any addition from all our pages at this point
        strBuffer = writer.GetNameDefinitionText(CECTssWriter::ENVIRONMENT, m_NameDefPage.m_strTestName, m_NameDefPage.m_strTestDescription);
        file.WriteString(strBuffer);

        strBuffer = writer.GetTemplateText(CECTssWriter::ENVIRONMENT);
        file.WriteString(strBuffer);

        // And do the rest for any other page in the future
        //
        // ...
        //


        // Definitions section
        // ===================

        strBuffer = writer.GetSectionTitle(CECTssWriter::DEFINITIONS);
        file.WriteString(strBuffer);

        // Variables
        // =========

        for(int nTest=0;nTest<m_GroupDepPage.GetNumSelectedTests();nTest++)
        {
            strBuffer = writer.GetTestStatement(CECTssWriter::DEFINITIONS,
                                                m_GroupDepPage.GetSelectedTest(nTest));

            file.WriteString(strBuffer);
        }

        strBuffer = writer.GetTemplateText(CECTssWriter::DEFINITIONS);
        file.WriteString(strBuffer);

        // And do the rest for any other page in the future
        //
        // ...
        //

        // Code Section
        // ============

        strBuffer = writer.GetSectionTitle(CECTssWriter::CODE);
        file.WriteString(strBuffer);

        for(nTest=0;nTest<m_GroupDepPage.GetNumSelectedTests();nTest++)
        {
            strBuffer = writer.GetTestStatement(CECTssWriter::CODE,
                                                m_GroupDepPage.GetSelectedTest(nTest));
            file.WriteString(strBuffer);
        }

        strBuffer = writer.GetTemplateText(CECTssWriter::CODE);
        file.WriteString(strBuffer);

        // And do the rest for any other page in the future
        //
        // ...
        //

        strBuffer = writer.GetTemplateText(CECTssWriter::EXIT);
        file.WriteString(strBuffer);

        file.Close();

        //Compile the script and clear the output window
        CECSourceFileUtils utils(m_pServer,strFileName);
        utils.Compile(TRUE);

        // And import it into the project
        m_pServer->CheckInFile(CYRServer::enSourceFolder,"",strTSSName);
    }
    catch (CFileException* e)
    {
        e->ReportError();
        e->Delete();
    }
}

/////////////////////////////////////////////////////////////////////////////
// CYRNameDefinitionPage dialog
CYRNameDefinitionPage::CYRNameDefinitionPage(CWnd* pParent /*=NULL*/)
        : CYRWizardBasePage(CYRNameDefinitionPage::IDD)
{
    //{{AFX_DATA_INIT(CYRNameDefinitionPage)
    m_strTestDescription = _T("");
    m_strTestName = _T("");
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}


void CYRNameDefinitionPage::DoDataExchange(CDataExchange* pDX)
{
    CYRWizardBasePage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRNameDefinitionPage)
    DDX_Text(pDX, IDC_TEST_DESCRIPTION, m_strTestDescription);
    DDV_MaxChars(pDX, m_strTestDescription, 50);
    DDX_Text(pDX, IDC_TEST_NAME, m_strTestName);
    DDX_Control(pDX, IDC_TEST_NAME, m_ctlTestName);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRNameDefinitionPage, CPropertyPage)
//{{AFX_MSG_MAP(CYRNameDefinitionPage)
ON_NOTIFY(NM_SETFOCUS, IDC_TEST_DESCRIPTION, OnSetfocusTestDescription)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRNameDefinitionPage message handlers
BOOL CYRNameDefinitionPage::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    m_ctlTestName.SetLength(13);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CYRNameDefinitionPage::OnOK()
{
    CYRGroupTestWizardSheet* psheet = (CYRGroupTestWizardSheet*) GetParent();
    psheet->OnWizardFinish();
}

void CYRNameDefinitionPage::OnSetfocusTestDescription(NMHDR* pNMHDR, LRESULT* pResult)
{
    UpdateData(TRUE);

    if(m_strTestDescription.IsEmpty())
    {
        m_strTestDescription = m_strTestName;
        UpdateData(FALSE);
    }

    *pResult = 0;
}

BOOL CYRNameDefinitionPage::OnSetActive()
{
    CYRGroupTestWizardSheet* psheet = (CYRGroupTestWizardSheet*) GetParent();
    psheet->SetWizardButtons(PSWIZB_NEXT);
    m_ctlTestName.EnableWindow(psheet->IsWizardMode());

    return CYRWizardBasePage::OnSetActive();
}

void CYRNameDefinitionPage::SavePageSettings(const CString& strWizardFile)
{
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_NAME,m_strTestName,strWizardFile);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_DESCRIPTION,m_strTestDescription,strWizardFile);
}

void CYRNameDefinitionPage::LoadPageSettings(const CString& strWizardFile)
{
    DWORD dwMaxBufferSize = 200;
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_NAME,"GROUP-TEST",m_strTestName.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    m_strTestName.ReleaseBuffer();
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_DESCRIPTION,GROUPTEST,m_strTestDescription.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    m_strTestDescription.ReleaseBuffer();
}

BOOL CYRNameDefinitionPage::OnKillActive()
{
    UpdateData(TRUE);
    if(m_strTestName.IsEmpty())
    {
        AfxMessageBox("Please specify a name for this test",MB_ICONEXCLAMATION);
        m_ctlTestName.SetFocus();
        return 0;
    }

    CYRGroupTestWizardSheet* pSheet = (CYRGroupTestWizardSheet*) GetParent();

    if(!pSheet->IsValidFilename(m_strTestName))
        return false;

    if(pSheet->IsWizardMode())
    {
        try
        {
            //Check if the test already exists
            if(pSheet->m_pServer->DoesFileExist(CYRServer::enTestsFolder,m_strTestName,m_strTestName + ".WIZ"))
            {
                //The test exists so ask to overwrite it
                CString strMessage;
                strMessage.Format("The test \"%s\" already exists.\n\nDo you want to overwrite it?",m_strTestName);
                if(AfxMessageBox(strMessage,MB_ICONWARNING|MB_YESNO) == IDNO)
                    return false;
            }
        }
        catch(...)
        {
        }

    }

    if(m_strTestDescription.IsEmpty())
    {
        m_strTestDescription = m_strTestName;
        UpdateData(FALSE);
    }
    else
        if(m_strTestDescription.Find("\"")!=-1)
        {
            AfxMessageBox("The test description must not contain \" s");
            return false;
        }

    return CPropertyPage::OnKillActive();
}

/////////////////////////////////////////////////////////////////////////////
// CYRSummaryPage dialog
IMPLEMENT_DYNAMIC(CYRSummaryPage, CPropertyPage)

CYRSummaryPage::CYRSummaryPage()
        : CYRWizardBasePage(CYRSummaryPage::IDD)
{
    //{{AFX_DATA_INIT(CYRSummaryPage)
    m_strSummary = _T("");
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}


void CYRSummaryPage::DoDataExchange(CDataExchange* pDX)
{
    CYRWizardBasePage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRSummaryPage)
    DDX_Control(pDX, IDC_SUMMARY, m_ctlSummary);
    DDX_Text(pDX, IDC_SUMMARY, m_strSummary);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRSummaryPage, CYRWizardBasePage)
//{{AFX_MSG_MAP(CYRSummaryPage)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRSummaryPage message handlers
BOOL CYRSummaryPage::OnSetActive()
{
    CYRGroupTestWizardSheet* psheet = (CYRGroupTestWizardSheet*) GetParent();
    psheet->SetWizardButtons(PSWIZB_FINISH|PSWIZB_BACK);

    ComposeSummary();

    m_ctlSummary.SetSel(-1,-1);
    return CYRWizardBasePage::OnSetActive();
}

void CYRSummaryPage::ComposeSummary()
{
    CYRGroupTestWizardSheet *pSheet = (CYRGroupTestWizardSheet *) GetParent();
    ASSERT (pSheet!=NULL);

    m_strSummary.Empty();

    // First the Name Page
    CYRNameDefinitionPage* pNameDefinitionPage = (CYRNameDefinitionPage*) pSheet->GetPage(CYRGroupTestWizardSheet::enNameDefPage);
    if(pNameDefinitionPage->GetSafeHwnd() != NULL)
    {
        m_strSummary += "Test Name:\t" + pNameDefinitionPage->m_strTestName + "\r\n";
        m_strSummary += "Description:\t" + pNameDefinitionPage->m_strTestDescription + "\r\n";
        m_strSummary += "\r\n";
    }

    // Now we add in some static text
    m_strSummary += "The Test will be structured in the following manner\r\n";
    m_strSummary += "\r\n";

    // First the Main Test Name
    m_strSummary += "Main Test:  " + pNameDefinitionPage->m_strTestName + "\r\n";
    m_strSummary += "\r\n";

    // Now the test selection page
    CString strNumber;
    CYRGroupDependencyPage* pGroupDepPage = (CYRGroupDependencyPage*) pSheet->GetPage(CYRGroupTestWizardSheet::enGroupDepPage);
    if(pGroupDepPage->GetSafeHwnd() != NULL)
    {
        for (int nNdx=0; nNdx<pGroupDepPage->GetNumSelectedTests(); nNdx++)
        {
            strNumber.Format("%d", nNdx+1);

            m_strSummary += "Slave " + strNumber + ":";

            m_strSummary += "\tName\t: " + pGroupDepPage->GetSelectedTest(nNdx) + "\r\n";
            m_strSummary += "\r\n";
        }
    }

    // And finally the end of test
    m_strSummary += "End of Test\r\n";
    m_strSummary += "\r\n";
    UpdateData(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CYRGroupDependencyPage dialog


CYRGroupDependencyPage::CYRGroupDependencyPage(CWnd* pParent /*=NULL*/)
        : CYRWizardBasePage(CYRGroupDependencyPage::IDD)
{
    //{{AFX_DATA_INIT(CYRGroupDependencyPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_bOnBack = false;
}


void CYRGroupDependencyPage::DoDataExchange(CDataExchange* pDX)
{
    CYRWizardBasePage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRGroupDependencyPage)
    DDX_Control(pDX, IDC_AVAILABLE, m_ctlAvailable);
    DDX_Control(pDX, IDC_SELECTED, m_ctlSelected);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRGroupDependencyPage, CPropertyPage)
//{{AFX_MSG_MAP(CYRGroupDependencyPage)
ON_BN_CLICKED(IDC_DESELECT, OnDeselect)
ON_BN_CLICKED(IDC_DESELECT_ALL, OnDeselectAll)
ON_BN_CLICKED(IDC_SELECT, OnSelect)
ON_BN_CLICKED(IDC_SELECT_ALL, OnSelectAll)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRGroupDependencyPage message handlers
BOOL CYRGroupDependencyPage::OnSetActive()
{
    CYRGroupTestWizardSheet* psheet = (CYRGroupTestWizardSheet*) GetParent();
    psheet->SetWizardButtons(PSWIZB_NEXT|PSWIZB_BACK);

    return CYRWizardBasePage::OnSetActive();
}

void CYRGroupDependencyPage::SavePageSettings(const CString& strWizardFile)
{
    int nNumSelectedTests = m_arSelectedTests.GetSize();

    //Write num of selected test to file
    CString str;
    str.Format("%d",nNumSelectedTests);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_NUM_SELECTED_TESTS,str,strWizardFile);

    CString strNumber;
    DWORD dwMaxBufferSize=200;
    GetPrivateProfileString(WIZARD_DEPENDENCIES,WIZARD_DEPENDENCIES_NUMBER,"0",strNumber.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    strNumber.ReleaseBuffer();
    int nNumDependencies = atoi(strNumber);

    //write all the selected tests to file
    CString strTestName;
    for(int nNdx=0;nNdx<nNumSelectedTests;nNdx++)
    {
        str.Format(WIZARD_TEST_SELECTED_TEST_FORMAT,nNdx+1);
        strTestName = m_arSelectedTests[nNdx];

        WritePrivateProfileString(WIZARD_TEST,str,strTestName,strWizardFile);

        //Write to dependencies section as well
        str.Format(WIZARD_DEPENDENCIES_FORMAT,nNumDependencies+1);
        WritePrivateProfileString(WIZARD_DEPENDENCIES,str,strTestName+".TSS",strWizardFile);
        nNumDependencies++;
    }

    strNumber.Format("%d",nNumDependencies);
    WritePrivateProfileString(WIZARD_DEPENDENCIES,WIZARD_DEPENDENCIES_NUMBER,strNumber,strWizardFile);
}

void CYRGroupDependencyPage::LoadPageSettings(const CString& strWizardFile)
{
    m_arSelectedTests.RemoveAll();

    const DWORD dwMaxBufferSize = 200;

    //Get num of selected test from file
    CString str;
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_NUM_SELECTED_TESTS,"",str.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    str.ReleaseBuffer();
    int nNumSelectedTests = atoi(str);

    //get all the selected tests from the file
    CYRTests Tests();
    CString strTestName,strNumVUs;
    for(int nNdx=0;nNdx<nNumSelectedTests;nNdx++)
    {
        str.Format(WIZARD_TEST_SELECTED_TEST_FORMAT,nNdx+1);
        GetPrivateProfileString(WIZARD_TEST,str,"",strTestName.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
        strTestName.ReleaseBuffer();

        m_arSelectedTests.Add(strTestName);
    }
}

BOOL CYRGroupDependencyPage::OnInitDialog()
{
    CYRWizardBasePage::OnInitDialog();

    m_ctlAvailable.InsertColumn(0,"Test",LVCFMT_CENTER,88,0);
    m_ctlAvailable.InsertColumn(1,"VU",LVCFMT_CENTER,28,1);
    m_ctlAvailable.SetExtendedStyle(LVS_EX_GRIDLINES|LVS_EX_FULLROWSELECT);

    m_ctlSelected.InsertColumn(0,"Test",LVCFMT_CENTER,88,0);
    m_ctlSelected.InsertColumn(1,"VU",LVCFMT_CENTER,28,1);
    m_ctlSelected.SetExtendedStyle(LVS_EX_GRIDLINES|LVS_EX_FULLROWSELECT);

    FillTestLists();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CYRGroupDependencyPage::OnSelect()
{
    m_ctlAvailable.SetRedraw(FALSE);
    m_ctlSelected.SetRedraw(FALSE);

    CString strTest,strVUs;

    int nNumItems = m_ctlAvailable.GetItemCount();
    for(int nItem=0;nItem<nNumItems;nItem++)
    {
        if(m_ctlAvailable.GetItemState(nItem,LVIS_SELECTED) & LVIS_SELECTED)
        {
            strTest = m_ctlAvailable.GetItemText(nItem,0);
            strVUs = m_ctlAvailable.GetItemText(nItem,1);

            int nNewItem = m_ctlSelected.InsertItem(0,strTest);
            if(nNewItem != -1)
            {
                m_ctlSelected.SetItemText(nNewItem,1,strVUs);
                m_ctlAvailable.DeleteItem(nItem--);
                nNumItems--;
            }
        }
    }

    m_ctlAvailable.SetRedraw();
    m_ctlSelected.SetRedraw();
}

void CYRGroupDependencyPage::OnSelectAll()
{
    m_ctlAvailable.SetRedraw(TRUE);
    m_ctlSelected.SetRedraw(TRUE);

    CString strTest,strVUs;
    int nNumItems = m_ctlAvailable.GetItemCount();
    for(int nItem=0;nItem<nNumItems;nItem++)
    {
        strTest = m_ctlAvailable.GetItemText(nItem,0);
        strVUs = m_ctlAvailable.GetItemText(nItem,1);

        int nNewItem = m_ctlSelected.InsertItem(0,strTest);
        if(nNewItem != -1)
            m_ctlSelected.SetItemText(nNewItem,1,strVUs);
    }
    m_ctlAvailable.DeleteAllItems();
    m_ctlAvailable.SetRedraw();
    m_ctlSelected.SetRedraw();
}

void CYRGroupDependencyPage::OnDeselect()
{
    m_ctlAvailable.SetRedraw(FALSE);
    m_ctlSelected.SetRedraw(FALSE);

    CString strTest,strVUs;

    int nNumItems = m_ctlSelected.GetItemCount();
    for(int nItem=0;nItem<nNumItems;nItem++)
    {
        if(m_ctlSelected.GetItemState(nItem,LVIS_SELECTED) & LVIS_SELECTED)
        {
            strTest = m_ctlSelected.GetItemText(nItem,0);
            strVUs = m_ctlSelected.GetItemText(nItem,1);

            int nNewItem = m_ctlAvailable.InsertItem(0,strTest);
            if(nNewItem != -1)
            {
                m_ctlAvailable.SetItemText(nNewItem,1,strVUs);
                m_ctlSelected.DeleteItem(nItem--);
                nNumItems--;
            }
        }
    }
    m_ctlAvailable.SetRedraw();
    m_ctlSelected.SetRedraw();
}

void CYRGroupDependencyPage::OnDeselectAll()
{
    m_ctlAvailable.SetRedraw(TRUE);
    m_ctlSelected.SetRedraw(TRUE);

    CString strTest,strVUs;
    int nNumItems = m_ctlSelected.GetItemCount();
    for(int nItem=0;nItem<nNumItems;nItem++)
    {
        strTest = m_ctlSelected.GetItemText(nItem,0);
        strVUs = m_ctlSelected.GetItemText(nItem,1);

        int nNewItem = m_ctlAvailable.InsertItem(0,strTest);
        if(nNewItem != -1)
            m_ctlAvailable.SetItemText(nNewItem,1,strVUs);
    }
    m_ctlSelected.DeleteAllItems();
    m_ctlAvailable.SetRedraw();
    m_ctlSelected.SetRedraw();
}

void CYRGroupDependencyPage::FillTestLists()
{
    CYRGroupTestWizardSheet* psheet = (CYRGroupTestWizardSheet*) GetParent();
    CYRTests Tests(psheet->m_pServer);

    CString strNumVUs;
    for(int nTest=0;nTest<m_arSelectedTests.GetSize();nTest++)
    {
        //Check that this is a simple test and that it still exists etc..
        if(Tests.IsSimpleTest(m_arSelectedTests[nTest]))
        {
            int nItem = m_ctlSelected.InsertItem(0,m_arSelectedTests[nTest]);

            int nNumVUs = Tests.GetNumVUsForSimpleTest(m_arSelectedTests[nTest]);
            strNumVUs.Format("%d",nNumVUs);
            m_ctlSelected.SetItemText(nItem,1,strNumVUs);
        }
    }

    CStringArray arstrSimpleTests;
    Tests.GetSimpleTests(arstrSimpleTests);

    LVFINDINFO lvFindInfo;
    lvFindInfo.flags = LVFI_STRING;
    for(nTest=0;nTest<arstrSimpleTests.GetSize();nTest++)
    {
        lvFindInfo.psz = arstrSimpleTests[nTest];
        if(m_ctlSelected.FindItem(&lvFindInfo,-1) != -1)
            continue;

        int nItem = m_ctlAvailable.InsertItem(0,arstrSimpleTests[nTest]);

        int nNumVUs = Tests.GetNumVUsForSimpleTest(arstrSimpleTests[nTest]);
        strNumVUs.Format("%d",nNumVUs);
        m_ctlAvailable.SetItemText(nItem,1,strNumVUs);
    }
}

BOOL CYRGroupDependencyPage::OnKillActive()
{
    //Always allow back!
    if (m_bOnBack)
    {
        m_bOnBack = false;
        return true;
    }

    int nNumSelectedTests = m_ctlSelected.GetItemCount();

    if(!nNumSelectedTests)
    {
        AfxMessageBox("Please select at least one test for this group test");
        return 0;
    }

    m_arSelectedTests.RemoveAll();

    CString strTestName;
    for(int nNdx=0;nNdx<nNumSelectedTests;nNdx++)
    {
        strTestName = m_ctlSelected.GetItemText(nNdx,0);

        m_arSelectedTests.Add(strTestName);
    }

    return CPropertyPage::OnKillActive();
}

LRESULT CYRGroupDependencyPage::OnWizardBack()
{
    m_bOnBack = true;
    return 0;
}

BOOL CYRWizardBasePage::OnWizardFinish()
{
    CYRGroupTestWizardSheet* psheet = (CYRGroupTestWizardSheet*) GetParent();
    psheet->OnWizardFinish();

    return true;
}




BOOL CYRSummaryPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArraySummary);

    return true;
}

BOOL CYRNameDefinitionPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArrayName);

    return true;
}

BOOL CYRGroupDependencyPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArrayDepends);

    return true;
}


bool CYRGroupTestWizardSheet::IsValidFilename(const CString &strPath,bool bTellUser)
{
    CString strFileName(strPath);

    int nSlashPos = strPath.ReverseFind('\\');
    if(nSlashPos != -1)
    {
        strFileName = strPath.Right(strPath.GetLength()-nSlashPos-1);
    }

    //remove file extension
    strFileName.TrimRight(".");
    strFileName.MakeUpper();

    //Check for reserved file names
    if(strFileName == "CON"
            || strFileName == "AUX"
            || strFileName == "CLOCK$"
            || strFileName == "PRN"

            //check for COM1,COM2... and LPT1,LPT2...
            || ((strFileName.GetLength()==4) && (isdigit(strFileName[3])) &&
                (strFileName == "COM" || strFileName == "LPT"))
      )
    {
        if(bTellUser)
        {
            CString strMsg;
            strMsg.Format("The filename \"%s\"\nis not allowed because it contains a name reserved for use by Windows",strPath);
            AfxMessageBox(strMsg,MB_ICONWARNING|MB_OK);
        }
        return false;
    }

    return true;
}
