/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
//  CYRThreadDebug.cpp
//  ==================
//
//  This is the implementation file for the CYRThreadDebug Class.
//
////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"

extern "C"
{
#include <psl_macros.h>
#include <psl_rtndef.h>

    //    function prototypes.
#include <vtest_rtndef.h>

    //    structures
#include <vtest_dtcdef.h>
#include <vtest_dtcovrdef.h>

#include <shrmsg.h>
#include <ipc_prmdef.h>
#include <pslerr_prmdef.h>      // Error Constants 

#include <psl_trc_prmdef.h>
#include <vtest_prmdef.h>
#include <vcbdef.h>
#include <psldate_prmdef.h>     // PSL Date Conversion 
}

//
//    Architecture includes
//

#include "CyrStAPI\CECComProject.h"

//
//    Product includes
//
#include "..\..\Framework\CECWorkspace.h"

//
//    Project includes
//
#include "CYRThreadDebug.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAX_OUTPUT_SIZE 512

static BOOL bDestroyed = FALSE;

/////////////////////////////////////////////////////////////////////////////
// CYRThreadDebug dialog

IMPLEMENT_DYNCREATE(CYRThreadDebug, CFormView)


CYRThreadDebug::CYRThreadDebug()
        : CFormView(CYRThreadDebug::IDD)

{
    //{{AFX_DATA_INIT(CYRThreadDebug)
    m_strFileName = _T("");
    m_strThreadName = _T("");
    m_strDebugOutput = _T("");
    m_strCurrentStatus = _T("");
    //}}AFX_DATA_INIT
    m_nLineNumber = -1;
    m_pfileScriptSource = NULL;
    m_strDebugOutput.GetBufferSetLength(MAX_OUTPUT_SIZE);
}


void CYRThreadDebug::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRThreadDebug)
    DDX_Control(pDX, IDC_THREADDEBUG_SORUCELIST, m_lcSourceCode);
    DDX_Text(pDX, IDC_THDBG_SOURCEFILE, m_strFileName);
    DDX_Text(pDX, IDC_THDBG_THREADNAME, m_strThreadName);
    DDX_Text(pDX, IDC_THDBG_OUTPUT, m_strDebugOutput);
    DDX_Text(pDX, IDC_THRDDBG_CURRENTSTATUS, m_strCurrentStatus);
    //}}AFX_DATA_MAP
} // CYRThreadDebug::DoDataExchange



BEGIN_MESSAGE_MAP(CYRThreadDebug, CFormView)
//{{AFX_MSG_MAP(CYRThreadDebug)
ON_BN_CLICKED(IDC_THREADDEBUG_GO, OnThreadDebugGo)
ON_WM_DESTROY()
ON_BN_CLICKED(IDC_THREADDEBUG_CLOSE, OnClose)
ON_WM_DRAWITEM()
//}}AFX_MSG_MAP
ON_WM_CREATE()
ON_BN_CLICKED(IDC_THREADDEBUG_STEP,OnDebugStep)


END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRThreadDebug message handlers

BOOL CYRThreadDebug::PreCreateWindow(CREATESTRUCT& cs)
{
    // As this window depends on certain criteria being met before it can be shown
    // we have determine here whether or not to continue with the creation

    return CFormView::PreCreateWindow(cs);
} // CYRThreadDebug::PreCreateWindow

int CYRThreadDebug::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CFormView::OnCreate(lpCreateStruct) == -1)
        return -1;

    // As this is a new invocation of the script debugger reset the
    // static destroy flag. This flag is used to ensure that we do not
    // kill the window twice
    bDestroyed = FALSE;         //+1757

    return 0;
} // CYRThreadDebug::OnCreate


void CYRThreadDebug::OnInitialUpdate()
{
    CFormView::OnInitialUpdate();

    // Make the window the size of the main dialog

    GetParentFrame()->RecalcLayout();

    ResizeParentToFit(FALSE);

    // Set the columns for the list control

    LV_COLUMN lvc;
    lvc.mask = LVCF_FMT | LVCF_TEXT | LVCF_TEXT | LVCF_WIDTH;
    lvc.fmt = LVCFMT_LEFT;
    lvc.pszText = "Check this out";
    lvc.cx = 400;
    m_lcSourceCode.InsertColumn(0, &lvc);

    // Keep user informed of our activity

    UpdateCurrentStatus("Initialising debugger...");

    // Initialise debug

    if ( DoDebug(TRUE) != PSL_NORMAL)
    {
        TRACE("Destroying window due to an error. Error was displayed on screen.\n");
        PostMessage(WM_COMMAND, IDC_THREADDEBUG_CLOSE, 0);
        return ;
    }


    // Keep user informed of our activity

    UpdateCurrentStatus("<Idle>");


} // CYRThreadDebug::OnInitialUpdate

void CYRThreadDebug::OnDebugStep()
{
    //
    // Keep user informed of our activity
    UpdateCurrentStatus("Executing current instruction ...");


    if ( DoDebug(FALSE) != PSL_NORMAL)
    {
        TRACE("Destroying window due to an error. Error was displayed on screen.\n");
        OnClose();
    }
    else
    {

        //
        // Keep user informed of our activity
        UpdateCurrentStatus("<Idle>");
    }
} // CYRThreadDebug::OnDebugStep

//
//
//      DoDebug     -   Routine to retrieve the debug status.
//
//      Functional summary:
//          Establish communications to VTEST. Retrieve current status
//          of debug. Retrieve the current source script file name,
//          load script source file if necessary,
//          current line of execution in that script file,
//
//
//
//      Calling convention
//          iret = DoDebug(intialising)
//
//
//
//      Where initialising  = BOOL specifying whether the routine is being
//                              called for initialisation or in response to
//                              step commands.
//
//      Return codes
//            iret          = Unsigned Long PSL Routine Return status
//                          = PSL_NORMAL if step/intialisation performed OK
//                          = IPC_LINKCLOSED if VTEST has performed a disappearing act
//                          = PSL_UNEXPMSG, unexpected error
//                          = Other error status if debug invalid.
//
//
//      Member variables modified/accessed
//          m_nLineNumber
//          m_strScriptFileName
//          m_strDebugOutput
//          m_pfileScriptSource
//
//
//
int CYRThreadDebug::DoDebug(BOOL initialising)
{
    return 0;
}
//
//
//      ReadScriptFile  -   Routine to read the source script file
//
//      Functional summary:
//          Read the source script file and add the read lines into
//          the list box.
//
//
//      Calling convention
//          iret = ReadScriptFile(p_fname)
//
//      Where p_fname       -   is a pointer to a CFile, that is already opened.
//
//
//      Return code         -   0 if encounters error during read
//                          -   non-zero if sucessful
//
//
//
int CYRThreadDebug::ReadScriptFile(CString& strFileName)
{
    char        buff[100];
    BOOL        bEOF;           // end-of-file flag
    BOOL        bError;         // error while reading the file
    int         iret;
    char*       pEOL;           // End-of-line pointer
    int         nLineCount;     // number of lines
    CString     str;


    //
    // We have not reached the end-of-file
    // and no error has occurred
    bEOF = bError = FALSE;
    nLineCount = 0;

    //
    // Open the source script file
    m_pfileScriptSource = fopen(strFileName, "r");
    if ( m_pfileScriptSource == NULL )
    {
        iret = FALSE;
        goto end;
    }



    //
    // Read the source script file - adding the lines to the
    // list box. Use buff+1 to insert a space character (a placeholder
    // for indicating current line of execution).
    buff[0] = ' ';
    while ( fgets(buff+1, sizeof(buff)-1, m_pfileScriptSource) != NULL )
    {
        //
        // Remove the Carriage Return character
        // Replace tabs by 4 white spaces
        // Add a blank character at start of string
        // this will be used to indicate CURRENT execution line
        pEOL = (char*) memchr(buff, '\n', sizeof(buff) );
        if (pEOL != NULL )
            *pEOL = '\0';
        //      PSL_SUBSTR(buff, sizeof(buff), "\t", "     ");
        str = buff;
        SetListCtrlItem(nLineCount, str, TRUE);

        nLineCount++;
    }

    //
    // Check if finished reading file
    if ( feof(m_pfileScriptSource) != 0)
    {
        str = " *** End of file *** ";
        SetListCtrlItem(nLineCount, str, TRUE);
    }
    else
    {
        iret  = FALSE;
        goto end;
    }

    //
    // Close the file
    iret = fclose(m_pfileScriptSource);
    if (iret != 0)
    {
        //
        // Error in closing file
        iret = FALSE;
        goto end;
    }
    else
    {
        iret = TRUE;
        goto end;
    }

end:
    return iret;
} // CYRThreadDebug::ReadScriptFile






//
// Retrieves the string at the zero-based index position nIndex
//
// Returns  FALSE   on failure
//          TRUE    on success
int CYRThreadDebug::RetrieveListCtrlItem(int nIndex, CString& str)
{
    int             iret;           // return code
    char            temp[200];      // pointer supplied to GetItem

    //
    // Set up and retrieve the intem
    LV_ITEM lvi;
    lvi.mask = LVIF_TEXT;
    lvi.iItem = nIndex;
    lvi.iSubItem = 0;
    lvi.cchTextMax = sizeof(temp);
    lvi.pszText = temp;
    iret = m_lcSourceCode.GetItem(&lvi);
    str = lvi.pszText;

    return iret;
}// CYRThreadDebug::RetrieveListCtrlItem




//
// Set the string at the zero-based index position nIndex to str
//
// Returns  FALSE   on failure
//          TRUE    on success

int CYRThreadDebug::SetListCtrlItem(int nIndex, CString& str, BOOL bNewItem)
{
    int iret;

    //
    // Set the text at position nIndex to str
    LV_ITEM lvi;
    lvi.mask = LVIF_TEXT;
    lvi.iItem = nIndex;
    lvi.iSubItem = 0;
    lvi.pszText = str.GetBuffer(1);
    lvi.cchTextMax = str.GetLength()+1;
    if (bNewItem)
        iret = m_lcSourceCode.InsertItem(&lvi);
    else
        iret = m_lcSourceCode.SetItem(&lvi);


    return iret;
} // CYRThreadDebug::SetListCtrlItem





//
// Message handler for the GO button on the Debug window
//
// Function simply resumes the terminal
void CYRThreadDebug::OnThreadDebugGo()
{


}





void CYRThreadDebug::OnClose()
{
}


void CYRThreadDebug::OnDestroy()
{
    CFormView::OnDestroy();

}



//
// Owner draw implementation for the source script list control
//
// This method is called when a child item that has OWNER draw style
// needs to be drawn (called for each item)
//
// The function paints the current source line in HIGHLIGHTed color
//
void CYRThreadDebug::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct)
{
    if (nIDCtl != IDC_THREADDEBUG_SORUCELIST)
        CFormView::OnDrawItem(nIDCtl, lpDrawItemStruct);



    //
    // Child item requesting draw is our source code control
    //

    //
    // If list control is empty, do nothing
    if (lpDrawItemStruct->itemID < 0)
        return;


    //
    // Create a temporary CDC object.
    CHeaderCtrl * pHdr = (CHeaderCtrl *)m_lcSourceCode.GetDlgItem(0);
    if( pHdr == NULL )
    {
        ASSERT(0);
        return;
    }


    CDC* pDC = CDC::FromHandle( lpDrawItemStruct->hDC );

    if( lpDrawItemStruct->itemAction & (ODA_DRAWENTIRE ) )
    {
        COLORREF crOldText;     // never permanently alter the original CDC attributes
        COLORREF crOldBack;   // never permanently alter the original CDC attributes

        //
        // This is to solve the problem when the highlighted item is at
        // the bottom of the view, and the next item is selected
        // We get an overspill of highlighted background left behind
        //
        //
        // If the item is selected then highlight the item.
        // If this is the current source code line then highlight it
        //
        if( lpDrawItemStruct->itemID == (UINT) m_nLineNumber-1)
        {
            crOldText = pDC->SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT) );
            crOldBack = pDC->SetBkColor(::GetSysColor(COLOR_HIGHLIGHT) );
            lpDrawItemStruct->rcItem.left += 2;
        }
        CString s;
        int subItem = 0;
        s = m_lcSourceCode.GetItemText(lpDrawItemStruct->itemID, subItem);

        //
        // Erase the entire area. Using ExtTextOut is a neat alternative to
        // FillRect and quicker, too!
        // We do not supply the actual string because this function uses a border
        // for its drawing the string, 2 in horizontal, 1 in vertical mode
        //
        pDC->ExtTextOut( lpDrawItemStruct->rcItem.left,
                         lpDrawItemStruct->rcItem.top,
                         ETO_OPAQUE, &(lpDrawItemStruct->rcItem),s, NULL );


        //
        // Restore resources
        if( lpDrawItemStruct->itemID == (UINT) m_nLineNumber-1)
        {
            pDC->SetTextColor( crOldText );
            pDC->SetBkColor( crOldBack );
        }



    } // end of if on   ODA_DRAWENTIRE | ODA_SELECT

} // CYRThreadDebug::OnDrawItem

//
// Update the current status

void CYRThreadDebug::UpdateCurrentStatus(CString stStatus)
{
    m_strCurrentStatus = stStatus;
    UpdateData(FALSE);
} // CYRThreadDebug::UpdateCurrentStatus




