/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECWebExecDlg.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "..\..\..\HTMLHelp\CYRImpact\Web_Exec.inl"

#include "CECWebExecDlg.h"
#include "..\..\Interfaces\CyrPlugIns.h"
#include "..\Common\CECTssWriter.h"
#include "solutions.h"
#include "..\Common\CYRTestProperties.h"
#include "..\..\Common\CECRegistryTools.h"
#include "..\..\Common\CECXServerError.h"
#include "Server.h"
#include "CECSourceFileUtils.h"
#include "htmlhelp.h"
#include "TestWizardDefs.h"
#include "ThreadModelDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECWebExecDlg dialog

CECWebExecDlg::CECWebExecDlg(CString strProject, CString strTest, CWnd* pParent /*=NULL*/)
        : CDialog(CECWebExecDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECWebExecDlg)
    //}}AFX_DATA_INIT

    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";


    m_Project = strProject;
    m_szTestToExecute = strTest;
    m_szNodeFile = "<Local>";
    m_bExecuting = FALSE;

	m_bThreadPool = TRUE;

    m_pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);
}


void CECWebExecDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECWebExecDlg)
    DDX_Control(pDX, IDC_TEST_COMBO, m_TestCombo);
    DDX_Control(pDX, IDC_APPLY_BUTTON, m_ApplyButton);
    DDX_Control(pDX, IDC_EXECUTE, m_ExecButton);
    DDX_Control(pDX, IDC_SPIN_USERS, m_SpinVUsers);
    DDX_Control(pDX, IDC_SPIN_INTERVAL, m_SpinInterval);
    DDX_Control(pDX, IDC_DESCRIPTION, m_DescriptionCtrl);
    DDX_Text(pDX, IDC_INTERVAL, m_nInterval);
    DDV_MinMaxInt(pDX, m_nInterval, 0, 2147483647);
    DDX_Text(pDX, IDC_USERS, m_UsersPerInterval);
    DDV_MinMaxInt(pDX, m_UsersPerInterval, 1, 999999999);
    //}}AFX_DATA_MAP

    DDX_Control(pDX, IDC_GRID, m_wndGrid);

}


BEGIN_MESSAGE_MAP(CECWebExecDlg, CDialog)
//{{AFX_MSG_MAP(CECWebExecDlg)
ON_WM_SIZE()
ON_COMMAND(ID_POPUP_INSERT_ROW, OnPopupInsertRow)
ON_COMMAND(ID_POPUP_DELETEROW, OnPopupDeleterow)
ON_COMMAND(ID_POPUP_NEWINJECTOR, OnPopupNewinjector)
ON_BN_CLICKED(IDC_APPLY_BUTTON, OnApply)
ON_EN_CHANGE(IDC_INTERVAL, OnChangeInterval)
ON_EN_CHANGE(IDC_USERS, OnChangeUsers)
ON_BN_CLICKED(IDC_EXECUTE, OnExecute)
ON_CBN_SELCHANGE(IDC_TEST_COMBO, OnSelchange)
ON_WM_HELPINFO()
ON_COMMAND(IDCANCEL, OnCancel)
	ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECWebExecDlg message handlers

BOOL CECWebExecDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    GetInjectorList();

    LoadTestList();

    // Get a handle to the Mainframe
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    // Now get the server login information from the framework
    m_szHost = m_pServer->GetHostName();
    m_szUsername = m_pServer->GetUserName();
    m_szPassword = m_pServer->GetPassword();


    // Initialize the spin controls
    CWnd* pInterval = GetDlgItem(IDC_INTERVAL);
    m_SpinInterval.SetBuddy(pInterval);
    m_SpinInterval.SetBase(0);
    m_SpinInterval.SetRange32(0, 50000);

    CWnd* pUsers = GetDlgItem(IDC_USERS);
    m_SpinVUsers.SetBuddy(pUsers);
    m_SpinVUsers.SetBase(1);
    m_SpinVUsers.SetRange32(1, 50000);

    ResetGrid();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECWebExecDlg::ResetGrid()
{
    m_wndGrid.DeleteAllItems();
    m_szDescription = "";
    m_nTotalVUsers = 0;
    m_bSimple = true;
    m_nNextRow = 2;
    m_nNextCol = 3;

    m_nDispatchRow = 11;
    m_nTotalRow = 12;
    m_nLastDummyRow = 10;

    m_nInterval = 0;
    m_UsersPerInterval = 1;
    m_nNumberOfTests = 0;

    // Load the test information into the grid and initialize cell controls
    SetupBasicGrid();

    CWnd* pInterval = GetDlgItem(IDC_INTERVAL);
    CWnd* pUsers = GetDlgItem(IDC_USERS);

    // Load information about the test from the Wizard generated ini file
    // and populate the grid.
    BOOL bCanExecute = LoadTestInfo();

    if (!bCanExecute)
    {
        m_ExecButton.EnableWindow(FALSE);

        if (m_UsersPerInterval == 0)
        {
            m_UsersPerInterval = m_nTotalVUsers;
            m_nInterval = 0;
        }
        if (m_nInterval == 0)
        {
            pInterval->EnableWindow(FALSE);
        }
        else
        {
            pInterval->EnableWindow(TRUE);
        }
    }
    else
    {
        m_ExecButton.EnableWindow(TRUE);
        m_ApplyButton.EnableWindow(FALSE);
    }

    m_SpinVUsers.SetRange32(1, m_nTotalVUsers);
    m_DescriptionCtrl.SetWindowText(m_szDescription);

    UpdateData(FALSE);

    UpdateTotals(FALSE);

    m_wndGrid.ExpandToFit();
}

void CECWebExecDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);


}

void CECWebExecDlg::SetupBasicGrid()
{

    // Set the initial size of the grid (it will get bigger!!).
    m_wndGrid.SetRowCount(13);
    m_wndGrid.SetColumnCount(3);
    m_wndGrid.SetFixedColumnCount();
    m_wndGrid.SetFixedRowCount();
    m_wndGrid.SetColumnResize();
    m_wndGrid.SetSingleColSelection();
    m_wndGrid.SetSingleRowSelection();

    // Setup grid headers and size
    m_wndGrid.SetItemText(0,1,"Injector");
    m_wndGrid.SetItemText(0,2,"Performance Monitor");


    // Add an empty Injector Row
    SetupControls(1);


    // Add "To Dispatch" and "Total" rows

    for (int i = 1; i<=10; i++)
    {
        CString str;
        str.Format("%d",i);
        m_wndGrid.SetItemText(i,0, str);
    }
    m_wndGrid.SetItemText(m_nDispatchRow,0, "To Dispatch");
    m_wndGrid.SetItemText(m_nTotalRow,0,"Total");
    /*
        m_wndGrid.SetStyleRange(CGXRange(m_nTotalRow,0,m_nTotalRow,2), CGXStyle()
    //      .SetInterior(RGB(122,172,172))
            .SetFont(CGXFont()
                    .SetSize(10)
                    .SetBold(TRUE))
            .SetEnabled(false));
     
      m_wndGrid.SetStyleRange(CGXRange(m_nDispatchRow,0,m_nDispatchRow,2), CGXStyle()
    //      .SetInterior(RGB(255,70,70))
            .SetFont(CGXFont()
                    .SetSize(10)
                    .SetBold(TRUE))
            .SetEnabled(false));
    */
    // Set up a number of dummy rows which will be overwritten as more
    // injectors are added
    for ( i = 2; i<=m_nLastDummyRow; i++)
    {
        for (int j = 0; j<=2; j++)
        {
            m_wndGrid.SetItemBkColour(i,j, RGB(172,172,172));
            m_wndGrid.SetCellEnabled(i,j, FALSE);
        }
    }
}

BOOL CECWebExecDlg::LoadTestInfo()
{
    // Open the wizard file for this test
    CString wizPath = CheckOutWizFile(m_szTestToExecute);

    // Determine the type of the test (Simple or Group)
    CString strType;
    DWORD dwMaxBufferSize=200;
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_TYPE,"",strType.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,wizPath);
    strType.ReleaseBuffer();
    m_szDescription = GetTestDescription(wizPath);

    // As this is a group test a "Load" column is required. For each subtest defined
    // in the wizard file add a column for it.
    if (strType == GROUPTEST)
    {
        m_bSimple = false;
        AddLoadColumn();
        int subTests = GetSubTestCount(wizPath);
        for (int i = 1; i<=subTests; i++)
        {
            CString testName = GetSubTest(wizPath, i);
            AddTestColumn(testName);
            m_nNumberOfTests++;
        }
    }

    // This is a simple test so just add a column for this test
    else if (strType == SIMPLETEST)
    {
        m_bSimple = true;
        AddTestColumn(m_szTestToExecute);
        m_nNumberOfTests++;
    }
    else
    {
        return FALSE;
    }

    // If the test has been run previously, load the last known parameters
    // so that the test can be run immediately rather than having to add
    // injectors and re-generating the tss file
    BOOL bCanExecute = LoadTestParams();

    //Force a recompilation of the TSS each time we
    //use the dialog
    return FALSE;//bCanExecute;
}


CString CECWebExecDlg::CheckOutWizFile(CString strTestName, BOOL bLock)
{
    m_pServer->CheckOutFile(CYRServer::enTestsFolder,strTestName,strTestName +".WIZ");

    // Return the full path to the file
    return m_pServer->GetFolderPath(CYRServer::enTestsFolder) + strTestName + "\\" + strTestName + ".WIZ";
}

CString CECWebExecDlg::GetTestDescription(CString wizFile)
{
    // Read the "Test Description" value from the specified wizard file
    CString strDesc;
    DWORD dwMaxBufferSize=200;
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_DESCRIPTION,"",strDesc.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,wizFile);
    strDesc.ReleaseBuffer();

    return strDesc;
}

int CECWebExecDlg::GetVUsers(CString wizFile)
{
    // Read the "Num V Users" value from the specified wizard file
    CString strUsers;
    DWORD dwMaxBufferSize=200;
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_NUM_V_USERS,"0",strUsers.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,wizFile);
    strUsers.ReleaseBuffer();
    int num = atoi(strUsers);
    return num;
}

int CECWebExecDlg::GetSubTestCount(CString wizFile)
{
    // Read the "Num selected Tests" value from the specified wizard file
    CString strTests;
    DWORD dwMaxBufferSize=200;
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_NUM_SELECTED_TESTS,"0",strTests.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,wizFile);
    strTests.ReleaseBuffer();

    int num = atoi(strTests);
    return num;
}
CString CECWebExecDlg::GetSubTest(CString wizFile, int index)
{
    // Read the name of the subtest for the given index in the specified wizard file
    CString strTest;
    CString section;
    section.Format(WIZARD_TEST_SELECTED_TEST_FORMAT,index);

    DWORD dwMaxBufferSize=200;
    GetPrivateProfileString(WIZARD_TEST,section.GetBuffer(0),"",strTest.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,wizFile);
    strTest.ReleaseBuffer();

    return strTest;
}

void CECWebExecDlg::GetInjectorList()
{
    m_nodeArray.RemoveAll();
    m_pServer->GetFileList(CYRServer::enNodesFolder,"",m_nodeArray);
}

void CECWebExecDlg::SetupControls(int nRow)
{

    // Make sure the row header is enabled and the correct colour as it may be
    // replacing a dummy row
    m_wndGrid.SetItemBkColour(nRow,0 ,RGB(192,192,192));
    for (int z = 1; z<m_nNextCol; z++)
    {
        m_wndGrid.SetItemBkColour(nRow,z ,RGB(255,255,255));
        m_wndGrid.SetCellEnabled(nRow,z ,TRUE);
    }
    // Apply a combo box control in the first column of this row for the node
    // combo setting the initial value to the first in the list
    m_wndGrid.CreateListCell(nRow,1, m_nodeArray);
    m_wndGrid.SetItemText(nRow, 1, m_nodeArray[0]);
    // Apply a check box control in the second column of this row for OTA
    // selection setting the initial value as checked
    m_wndGrid.CreateCheckCell(nRow,2);
    m_wndGrid.SetCheckBoxCellState(nRow,2, TRUE);

    // All other controls on the current row are test colums so spin controls
    // are added for setting the number of Users to be simulated on this injector.
    int nStartCol = 3;
    if(!m_bSimple) // In Group test mode we have a load column so start with the next col
    {
        nStartCol = 4;
    }

    for (int i = nStartCol; i<m_nNextCol; i++)
    {
        m_wndGrid.CreateSpinCell(nRow,i);
    }

}

void CECWebExecDlg::AddTestColumn(CString strTest)
{
    CString wizPath = CheckOutWizFile(strTest);
    // Add total users for this test
    int VUsers = GetVUsers(wizPath);
    m_nTotalVUsers+=VUsers;

    // First insert a new column
    int nCol = m_wndGrid.InsertColumn(strTest, DT_CENTER | DT_VCENTER);
    m_wndGrid.RedrawColumn(nCol);
    m_nNextCol++;


    CString strUsers = "";
    strUsers.Format("%d", VUsers);
    m_wndGrid.SetItemText(m_nTotalRow, nCol, strUsers);

    // Add the Spin control
    m_wndGrid.CreateSpinCell(1,nCol);

    for (int i = m_nNextRow; i<=m_nLastDummyRow; i++)
    {
        m_wndGrid.SetItemBkColour(i,m_nNextCol-1,RGB(172,172,172));
        m_wndGrid.SetCellEnabled(i,m_nNextCol-1,FALSE);
    }
}

void CECWebExecDlg::AddLoadColumn()
{

    // First insert a new column
    int nCol = m_wndGrid.InsertColumn("VU/Injector", DT_CENTER | DT_VCENTER);
    m_nNextCol++;

    for (int i = 2; i<=m_nLastDummyRow; i++)
    {
        m_wndGrid.SetItemBkColour(i,nCol, RGB(172,172,172));
    }

}

void CECWebExecDlg::OnPopupInsertRow()
{

    if (m_nNextRow>m_nLastDummyRow)
    {
        CString rowNum = "";
        rowNum.Format("%d", m_nNextRow);
        m_wndGrid.InsertRow(rowNum, m_nNextRow);
        m_nTotalRow++;
        m_nDispatchRow++;
        m_nLastDummyRow++;
    }
    SetupControls(m_nNextRow);
    m_nNextRow++;

    m_wndGrid.Invalidate();

}

void CECWebExecDlg::OnPopupDeleterow()
{
    //remove all the selected rows from the grid
    CCellRange range = m_wndGrid.GetSelectedCellRange();
    int nSelRow = range.GetMaxRow();

    if (nSelRow>10 && nSelRow<=m_nLastDummyRow)
    {
        m_wndGrid.DeleteRow(nSelRow);
        m_nNextRow--;
        m_nDispatchRow--;
        m_nTotalRow--;
        m_nLastDummyRow--;
    }
    else if (nSelRow>=1 && nSelRow<m_nNextRow)
    {
        m_wndGrid.DeleteRow(nSelRow);
        CString rowNum = "";
        rowNum.Format("%d", m_nNextRow-1);
        m_wndGrid.InsertRow(rowNum, m_nNextRow-1);

        for (int j = 0; j<m_nNextCol; j++)
        {
            m_wndGrid.SetItemBkColour(m_nNextRow-1,j, RGB(172,172,172));
            m_wndGrid.SetCellEnabled(m_nNextRow-1,j, FALSE);
        }
        m_nNextRow--;
    }
}

void CECWebExecDlg::OnPopupNewinjector()
{

    CWnd* pMainWnd = AfxGetMainWnd();
    pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT ,(LPARAM)STUDIO_CUSTOM_LINK_3);

    GetInjectorList();

    for (int i = 1; i<m_nNextRow; i++)
    {
        CString Value = m_wndGrid.GetItemText(i,1);
        m_wndGrid.CreateListCell(i,1, m_nodeArray);
        m_wndGrid.SetItemText(i,1, Value);
    }

}

void CECWebExecDlg::OnApply()
{
    CWaitCursor wc;
    m_wndGrid.EnableWindow(FALSE);

    BOOL bValidated = FALSE;
    if (m_bSimple)
    {
        bValidated = ValidateGrid();
        if (bValidated)
        {
            BOOL bCanExecute = WriteTssSimple(m_szTestToExecute, m_UsersPerInterval);
            StoreTestParams(bCanExecute);
        }
        else
        {
            m_wndGrid.EnableWindow(TRUE);
            return;
        }
    }
    else
    {
        bValidated = ValidateGrid();
        if (bValidated)
        {
            BOOL bCanExecute = WriteTssGroup();
            StoreTestParams(bCanExecute);
        }
        else
        {
            m_wndGrid.EnableWindow(TRUE);
            return;
        }

    }

    BOOL bCompiled = Compile();
    m_ExecButton.EnableWindow(bCompiled);
    m_ApplyButton.EnableWindow(FALSE);

    m_wndGrid.EnableWindow(TRUE);
}

BOOL CECWebExecDlg::ValidateGrid()
{

    // Make sure for each injector row that an Injector has been chosen and that it is unique
    CStringArray allInjectors;
    for (int i = 1; i<m_nNextRow; i++)
    {
        CString Value = m_wndGrid.GetItemText(i,1);

        BOOL bFound = FALSE;
        for (int z = 0; z<allInjectors.GetSize(); z++)
        {
            if (Value.Compare(allInjectors[z]) == 0)
            {
                bFound = TRUE;
            }
        }

        if (bFound)
        {
            AfxMessageBox("Please remove duplicate injectors!");
            return FALSE;
        }
        allInjectors.Add(Value);

        if(Value.IsEmpty() || Value == "<No Injectors Defined>")
        {
            AfxMessageBox("Please select an injector!");
            return FALSE;
        }
    }


    // Make sure for each Test column the "To Dispatch" value is zero
    int testColIndex = 4;
    if (m_bSimple)
    {
        testColIndex = 3;
    }
    for (i = testColIndex; i<m_nNextCol; i++)
    {
        CString szToDispatch = m_wndGrid.GetItemText(m_nDispatchRow, i);

        if (atoi(szToDispatch) != 0)
        {
            AfxMessageBox("There are still some Virtual Users left to dispatch.");
            return FALSE;
        }
    }

    return TRUE;
}

void CECWebExecDlg::EditStarted(int nRow, int nCol)
{
    m_ExecButton.EnableWindow(FALSE);
    m_ApplyButton.EnableWindow(TRUE);
}

void CECWebExecDlg::OnChangeInterval()
{
    CWnd* pUsers = GetDlgItem(IDC_USERS);
    UpdateData();
    if(m_nInterval == 0)
    {
        pUsers->EnableWindow(FALSE);
        m_UsersPerInterval = m_nTotalVUsers;
        UpdateData(FALSE);
    }
    else
    {
        if(m_UsersPerInterval == m_nTotalVUsers)
        {
            m_UsersPerInterval =1;
            UpdateData(FALSE);
        }

        pUsers->EnableWindow();
    }

    m_ApplyButton.EnableWindow(TRUE);
    m_ExecButton.EnableWindow(FALSE);
}

void CECWebExecDlg::OnChangeUsers()
{
    CWnd* pInterval = GetDlgItem(IDC_INTERVAL);
    UpdateData();
    if (m_UsersPerInterval > m_nTotalVUsers)
    {
        m_UsersPerInterval = m_nTotalVUsers;
        UpdateData(FALSE);
    }
    if (m_UsersPerInterval == m_nTotalVUsers)
    {
        pInterval->EnableWindow(FALSE);
        m_nInterval = 0;
        UpdateData(FALSE);
    }
    else
    {
        if(m_nInterval == 0)
        {
            m_nInterval =1;
            UpdateData(FALSE);
        }
        pInterval->EnableWindow();
    }

    m_ApplyButton.EnableWindow(TRUE);
    m_ExecButton.EnableWindow(FALSE);
}

int CECWebExecDlg::GetColForTest(CString strTest)
{
    for (int i = 0; i<m_nNextCol; i++)
    {
        CString name = m_wndGrid.GetItemText(0,i);
        if (name == strTest)
        {
            return i;
        }
    }
    return -1;
}

BOOL CECWebExecDlg::WriteTssSimple(CString strTest, int ramp)
{

    CString strTSSName = strTest + ".TSS";
    int nCol = GetColForTest(strTest);

    //  m_pServer->CheckOutFile(CYRServer::enSourceFolder,"",strTSSName);

    CString strFileName = m_pServer->GetFolderPath(CYRServer::enSourceFolder) + strTSSName;
    strFileName.MakeUpper();

    // Now declare a file object
    CStdioFile file;

    // And a writer
    CECTssWriter writer;

    CString strBuffer;
    // And away we go ...
    try
    {
        CStdioFile file(strFileName, CFile::modeCreate | CFile::modeWrite | CFile::typeText);
        // Header Section
        // ==============

        // Formalate the Header Text
        strBuffer = writer.GetTemplateText(CECTssWriter::HEADER, strTest);
        file.WriteString(strBuffer);

        // Environment section
        // ===================

        strBuffer = writer.GetSectionTitle(CECTssWriter::ENVIRONMENT);
        file.WriteString(strBuffer);

        // And add in any addition from all our pages at this point
        strBuffer = writer.GetNameDefinitionText(CECTssWriter::ENVIRONMENT, strTest, strTest);
        file.WriteString(strBuffer);

        strBuffer = writer.GetTemplateText(CECTssWriter::ENVIRONMENT);
        file.WriteString(strBuffer);

        // Definitions section
        // ===================

        strBuffer = writer.GetSectionTitle(CECTssWriter::DEFINITIONS);
        file.WriteString(strBuffer);


        // Constants
        strBuffer.Format("\tConstant NUM-INJ = %d\n", m_nNextRow-1);
        file.WriteString(strBuffer);
        strBuffer.Format("\tConstant RAMP = %d\n", ramp);
        file.WriteString(strBuffer);
        strBuffer.Format("\tConstant WAIT-TIME = %d\n", m_nInterval);
        file.WriteString(strBuffer);

        // Variables
		if (m_bThreadPool)
		{
	        strBuffer = "\tThread\t TEST-TH For Target WEB\n";
		}
		else
		{
	        strBuffer = "\tThread\t TEST-TH For Target WDT\n";
		}
        file.WriteString(strBuffer);

        BOOL bNeedOTA = FALSE;
        for(int row=1; row<m_nNextRow; row++)
        {
            BOOL bOTA = m_wndGrid.GetCheckBoxCellState(row,2);
            if (bOTA)
            {
                bNeedOTA = TRUE;
            }

        }

        if(bNeedOTA && m_bSimple)
        {
            strBuffer = writer.GetThreadStatement(CECTssWriter::DEFINITIONS,
                                                  "MON-TH", 0,0,"MON");
            file.WriteString(strBuffer);
        }

        strBuffer = "\tInteger TO-DO\n";
        file.WriteString(strBuffer);
        strBuffer = "\tInteger INJ\n";
        file.WriteString(strBuffer);
        strBuffer = "\tInteger INJECTOR\n";
        file.WriteString(strBuffer);
        strBuffer = "\tCharacter*16 INJ-NAME[NUM-INJ]\n";
        file.WriteString(strBuffer);
        strBuffer = "\tInteger INJ-TOTAL[NUM-INJ]\n";
        file.WriteString(strBuffer);

        strBuffer = "\t\n";
        file.WriteString(strBuffer);
        strBuffer = "\tInteger THIS-TOTAL\n";
        file.WriteString(strBuffer);
        strBuffer = "\tCharacter*16 THIS-INJ";
        file.WriteString(strBuffer);


        strBuffer = writer.GetTemplateText(CECTssWriter::DEFINITIONS);
        file.WriteString(strBuffer);

        // Code Section
        // ============

        strBuffer = writer.GetSectionTitle(CECTssWriter::CODE);
        file.WriteString(strBuffer);
        // First write the Execute OTA lines
        for(row=1; row<m_nNextRow; row++)
        {
            CString node = m_wndGrid.GetItemText(row,1);
            BOOL bOTA = m_wndGrid.GetCheckBoxCellState(row,2);

            if (bOTA && m_bSimple)
            {
                strBuffer.Format("\tExecute Thread\tMON-TH\tUsing Tof \"%s-MON\", on node \"%s\"\n", node, node);
                file.WriteString(strBuffer);
            }
        }

        for (row = 1; row<m_nNextRow; row++)
        {
            CString total = m_wndGrid.GetItemText(row, nCol);
            CString node = m_wndGrid.GetItemText(row,1);

            int nTotal = atoi(total);
            strBuffer.Format("\tSet INJ-TOTAL[%d] = %d\n", row, nTotal);
            file.WriteString(strBuffer);

            strBuffer.Format("\tSet INJ-NAME[%d] = \"%s\"\n", row, node);
            file.WriteString(strBuffer);

        }

        strBuffer = "\tSet INJ = 1\n";
        file.WriteString(strBuffer);
        strBuffer = "\tSet TO-DO = RAMP\n";
        file.WriteString(strBuffer);

        strBuffer = "\n";
        file.WriteString(strBuffer);

        strBuffer = "\n";
        file.WriteString(strBuffer);
        strBuffer = "WHILE-LOOP:\n";
        file.WriteString(strBuffer);
        strBuffer = "\tIf (INJ <= NUM-INJ) then\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\tSet THIS-INJ = INJ-NAME[INJ]\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\tIf (INJ-TOTAL[INJ] >= TO-DO) then\n";
        file.WriteString(strBuffer);
        strBuffer.Format("\t\t\tExecute Thread TEST-TH Using Script \"%s-TH\", on node THIS-INJ, number TO-DO\n", strTest);
        file.WriteString(strBuffer);
        strBuffer = "\t\t\tSet INJ-TOTAL[INJ] = INJ-TOTAL[INJ] - TO-DO\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\t\tWait WAIT-TIME\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\t\tSet TO-DO = RAMP\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\tElseIf (INJ-TOTAL[INJ] > 0) then\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\t\tSet THIS-TOTAL = INJ-TOTAL[INJ]\n";
        file.WriteString(strBuffer);
        strBuffer.Format("\t\t\tExecute Thread TEST-TH Using Script \"%s-TH\", on node THIS-INJ, number THIS-TOTAL\n", strTest);
        file.WriteString(strBuffer);
        strBuffer = "\t\t\tSet TO-DO = TO-DO - INJ-TOTAL[INJ]\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\t\tSet INJ-TOTAL[INJ] = 0\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\tElse\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\t\tSet INJ = INJ + 1\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\tEndIf\n";
        file.WriteString(strBuffer);
        strBuffer = "\t\tGoto WHILE-LOOP\n";
        file.WriteString(strBuffer);
        strBuffer = "\tEndIf\n";
        file.WriteString(strBuffer);

        strBuffer = writer.GetTemplateText(CECTssWriter::EXIT);
        file.WriteString(strBuffer);

        file.Close();

        // Check the file into OpenSTARep
        m_pServer->CheckInFile(CYRServer::enSourceFolder,"",strTSSName, strTSSName);

        return TRUE;

    }
    catch (CFileException* e)
    {
        e->ReportError();
        e->Delete();
    }

    return FALSE;
}

void CECWebExecDlg::OnExecute()
{
    // Make sure all scripts are compiled (only compiles scripts where the source file
    // is newer than the current object file residing on the local machine).
    if(!Compile())
    {
        m_ExecButton.EnableWindow(FALSE);
        return;
    }

    // Save the name of this test to the regisrty
    CString KeyName = SOLUTIONS_FRAMEWORK_KEY;
    CString ValueName = "LastExec";
    CString Value = m_Project+"|"+m_szTestToExecute;

    CECRegistryTools reg;
    reg.SetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName, Value.GetBuffer(0));


    CDialog::OnOK();
}

BOOL CECWebExecDlg::GetNodeInfo(CString& physNode, CString& userName, CString& password)
{
    // Get the work directory
    CString nodePath = m_pServer->GetFolderPath(CYRServer::enNodesFolder)+m_szNodeFile;

    try
    {
        CStdioFile f(nodePath, CFile::modeRead);
        f.ReadString(physNode);
    }
    catch (CFileException e)
    {}

    return TRUE;
}

void CECWebExecDlg::OnCancel()
{
    CDialog::OnCancel();
}


void CECWebExecDlg::StoreTestParams(BOOL bCompiled)
{

    CString wizPath = CheckOutWizFile(m_szTestToExecute, TRUE);

    UpdateData(FALSE);

    int nInjectors = m_nNextRow - 1;
    CString strInjectors;
    strInjectors.Format("%d", nInjectors);
    WritePrivateProfileString(WIZARD_TEST_EXEC, WIZARD_TEST_EXEC_INJECTOR_COUNT, strInjectors.GetBuffer(0), wizPath.GetBuffer(0));

    CString strInterval;
    strInterval.Format("%d", m_nInterval);
    WritePrivateProfileString(WIZARD_TEST_EXEC, WIZARD_TEST_EXEC_INTERVAL, strInterval.GetBuffer(0), wizPath.GetBuffer(0));

    CString strUsersPerInterval;
    strUsersPerInterval.Format("%d", m_UsersPerInterval);
    WritePrivateProfileString(WIZARD_TEST_EXEC, WIZARD_TEST_EXEC_USERS_PER_INTERVAL, strUsersPerInterval.GetBuffer(0), wizPath.GetBuffer(0));

    for (int i=0; i<nInjectors; i++)
    {
        CString strRow;
        strRow.Format(WIZARD_TEST_EXEC_ROW_FORMAT, i+1);

        CString injectorName = m_wndGrid.GetItemText(i+1,1);
        BOOL bOta = m_wndGrid.GetCheckBoxCellState(i+1,2);
        CString params = "";

        if (m_bSimple)
        {
            CString number = m_wndGrid.GetItemText(i+1,3);
            params.Format("%s,%d,%s",injectorName,(int)bOta,number);
        }
        else
        {
            params.Format("%s,%d",injectorName,(int)bOta);
            for (int z = 4; z<m_nNextCol; z++)
            {
                CString number = m_wndGrid.GetItemText(i+1,z);
                params += "," + number;
            }
        }

        WritePrivateProfileString(WIZARD_TEST_EXEC, strRow.GetBuffer(0), params.GetBuffer(0), wizPath.GetBuffer(0));

    }

    // Check In the updated wizard file
    CString strTestFile(m_szTestToExecute + ".WIZ");
    m_pServer->CheckInFile(CYRServer::enTestsFolder,m_szTestToExecute,strTestFile);


}

BOOL CECWebExecDlg::LoadTestParams()
{

    CString wizPath = CheckOutWizFile(m_szTestToExecute);

    CString strInjectors;
    DWORD dwMaxBufferSize=200;
    GetPrivateProfileString(WIZARD_TEST_EXEC,WIZARD_TEST_EXEC_INJECTOR_COUNT,"0",strInjectors.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,wizPath);
    strInjectors.ReleaseBuffer();

    int nInjectors = atoi(strInjectors);
    if (nInjectors == 0)
        return FALSE;

    CString strInterval;
    GetPrivateProfileString(WIZARD_TEST_EXEC,WIZARD_TEST_EXEC_INTERVAL,"0",strInterval.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,wizPath);
    strInterval.ReleaseBuffer();

    int interval = atoi(strInterval);
    m_nInterval = interval;

    CString strUsersPerInterval;
    GetPrivateProfileString(WIZARD_TEST_EXEC,WIZARD_TEST_EXEC_USERS_PER_INTERVAL,"0",strUsersPerInterval.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,wizPath);
    strUsersPerInterval.ReleaseBuffer();

    int users = atoi(strUsersPerInterval);
    m_UsersPerInterval = users;
    UpdateData(FALSE);


    int nUsersAllocated = 0;

    BOOL bInjectorsOK = TRUE;
    BOOL bModified = FALSE;

    for (int i = 0; i<nInjectors; i++)
    {
        CString strField;
        strField.Format(WIZARD_TEST_EXEC_ROW_FORMAT,i+1);

        CString strParams;
        GetPrivateProfileString(WIZARD_TEST_EXEC,strField,"",strParams.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,wizPath);
        strParams.ReleaseBuffer();

        if (!strParams.IsEmpty())
        {
            if (i!=0)
            {
                OnPopupInsertRow();
            }

            char *szInjector = strtok(strParams.GetBuffer(0),",");
            char* szOTA = strtok(NULL,",");

            if ( NodeExists(szInjector) )
            {
                m_wndGrid.SetItemText(i+1, 1, szInjector);
            }
            else
            {
                bInjectorsOK = FALSE;
            }

            m_wndGrid.SetCheckBoxCellState(i+1, 2, atol(szOTA));

            int nStartTestCol = 3;
            if (!m_bSimple)
            {
                nStartTestCol = 4;
            }

            for (int z = nStartTestCol; z<m_nNextCol; z++)
            {
                CString Number = strtok(NULL,",");

                int nNumber = atoi(Number);
                if (nUsersAllocated >= m_nTotalVUsers)
                {
                    nNumber = 0;
                    bModified = TRUE;
                }
                else if (nNumber + nUsersAllocated > m_nTotalVUsers)
                {
                    nNumber = m_nTotalVUsers - nUsersAllocated;
                    nUsersAllocated = m_nTotalVUsers;
                    bModified = TRUE;
                }
                else
                {
                    nUsersAllocated += nNumber;
                }

                CString strNum = "";
                strNum.Format("%d", nNumber);
                m_wndGrid.SetItemText(i+1, z, strNum);
            }
        }
    }

    if (bModified || nUsersAllocated < m_nTotalVUsers)
        return FALSE;

    if (bInjectorsOK)
        return TRUE;

    return FALSE;
}

void CECWebExecDlg::LoadTestList()
{
    try
    {
        CStringArray arTests;
        m_pServer->GetFolderList(CYRServer::enTestsFolder,"",arTests);
        for(int nNdx=0;nNdx<arTests.GetSize();nNdx++)
        {
            m_TestCombo.AddString(arTests[nNdx]);
        }

        int nPos = m_TestCombo.FindStringExact(0,m_szTestToExecute);

        if (nPos == -1)
        {
            m_TestCombo.SetCurSel(0);
            m_TestCombo.GetLBText(0, m_szTestToExecute);
        }
        else
        {
            m_TestCombo.SetCurSel(nPos);
        }
    }
    catch(...)
    {}

}

void CECWebExecDlg::OnSelchange()
{
    int nSel = m_TestCombo.GetCurSel();
    m_TestCombo.GetLBText(nSel, m_szTestToExecute);
    ResetGrid();
    m_wndGrid.Invalidate();
    Invalidate();
}

BOOL CECWebExecDlg::WriteTssGroup()
{

    CString wizPath = CheckOutWizFile(m_szTestToExecute);
    int nSubTests = GetSubTestCount(wizPath);

    int RAMPED = 0;

    for (int i = 1; i<=nSubTests; i++)
    {
        CString subTest = GetSubTest(wizPath, i);

        int  nCol = GetColForTest(subTest);
        CString strUsers = m_wndGrid.GetItemText(m_nTotalRow, nCol);
        int VUsers = atoi(strUsers);
        int rampVal = VUsers * m_UsersPerInterval / m_nTotalVUsers;
        RAMPED += rampVal;

        if (i == nSubTests && RAMPED != m_UsersPerInterval)
        {
            rampVal = rampVal + (m_UsersPerInterval - RAMPED);
        }

        if (rampVal == 0)
        {
            rampVal =1;
        }

        WriteTssSimple(subTest, rampVal);
    }

    // Write the top-level test
    CString strTSSName = m_szTestToExecute + ".TSS";


    // If the TSS file already exists for this test, check it out and lock it
    m_pServer->CheckOutFile(CYRServer::enSourceFolder,"",strTSSName);

    // Create the path to the local copy of the TSS file
    CString strFileName = m_pServer->GetFolderPath(CYRServer::enSourceFolder) + strTSSName;
    strFileName.MakeUpper();

    // And away we go ...
    CStdioFile file;
    CECTssWriter writer;
    CString strBuffer;
    try
    {
        CStdioFile file(strFileName, CFile::modeCreate | CFile::modeWrite | CFile::typeText);
        // Header Section
        // ==============

        // Formalate the Header Text
        strBuffer = writer.GetTemplateText(CECTssWriter::HEADER, m_szTestToExecute);
        file.WriteString(strBuffer);

        // Environment section
        // ===================

        strBuffer = writer.GetSectionTitle(CECTssWriter::ENVIRONMENT);
        file.WriteString(strBuffer);

        // And add in any addition from all our pages at this point
        strBuffer = writer.GetNameDefinitionText(CECTssWriter::ENVIRONMENT, m_szTestToExecute, m_szTestToExecute);
        file.WriteString(strBuffer);

        strBuffer = writer.GetTemplateText(CECTssWriter::ENVIRONMENT);
        file.WriteString(strBuffer);

        // Definitions section
        // ===================

        strBuffer = writer.GetSectionTitle(CECTssWriter::DEFINITIONS);
        file.WriteString(strBuffer);

        BOOL bNeedOTA = FALSE;
        for(int row=1; row<m_nNextRow; row++)
        {
            CString node = m_wndGrid.GetItemText(row,1);
            BOOL bOTA = m_wndGrid.GetCheckBoxCellState(row,2);
            if (bOTA)
            {
                bNeedOTA = TRUE;
            }
        }
        if(bNeedOTA)
        {
            strBuffer = writer.GetThreadStatement(CECTssWriter::DEFINITIONS,
                                                  "MON-Thread", 0,0,"MON");
            file.WriteString(strBuffer);
        }

        CString wizPath = CheckOutWizFile(m_szTestToExecute);
        int nSubTests = GetSubTestCount(wizPath);
        for (int sub = 1; sub<=nSubTests; sub++)
        {
            CString subTest = GetSubTest(wizPath, sub);
            strBuffer = "\tTest\t"+subTest+"\n";
            file.WriteString(strBuffer);
        }
        file.WriteString("\n");

        // Code Section
        // ============

        strBuffer = writer.GetSectionTitle(CECTssWriter::CODE);
        file.WriteString(strBuffer);

        // First write the Execute OTA lines
        for(row=1; row<m_nNextRow; row++)
        {
            CString node = m_wndGrid.GetItemText(row,1);
            BOOL bOTA = m_wndGrid.GetCheckBoxCellState(row,2);

            if (bOTA && !m_bSimple)
            {
                strBuffer.Format("\tExecute Thread\tMON-THREAD\tUsing Tof \"%s-MON\", on node \"%s\"\n", node, node);
                file.WriteString(strBuffer);
            }
        }

        file.WriteString("\n");
        for (sub = 1; sub<=nSubTests; sub++)
        {
            CString subTest = GetSubTest(wizPath, sub);
            strBuffer = "\tExecute Test\t"+subTest+"\n";
            file.WriteString(strBuffer);
        }

        strBuffer = writer.GetTemplateText(CECTssWriter::EXIT);
        file.WriteString(strBuffer);

        file.Close();

        m_pServer->CheckInFile(CYRServer::enSourceFolder,"",strTSSName, strTSSName);
    }
    catch (CFileException* e)
    {
        e->ReportError();
        e->Delete();
    }

    return FALSE;

}

BOOL CECWebExecDlg::NodeExists(CString nodeName)
{
    for (int i = 0; i<m_nodeArray.GetSize(); i++)
    {
        if (nodeName == m_nodeArray[i])
        {
            return TRUE;
        }
    }

    return FALSE;
}

CString CECWebExecDlg::GetTestToExecute()
{
    return m_szTestToExecute;
}


BOOL CECWebExecDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
		HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

BOOL CECWebExecDlg::Compile()
{
    CString strTSSName = m_szTestToExecute + ".TSS";

    CString strFileName = m_pServer->GetFolderPath(CYRServer::enSourceFolder) + strTSSName;
    strFileName.MakeUpper();

    CECSourceFileUtils SU(m_pServer,strFileName);
    BOOL bCompiled = SU.Compile(TRUE, TRUE);
    if (!bCompiled)
    {
        CString msg;
        msg += "Compilation Failed!\n\n";
        msg += "Please correct all script errors before\n";
        msg += "executing this test.\n";

        MessageBox(msg, "Compilation Error", MB_OK|MB_ICONEXCLAMATION);
    }
    return bCompiled;

}

void CECWebExecDlg::UpdateTotals(BOOL bRedraw)
{
    int testColIndex = 4;
    if (m_bSimple)
    {
        testColIndex = 3;
    }

    for (int i = testColIndex; i<=m_wndGrid.GetColumnCount(); i++)
    {
        int nDespatched = 0;
        for (int j = 1; j<m_nNextRow; j++)
        {
            CString strTemp = m_wndGrid.GetItemText(j,i);
            nDespatched += atoi(strTemp);
        }

        CString strTotal = m_wndGrid.GetItemText(m_nTotalRow,i);
        int nToDespatch = atoi(strTotal) - nDespatched;
        CString strToDesp = "";
        strToDesp.Format("%d",nToDespatch);
        m_wndGrid.SetItemText(m_nDispatchRow,i, strToDesp);
        if (bRedraw)
            m_wndGrid.RedrawCell(m_nDispatchRow, i);

        if (nToDespatch < 0)
        {
            AfxMessageBox("Too many Virtual Users allocated!");
        }
    }

    if (!m_bSimple)
    {
        int nTotalDespatched = 0;
        for (int i = 1; i<m_nNextRow; i++)
        {
            int nDespatched = 0;
            for (int j = testColIndex; j<=m_wndGrid.GetColumnCount(); j++)
            {
                CString strTemp = m_wndGrid.GetItemText(i,j);
                nDespatched += atoi(strTemp);
            }

            CString strDesp = "";
            strDesp.Format("%d",nDespatched);

            m_wndGrid.SetItemText(i,3, strDesp);
            if (bRedraw)
                m_wndGrid.RedrawCell(i,3);
            nTotalDespatched+=nDespatched;
        }

        CString strTemp = "";
        strTemp.Format("%d",m_nTotalVUsers);
        m_wndGrid.SetItemText(m_nTotalRow,3, strTemp);
        if (bRedraw)
            m_wndGrid.RedrawCell(m_nTotalRow,3);

        strTemp.Format("%d",m_nTotalVUsers - nTotalDespatched);
        m_wndGrid.SetItemText(m_nDispatchRow,3, strTemp);
        if (bRedraw)
            m_wndGrid.RedrawCell(m_nDispatchRow,3);
    }
}

void CECWebExecDlg::OnAdvanced() 
{
	CThreadModelDlg dlg(m_bThreadPool);
	int nRet = dlg.DoModal();

	if (nRet == IDCANCEL)
		return;

	m_bThreadPool = dlg.IsUsingThreadPool();

    m_ApplyButton.EnableWindow(TRUE);
    m_ExecButton.EnableWindow(FALSE);
}
