/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRImage.cpp
// ------------
//
//    This is the implementation file for the CYRImage class.
//
//=========================================================================

#include "stdafx.h"
#include "CYRImage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CYRImage

CYRImage::CYRImage(HWND hParent, CString szImage)
{
    m_szImage = szImage;
    m_hParent = hParent;
}

CYRImage::~CYRImage()
{}

/////////////////////////////////////////////////////////////////////////////
// CYRImage message handlers

//
// Run
// ===
//
// Runs the current image as a seperate process
//

BOOL CYRImage::Run()
{
    HINSTANCE                   hInstance;
    char                        szProgPath[256];

    // Retrieve the path to the document
    char szTemp[256];

    CString szTempCString;
    GetModuleFileName(NULL, szTemp, sizeof(szTemp) );

    szTempCString = szTemp;

    CString szPath = szTempCString.Left(szTempCString.ReverseFind('\\') + 1);

    CString szFullPathName = szPath + m_szImage;

    // Retrieve the executable file name for a document of this type
    // ie retrieve the full location of the executable.
    hInstance = FindExecutable(m_szImage.GetBuffer(0), szPath, szProgPath);
    if ((int) hInstance <= 32)
    {
        ReportError((int) hInstance);
        return FALSE;
    }

    // Now run the image
    hInstance = ShellExecute(m_hParent,     // handle to parent window
                             "open",                         // pointer to string that specifies operation to perform
                             m_szImage.GetBuffer(0),         // pointer to filename or folder name string
                             NULL,                           // pointer to string that specifies executable-file parameters
                             szPath.GetBuffer(0),            // pointer to string that specifies default directory
                             SW_SHOWNORMAL);                 // whether file is shown when opened
    if ((int) hInstance <= 32)
    {
        ReportError((int) hInstance);
        return FALSE;
    }

    return TRUE;
}

//
// ReportError
// ===========
//
// Report any errors which may of occurred while trying to find or execute the
// image we have bee asked to invoke.
//
void CYRImage::ReportError(int error)
{
    CString szErrorMsg;

    switch (error)
    {
    case 0:
        {
            szErrorMsg = "The operating system is out of memory or resources";
            break;
        }
    case ERROR_FILE_NOT_FOUND:
        {
            szErrorMsg = "The specified file was not found";
            break;
        }

    case ERROR_PATH_NOT_FOUND:
        {
            szErrorMsg = "The specified path was not found";
            break;
        }
    case ERROR_BAD_FORMAT:
        {
            szErrorMsg = "The .exe file is invalid (non-Win32 .exe or error in .exe image)";
            break;
        }
    case SE_ERR_ACCESSDENIED:
        {
            szErrorMsg = "The operating system denied access to the specified file";
            break;
        }
    case SE_ERR_ASSOCINCOMPLETE:
        {
            szErrorMsg = "The file name association is incomplete or invalid";
            break;
        }
    case SE_ERR_DDEBUSY:
        {
            szErrorMsg = "The DDE transaction could not be completed because other DDE transactions were being processed";
            break;
        }
    case SE_ERR_DDEFAIL:
        {
            szErrorMsg = "The DDE transaction failed";
            break;
        }
    case SE_ERR_DDETIMEOUT:
        {
            szErrorMsg = "The DDE transaction could not be completed because the request timed out";
            break;
        }
    case SE_ERR_DLLNOTFOUND:
        {
            szErrorMsg = "The specified dynamic-link library was not found";
            break;
        }
    case SE_ERR_NOASSOC:
        {
            szErrorMsg = "There is no application associated with the given file name extension";
            break;
        }
    case SE_ERR_OOM:
        {
            szErrorMsg = "There was not enough memory to complete the operation";
            break;
        }
    case SE_ERR_SHARE:
        {
            szErrorMsg = "A sharing violation occurred";
            break;
        }
    }
    AfxMessageBox(szErrorMsg, MB_OK | MB_ICONSTOP);
}

