/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// AtvfRecRule.h
#ifndef _CYR_ATVFRECRULE_H
#define _CYR_ATVFRECRULE_H
/**
 * Virtual class defining the structure of a record rule
 **/
#include <string>
#ifdef HPUX
#include <iostream.h>
#else
#include <iostream>
#endif
#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif
#if defined (WIN32)
#pragma warning(disable:4786)
#endif



// local includes
#include "AtvfKey.h"
#include "AtvfInput.h"

// class definition
class AtvfRecRule
{
    const AtvfKey _key;
#ifndef TH_POOL_VER
    char* _token;
#else
    string _token;
#endif
    const bool _hasValue;
    const bool _leadWSpace;
    int     _number_of_refs;

protected:

    //Constructors & destructor
    AtvfRecRule(const string& token,
                bool hasValue=false,
                bool leadWSpace=false);


public:

#ifndef TH_POOL_VER
    virtual ~AtvfRecRule()
    {
        delete[] _token;
    };
#else
    virtual ~AtvfRecRule()
    {}
    ;
#endif

    // Accessors
    string token() const;
    AtvfKey key() const;
    bool hasValue() const;
    bool leadWSpace() const;
    const int numberOfReferences() const
    {
        return _number_of_refs;
    }


    virtual string type() const = 0;
    virtual void saveValue(const string& value, ostream& out) const;
    virtual string* parseInput(AtvfInput& in) const;

    // Mutators

    void addReference()
    {
        _number_of_refs++;
    }
    void removeReference()
    {
        _number_of_refs--;
    }

}
; // AtvfRecRule



ostream& operator<<(ostream& os, const AtvfRecRule& r);



#endif // _CYR_ATVFRECRULE_H

