/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//AtvfRecord.h
#ifndef _CYR_ATVFRECORD_H
#define _CYR_ATVFRECORD_H
/**
 * Class definition for an AtvfRecord.
 * Although there are different types of AtvfRecords these type
 * differences are only defined by the rule contained within.
 * Invalid records found in files are also saved and these are
 * represented by having no rule, as are empty lines.
 **/
#include <string>
#ifdef HPUX
#include <iostream.h>
#else
#include <iostream>
#endif
#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif
#if defined (WIN32)
#pragma warning(disable:4786)
#endif

// local includes
#include "AtvfRecRule.h"

class AtvfRecord
{
    string* _tokP;
    string* _valP;
    const AtvfRecRule* _ruleP;

public:
    //Constructors & destructors
    AtvfRecord(string* valP = NULL);
    AtvfRecord(const AtvfRecRule* ruleP, string* tokP = NULL);
    ~AtvfRecord();

    //Accessors
    string value() const;
    const AtvfRecRule& rule() const;
    bool isBlank() const;
    bool isKnown() const;
    AtvfKey key() const;
    void save(ostream& out) const;

    //Mutators
    void value(const string& newVal);
    void getValue(AtvfInput& in);

}
; //AtvfRecord

ostream& operator<<(ostream& os, const AtvfRecord& record);

#endif // _CYR_ATVFRECORD_H
