/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif


#include "TPLog.h"
#ifdef HPUX
#include <unistd.h>
#include <strstream.h>
#include <fstream.h>
#else
#include <strstream>
#include <fstream>
#endif
#include <string>
#include <mutex.hpp>
#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif

const char* STARTUP="Startup";//Startup trace messages
const char* SHUTDOWN="Shutdown";//shutdown trace messages
const char* SCRIPTCOMMAND="ScriptCommand";//script command
const char* DBGSTD="Debug-Generic";//non-specific debug message
const char* AUDITFAIL="Failed Audit Message";//message failed to get to Audit type logging
const char* NOTE="Note";//Note message
const char* USERTRACE="User Trace";//Trace Message
const char* AUDIT="Audit";//Copy of Audit message
const char* REPOSITORY="Repository";//Repository Debug messages
const char* THREADING="Threading";//Threading messages
const int   ERRORLEVEL=1;//Important Trace error
const int   WARNINGLEVEL=10;
const int   DEBUGLEVEL=50;//Sort of mid range level
const int   DETAILLEVEL=500;//Very specific Trace messages
const int   AUDITLEVEL=10;//Level at which Audit logs are traced
const int   TRACELEVEL=3;//Tracing level
const int   NOTELEVEL=4;//Level at which Notes should be logged
TPLog*	    logr= NULL;

#ifdef __DLLSA__
HWND gHwnd = NULL;
#endif

#ifndef CYRANO_TP_STANDALONE
ofstream Errlog;
CTMutex LogMtx;
TPLog::TPLog()
{
    m_Log=NULL;
}
TPLog::~TPLog()
{
    if(!m_TraceList.empty())
    {
        Errlog.open("TProcess.log",ios::out);
        for(TraceList_t::iterator i=m_TraceList.begin();i!=m_TraceList.end();i++)
        {
            Errlog<<(*i).category<<","<<(*i).level<<","<<(*i).msg<<endl;
        }
        Errlog.close();
    }
}
void TPLog::Setup( CORBAServer* pServer, const char* name, long level )
{
#ifndef CYRANO_TP_STANDALONE
    m_Log=new CECLog(pServer, name,level);
    //
    if(!m_TraceList.empty())
    {
        for(TraceList_t::iterator i=m_TraceList.begin();i!=m_TraceList.end();i++)
        {
            AddTrace((*i).category.c_str(),(*i).level,(*i).msg.c_str());
            m_TraceList.pop_front();
        }
    }
#else
    m_level=level;
#endif
}
void TPLog::AddTrace( const char* category, long level, const char* msg )
{
#ifndef CYRANO_TP_STANDALONE
    CTLockGuard<CTMutex> guard(LogMtx);
    if(m_Log)
    {
        m_Log->AddTrace(category,level,msg);
    }
    else
    {
        TraceMsg t(category,level,msg);
        m_TraceList.push_back(t);
    }
#endif
}
void TPLog::Store( CECOrbFolder* pFolder )
{
#ifndef CYRANO_TP_STANDALONE
    m_Log->Store(pFolder);
    delete m_Log;
    m_Log=NULL;
#endif
}
void TPLog::SetLevel( const char* category, long level)
{
#ifndef CYRANO_TP_STANDALONE
    m_Log->SetLevel(category,level);
#endif
}

#endif
