/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _TEThread_H_
#define _TEThread_H_
#ifndef CYRANO_TP_STANDALONE
#include "TestExecuter.h"
#else
#include <assert.h>
#endif
#include "mdbthread.h"
#include "tofprmdef.h"
class TEThread;
template <class T> class Threadcontrol;
typedef Threadcontrol<TEThread> TEthreadcontrol;
class TEThread:public Cmdbthread
{
public:
    TEThread(TEthreadcontrol* threadcontrol,string filename);
    virtual ~TEThread();
    // methods related to functionality in TExecuterRcrds's
    virtual int ExitCommand(bool keepalive,const string& retvar);
    int Entry(TtValList parameters);
    virtual int CallScript(const string& name,TtValList params);//Call another script in this Context
    string  GetLastError()
    {
        return m_last_error;
    }
    ;//Get the last error string
    //////////////////////////////////////////////////////
    virtual void    StartScript(const string& scriptname);
    virtual void    EndScript();
    ///////////////////////////////////
#ifndef CYRANO_TP_STANDALONE
    virtual void SetThreadInfo(const string& TestName,const ThreadDef & thrdef);
#endif
    virtual void CleanUp();
    virtual void SetNoteText(const string& mesg);
#ifndef CYRANO_TP_STANDALONE
    virtual ThreadDef GetThreadDef();
    inline long GetTestCaseResults(TestCaseStatus  Status)
    {
        if(Status==PASSED)
            return m_TestCasePass;
        else
            return m_TestCaseFail;
    }
#endif
    virtual string get_test_name(void);
    virtual int TCterminate(int result,string desc);

protected:
    virtual void Initialise();
    virtual void LogStartMessage();
    virtual void LogEndMessage();

#ifndef CYRANO_TP_STANDALONE    // No parent for TestExecuter
    ThreadDef               m_ThreadDef;
#endif
    bool                have_cleaned_up;//should only go to CleanUp once but must go at least once
    string              m_TestName;
    string              m_NoteText;
    long                m_TestCasePass;
    long                m_TestCaseFail;
    list<string>            m_Params;//Current parameter list (passing by value only so stack not required)
    TtValList*              m_ParamsInfo;//ParameterListInfo
    TEthreadcontrol*            m_tethreadcontrol;//saved pointer threadcontrol to save on dynamic casting but is duplicate of m_threadcontrol in base class
    static CTMutex             m_ThreadMap_mutex;   // @cmember ThreadMap modifier mutex
    static CTMutex              m_cache_mutex;
    string                  m_last_error;

};
#endif
