/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     TestExecuter
///////////////////////////////////////////////////////////////////////////////

#ifndef _TESTEXECUTER_I_H_
#define _TESTEXECUTER_I_H_


#include <string>
#include <list>

#ifndef CYRANO_TP_STANDALONE
#   include <TestExecuter.h>
#include <OSTAORB/OSTAORB.h>
#else
#	include <CECSACORBA.h>
#endif

class TScriptRule;

#ifndef TH_POOL_VER
#include "TEthreadcontrol.h"
#endif

#ifdef WIN32
#   pragma warning(disable:4786)
#   pragma warning(disable:4503)
#endif

#ifndef CYR_NO_STD_NAMESPACE
using std::string;
using std::list;
#endif

#ifndef TH_POOL_VER
extern CTSemaphore blocking_semaphore;
#endif

// @class CyranoManager
// @base public | <c _sk_TestExecuter>
class TestExecuterImpl
#ifndef CYRANO_TP_STANDALONE
            : public POA_TestExecuter, public PortableServer::RefCountServantBase
#endif
{
public:
#ifndef TH_POOL_VER
    TestExecuterImpl(CORBAServer* pserver,string &regname,string node,int pid,string exename,TScriptRule* rule,int max_threads);
#else
    TestExecuterImpl(CORBAServer* pserver,string &regname,string node,int pid,string exename,TScriptRule* rule);
#endif
    virtual ~TestExecuterImpl();
    virtual CORBA::Long  Initialise(const char*  node, const char*  user, const char*  password, const char*  TestRunString);

    virtual CORBA::Long  ExecuteThread( const char*  ProjectName
                                        ,const char*  ScriptName
#ifndef CYRANO_TP_STANDALONE
                                        ,TestLevel_ptr & parent
                                        ,const ThreadDef & thrdef
#endif
                                      );
    virtual void Stop();
    virtual void Kill();

#ifndef CYRANO_TP_STANDALONE
    virtual char*  GetNode();
    virtual char*  GetNodeName();
    virtual char*  Name();
    virtual char*  GetThreadIdents();
    virtual CORBA::ULong  GetNoThreads(char* & ScriptName);
    virtual CORBA::ULong  GetNoActiveThreads();
    virtual ThreadStatus  GetStatus(char* & ident);
    virtual char*  GetTarget();
    virtual char*  TestName();
    virtual CORBA::Boolean  Suspend(char* & ident);
    virtual CORBA::Boolean  Activate(char* & ident);
    virtual CORBA::Boolean  Detatch(char* & ident);
    virtual char*  ScriptName(char* & ident);
    virtual CORBA::ULong  StartTime(char* & ident);
    virtual char*  GetNoteText(char* & ident);
    virtual CORBA::Long  GetTestCaseResults(TestCaseStatus  Status);
    virtual CORBA::ULong  GetThreadInformation(ThreadInfoList & TList);
#endif
    void SetToCloseDown()//Set flag to say closing down is imminent
    {
        m_closing_down=true;
        CloseDown();
    }
    bool ReportOnThreads();//reports while running
    // @access Protected Members
protected:


private:
    void CloseDown();

    string			m_TestRunString;
    string			m_nodename;
    int			    m_pid;
    string			m_exename;
    CORBAServer*	m_pServer;
    string			m_RegisterName;
#ifndef TH_POOL_VER
    TEthreadcontrol*m_threadcontrol;
#endif
#ifndef CYRANO_TP_STANDALONE
    ThreadDef		m_ThreadDef;
    TestLevel_var	m_ParentTestLevel;
#endif
    bool			m_closing_down;//set when this object believes it is closing down
}
;

#endif
