/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc
///////////////////////////////////////////////////////////////////////////////

#ifndef _OtaQuery_h
#define _OtaQuery_h

#include <OtaQueryHandler.h>
#include <OtaDetail.h>
#include <CyrWrapLib/CyrWrapLib.h>
#include "OtaTarget.h"

///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <string>

#ifndef VMS
using std::string;
#endif

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

///////////////////////////////////////////////////////////////////////////////
#include <ctime.hpp>

///////////////////////////////////////////////////////////////////////////////
// @class   OtaQuery | This class contains details about a single OTA query.
//          Such details include the executor that handles the query, the query
//          itself, the interval between queries and the actual results.
class OtaQuery
{
    // @access Public Members
public:
    static  const char * PARAM_DEBUG;
    static  const char * PARAM_DETAIL;
    // @cmember Constructor
    OtaQuery( CECOrbTestRun* pTestRun, string query, string nodename, string desc,
              int interval, bool delta, bool detail, int querytype,
              OtaTarget* pTarget, OtaQueryHandler* pHandler );
    virtual ~OtaQuery();                    // @cmember Destructor

    string GetQuery();                      // @cmember Returns query string
    string GetDescription();                // @cmember Returns query description
    OtaQueryHandler* GetHandler();          // @cmember Returns handler reference
    OtaTarget* GetTarget();                 // @cmember Returns target reference
    long GetInterval();                     // @cmember Returns interval between queries
    CTTime GetLastTimestamp();              // @cmember Returns last time query executed
    bool IsDifferenceQuery();               // @cmember True if a difference query
    bool IsDetailQuery();                   // @cmember True if a detail query
    void AddResultHeader(CECOrbTestRun* pTestRun, string nodename, string desc );
    void AddResult(CECOrbTestRun* pTestRun, long start, long stop, long result );
    void AddResult( long start, long stop, const char* result );

    bool IsActive();
    void SetActive( bool active );
    void Disable();
    bool IsDisabled();

    void StoreResults( CECOrbTestRun* pTestRun );
    void Log();

    // @access Private Members
private:
    CECOtaSimpleQuery* m_sQuery;        // @cmember Contains simple query information
    CECOtaHeader* m_sHeader;        // @cmember Contains simple query information
    OtaDetailQuery* m_dQuery;           // @cmember Contains detailed query information

    string  m_query;            // @cmember Query text
    string  m_node;             // @cmember Node Name
    string  m_desc;             // @cmember Description text
    int     m_interval;         // @cmember Interval between queries
    bool    m_delta;            // @cmember True if query stores deltas
    bool    m_detail;           // @cmember True if detail query
    int     m_type;             // @cmember Query type
    int     m_index;            // @cmember Query index
    int     m_lastResult;       // @cmember Last result
    bool    m_savedLastResult;  // @cmember Saves last result

    OtaQueryHandler*   m_pHandler;   // @cmember Reference to query handler
    OtaTarget*         m_pTarget;    // @cmember Reference to target
    bool    m_active;       // @cmember True if query is active
    bool    m_disabled;     // @cmember True if query has been disabled
    CTTime m_time;         // @cmember Time last query was executed
}
;

#endif
