/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __ASYN_FILE_IO_HANDLER_HPP__
#define __ASYN_FILE_IO_HANDLER_HPP__

#include "AsyncFile.hpp"
#include "Trace.hpp"
#include "iocp.hpp"

class CFileManager;

class CAsyncFileIOHandler : public IocpEvent
{
public:
    CAsyncFileIOHandler(CFileManager *pFileMgr, CIOCP &iocp, CTrace *pTrace);
    ~CAsyncFileIOHandler() {};

    // Callback methods.
    virtual void OnEvent      (DWORD dwNumberOfBytesTransferred, BOOL bIsIo, DWORD dwEventType);
    virtual void OnEventFailed(DWORD dwNumberOfBytesTransferred, BOOL bIsIo, DWORD dwEventType, DWORD dwError);

    // User methods.
    DWORD Open(LPCSTR zFilePath, DWORD dwDesiredAccess, DWORD dwOptions);
    DWORD Close();

    // TODO: remplacer zBuffer par un std::string ? (assurerait la libration de la mmoire mme en cas d'erreur).
    DWORD Read(LPSTR zBuffer, DWORD dwBufferLength);
    DWORD Write(const std::string &zBuffer);

    void Rewind();
    BOOL  IsOpen();

    DWORD GetResults(char *&ppBuffer)   const;
    DWORD FileControl(int operation);

protected:
    // Helper methods.
    BOOL ProcessReadData(DWORD dwNumberOfBytesRead);

protected:
    CAsyncFile          m_file;
    CTrace *            m_pTrace;
    CFileManager *      m_pFileMgr; // Object we have to signal when the IO is over.

    DWORD   m_dwOffset;
    DWORD   m_dwBytesToTransfer;    // Number of bytes that should be written or read.

    DWORD   m_dwLastError;          // Error code of the last read / write op.
    char *  m_sReadBuffer;          // Buffer the read data is stored into. Allocated and freed by the calling thread.
    std::string m_zSendBuffer;          // Buffer storing the data that must be sent.

    enum { SKIP_DATA_BUFF_SIZE = 512 };
    char    m_acSkipData[SKIP_DATA_BUFF_SIZE];

    DWORD   m_dwDesiredAccess;      // Store the access mode for access checkings.
    DWORD   m_dwOptions;            // Options like binary ...

    enum { READING_DATA, SKIPPING_DATA }    m_eReadPhase;

private:
    // Forbidden.
    void operator= (const CAsyncFileIOHandler &);
};

#endif
