/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _OPENSTA_PERFCOUNTERMAP_H_
#define _OPENSTA_PERFCOUNTERMAP_H_

#include "PerfCounterKeys.hpp"

BEGIN_PERFORMANCE(OpenSTAPerf)

    PERF_OBJECT_NO_INSTANCE(PRFOBJ_OPENSTA_THREADPOOL,
        L"OpenSTA Thread Pool",
        L"The OpenSTA Thread Pool object type includes the counters that apply to the Test Executer's thread pool.",
        PERF_DETAIL_NOVICE,
        PRFCNT_OPENSTA_THREADPOOL_IOCPQUEUELENGTH)

        PERF_COUNTER(PRFCNT_OPENSTA_THREADPOOL_THREADS,
            L"Threads",
            L"The number of threads currently in the pool.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            0)

        PERF_COUNTER(PRFCNT_OPENSTA_THREADPOOL_IOCPQUEUELENGTH,
            L"Completion queue length",
            L"The number of entries in the main I/O completion port's completion queue.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            0)

        PERF_COUNTER(PRFCNT_OPENSTA_THREADPOOL_ACTIVEUSERS,
            L"Active virtual users",
            L"The number of active virtual users.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            -1)

        PERF_COUNTER(PRFCNT_OPENSTA_THREADPOOL_QUERYRATE,
            L"Queries / sec",
            L"The number of HTTP queries generated per second.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_COUNTER,
            -1)

        PERF_COUNTER(PRFCNT_OPENSTA_THREADPOOL_PENDINGQUERIES,
            L"Pending queries",
            L"The number of pending queries.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            -1)

        PERF_COUNTER(PRFCNT_OPENSTA_THREADPOOL_ACTIVETHREADS,
            L"Active Threads",
            L"The number of pool threads not currently waiting on the IOCP's completion queue.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            0)

        PERF_COUNTER(PRFCNT_OPENSTA_THREADPOOL_TIMERQUEUELENGTH,
            L"Timer Queue length",
            L"The number of entries in the timer queue.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            -1)

        PERF_COUNTER(PRFCNT_OPENSTA_RECV_BUFF_ALLOCATED_MEM,
            L"Allocated data buffers (bytes)",
            L"Whole memory allocated for buffers storage.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            -6)

        PERF_COUNTER(PRFCNT_OPENSTA_RECV_BUFF_USED_MEM,
            L"Used buffers memory (bytes)",
            L"Memory used or wasted by used buffers.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            -6)

        PERF_COUNTER(PRFCNT_OPENSTA_DATA_STORED,
            L"Buffers memory (bytes)",
            L"Amount of bytes stored in memory buffers.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            -5)

        PERF_COUNTER(PRFCNT_OPENSTA_INCOMING_THROUGHPUT,
            L"Received data bytes/sec",
            L"Data received per second.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_COUNTER,
            -5)

        PERF_COUNTER(PRFCNT_OPENSTA_OUTGOING_THROUGHPUT,
            L"Sent data bytes/sec",
            L"Data bytes sent per second.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_COUNTER,
            -4)

        PERF_COUNTER(PRFCNT_OPENSTA_CONNECTIONS_PER_SECOND,
            L"Connections / sec",
            L"Total number of connections per second.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_COUNTER,
            -1)

        PERF_COUNTER(PRFCNT_OPENSTA_SSL_ALLOCATED_MEM_BUFFS,
            L"Allocated SSL Buffers",
            L"Total number of SSL buffers allocated.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            -2)

        PERF_COUNTER(PRFCNT_OPENSTA_SSL_USED_MEM_BUFFS,
            L"Used SSL Buffers",
            L"Number of SSL buffers used at current time.",
            PERF_DETAIL_NOVICE, PERF_COUNTER_RAWCOUNT,
            -2)

END_PERFORMANCE

#endif  // _OPENSTA_PERFCOUNTERMAP_H_
