/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __TOOLS_H__
#define __TOOLS_H__

// Performs a case-insensitive search of item in str, ASSUMING item is LOWER CASE.
// In case 'item' does not exist, returns ULONG_MAX.
inline DWORD strinfind(LPCSTR str, DWORD dwStrLen, LPCSTR item, DWORD dwItemLen)
{
	DWORD dwCurs = 0;
    char c;

	for(DWORD i=0 ; dwItemLen-dwCurs <= dwStrLen-i && dwCurs < dwItemLen ; i++)
	{
        c = str[i];
        if(c<='Z' && c>='A')
            c += 'a'-'A';

		if(c == item[dwCurs])
		{
			dwCurs ++;
		}
		else
		{
			i -= dwCurs;
			dwCurs = 0;
		}
	}

	if(dwCurs == dwItemLen)
        return i-dwCurs;
    else
        return ULONG_MAX;
}

// Performs a case-insensitive search of item in str, NOT assuming item is LOWER CASE.
inline DWORD striinfind(LPCSTR str, DWORD dwStrLen, LPCSTR item, DWORD dwItemLen)
{
	DWORD dwCurs = 0;
    char c1, c2;

	for(DWORD i=0 ; dwItemLen-dwCurs <= dwStrLen-i && dwCurs < dwItemLen ; i++)
	{
        c1 = str[i];
        if(c1<='Z' && c1>='A')
            c1 += 'a'-'A';
        c2 = item[dwCurs];
        if(c2<='Z' && c2>='A')
            c2 += 'a'-'A';

		if(c1 == c2)
		{
			dwCurs ++;
		}
		else
		{
			i -= dwCurs;
			dwCurs = 0;
		}
	}

	if(dwCurs == dwItemLen)
        return i-dwCurs;
    else
        return ULONG_MAX;
}

inline BOOL IsWS(char c)
{
    return c == ' ' || c == '\t';
}

inline BOOL IsEOL(CHAR c)
{
    return c == '\r' || c == '\n';
}

inline BOOL IsDigit(char c)
{
    return c > '/' && c < ':';
}

inline BOOL IsAlpha(char c)
{
    return (c > '@' && c < '[') || (c > '`' && c < '{');
}

#endif
