/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     TManagerRcrd.h
///////////////////////////////////////////////////////////////////////////////
#ifndef CYR_TMANAGERRECORDS
#define CYR_TMANAGERRECORDS

#ifdef WIN32
#pragma warning(disable:4786)
#endif

// *****************************************************************************
// * NEW TEST CONTROL INSTRUCTIONS
// *****************************************************************************
//
// TRcrdExecuteTest
class TRcrdExecuteTest : public TRecord
{

public:
    static const AtvfKey KEY;

    // Constructors & Destructor
TRcrdExecuteTest(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}

    // Control methods

    virtual int execute(TContext& ctxt);

    virtual TRecord* duplicate() const
    {
        return new TRcrdExecuteTest(*this);
    };

private:

    static const bool _registeredCreator;
    static const bool _RegisterCreator();

}
; // TRcrdExecuteTest

// TRcrdExecuteThread
class TRcrdExecuteThread : public TRecord
{

public:
    static const AtvfKey KEY;

    // Constructors & Destructor
TRcrdExecuteThread(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}

    // Control methods

    virtual int execute(TContext& ctxt);

    virtual TRecord* duplicate() const
    {
        return new TRcrdExecuteThread(*this);
    };

private:

    static const bool _registeredCreator;
    static const bool _RegisterCreator();

}
; // TRcrdExecuteThread
//
// TRcrdWtest
class TRcrdWtest : public TRecord
{

public:
    static const AtvfKey KEY;

    // Constructors & Destructor
TRcrdWtest(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}

    // Control methods

    virtual int execute(TContext& ctxt);
    virtual int prepare(TScript* script, int objIndex, bool verbose=false);

    virtual TRecord* duplicate() const
    {
        return new TRcrdWtest(*this);
    };

protected:
    int     _labelIndex;        // Destination label index in script
    vector<string>  NamesToWaitFor;

private:

    static CTMutex m_mutex; // used to allow safe writes during run-phase
    static const bool _registeredCreator;
    static const bool _RegisterCreator();

}
; // TRcrdWtest


// *****************************************************************************
// * TEST CONTROL INSTRUCTIONS FOR THREAD DEFINITION
// *****************************************************************************
class ThrDefRcrd : public TRecord
{

public:
    static const AtvfKey KEY;

    // Constructors & Destructor
ThrDefRcrd(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}

    // Control methods

    virtual int prepare(TScript* script, int objIndex, bool verbose=false);
    virtual TRecord* duplicate() const
    {
        return new ThrDefRcrd(*this);
    };

private:
    static const bool _registeredCreator;
    static const bool _RegisterCreator();

}
; // ThrDefRcrd


#endif // CYR_TCMNRECORDS
