/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <string.h>
#include "Tof2SclParser.h"
#include "SclWriter.h"

char szSCLFilename[_MAX_PATH] = {NULL};
char szHeadersIniPath[MAX_PATH] = {NULL};
bool bDebugOn =false;
SCLWriter *pSclWriter = NULL;

#ifdef TOF2SCL_EXPORTS
#include "Tof2Scl.h"

TOF2SCL_API void SetHeadersIniPath(const char* szPath)
{
    strcpy(szHeadersIniPath,szPath);
}

TOF2SCL_API int ParseTof(const char* szTOFFilename)
{
    //check if tof file exists
    if((_access(szTOFFilename, 0 )) == -1 )
        return 2;

    //If the tofs extension is .tof convert it to .scl for the scl file
    //otherwise just add .scl
    strcpy(szSCLFilename,szTOFFilename);
    int nLen = strlen(szTOFFilename);
    if(nLen>=4 && !stricmp(&szTOFFilename[nLen-4],".tof"))
    {
        strcpy(&szSCLFilename[nLen-4],".SCL");
    }
    else
    {
        strcat(szSCLFilename,".SCL");
    }

    pSclWriter = new SCLWriter;

    Tof2SclParser parser(szTOFFilename);
    parser.Parse();

    delete pSclWriter;
    pSclWriter = NULL;
    return 1;
}

#else

int main(int argc, char** argv)
{
	char szTOFFilename[_MAX_PATH] = {NULL};
    for(int nArg=1;nArg<argc;nArg++)
    {
        if(strcmp(argv[nArg],"/i")==0 && ++nArg<argc)
        {
            strcpy(szTOFFilename,argv[nArg]);
        }
        else
            if(strcmp(argv[nArg],"/d")==0)
            {
                bDebugOn=true;
            }
    }
    
    if(szTOFFilename[0] == NULL)
    {
        printf("\nUsage: Tof2Scl /i <Tof filename>\n\n");
        return -1;
    }

    //check if tof file exists
    if((_access(szTOFFilename, 0 )) == -1 )
    {
        printf("\nCould not find \"%s\"",szTOFFilename);
        return -2;
    }

    //If the tofs extension is .tof convert it to .scl for the scl file
    //otherwise just add .scl
    strcpy(szSCLFilename,szTOFFilename);
    int nLen = strlen(szTOFFilename);
    if(nLen>=4 && !stricmp(&szTOFFilename[nLen-4],".tof"))
    {
        strcpy(&szSCLFilename[nLen-4],".SCL");
    }
    else
    {
        strcat(szSCLFilename,".SCL");
    }

    pSclWriter = new SCLWriter;

    Tof2SclParser parser(szTOFFilename);
	parser.Parse();

    delete  pSclWriter;
    return 0;
}

#endif
