#ifndef PSL__SCPDEF_LOADED
#define PSL__SCPDEF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*                  **** L G N ****                  */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: SCPDEF                  */
/*          ============                         */
/*                                       */
/*  Description:                                 */
/*  ============                                 */
/*                                       */
/*  This module contains a macro, SCPDEF, which defines the fields       */
/*  held in the SCP file header block.                   */
/*                                       */
/*                                       */
/*===========================================================================*/

/*** MODULE SCPDEF ***/

/*    The macro SCPDEF defines the layout of the Script Object File Header */
/*    Record. */

struct scph
{

    /*    This field defines the type of file. It should contain the ascii text */
    /*    'SCP ' */

    char                    scph_ftyp_c4[4];

    /*    This field defines the version of the SCP file */
    /*    NOTE THAT WE ALSO HAVE A COMPILATION VERSION WHICH MUST BE KEPT UP-TO-DATE */

    PSL_LONGWORD_T          scph_fvrn_l;
#define            SCP_FVRN 0X52


    /*    This field defines the date and time at which the SCP was created i.e. */
    /*    the time at which the script was compiled. */

    PSL_U_LONGWORD_T        scph_ctim_q[2];

    /*    This field defines the dat and time at which this SCP was inserted into */
    /*    the script library. */

    PSL_U_LONGWORD_T        scph_ltim_q[2];

    /*    This field defines the total length of the SCP file in bytes. */


    /*    Please maintain the order and position of the above fields as the SLB */
    /*    checking routines need them to be in the same place as those in the */
    /*    TLODEF macro. */

    PSL_LONGWORD_T          scph_scpl_l;

    /*    This field defines the name of the script. */

    char                    scph_snam_c16[16];

    /*    This field contains up to 50 characters of descriptive text for the */
    /*    script. */

    char                    scph_sdsc_c50[50];

    /*    This field defines the type of the terminal defined for this script. It */
    /*    not currently used. */

    char                    scph_ttyp_b;

    /*    This field defines the baud rate of the terminal defined for this script. */
    /*    It is not currently used. */

    char                    scph_tspd_b;

    /*    This field defines the size in bytes of the local data area which this */
    /*    script requires. It includes the pre-allocated data area for script */
    /*    parameter passing. */

    PSL_LONGWORD_T          scph_localsiz_l;

    /*    This field defines the size in bytes of the script data area which this */
    /*    script requires. */

    PSL_LONGWORD_T          scph_sdasiz_l;

    /*    This field defines the size in bytes of the global data area which this */
    /*    script requires. */

    PSL_LONGWORD_T          scph_gdasiz_l;

    /*    This field defines the offset of the timer list within the script. */

    PSL_LONGWORD_T          scph_timoff_l;

    /*    This field defines the offset of the global variables list */

    PSL_LONGWORD_T          scph_gbloff_l;

    /*    This field defines the length in bytes of the code section of the script. */

    PSL_LONGWORD_T          scph_codlen_l;

    /*    This field defines the offset of the code section of the script. */

    PSL_LONGWORD_T          scph_codoff_l;

    /*    This field, when set, defines that this script is marked for deletion */
    /*    from the library and must therefore not be used. */

    PSL_LONGWORD_T          scph_delp_l;

    /*    This field defines the initial input delay for input characters fed by */
    /*    this script. */

    PSL_WORD_T              scph_inpdly_w;

    /*    This field defines the mode of operation of the script. */

    char                    scph_mode_b;

    /*    If the MODE field is set to this value then a screen image will be */
    /*    maintained. */

#define            MODE_SCREEN 0X1
#define            MODE_DEBUG 0X2
#define            MODE_CHR_CELL 0X4
#define            MODE_IMPACT 0X8
#define            MODE_TEST 0X10

    /*    This field is undefined and is reserved for future expansion. */

    char                    scph_spr4_b;

    /*    This field defines the username of the owner of the script. It will */
    /*    be filled in by SLB to the username of the user who adds the script to */
    /*    the script library. */

    char                    scph_ownusrnm_c12[12];

    /*    This field defines the protection mask to be set for the script. It will */
    /*    be filled in by SLB to the default protection mask of user who adds the */
    /*    script to the script library unless over-ridden by the /PROTECTION */
    /*    qualifier. */

    PSL_LONGWORD_T          scph_sprot_l;
#define            SPROT_SYSTEM_M 0X1
#define            SPROT_SYSTEM_V 0
#define            SPROT_OWNER_M 0X2
#define            SPROT_OWNER_V 1
#define            SPROT_GROUP_M 0X4
#define            SPROT_GROUP_V 2
#define            SPROT_WORLD_M 0X8
#define            SPROT_WORLD_V 3

#define            SPTP_RED_M 1
#define            SPTP_WRT_M 2
#define            SPTP_EXE_M 4
#define            SPTP_DEL_M 8
#define            SPTP_RED_V 0
#define            SPTP_WRT_V 1
#define            SPTP_EXE_V 2
#define            SPTP_DEL_V 3

    /*    This field defines the UIC of the owner of the script. It will */
    /*    be filled in by SLB to the UIC of the user who adds the script to */
    /*    the script library. */

    PSL_LONGWORD_T          scph_ownuic_l;

    /*    This field defines a unique pattern string which identifies this file */
    /*    as being an SCP file. */

    PSL_LONGWORD_T          scph_fptn_l;
#define            SCP_FPN 0XFFFFFFFF

    /*    This field defines the length of the script name. */

    PSL_U_LONGWORD_T        scph_snamlen_l;

    /*    This field defines the length of the script description. */

    PSL_U_LONGWORD_T        scph_sdsclen_l;

    /*    This field defines the length of the script owner username length */

    PSL_U_LONGWORD_T        scph_ownnamlen_l;

    /*    This field defines the offset of the start of the script file names */
    /*    section of the SCP. It is used for debugging and is only present if the */
    /*    /DEBUG qualifier was present during compilation. */

    PSL_U_LONGWORD_T        scph_fil_symtbl_off_l;
    PSL_U_LONGWORD_T        scph_fil_symtbl_size_l;

    /*    This field maintains the offset of the MODULES symbol table added to */
    /*    the SCP code which is used during debugging */

    PSL_U_LONGWORD_T        scph_mod_symtbl_off_l;
    PSL_U_LONGWORD_T        scph_mod_symtbl_size_l;

    /*    This field maintains the offset of the LABELS symbol table added to */
    /*    the SCP code which is used during debugging */

    PSL_U_LONGWORD_T        scph_lab_symtbl_off_l;
    PSL_U_LONGWORD_T        scph_lab_symtbl_size_l;

    /*    This field maintains the offset of the Variable symbol table added to */
    /*    the SCP code which is used during debugging */

    PSL_U_LONGWORD_T        scph_var_symtbl_off_l;
    PSL_U_LONGWORD_T        scph_var_symtbl_size_l;

    /*    This field defines the sub-version number of this script. */

    PSL_LONGWORD_T          scph_subvrn_l;
#define            SCP_SVRN 0X2

    /*    This field defines whether or not the Script Protection was defauled */
    /*    or not.  If so, the protection comes from the "$DEFAULT" Test Definition. */

    Boolean                 scph_prot_defaulted_n;

    /*    The following field is spare and reserved for future expansion */

    char                    scph_spare1_c48[48];

    /*    This field defines the maximum number of lines which should be allocated */
    /*    for the screen image. It is set from the value specified (or defaulted) */
    /*    for the MODE statement. */

    PSL_WORD_T              scph_sline_w;

    /*    This field defines the maximum number of columns which should be allocated */
    /*    for the screen image. It is set from the value specified (or defaulted) */
    /*    for the MODE statement. */

    PSL_WORD_T              scph_scols_w;

    /*    This field defines the offset of the Script Parameter Block. */

    PSL_LONGWORD_T          scph_prmoff_l;

    /*    This field defines the number of parameters required by this script. */

    PSL_WORD_T              scph_prmcnt_w;

    PSL_WORD_T              scph_spare2_w;

    /*    This field is UNIX specific. It defines the User Id of the owner */
    /*    of the script. It is set by SLB to the effective user id. of the */
    /*    user who adds the script to the script library. */

    PSL_LONGWORD_T          scph_ownuid_l;

    /*    This field is UNIX specific. It defines the Group Id of the owner */
    /*    of the script. It is set by SLB to the effective group id. of the */
    /*    user who adds the script to the script library. */

    PSL_LONGWORD_T          scph_owngid_l;

    /*    This field defines the version of SCL that compiled the script. */

    PSL_LONGWORD_T          scph_compile_vrn_l;
#define            SCP_CMPVRN 0X52



    /* The following fields are to indicate if there is any platform */
    /* or operating system specific code in the script. It is stored in */
    /* the header and then checked in LGR to check the validity of running */
    /* the script on the current platform or OS. */


    /*    This field holds a longword and determines the platform the */
    /*    script was compiled on. */

    PSL_LONGWORD_T          scph_compile_plat_l;
#define            SCPH_PLAT_NTI 1
#define            SCPH_PLAT_VAX 3
#define            SCPH_PLAT_ALPHA 4
#define            SCPH_PLAT_SUN 5
#define            SCPH_PLAT_HP 6
#define            SCPH_PLAT_IBM 7
#define            SCPH_PLAT_NCR 8
#define            SCPH_PLAT_OSF 9

    /*    This field holds a longword and determines the operating system the */
    /*    script was compiled on. */

    PSL_LONGWORD_T          scph_compile_opsys_l;
#define            SCPH_OS_VMS 1
#define            SCPH_OS_UNIX 2
#define            SCPH_OS_WIN32 3

    /*    This field indicates if there was any platform specific */
    /*    code in the script. */

    Boolean                 scph_plat_spec_code_n;

    /*    This field indicates if there was any operting system specific */
    /*    code in the script. */

    Boolean                 scph_opsys_spec_code_n;


    /*  MORE SPARE BYTES */


#ifdef VMS
    char                    scph_spare3_c216[216];
#endif
#ifdef WIN32
    char                    scph_spare3_c216[216];
#endif
#ifdef UNIX
#ifdef OSF
    char scph_spare3_c216[48];
#else
char scph_spare3_c216[216];
#endif
#endif


} ;


/*  End of C Include File */

#endif  /* PSL__SCPDEF_LOADED */
